
package org.netbeans.modules.asm.core.editor;

import java.util.Collections;
import java.util.HashSet;
import org.netbeans.editor.Syntax;
import org.netbeans.editor.TokenID;

/**
 *
 * @author am204439
 */
public class AsmSyntax extends Syntax {
    
    private static final int ISI_INDEF = 1;
    private static final int ISI_REG = 2;
    private static final int ISI_BLOCK = 3;
    private static final int ISI_NUM = 4;
    private static final int ISI_COMMENT = 5;
      
    private StringBuilder lex;
    
    public AsmSyntax() {
        tokenContextPath = AsmTokenContext.contextPath;
        lex = new StringBuilder();              
    }
    
    protected TokenID parseToken() {
        TokenID result = doParseToken();              
        return result;
    }
    
    private TokenID doParseToken() {
        char actChar;
        
        while (offset < stopOffset) {
            actChar = buffer[offset];
             
            switch (state) {
                case INIT:
                    if (Character.isLetter(actChar)) {
                        state = ISI_INDEF;                        
                    }
                    else if (Character.isDigit(actChar)) {
                        state = ISI_NUM;
                    }
                    else if (actChar == '%') {
                        state = ISI_REG;
                    }
                    else if (actChar == '.') {
                        state = ISI_BLOCK;
                    } 
                    else if (actChar == '/') {
                        state = ISI_COMMENT;
                    }
                    else if (actChar == '(' || actChar == ')' ||
                             actChar == ',') {
                        offset++; 
                        return AsmTokenContext.COLON;
                    }                                         
                    lex.setLength(0);                        
                    lex.append(actChar);        
                    break;
                    
                case ISI_NUM:
                    if (actChar == ':') {
                        state = INIT;
                        offset++;
                        return AsmTokenContext.LABEL;
                    }
                    if (!Character.isDigit(actChar)) {
                        state = INIT;
                        if (Character.isLetter(actChar))
                            offset++;
                        return AsmTokenContext.NUMBER;
                    }
                    break;
                    
                case ISI_COMMENT:
                    if (actChar == '\n') {
                        state = INIT;
                        offset++;
                        return AsmTokenContext.COMMENT;
                    }
                    break;
                    
                case ISI_INDEF: case ISI_REG: case ISI_BLOCK:
                    if (state == ISI_INDEF && actChar == ':') {
                        state = INIT;
                        offset++;
                        return AsmTokenContext.LABEL;
                    }
                    else if (!Character.isJavaIdentifierPart(actChar)) {                                                
                        //offset++;
                        if (state == ISI_REG) {
                            state = INIT;
                            return AsmTokenContext.REGISTER;
                        }
                        else if (state == ISI_BLOCK) {
                            state = INIT;
                            return AsmTokenContext.SECTION;
                        }
                        else if(state == ISI_INDEF) {
                            state = INIT;
                            return AsmTokenContext.INSTRUCTION;
                        }                        
                    }
                    
                    lex.append(actChar);
                    break;
            }
           
            offset++;
        }
                     
        return null;
    }
   
}
