/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml;

import java.io.*;
import org.w3c.dom.*;
import java.util.Properties;

/**
 *
 * @author Robert Novak
 */
public class ModuleConfigData {

    private String module_code_name;
    private Properties module_params = new Properties();

    ModuleConfigData(NBMfile nbmFile, String moduleCodename) {
        this.module_code_name = moduleCodename;

        InputStream inp_stream = nbmFile.getModuleConfigStream(moduleCodename);
        if (inp_stream != null) {
            loadFromDocument(XMLutils.openXML(inp_stream));
        }

    }

    private void loadFromDocument(Document configXMLFile) {
        Element element;
        element = configXMLFile.getDocumentElement();

        if ((element != null) && element.getTagName().equals("module")) {
            scanElementModule(element);
        }
    }

    /**
     * Scans module element
     */
    private void scanElementModule(Element element) { // <module>
        String attr_name, attr_value;

        NamedNodeMap attrs = element.getAttributes();
        for (int i = 0; i < attrs.getLength(); i++) {
            Attr attr = (Attr) attrs.item(i);
            attr_name = attr.getName();
            attr_value = attr.getValue();

            if (attr_name.equals("name") && attr_value.equals(module_code_name)) {

                NodeList nodes = element.getChildNodes();

                scanParamSection(nodes);
                break;
            }
        }
    }

    private void scanParamSection(NodeList nodes) {
        for (int i = 0; i < nodes.getLength(); i++) {
            Node one_node = nodes.item(i);

            if (one_node.getNodeType() == Node.ELEMENT_NODE) {
                Element nodeElement = (Element) one_node;
                if (nodeElement.getTagName().equals("param")) {
                    scanElementParam(nodeElement);
                }
            }
        }

    }

    private void scanElementParam(Element element) {
        String attr_name;
        String param_name = null;
        String param_value = null;

        NamedNodeMap attrs = element.getAttributes();
        for (int i = 0; i < attrs.getLength(); i++) {
            Attr attr = (Attr) attrs.item(i);
            attr_name = attr.getName();

            if (attr_name.equals("name")) {
                param_name = attr.getValue();
                break;
            }
        }

        NodeList nodes = element.getChildNodes();

        Node one_node;
        for (int i = 0; i < nodes.getLength(); i++) {
            one_node = nodes.item(i);

            if (one_node.getNodeType() == Node.TEXT_NODE) {
                param_value = ((Text) one_node).getData();
            }
        }

        if (param_name != null) {
            module_params.setProperty(param_name, param_value);
        }
    }

    public String getEagerValue() {
        return (module_params.getProperty("eager"));
    }

    public String getAutoloadValue() {
        return (module_params.getProperty("autoload"));
    }
}
