/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.auxml.web;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.netbeans.auxml.Version;
import org.netbeans.auxml.service.NewTypeForm;
import org.springframework.web.servlet.ModelAndView;
import org.springframework.web.servlet.mvc.SimpleFormController;
import org.springframework.web.servlet.view.RedirectView;
import static org.netbeans.auxml.web.Config.getManager;

/**
 *
 * @author mschovanek@netbeans.org
 */
public class NewTypeFormController extends SimpleFormController {

    /** Logger for this class and subclasses */
    protected final Log LOG = LogFactory.getLog(getClass());

    @Override
    public ModelAndView onSubmit(Object command) throws ServletException, IOException {
        NewTypeForm type = (NewTypeForm) command;
        getManager().createUpdatCenter(type.getVersion(), type.getName(), type.getTemplate());
        return new ModelAndView(new RedirectView(getSuccessView()));
    }
    
    @Override
    public ModelAndView handleRequest(HttpServletRequest request, HttpServletResponse response)
            throws ServletException, Exception {
        ModelAndView model = super.handleRequest(request, response);
        List<Version> versions = getManager().getVersions();
        List<String> verNames = new ArrayList<String>(versions.size());
        for (Version version : versions) {
            verNames.add(version.getName());
        }
        File[] templates = getManager().getTemplates();
        List<String> tempNames = new ArrayList<String>(templates.length);
        for (File file : templates) {
            tempNames.add(file.getName());
        }
        Map map = model.getModel();
        map.put("versions", verNames);
        map.put("templates", tempNames);
        return model;
     }

    @Override
    protected Object formBackingObject(HttpServletRequest request) throws ServletException {
        NewTypeForm ver = new NewTypeForm();
        return ver;
    }
}