/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.actionperformers;

import java.util.Enumeration;
import javax.swing.SwingUtilities;
import org.netbeans.api.java.project.JavaProjectConstants;
import org.netbeans.api.progress.ProgressHandle;
import org.netbeans.api.progress.ProgressHandleFactory;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.api.project.SourceGroup;
import org.netbeans.modules.coverage.ProjectCoverageActivityOnAction;
import org.openide.cookies.EditorCookie;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.loaders.DataObjectNotFoundException;
import org.openide.nodes.Node;
import org.openide.util.NbBundle;

/**
 * The class updates the markup for all opened files
 * @author mikk@netbeans.org
 */
public class RefreshProjectOpenedEditors implements Runnable{
    
    private Project project = null;
    private String name = null;
    
    /**
     * Creates new instance based on the project and command name
     * @param project Project
     * @param name Command name
     */
    public RefreshProjectOpenedEditors(Project project, String name){
        this.project = project;
        this.name = name;
    }
    
    /**
     * Refreshes project's opened java files markup with new coverage data
     * @param project Project which sources' markup will be refreshed
     */
    public static void refreshProject(Project project){
        SwingUtilities.invokeLater(new Refresher(project));
        
    }
    
    /**
     *Runs the refresher
     **/
    public void run() {
        ProgressHandle progress = ProgressHandleFactory.createHandle(name); // NOI18N
        progress.start();
        
        refreshProject(project);
        
        if (progress != null) progress.finish();
    }
    
    /**
     *The class iterates through the project nodes and refreshes markup in case 
     * the associated class is opened.  
     **/
    public static class Refresher implements Runnable{
        private Project project = null;
        /**
         * Creates new Refresher instance
         * @param projectToRefresh Project which sources will be refreshed
         */
        public Refresher(Project projectToRefresh){
            this.project = projectToRefresh;
        }
        
        /**
         *Refreshes all opened java files of the project
         **/
        public void run() {
            SourceGroup[] sourceGroups = ProjectUtils.getSources(project).getSourceGroups(JavaProjectConstants.SOURCES_TYPE_JAVA);
            for(int i=0;i<sourceGroups.length;i++){
                Enumeration fileObjEnum = sourceGroups[i].getRootFolder().getChildren(true);
                while(fileObjEnum.hasMoreElements()){
                    FileObject fileObj = (FileObject) fileObjEnum.nextElement();
                    if (!fileObj.getExt().equals("java")){
                        continue;
                    }
                    try{
                        DataObject dataObj = DataObject.find(fileObj);
                        
                        Node node = dataObj.getNodeDelegate();
                        EditorCookie editorCookie = (EditorCookie)node.getCookie(EditorCookie.class);
                        if (editorCookie!=null && editorCookie.getOpenedPanes()!=null){
                            new ClassMarkupCoverage(node,
                                    NbBundle.getMessage(ProjectCoverageActivityOnAction.class, "CTL_Class_MarkupCoverage")).run();
                        }
                        
                    }catch(DataObjectNotFoundException ex){
                        continue;
                    }
                    
                }
            }
            
        }
    }
    
}
