/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.config;

import java.util.Properties;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.api.project.ui.OpenProjects;
import org.netbeans.modules.coverage.annotations.AbstractCoverageAnnotation;
import org.netbeans.modules.coverage.annotations.OpenEditorListener;
import org.netbeans.modules.coverage.exceptions.PluginNotConfiguredException;
import org.netbeans.modules.coverage.instrumentation.CoveragePreparers;
import org.netbeans.modules.coverage.instrumentation.ProjectCoveragePreparer;
import org.netbeans.modules.coverage.utils.ModuleActivityModes;
import org.openide.modules.ModuleInstall;
import org.openide.windows.TopComponent;

/**
 * The class performs some specific actions when the module is installed or uninstalled
 * @author mikk@netbeans.org
 */
public class CoverageModuleInstall extends ModuleInstall{
    
    
    /**
     * The method is called when the module is loaded/activated (See ModuleInstall class for details)
     * We add listener that checks for all new opened editors and colors them according to coverage data
     */
    public void restored(){
        //TODO: check how remove and add work. Probably trycatch may be removed
        TopComponent.Registry registry = TopComponent.getRegistry();
        try{
            registry.removePropertyChangeListener(OpenEditorListener.getInstance());
        }catch(Exception e){}
        registry.addPropertyChangeListener(OpenEditorListener.getInstance());
    }
    
    /**
     * This is called when the IDE is closed
     * Currently we turn off the coverrage collection every time user closes IDE
     */
    public void close(){
        Project[] projects = OpenProjects.getDefault().getOpenProjects();
        for(int i=0;i<projects.length;i++){
            Properties props = null;
            try{
                Config config = Config.getInstanceForProject(projects[i]);
                props = config.getPluginProperties();
            }catch(PluginNotConfiguredException e){
                continue;
            }
            if (!props.get("coverage.activity").equals(ModuleActivityModes.ON)){
                continue;
            }
            
            props.put("coverage.activity",ModuleActivityModes.OFF);
            
            Config.storeProjectProperties(projects[i], props);
            //cleanup the project
            ProjectCoveragePreparer preparer = CoveragePreparers.getPreparerForProject(projects[i]);
            if(preparer != null){
                preparer.cleanup();
            }
        }
    }
    
    
    /**
     * This is called when user uninstalls the module and removes all added annotations
     * To avoid memory leaks and some exceptions all annotations have to be removed
     */
    public void uninstalled(){
        Project[] projects = OpenProjects.getDefault().getOpenProjects();
        for(int i=0;i<projects.length;i++){
            AbstractCoverageAnnotation.removeAll(ProjectUtils.getInformation(projects[i]).getName());            
        }
    }
    
}
