/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.data;

import java.util.Hashtable;
import org.netbeans.modules.coverage.utils.CountRate;
import org.netbeans.modules.coverage.utils.LineCoverageStates;

/**
 * Interface that represents the object that holds class coverage data
 * @author mikk@netbeans.org
 */
public interface IClassCoverageData {
    /**
     * Returns covered lines number and associated states for the specified range of lines
     * @param firstLineNum Starting line number
     * @param lastLineNum End line number
     * @return Hashtable containing mapping between line numbers and their coverage states
     */
    public Hashtable<Integer, LineCoverageStates> getCoveredLines(int firstLineNum, int lastLineNum);
    
    /**
     * Returns all covered lines and their states
     * @return The hash that contains mapping between covered lines numbers and their state
     */
    public Hashtable<Integer, LineCoverageStates> getAllCoveredLines();
    
    /**
     * Returns the specific line state
     * @param lineNum Nubmer of line
     * @return Coverage state of the specified line
     */
    public LineCoverageStates getLineCoverage(int lineNum);
    
    String getName();
    
    boolean isCovered();
    
    CountRate getOverallLineCoverage();
}
