/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.instrumentation;

import java.io.File;
import java.util.HashMap;
import org.apache.tools.ant.module.spi.AntEvent;
import org.apache.tools.ant.module.spi.AntLogger;
import org.apache.tools.ant.module.spi.AntSession;
import org.netbeans.api.project.Project;
import org.netbeans.modules.coverage.actionperformers.RefreshProjectOpenedEditors;
import org.netbeans.modules.coverage.config.Config;
import org.netbeans.modules.coverage.instrumentation.AntSessionContext;
import org.netbeans.modules.coverage.instrumentation.CoverageUtils;
import org.netbeans.modules.coverage.utils.ModuleActivityModes;

/**
 * This class is used to track actions performed on the project and invokes some
 * coverage module specific targets to generate coverage data when tests are executed
 * TODO: This class will be moved to different package later
 * @author mikk@netbeans.org, Alexander Pepin
 */
public class CoverageAntLogger extends AntLogger{
    
    private StartTargetDispatcher startTarget = new StartTargetDispatcher();
    private FinishTargetDispatcher finishTarget = new FinishTargetDispatcher();
    /** Creates a new instance of CoverageAntLogger */
    public CoverageAntLogger() {
        initTargetsMap();
    }
    
        
    public boolean interestedInSession(AntSession session) {
        return true;
    }
    
    public boolean interestedInAllScripts(AntSession session){
        return true;
    }
    
    public boolean interestedInScript(File script, AntSession session) {
        return true;
    }
    
    public String[] interestedInTargets(AntSession session) {
        return AntLogger.ALL_TARGETS;
    }
    
    public String[] interestedInTasks(AntSession session) {
        return AntLogger.NO_TASKS;
    }
    
    public void targetStarted(AntEvent antEvent) {
        processAntEvent(antEvent, startTarget);
    }
    
    public void targetFinished(AntEvent antEvent) {
        processAntEvent(antEvent, finishTarget);
    }
    
    
   
    
    private void processAntEvent(AntEvent antEvent, TargetDispatcher dispatcher){
        String antTarget = antEvent.getTargetName();
        if (isAcceptedTarget(antTarget)){
            AntSessionContext context = new AntSessionContext(antEvent);
           
            Project project = context.getOwnerProject();
            if (project != null && ModuleActivityModes.isModuleActive(project)){
                //perform actions based on the target
                dispatcher.dispatch(targets.get(antTarget), context);                
            }
        }
    }
    
    class StartTargetDispatcher implements TargetDispatcher{
        public void dispatch(Targets target, AntSessionContext context){
            switch(target){
                //for java project
                case DO_TEST_RUN:
                case DO_TEST_RUN_SINGLE:
                    if(Config.isJavaProject(context.getOwnerProject())){
                        CoverageUtils.prepareJavaProject(context);
                    }
                    break;
                    //for nbmodule project
                case TEST_START:
                    if(Config.isNbModuleProject(context.getOwnerProject())){
                        CoverageUtils.prepareNbProject(context);
                    }
                    break;
            }
        }
    }
    
    class FinishTargetDispatcher implements TargetDispatcher{
        public void dispatch(Targets target, AntSessionContext context){
            switch(target){

                //for java project
                case DO_TEST_RUN:
                case DO_TEST_RUN_SINGLE:
                    if(Config.isJavaProject(context.getOwnerProject())){
                        CoverageUtils.completeJavaProject(context);
                        RefreshProjectOpenedEditors.refreshProject(context.getOwnerProject());
                    }
                    break;

                case TEST_END:
                case TEST_END_UNIT:
                    if(Config.isNbModuleProject(context.getOwnerProject())){
                        //return jar get coverage
                        CoverageUtils.completeNbProject(context);
                        RefreshProjectOpenedEditors.refreshProject(context.getOwnerProject());
                    }
                    break;
            }
        }
    }
    
    
    private HashMap<String, Targets> targets = new HashMap<String, Targets>();
    
    private void initTargetsMap(){
        for(Targets target: Targets.values()){
            targets.put(target.toString(), target);
        }
    }
    
    private boolean isAcceptedTarget(String target){
        return targets.containsKey(target);
    }
    
    //desired targets should be defined here
    enum Targets {//COV_RESTORE("cov-restore"),
        //java project targets
        DO_TEST_RUN("-do-test-run"),
        DO_TEST_RUN_SINGLE("-do-test-run-single"),
        TEST_START("test-dep-build"),
        TEST_END("test"),
        TEST_END_UNIT("test-unit");
        
        private Targets(String name){targetName = name;}
        public String toString(){return targetName;}
        private String targetName;
    };
    
    interface TargetDispatcher{
        public void dispatch(Targets target, AntSessionContext context);
    }
}

