/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.coverage.instrumentation;

import java.io.File;
import java.io.IOException;
import java.util.Properties;
import org.apache.tools.ant.module.api.AntProjectCookie;
import org.apache.tools.ant.module.api.AntTargetExecutor;
import org.openide.execution.ExecutorTask;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;

/**
 *
 * @author apepin
 */
public class CoverageUtils {
    static final String fsRoot = System.getProperty("os.name").toLowerCase().startsWith("windows")? "" : "/";
    static private String emmaPath = "";
    
    public static void setEmmaPath(String pathToEmma){
        emmaPath = normalizePath(pathToEmma);
    }
    
    public static String getEmmaPath(){
        return emmaPath;
    }
    
    /** Creates a new instance of CoverageUtils */
    public CoverageUtils() {
    }
    
       
    public static void prepareJavaProject(AntSessionContext context){
        try{
            FileObject projectDir = context.getOwnerProject().getProjectDirectory();
            FileObject pluginDir = projectDir.getFileObject("coverage");
            FileObject classesDir = projectDir.getFileObject(context.getProperty("build.classes.dir"));
            //copy classes to ${plugindir}/tmp
            FileObject tmp = recreateDir(pluginDir, "tmp");
            copyDir(classesDir, tmp);
            //instrument ${build.dir}/classes
            FileObject script = pluginDir.getFileObject("emmascript.xml");
            int res = instrumentSourceFiles(script,
                    fsRoot+classesDir.getPath(),
                    fsRoot+pluginDir.getPath());
            if (res!=0){
                System.out.println("Project instrumentation failed");
                return;
            }
        }catch(IOException e){
            e.printStackTrace();
        }
    }
    
    
    
    public static void completeJavaProject(AntSessionContext context){
        try{
            FileObject projectDir = context.getOwnerProject().getProjectDirectory();
            FileObject pluginDir = projectDir.getFileObject("coverage");
            //copy ${plugindir}/tmp to ${builddir}/classes
            FileObject tmp = pluginDir.getFileObject("tmp");
            FileObject classesDir = recreateDir(normalizePath(fsRoot+
                    projectDir.getFileObject(context.getProperty("build.classes.dir")).getPath()));
            copyDir(tmp, classesDir);
            tmp.delete();
            //copy coverage.ec to ${plugindir}/coverage.emma
            FileObject old_coverage = pluginDir.getFileObject("coverage", "emma");
            if(old_coverage != null)
                old_coverage.delete();
            FileObject coverage = toFileObject(context.getProperty("work.dir")+"/coverage.ec");
            // skip if the coverage.ec was not created during the test execution
            if (coverage != null ) {
                coverage.copy(pluginDir, "coverage", "emma");
                coverage.delete();
            }
            
        }catch(IOException e){
            e.printStackTrace();
        }
    }
    
    public static void prepareNbProject(AntSessionContext context){
        try{
            FileObject projectDir = context.getOwnerProject().getProjectDirectory();
            FileObject pluginDir = projectDir.getFileObject("coverage");
            FileObject jarfile = toFileObject(normalizePath(context.getProperty("cluster")+"/"+context.getProperty("module.jar")));
            //copy jar to ${plugindir}/tmp
            FileObject tmp = recreateDir(pluginDir, "tmp");
            jarfile.copy(tmp, context.getProperty("module.jar.basename"), null);
            //instrument jar file
            FileObject script = pluginDir.getFileObject("emmascript.xml");
            int res = instrumentSourceFiles(script,
                    fsRoot+jarfile.getPath(),
                    fsRoot+pluginDir.getPath());
            if (res!=0){
                System.out.println("Project instrumentation failed");
                return;
            }
        }catch(IOException e){
            e.printStackTrace();
        }
    }
    
    public static void completeNbProject(AntSessionContext context){
        /*
        try{
            FileObject projectDir = context.getOwnerProject().getProjectDirectory();
            FileObject pluginDir = projectDir.getFileObject("coverage");
            
            FileObject jarfile = toFileObject(normalizePath(context.getProperty("cluster")+"/"+context.getProperty("module.jar")));
            jarfile.delete();
            //copy jar to original location
            FileObject moduleDir = toFileObject(normalizePath(context.getProperty("cluster")+"/"+context.getProperty("module.jar.dir")));
            FileObject tmp = pluginDir.getFileObject("tmp");
            String jarname = context.getProperty("module.jar.basename");
            tmp.getFileObject(jarname).copy(moduleDir, jarname, null);
            tmp.delete();
            //copy coverage.ec to pluginDir/coverage.emma
            FileObject old_coverage = pluginDir.getFileObject("coverage", "emma");
            if(old_coverage != null)
                old_coverage.delete();
            FileObject coverage = toFileObject(context.getProperty("user.dir")+"/coverage.ec");
            coverage.copy(pluginDir, "coverage", "emma");
            coverage.delete();
            
        }catch(IOException e){
            e.printStackTrace();
        }*/
        try{
            
            /*
             The current 'problem' with the implementation below is that I can not                
             restore the original jar after finishing running tests. Why? Because of some
             threading issue the instrumented jar is not released even after task.result() returns
             *
             TODO: recheck this!
            */
            FileObject projectDir = context.getOwnerProject().getProjectDirectory();
            FileObject pluginDir = projectDir.getFileObject("coverage");
            
            FileObject old_coverage = pluginDir.getFileObject("coverage", "emma");
            if(old_coverage != null)
                old_coverage.delete();
            FileObject coverage = toFileObject(context.getProperty("user.dir")+"/coverage.ec");
            coverage.copy(pluginDir, "coverage", "emma");
            coverage.delete();
            
            FileObject scriptToExecute = projectDir.getFileObject("build", "xml");
            DataObject dataObj = DataObject.find(scriptToExecute);
            AntProjectCookie antCookie = (AntProjectCookie) dataObj.getCookie(AntProjectCookie.class);
            AntTargetExecutor.Env env = new AntTargetExecutor.Env();
            AntTargetExecutor executor = AntTargetExecutor.createTargetExecutor(env);

            Properties props = new Properties();

            env.setProperties(props);
            ExecutorTask task = executor.execute(antCookie, new String[]{"clean"});
            task.result();
            
        }catch(IOException e){
            e.printStackTrace();
        }
    }
    
    public static int instrumentSourceFiles(FileObject scriptToExecute, String sourcePath, String outputPath) throws IOException{
        
        //delete old template.emma
        File old_template = new File(outputPath+"/template.emma");
        if(old_template.exists())
            old_template.delete();
        
        DataObject dataObj = DataObject.find(scriptToExecute);
        AntProjectCookie antCookie = (AntProjectCookie)dataObj.getCookie(AntProjectCookie.class);
        AntTargetExecutor.Env env = new AntTargetExecutor.Env();
        AntTargetExecutor executor = AntTargetExecutor.createTargetExecutor(env);

        Properties props = new Properties();
        props.setProperty("jarfiles", sourcePath);
        props.setProperty("emma", emmaPath);
        props.setProperty("output.dir", outputPath);
        env.setProperties(props);
        ExecutorTask task = executor.execute(antCookie, new String[]{"instr"});
        return task.result();
    }
    
    public static FileObject toFileObject(String absolutePath){
        File file = new File(absolutePath);
        return FileUtil.toFileObject(file);
    }
    
    public static void copyDir(FileObject source, FileObject target) throws IOException{
        if(source.isFolder()){
            for(FileObject child: source.getChildren()){
                if(child.isFolder()){
                    FileObject subTarget = target.createFolder(child.getName());
                    copyDir(child, subTarget);
                }
                else{
                    child.copy(target, child.getName(), child.getExt());
                }
            }
        }
    }
    
    public static FileObject recreateDir(FileObject parentDir, String dirName) throws IOException{
        FileObject tmp = parentDir.getFileObject(dirName);
        if(tmp != null){
            tmp.delete();
        }
        return parentDir.createFolder(dirName);
    }
    
    public static FileObject recreateDir(String dirPath) throws IOException{
        File dir = new File(dirPath);
        FileObject tmp = FileUtil.toFileObject(dir);
        if(tmp != null){
            tmp.delete();
        }
        if(dir.mkdir())
            return FileUtil.toFileObject(dir);
        else
            return null;
    }
    
    public static String normalizePath(String path){
        String newPath = path.replace('\\','/');
        int index = -1;
        while((index = newPath.indexOf("/..")) > 0 ){
            int baseIndex = newPath.substring(0, index).lastIndexOf('/');
            String tail = newPath.substring(index+3);
            newPath = newPath.substring(0, baseIndex)+tail;
        }
        return newPath;
    }
    
    
}
