/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.coverage.utils;

import org.openide.windows.IOProvider;
import org.openide.windows.InputOutput;
import org.openide.windows.OutputWriter;

/**
 * The class is used to write some messages or debug information to the output tabs in NB
 * @author Andrey Korostelev
 */
public class Debugger {
    
    private final static String TAB_TITLE = "Coverage module output";
    
    
    /** Creates a new instance of Debugger */
    public Debugger() {
    }
    
    /**
     * Logs specified message in the specified tab
     * @param tabTitle Tab title
     * @param msg message to log
     */
    public static void log(String tabTitle, String msg){
        InputOutput io = IOProvider.getDefault().getIO(tabTitle, false);
        io.select();
        OutputWriter writer = io.getOut();
        writer.println("\n"+msg); //write tag to output window
        writer.flush();
        writer.close();
    }

    
    /**
     * Logs message into default tab
     * @param msg Message to log
     */
    public static void log(String msg){
        log(TAB_TITLE, msg);
    }
    
    /**
     * Writes error message to the specified tab
     * @param tabTitle Tab to write message to
     * @param msg Message to write
     */
    public static void error(String tabTitle, String msg){
        InputOutput io = IOProvider.getDefault().getIO(tabTitle, false);
        io.select();
        OutputWriter writer = io.getErr();
        writer.println("\nERROR: "+msg); //write tag to output window
        writer.flush();
        writer.close();
    }

    
    /**
     * Writes error message to the default tab
     * @param msg Message to write
     */
    public static void error(String msg){
        error(TAB_TITLE, msg);
    }
    
    /**
     * Writes debug message to the specified tab
     * @param tabTitle Tab to write message to
     * @param msg Message to write
     */
    public static void debug(String tabTitle, String msg){
        InputOutput io = IOProvider.getDefault().getIO(tabTitle, false);
        io.select();
        OutputWriter writer = io.getOut();
        writer.println("\nDEBUG: "+msg); //write tag to output window
        writer.flush();
        writer.close();
    }

    
    /**
     * Writes debug message to default tab
     * @param msg Message to write
     */
    public static void debug(String msg){
        debug(TAB_TITLE, msg);
    }
    
    
}
