/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm;

import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.PrintWriter;
import org.openide.DialogDisplayer;
import org.openide.NotifyDescriptor;
import org.openide.compiler.Compiler;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataObject;

public class JRAWCompiler extends org.openide.compiler.Compiler {

    DataObject dao;
    
    public JRAWCompiler (DataObject dao) {
        this.dao = dao;
    }
    
    public Class compilerGroupClass() {
        return JRAWCompilerGroup.class;
    }
    
    protected boolean isUpToDate() {
        return false;
    }
    
    public boolean compile () {
        final boolean[] ret = new boolean[] { false };
        try {
            final FileObject input = dao.getPrimaryFile ();
            input.getFileSystem ().runAtomicAction (new FileSystem.AtomicAction () {
                public void run () throws IOException {
                    FileObject output = FileUtil.findBrother(input, JRAWDataLoader.EXT_CLASS);
                    if (output == null)
                        output = input.getParent ().createData (input.getName (), JRAWDataLoader.EXT_CLASS);
                    FileLock fl = null;
                    InputStream is = null;
                    OutputStream os = null;
                    try {
                        fl = output.lock ();
                        is = input.getInputStream ();
                        os = output.getOutputStream (fl);
                        org.netbeans.modules.jasm.parser.Parser.jraw2classIO (is, os);
                        ret[0] = true;
                    } catch (Exception e) {
                        DialogDisplayer.getDefault().notify(new NotifyDescriptor.Exception (e));
                    } finally {
                        if (fl != null)
                            try { fl.releaseLock (); } catch (Exception e) {}
                        if (is != null)
                            try { is.close (); } catch (Exception e) {}
                        if (os != null)
                            try { os.close (); } catch (Exception e) {}
                        input.getParent ().refresh (true);
                    }
                }
            });
        } catch (Exception e) {
            DialogDisplayer.getDefault().notify(new NotifyDescriptor.Exception (e));
        }
        return ret[0];
    }
    
}
