/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.*;
import java.util.ArrayList;
import java.util.Hashtable;

public class ConstantPool {

	Parser parent;
	int count;
	CPElement[] array;
    ExtStringArray esa;
    Hashtable labels;
	
	public ConstantPool (Parser p) {
		parent = p;
		count = 0;
        esa = null;
        labels = new Hashtable ();
	}
	
	void fromClass (DataInputStream dis) throws Exception {
		count = dis.readUnsignedShort () - 1;
		array = new CPElement[count];
		for (int a = 0; a < count; a ++) {
			int b = dis.readUnsignedByte ();
			CPElement o;
			switch (b) {
				case  1: o = new CPUTF8 (this); break;
				case  3: o = new CPInteger (this); break;
				case  4: o = new CPFloat (this); break;
				case  5: o = new CPLong (this); break;
				case  6: o = new CPDouble (this); break;
				case  7: o = new CPClass (this); break;
				case  8: o = new CPString (this); break;
				case  9: o = new CPField (this); break;
				case 10: o = new CPMethod (this); break;
				case 11: o = new CPIntMethod (this); break;
				case 12: o = new CPNameType (this); break;
				default: throw new ParserException ("Unknown type: " + b + ", pos: " + (a + 1));
			}
			o.fromClass (dis);
			array[a] = o;
			if (b == 5  ||  b == 6) {
				a ++;
				array[a] = null;
			}
				
		}
	}
	
	void toClass (DataOutputStream dos) throws Exception {
		dos.writeShort (count + 1);
		for (int a = 0; a < count; a ++)
			if (array[a] != null)
				array[a].toClass (dos);
	}
	
	void fromJRAW (Lexan lex) throws Exception {
		ArrayList al = new ArrayList ();
        labels = new Hashtable ();
		lex.cmpLexel ("ConstantPool");
		lex.cmpLexel ("{");
		boolean add = false;
		for (;;) {
			String s = lex.getLexel ();
			lex.cmpLexel ();
			CPElement o;
			if ("}".equals (s))
				break;
            else if (s != null  &&  s.endsWith(":")) {
                String ls = "#" + s.substring(0, s.length () - 1);
                if (labels.get (ls) != null) {
                    throw new ParserException ("Already defined label: " + ls);
                }
                labels.put (ls, new Integer (al.size () + 1));
                continue;
            } else if (CPUTF8.ID.equals (s))
				o = new CPUTF8 (this);
			else if (CPInteger.ID.equals (s))
				o = new CPInteger (this);
			else if (CPFloat.ID.equals (s))
				o = new CPFloat (this);
			else if (CPLong.ID.equals (s)) {
				o = new CPLong (this);
				add = true;
			} else if (CPDouble.ID.equals (s)) {
				o = new CPDouble (this);
				add = true;
			} else if (CPClass.ID.equals (s))
				o = new CPClass (this);
			else if (CPString.ID.equals (s))
				o = new CPString (this);
			else if (CPField.ID.equals (s))
				o = new CPField (this);
			else if (CPMethod.ID.equals (s))
				o = new CPMethod (this);
			else if (CPIntMethod.ID.equals (s))
				o = new CPIntMethod (this);
			else if (CPNameType.ID.equals (s))
				o = new CPNameType (this);
			else
				throw new ParserException ("Unknown type: " + s);
			o.fromJRAW (lex);
			al.add (o);
			if (add) {
				al.add (null);
				add = false;
			}
		}
		count = al.size ();
		array = new CPElement[count];
        for (int a = 0; a < count; a ++) {
			array[a] = (CPElement) al.get (a);
            if (array[a] != null)
                array[a].solveLabel ();
        }
	}

	void toJRAW (ExtStringBuffer esb) {
		esb.println ("ConstantPool {");
		esb.incIndent ();
		for (int a = 0; a < count; a ++)
			if (array[a] != null)
				esb.println ("cp" + (a + 1) + ": " + array[a].toString () + " // " + array[a].publish ());
		esb.decIndent ();
		esb.println ("}");
	}
    
    public int getLabelPos (String label) throws Exception {
        Object o = labels.get (label);
        if (o == null) {
            throw new ParserException ("Undefined label: " + label);
        }
        return ((Integer) o).intValue();
    }
	
	public CPElement get (int index) {
		index --;
		if (index < 0  ||  index >= count)
			return null;
		return array[index];
	}
	
	public String getUTF8Value (int index) {
		index --;
		if (index < 0  ||  index >= count)
			return null;
		if (!(array[index] instanceof CPUTF8))
			return null;
		return ((CPUTF8) array[index]).getValue ();
	}

	public String getUTF8String (int index) {
		index --;
		if (index < 0  ||  index >= count)
			return null;
		if (!(array[index] instanceof CPUTF8))
			return null;
		return array[index].toString ();
	}
    
    public String publish (int index) {
        index --;
		if (index < 0  ||  index >= count  ||  array[index] == null)
			return "";
		return array[index].publish ();
    }

}
