/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.parser;

import java.io.BufferedReader;
import java.io.IOException;

public class Lexan {

	BufferedReader buf;
	boolean characterReaded;
	int character;
	boolean lexelReaded, doubleReaded, longReaded;
	boolean isDouble, isLong, isString;
	String lexelValue;
	double doubleValue;
	long longValue;
	String stringValue;
	
	class EOFException extends Exception { }
	
	public Lexan (BufferedReader br) {
		buf = br;
		isDouble = isLong = false;
		characterReaded = false;
	}
	
	public void close () throws IOException {
		buf.close ();
	}

	public int get () throws Exception {
		if (!characterReaded) {
			character = buf.read ();
			if (character == -1)
				throw new EOFException ();
			characterReaded = true;
		}
		return character;
	}
	
	public int next () throws Exception {
		if (!characterReaded)
			buf.read ();
		else
			characterReaded = false;
		return get ();
	}
	
	public String getLexel () throws Exception {
		if (!lexelReaded) {
			doubleReaded = longReaded = false;
			StringBuffer sb = new StringBuffer ();
			int c = get ();
			for (;;) {
				if (c <= ' ')
					c = next ();
				else if (c == '/') {
					c = next ();
					if (c == '/') {
						c = next ();
						while (c != '\n')
							c = next ();
					} else if (c == '*') {
						c = next ();
						for (;;) {
							if (c == '*') {
								c = next ();
								if (c == '/') {
									c = next ();
									break;
								}
							} else
								c = next ();
						}
					} else {
						sb.append ((char) c);
						break;
					}
				} else
					break;
			}
			try {
				if (c == '"') {
					isString = true;
					sb.append ((char) c);
					c = next ();
					while (c != '"') {
						if (c == '\\') {
						    sb.append ((char) c);
						    c = next ();
						    sb.append ((char) c);
						    if (c == '0') {
							c = next ();
							sb.append ((char) c);
							c = next ();
							sb.append ((char) c);
						    }
					    } else
						sb.append ((char) c);
					    c = next ();
					}
					sb.append ((char) c);
					c = next ();
					sb.append ('"');
					stringValue = StringUtil.convertFromProg (sb.substring (1, sb.length () - 2));
					if (stringValue == null)
						throw new LexanException ("Not a string: " + sb.toString ());
				} else {
					isString = false;
					while (c > ' ') {
						sb.append ((char) c);
						c = next ();
					}
				}
			} catch (EOFException e) {
			}
			lexelValue = sb.toString ();
			lexelReaded = true;
		}
		return lexelValue;
	}
	
	public double getDouble () throws Exception {
		getLexel ();
		if (!doubleReaded) {
			try {
				doubleValue = new Double (lexelValue).doubleValue ();
			} catch (NumberFormatException e) {
				throw new LexanException ("Cannot convert lexel \"" + lexelValue + "\" to double");
			}
			doubleReaded = true;
		}
		return doubleValue;
	}
	
	public long getLong () throws Exception {
		getLexel ();
		if (!longReaded) {
			try {
				longValue = new Long (lexelValue).longValue ();
			} catch (NumberFormatException e) {
				throw new LexanException ("Cannot convert lexel \"" + lexelValue + "\" to long");
			}
			longReaded = true;
		}
		return longValue;
	}
	
	public String getString () throws Exception {
		getLexel ();
		if (!isString)
			throw new LexanException ("Cannot convert lexel \"" + lexelValue + "\" to string");
		return stringValue;
	}

	public void cmpLexel () throws Exception {
		cmpLexel (null);
	}

	public void cmpLexel (String str) throws Exception {
		getLexel ();
		if (str != null)
			if (!str.equals (lexelValue))
				throw new LexanException ("Cannot compare lexel \"" + lexelValue + "\"");
		lexelReaded = false;
	}

}
