/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.jasm.settings;

import java.beans.PropertyChangeSupport;

import java.io.Serializable;
import org.openide.filesystems.FileObject;

import org.openide.options.SystemOption;
import org.openide.util.NbBundle;


public class JASMSettings extends SystemOption implements Serializable {

    static final long serialVersionUID = 3900748952306637342L;

    transient protected PropertyChangeSupport propertyChangeSupport = new PropertyChangeSupport (this);

    transient public static final String PROP_DISASSEMBLER = "disassembler"; // NOI18N
    transient public static final String DISP_DISASSEMBLER = NbBundle.getMessage (JASMSettings.class, "DISP_Disassembler"); // NOI18N
    transient public static final String HINT_DISASSEMBLER = NbBundle.getMessage (JASMSettings.class, "HINT_Disassembler"); // NOI18N
    
    transient public static final String PROP_RENAME_FILE = "renameFile"; // NOI18N
    transient public static final String DISP_RENAME_FILE = NbBundle.getMessage (JASMSettings.class, "DISP_Rename_File"); // NOI18N
    transient public static final String HINT_RENAME_FILE = NbBundle.getMessage (JASMSettings.class, "HINT_Rename_File"); // NOI18N
    
    transient public static final String PROP_JODE_JAR_FILE = "jodeJarFile"; // NOI18N
    transient public static final String DISP_JODE_JAR_FILE = NbBundle.getMessage (JASMSettings.class, "DISP_JODE_JAR_File"); // NOI18N
    transient public static final String HINT_JODE_JAR_FILE = NbBundle.getMessage (JASMSettings.class, "HINT_JODE_JAR_File"); // NOI18N
    
    transient public static final String PROP_JAD_BINARY = "jadBinary"; // NOI18N
    transient public static final String DISP_JAD_BINARY = NbBundle.getMessage (JASMSettings.class, "DISP_JAD_Binary"); // NOI18N
    transient public static final String HINT_JAD_BINARY = NbBundle.getMessage (JASMSettings.class, "HINT_JAD_Binary"); // NOI18N
    
    transient private static Disassembler[] array = null;
    private String disassembler = null;
    transient private Disassembler disassemblerClass = null;
    
    private boolean renameFile = false;
    private String jodeJarFile = "jode.jar"; // NOI18N
    private String jadBinary = "jad"; // NOI18N
    
    public static JASMSettings getDefault () {
        return (JASMSettings) SystemOption.findObject (JASMSettings.class, true);
    }
    
    public JASMSettings () {
        array = new Disassembler[3];
        array[0] = new JRAWDisassembler ();
        array[1] = new JodeDisassembler ();
        array[2] = new JadDisassembler ();
    }
    
    public String displayName () {
        return NbBundle.getMessage (JASMSettings.class, "DISP_JASM_Setting"); // NOI18N
    }
    
    public boolean isGlobal () {
        return false;
    }
    
    public String[] getTags () {
        String[] strs = new String[array.length];
        for (int a = 0; a < array.length; a ++)
            strs[a] = array[a].getName ();
        return strs;
    }
    
    public String getDisassembler () {
        if (disassembler == null)
            setDefaultDisassembler ();
        return disassembler;
    }
    
    public Disassembler getDisassemblerClass () {
        if (disassembler == null)
            setDefaultDisassembler ();
        return disassemblerClass;
    }
    
    public Disassembler getDisassemblerClass (String name) {
        if (array == null)
            return null;
        for (int a = 0; a < array.length; a ++)
            if (array[a].getName ().equals (name))
                return array[a];
        return null;
    }
    
    public void setDefaultDisassembler () {
        if (array != null  &&  array.length >= 1  &&  array[0] != null) {
            disassembler = array[0].getName ();
            disassemblerClass = array[0];
        }
    }
    
    public void setDisassembler (String value) {
        String old = disassembler;
        Disassembler disClass = getDisassemblerClass (value);
        if (disClass != null) {
            disassembler = value;
            disassemblerClass = disClass;
        } else
            setDefaultDisassembler ();
        propertyChangeSupport.firePropertyChange (PROP_DISASSEMBLER, old, disassembler);
    }
    
    public void decompile (FileObject file) {
        if (getDisassemblerClass () != null)
            disassemblerClass.decompile (file);
    }

    public boolean supportsBrowse () {
        if (getDisassemblerClass () != null)
            return disassemblerClass.supportsBrowse ();
        return false;
    }
    
    public void browse () {
        if (getDisassemblerClass () != null)
            disassemblerClass.browse ();
    }
    
    public String getJodeJarFile() {
        return jodeJarFile;
    }
    
    public void setJodeJarFile(String value) {
        String old = jodeJarFile;
        jodeJarFile = value;
        propertyChangeSupport.firePropertyChange (PROP_JODE_JAR_FILE, old, value);
    }
    
    public String getJadBinary() {
        return jadBinary;
    }
    
    public void setJadBinary(String value) {
        String old = jadBinary;
        jadBinary = value;
        propertyChangeSupport.firePropertyChange (PROP_JAD_BINARY, old, value);
    }
    
    public boolean isRenameFile() {
        return renameFile;
    }
    
    public void setRenameFile(boolean value) {
        boolean old = renameFile;
        renameFile = value;
        propertyChangeSupport.firePropertyChange (PROP_RENAME_FILE, old, value);
    }
    
}
