/**
 * Generate a sdoc from the JQuery documentation createjQueryXMLDocs.py.xml.
 * It can be generated from
 * createjQueryXMLDocs.py
 * in the jQuery source tree.
 *
 */
package jquerysdoc;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.StringReader;
import java.io.Writer;
import java.util.ArrayList;
import java.util.List;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

public class Main {

    public static void main(String[] args) {
        try {
            new Main().process();
        } catch (IOException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
        }
    }

    void process() throws IOException {
        File file = new File("createjQueryXMLDocs.py.xml");
        file = file.getCanonicalFile();
        assert file.exists() : file.getPath() + " not found; it's available in JQuery's SVN - under trunk/tools/api-browser/createjQueryXMLDocs.py.xml";
        Document doc = parse(file);
        assert doc != null;
        
        // Walk through the document and spit out an appropriate sdoc
        Writer writer = new BufferedWriter(new FileWriter("jquery.sdoc"));
        
        writer.write("/* Doc definitions for JQuery support in NetBeans, \n"+
                " * automatically generated from the JQuery documentation\n" +
                " * using the generator in misc/javascript.generatestubs/jquery-sdoc\n" +
                " * Version: 1.0\n" +
                " */\n\n");
        
        writer.write("/**\n" +
                    " * @id jQuery\n" +
                    " * @global jQuery\n" +
                    " * @type jQuery\n" +
                    " */\n" +
                    "\n");
        writer.write("/**\n" +
                    " * @id $\n" +
                    " * @global $\n" +
                    " * @type jQuery\n" +
                    " */\n" +
                    "\n");
        
        Element root = doc.getDocumentElement();
        NodeList nl = root.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node n = nl.item(i);
            if (n.getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element category = (Element)n;
            category(doc, category, writer);
        }
        
        writer.close();
    }
    
    private void category(Document doc, Element category, Writer writer) {
        NodeList nl = category.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node n = nl.item(i);
            if (n.getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element subcat = (Element)n;
            subcategory(doc, subcat, writer);
        }
    }

    private static String htmlify(String code) {
        code = code.replace("&", "&amp;"); // NOI18N
        code = code.replace("<", "&lt;"); // NOI18N
        code = code.replace(">", "&gt;"); // NOI18N
        
        return code;
    }

    private void subcategory(Document doc, Element subcategory, Writer writer) {
        NodeList nl = subcategory.getChildNodes();
        for (int i = 0; i < nl.getLength(); i++) {
            Node n = nl.item(i);
            if (n.getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element element = (Element)n;
            if (element.getTagName().equals("function")) {
                functionOrProperty(doc, element, writer, ElementType.FUNCTION);
            } else if (element.getTagName().equals("property")) {
                functionOrProperty(doc, element, writer, ElementType.PROPERTY);
            } else if (element.getTagName().equals("selector")) {
                // ??
            } else {
                assert false : element.getTagName();
            }
        }
    }
    
    enum ElementType { FUNCTION, PROPERTY };

    private void functionOrProperty(Document doc, Element function, Writer writer, ElementType elementType) {
        String name = function.getAttribute("name");
        String type = function.getAttribute("return");
        if (type.indexOf(',') != -1) {
            String[] types = type.split(",");
            StringBuilder sb = new StringBuilder();
            for (String t : types) {
                if (sb.length() > 0) {
                    sb.append("|");
                }
                sb.append(t.trim());
            }
            type = sb.toString();
        }
        assert type != null;
        
        String desc = null;
        String longDesc = null;
        String added = null;
        String removed = null;
        String deprecated = null;
        List<String> paramList = new ArrayList<String>();
        NodeList nl = function.getChildNodes();
        String codeExample = null;
        for (int i = 0; i < nl.getLength(); i++) {
            Node n = nl.item(i);
            if (n.getNodeType() != Node.ELEMENT_NODE) {
                continue;
            }
            Element element = (Element)n;
            String tag = element.getTagName();
            if ("desc".equals(tag)) {
                desc = getText(element);
            } else if ("longdesc".equals(tag)) {
                longDesc = getText(element);
            } else if ("added".equals(tag)) {
                added = getText(element);
            } else if ("removed".equals(tag)) {
                removed = getText(element);
            } else if ("deprecated".equals(tag)) {
                deprecated = getText(element);
            } else if ("params".equals(tag)) {
                String paramName = element.getAttribute("name");
                String paramType = element.getAttribute("type");
                String paramDesc = getPreText(element);
                //paramDesc = htmlify(paramDesc);
                String param;
                if (paramType != null) {
                    param = "@param {" + paramType + "} " + paramName + " " + paramDesc;
                } else {
                    param = "@param " + paramName + " " + paramDesc;
                }
                paramList.add(param);
            } else if ("example".equals(tag)) {
                // Find the HTML element
                NodeList enl = element.getChildNodes();
                for (int j = 0; j < enl.getLength(); j++) {
                    Node en = enl.item(j);
                    if (en.getNodeType() != Node.ELEMENT_NODE) {
                        continue;
                    }
                    Element html = (Element)en;
                    if (html.getTagName().equals("code")) {
                    //if (html.getTagName().equals("html")) {
                        codeExample = getPreText(html).trim();
                        if (codeExample.length() > 0) {
                            StringBuilder sb = new StringBuilder();
                            for (String s : codeExample.split("\n")) {
                                sb.append(" * ");
                                sb.append(s);
                                sb.append("\n");
                            }
                            codeExample = sb.toString().trim();
                        }
                        break;
                    }
                }
            } else {
                System.err.println("WARNING: ignoring attribute " + tag);
            }
        }

        StringBuilder sb = new StringBuilder();
        sb.append("/**\n");

        if (desc != null) {
            for (String s : desc.split("\n")) {
                sb.append(" * ");
                sb.append(s);
                sb.append("\n");
            }
        }

        if (longDesc != null) {
            for (String s : longDesc.split("\n")) {
                sb.append(" * ");
                sb.append(s);
                sb.append("\n");
            }
        }
        
        if (codeExample != null) {
            sb.append(" * @code\n ");
            sb.append(codeExample);
            sb.append("\n");
        }
        
        sb.append(" * ");
        sb.append("@id ");
        if (name.startsWith("$.")) {
            sb.append("jQuery.");
            sb.append(name.substring(2));
        } else if (name.startsWith("jQuery.") || name.equals("$")) {
            sb.append(name);
        } else {
            sb.append("jQuery.");
            sb.append(name);
        }
        sb.append("\n");
        
        if (desc != null) {
            for (String param : paramList) {
                for (String s : param.split("\n")) {
                    sb.append(" * ");
                    sb.append(s);
                    sb.append("\n");
                }
            }
        }

        if (type != null) {
            sb.append(" * @type ");
            sb.append(type);
            sb.append("\n");
        }
        
        if (elementType == ElementType.PROPERTY) {
            sb.append(" * @property\n");
        }
        
        if (added != null) {
            sb.append(" * @since ");
            sb.append(added);
            sb.append("\n");
        }
        
        if (removed != null) {
            sb.append(" * @removed ");
            sb.append(removed);
            sb.append("\n");
        }
        
        if (deprecated != null) {
            sb.append(" * @deprecated ");
            sb.append(deprecated);
            sb.append("\n");
        }
        // From http://docs.jquery.com/Browser_Compatibility
        sb.append(" * @compat=IE6|IE7|FF1|FF2|FF3|OPERA|SAFARI2|SAFARI3|KONQ\n");
        sb.append(" */\n\n");
        try {
            writer.write(sb.toString());
        } catch (IOException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            System.exit(0);
        }
    }

    private boolean retried = false;

    private Document parse(File file) throws IOException {
        try {
            DocumentBuilderFactory docFactory = DocumentBuilderFactory.newInstance();
            docFactory.setValidating(false);

            DocumentBuilder docBuilder = docFactory.newDocumentBuilder();
//            docBuilder.setEntityResolver(new EntityResolver() {
//                      public InputSource resolveEntity(java.lang.String publicId, java.lang.String systemId)
//                             throws SAXException, java.io.IOException
//                      {
//                        if (publicId.equals("--myDTDpublicID--"))
//                          // this deactivates the open office DTD
//                          return new InputSource(new ByteArrayInputStream("<?xml version='1.0' encoding='UTF-8'?>".getBytes()));
//                        else return null;
//                          return null;
//                      }
//            });            
            //Document doc = docBuilder.parse(file);
            // Read file in and skip DOCTYPE
            BufferedReader f = new BufferedReader(new FileReader(file));
            StringBuilder sb = new StringBuilder();
            while (true) {
                String line = f.readLine();
                if (line == null) {
                    break;
                }
                if (line.equals("<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\">")) {
                    line = "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Transitional//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd\">";
                }
//                if (line.startsWith("<!DOCTYPE html PUBLIC")) {
//                    continue;
//                }
                sb.append(line);
                sb.append("\n");
            }
            f.close();
            String buffer = sb.toString();
//            if (!buffer.startsWith("<?xml")) {
//                buffer = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n" + buffer;
//            }
            InputSource is = new InputSource(new StringReader(buffer));
            Document doc = docBuilder.parse(is);

            if (retried) {
                System.err.println("Dodged a bullet - tidying the source file succeeded.");
            }

            return doc;
        } catch (SAXException ex) {
            if (retried) {
                System.err.println("Parsing tidied file " + file.getAbsolutePath() + " somehow failed...");
                System.exit(0);
            }
            // Try reparsing!!
            if (!new File("/usr/bin/tidy").exists()) {
                System.err.println("There's a fatal parsing error for " + file.getPath());
                System.err.println("I'd like to fix it, but you need to have /usr/bin/tidy installed");
                System.exit(0);
            }

            System.err.println("Warning: There's a fatal parsing error for " + file.getPath());
            File output = new File("/tmp", file.getName());
            if (output.exists()) {
                output.delete();
            }
            ProcessBuilder pb = new ProcessBuilder("/usr/bin/tidy", "-asxhtml", "-o", output.getAbsolutePath(), file.getAbsolutePath());
            Process start = pb.start();
            try {
                start.waitFor();
                retried = true;
                return parse(output);
            } catch (InterruptedException ex1) {
                Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex1);
                System.exit(0);
            } finally {
                retried = false;
            }
            System.err.println("The document is not well formed.");
            System.err.println("Cannot continue - exiting");
            System.exit(0);

            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            IOException ioe = new IOException();
            ioe.initCause(ex);
            throw ioe;
        } catch (ParserConfigurationException ex) {
            Logger.getLogger(Main.class.getName()).log(Level.SEVERE, null, ex);
            IOException ioe = new IOException();
            ioe.initCause(ex);
            throw ioe;
        }
    }
    
    /** Compute the text under the given element */
    private String getText(Element element) {        
//        StringBuilder sb = new StringBuilder();
//        addNodeText(element, sb);
//        return sb.toString();
        return getFlattenedText(element);
    }

    /** Compute the text under the given element */
    private String getPreText(Element element) {        
        StringBuilder sb = new StringBuilder();
        addNodeText(element, sb);
        return sb.toString();
    }
    
    private void addNodeText(Node n, StringBuilder sb) {
        // This doesn't pick up text in comments, which is usually the correct thing to do,
        // but for things like JavaScript code in HTML pages, it's sometimes embedded in
        // comments for backwards compatibility reasons
        if (n.getNodeType() == Node.TEXT_NODE) {
            String v = n.getNodeValue();
            sb.append(v);
        } else if (n.getNodeType() == Node.CDATA_SECTION_NODE) {
            String v = n.getNodeValue();
            sb.append(v);
        } else if (n.hasChildNodes()) {
            NodeList nl = n.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                addNodeText(nl.item(i), sb);
            }
        }
    }

    /** Compute the text under the given element */
    private String getFlattenedText(Element element) {
        StringBuilder sb = new StringBuilder();
        addFlattenedNodeText(element, sb);
        return sb.toString();
    }
    
    private void addFlattenedNodeText(Node n, StringBuilder sb) {
        // This doesn't pick up text in comments, which is usually the correct thing to do,
        // but for things like JavaScript code in HTML pages, it's sometimes embedded in
        // comments for backwards compatibility reasons
        if (n.getNodeType() == Node.TEXT_NODE) {
            String v = n.getNodeValue();
            v = v.trim();
            if (v.length() > 0) {
                boolean lastWasSpace = sb.length() == 0 || sb.charAt(sb.length()-1) == '\n';
                if (!lastWasSpace) {
                    sb.append(' ');
                }
                for (int i = 0; i < v.length(); i++) {
                    char c = v.charAt(i);
                    boolean space = Character.isWhitespace(c);
                    if (space && !lastWasSpace) {
                        sb.append(' ');
                    } else if (!space) {
                        sb.append(c);
                    }
                    lastWasSpace = space;
                }
            }
        } else if (n.getNodeType() == Node.CDATA_SECTION_NODE) {
            String v = n.getNodeValue();
            sb.append(v);
        } else if (n.getNodeType() == Node.ELEMENT_NODE) {
            Element e = (Element)n;
            if (e.getTagName().equals("br")) {
                sb.append("\n");
            }
        } 
        
        if (n.hasChildNodes()) {
            NodeList nl = n.getChildNodes();
            for (int i = 0; i < nl.getLength(); i++) {
                addFlattenedNodeText(nl.item(i), sb);
            }
        }
    }
}

