/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.dom;

import java.io.IOException;
import java.util.List;
import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.GeneratorUtils;
import org.netbeans.jsstubsgenerator.stub.Origin;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 *
 * @author Martin Fousek <marfous@netbeans.org>
 */
public class HistoryGenerator {

    public static void generate(Map<String, StubObject> allStubs) throws IOException {
        // general definition
        StubObject historyInterface = new StubObject(Origin.DOM, "History", StubObject.GLOBAL);
        historyInterface.setPrune(true);
        historyInterface.setType(Type.OBJECT);
        historyInterface.setSummary("The joint session history of a History object is the union of all the session histories of all browsing contexts of all the fully active Document objects that share the History object's top-level browsing context, with all the entries that are current entries in their respective session histories removed except for the current entry of the joint session history.");

        // attributes definition
        String[][] attributes = new String[][]{
            {"length", "Number", "Read-only. Returns the number of elements in the session history, including the currently loaded page. For example, for a page loaded in a new tab this property returns 1."},
            {"state", "Object", "Returns the state at the top of the history stack. This is a way to look at the state without having to wait for a popstate event. Read only."}
        };
        List<StubObject> generatedInstanceProperties = GeneratorUtils.generateProperties(Origin.DOM, attributes, "History", false);
        for (StubObject instanceProperty : generatedInstanceProperties) {
            historyInterface.putAttribute(instanceProperty);
        }

        // methods definition
        String[][] methods = new String[][]{
            {"back", "undefined", "Goes to the previous page in session history, the same action as when the user clicks the browser's Back button. Equivalent to history.go(-1)."},
            {"forward", "undefined", "Goes to the next page in session history, the same action as when the user clicks the browser's Forward button; this is equivalent to history.go(1)."},
            {"go", "undefined", "Loads a page from the session history, identified by its relative location to the current page, for example -1 for the previous page or 1 for the next page. When integerDelta is out of bounds (e.g. -1 when there are no previously visited pages in the session history), the method doesn't do anything and doesn't raise an exception. Calling go() without parameters or with a non-integer argument has no effect (unlike Internet Explorer, which supports string URLs as the argument)."},
            {"pushState", "undefined", "Pushes the given data onto the session history stack with the specified title and, if provided, URL. The data is treated as opaque by the DOM; you may specify any JavaScript object that can be serialized.  Note that Firefox currently ignores the title parameter; for more information, see manipulating the browser history."},
            {"replaceState", "undefined", "Updates the most recent entry on the history stack to have the specified data, title, and, if provided, URL. The data is treated as opaque by the DOM; you may specify any JavaScript object that can be serialized.  Note that Firefox currently ignores the title parameter; for more information, see manipulating the browser history."},
        };
        List<StubObject> generatedInstanceMethods = generateInstanceMethods(methods);
        for (StubObject instanceMethod : generatedInstanceMethods) {
            historyInterface.putAttribute(instanceMethod);
        }

        allStubs.put("History", historyInterface);
    }

    private static List<StubObject> generateInstanceMethods(String[][] methods) {
        List<StubObject> methodStubs = GeneratorUtils.generateMethods(methods, "History", false);
        for (StubObject method : methodStubs) {
            if ("go".equals(method.getName())) {
                method.addParameter(new Parameter("integerDelta", "Number", ""));
            } else if ("pushState".equals(method.getName()) || "replaceState".equals(method.getName())) {
                method.addParameter(new Parameter("data", "Object", "The state object is a JavaScript object which is associated with the new history entry created by pushState(). Whenever the user navigates to the new state, a popstate event is fired, and the state property of the event contains a copy of the history entry's state object."));
                method.addParameter(new Parameter("title", "String", "Passing the empty string here should be safe against future changes to the method. Alternatively, you could pass a short title for the state to which you're moving."));
                method.addParameter(new Parameter("url", "String", "The new history entry's URL is given by this parameter. Note that the browser won't attempt to load this URL after a call to pushState(), but it might attempt to load the URL later, for instance after the user restarts her browser. The new URL does not need to be absolute; if it's relative, it's resolved relative to the current URL. The new URL must be of the same origin as the current URL; otherwise, pushState() will throw an exception. This parameter is optional; if it isn't specified, it's set to the document's current URL.", true));
            }
        }
        return methodStubs;
    }
}
