/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.print;

import java.util.ArrayList;
import java.util.List;
import org.jsoup.Jsoup;
import org.netbeans.jsstubsgenerator.JsStubsGenerator;
import org.netbeans.jsstubsgenerator.stub.Parameter;
import org.netbeans.jsstubsgenerator.stub.Return;
import org.netbeans.jsstubsgenerator.stub.StubObject;

/**
 *
 * @author marfous
 */
public class DocSummary extends Construct {

    private static final String WRAPPER_ELEMENT = "b";
    private static final String WRAPPER_HEADER = "h3";

    public DocSummary(StubObject stubObject, int nestedLevel) {
        super(stubObject, nestedLevel);
    }

    private void elementName() {
        jsBuilder.appendComment("<" + WRAPPER_ELEMENT + ">" + stubObject.getName() + "</" + WRAPPER_ELEMENT + ">");
    }

    private void summary() {
        if (!"".equals(stubObject.getSummary().trim())) {
            jsBuilder.appendComment(stubObject.getSummary());
        }
    }

    private static String getWithoutWS(String string) {
        // the second char is not common space, it's /160
        return string.replace(" ", "").replace(" ", "");
    }

    private static String getCleanedSyntax(String originalSyntax) {
        // issue #224105 … is not well displayed in non-unix OSes
        return originalSyntax.replace("…", "...");
    }

    private void syntax() {
        if (!"".equals(stubObject.getSyntax().trim())) {
            jsBuilder.appendComment("@syntax " + getCleanedSyntax(stubObject.getSyntax()));
        }
    }

    private void parameters() {
        if (!stubObject.getParameters().isEmpty()) {
            String[] paramNames = getParameterNamesFromSyntax(stubObject.getSyntax());
            List<Parameter> parameters = stubObject.getParameters();
            for (int i = 0; i < parameters.size(); i++) {
                Parameter param = parameters.get(i);
                if (JsStubsGenerator.GENERATE_FULL || !stubObject.isPrune()) {
                    // removed dashes from parameter names if any
                    String paramName = param.getName().contains("-") ? param.getName().replace("-", "") : param.getName();
                    jsBuilder.appendComment("@param {" + param.getType() + "} " + getWithoutWS(paramName) + " " + param.getDesc());
                } else {
                    // update parameter names to be able to complete at least their type where possible
                    if (i < paramNames.length) {
                        String paramName = paramNames[i].contains("-") ? paramNames[i].replace("-", "") : paramNames[i];
                        jsBuilder.appendComment("@param {" + param.getType() + "} " + paramName);
                    }
                }
            }
        }
    }

    private String[] getParameterNamesFromSyntax(String syntax) {
        List<String> params = new ArrayList<String>();
        String originalSyntax = Jsoup.parse(syntax.trim()).text().trim();
        if (originalSyntax.indexOf("(") == -1 || originalSyntax.indexOf(")") == -1) {
            return new String[0];
        }
        String parametersString = syntax.substring(originalSyntax.indexOf("(") + 1, originalSyntax.indexOf(")"));
        // without optional and more arg parameters
        String minSyntax = (parametersString.indexOf("[") > -1) ? parametersString.substring(0, parametersString.indexOf("[")) : parametersString;
        minSyntax = (minSyntax.indexOf(", ...") > -1) ? minSyntax.substring(0, minSyntax.indexOf(", ...")) : minSyntax;
        for (String string : minSyntax.split("[,]")) {
            params.add(string.trim());
        }
        return params.toArray(new String[params.size()]);
    }

    private void returns() {
        Return returns = stubObject.getReturns();
        if (returns != null) {
            if (JsStubsGenerator.GENERATE_FULL || !stubObject.isPrune()) {
                jsBuilder.appendComment("@returns {" + returns.getType() + "} " + ((returns.getDesc() != null) ? returns.getDesc() : ""));
            } else {
                jsBuilder.appendComment("@returns {" + returns.getType() + "}");
            }
        }
    }

    private void statics() {
        if (stubObject.isStatic()) {
            jsBuilder.appendComment("@static");
        }
    }

    public String print() {
        jsBuilder.append("/**");

//        elementName();
        // text descriptions generate only for complete documentation
        if (JsStubsGenerator.GENERATE_FULL || !stubObject.isPrune()) {
            summary();
        }
        syntax();
        parameters();
        returns();
        statics();
        
        jsBuilder.append(" */", false);
        return jsBuilder.toString();
    }

}
