/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.jsstubsgenerator.print;

import java.util.Map;
import org.netbeans.jsstubsgenerator.stub.StubObject;
import org.netbeans.jsstubsgenerator.stub.Type;

/**
 * Prints small structures in the jsDoc documentation.
 *
 * @author marfous
 */
public class JsObject extends JsConstruct {

    private static final String PROTOTYPE_PROPERTY = "prototype";

    public JsObject(StubObject stubObject, int nestedLevel) {
        super(stubObject, nestedLevel);
    }

    private boolean begin(String objectName) {
        if (hasSyntax() || hasConstructor()) {
            jsBuilder.append("function " + objectName + "(", false);
            if (hasSyntax()) {
                Utils.fulfilArguments(jsBuilder, stubObject);
            }
            jsBuilder.append(") {");
        } else {
            if (!stubObject.getParent().equals(StubObject.GLOBAL)) {
                jsBuilder.append("var " + objectName + " = new " + stubObject.getParent() + "();");
                return false;
            } else {
                jsBuilder.append("var " + objectName + " = {");
            }
        }
        return true;
    }

    private boolean hasSyntax() {
        return stubObject.getSyntax() != null && !"".equals(stubObject.getSyntax());
    }

    private boolean hasConstructor() {
        for (Map.Entry<String, StubObject> entry : stubObject.getAttributes().entrySet()) {
            if (entry.getKey().equals("constructor")) {
                return true;
            }
        }
        return false;
    }

    private void end() {
        jsBuilder.append("}");
    }

    private void exposeAttributes() {
        for (Map.Entry<String, StubObject> entry : stubObject.getAttributes().entrySet()) {
            StubObject attrStub = entry.getValue();
            if (!"prototype".equals(attrStub.getName())) {
                jsBuilder.append(entry.getValue().toDoc(true));
            }
        }

        // every object should have prototype attribute (also inheritance impacts)
        exposePrototypeProperty();
    }

    private void exposePrototypeProperty() {
        StubObject stub = new StubObject(stubObject.getOrigin(), PROTOTYPE_PROPERTY, stubObject.getName());
        if (stubObject.getAttributes().containsKey(PROTOTYPE_PROPERTY)) {
            stub = stubObject.getAttributes().get(PROTOTYPE_PROPERTY);
        } else {
            stub.setType(Type.PROPERTY);
            stub.setSummary("Represents the " + stubObject.getName() + " prototype object.");
            stub.setSyntax(stubObject.getName() + "." + PROTOTYPE_PROPERTY);
            stub.setStatic(true);
        }
        jsBuilder.append(stub.toDoc(true, stubObject.getSuperClass()));
    }

    public String print() {
        appendDoc(new DocSummary(stubObject, nestedLevel));
        if (!stubObject.isInstance()) {
            boolean needEnding = begin(stubObject.getName());
//            // workaround to mark object with no properties still as objects
//            jsBuilder.append("    this.prototype");
            if (needEnding) {
                end();
            }
        } else {
            // instantiated object
            jsBuilder.append("var " + stubObject.getName() + " = new " + stubObject.getSuperClass() + "();");
        }
        exposeAttributes();
        return jsBuilder.toString();
    }

}
