/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2009 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.server.services;

import java.io.IOException;
import java.sql.Connection;
import java.sql.DriverManager;
import java.sql.SQLException;
import java.util.Properties;
import java.util.logging.Level;
import java.util.logging.Logger;

/**
 *
 * @author Jindrich Sedek
 */
public class BugzillaDBConnector {

    private static BugzillaDBConnector instance = null;

    private BugzillaDBConnector() {
    }

    public static BugzillaDBConnector getInstance() {
        if (instance == null) {
            try {
                Class.forName("com.mysql.jdbc.Driver").newInstance();
            } catch (InstantiationException ex) {
                Logger.getLogger(BugzillaDBConnector.class.getName()).log(Level.SEVERE, null, ex);
            } catch (IllegalAccessException ex) {
                Logger.getLogger(BugzillaDBConnector.class.getName()).log(Level.SEVERE, null, ex);
            } catch (ClassNotFoundException ex) {
                Logger.getLogger(BugzillaDBConnector.class.getName()).log(Level.SEVERE, null, ex);
            }
            instance = new BugzillaDBConnector();
        }
        return instance;
    }

    public <T> T process(DBTask<T> task) {
        Connection conn = null;
        T result = null;
        try {
            Properties p = loadProperties();
            String userName = p.getProperty("USERNAME");
            String password = p.getProperty("PASSWD");
            String url = "jdbc:mysql://" + p.getProperty("SERVER") + "/" + p.getProperty("BUGZILLA_DB_NAME");
            conn = DriverManager.getConnection(url, userName, password);
            result = task.process(conn);
        } catch (SQLException ex) {
            Logger.getLogger(BugzillaDBConnector.class.getName()).log(Level.SEVERE, null, ex);
        } catch (IOException ex) {
            Logger.getLogger(BugzillaDBConnector.class.getName()).log(Level.SEVERE, null, ex);
        } finally {
            if (conn != null) {
                try {
                    conn.close();
                } catch (SQLException e) {
                    /* ignore close errors */
                }
            }
        }
        return result;
    }

    private static Properties loadProperties() throws IOException {
        Properties p = new Properties();
        p.load(BugzillaDBConnector.class.getResourceAsStream("connection.properties"));
        return p;
    }

    public static interface DBTask<T> {

        public T process(Connection conn) throws SQLException;
    }
}
