/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2009 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.test.uihandler.selenium;

import java.util.Arrays;
import java.util.List;

/**
 *
 * @author Jindrich Sedek
 */
public class ListPageTest extends SeleniumTestBase {

    public void testComponents() throws Exception {
        selenium.open("analytics/list.do");
        assertFalse(selenium.isSomethingSelected("name=subcomponents"));
        selenium.type("name=component", "javaee");
        selenium.type("name=subcomponents", "JSP");
        assertEquals("JSP", selenium.getSelectedLabel("name=subcomponents"));
    }

    public void testVersions() throws Exception {
        selenium.open("analytics/list.do");
        List<String> versions = Arrays.asList(selenium.getSelectOptions("name=versions"));
        assertTrue(versions.contains("6.1"));
        assertTrue(versions.contains("6.5"));
        assertTrue(versions.contains("6.0 Beta 2"));
    }

    public void testCount() throws Exception {
        selenium.open("analytics/list.do");
        selenium.type("name=component", "javaee");
        selenium.type("name=subcomponents", "JSP");
        selenium.addSelection("name=versions", "6.5");
        String text = waitCountReload();
        assertTrue(text.contains("Query will find"));

        selenium.addSelection("name=versions", "6.5 Beta");
        selenium.addSelection("name=versions", "6.5 RC1");
        selenium.addSelection("name=versions", "6.5 RC2");
        String text2 = waitCountReload();
        assertTrue(text2.contains("Query will find"));
        assertNotEquals(text, text2);
    }

    public void testShowDups() throws Exception {
        selenium.open("analytics/list.do");
        selenium.type("name=component", "javaee");
        selenium.type("name=subcomponents", "JSP");
        selenium.addSelection("name=versions", "6.5");
        String text1 = waitCountReload();
        assertFalse(selenium.isChecked("name=showDuplicates"));

        selenium.click("name=showDuplicates");
        assertTrue(selenium.isChecked("name=showDuplicates"));
        String text2 = waitCountReload();
        assertNotEquals(text1, text2);

        selenium.click("name=showDuplicates");
        assertFalse(selenium.isChecked("name=showDuplicates"));
        String text3 = waitCountReload();
        assertTrue(text3.contains("Query will find"));
    }

    public void testSearch() throws Exception {
        selenium.open("analytics/list.do");
        selenium.type("name=component", "javaee");
        selenium.type("name=subcomponents", "JSP");
        selenium.addSelection("name=versions", "6.5");
        selenium.click("xpath=//input[@value='Search']");
        selenium.waitForPageToLoad("300000");
        assertTrue(selenium.isTextPresent("Exceptions"));
        assertTrue(selenium.isTextPresent("count:"));
        assertTrue(selenium.isTextPresent("Summary"));
        assertTrue(selenium.isTextPresent("Component"));
        assertTrue(selenium.isTextPresent("Bugzilla"));
    }

    private String waitCountReload() throws InterruptedException {
        Thread.sleep(1000);
        String text = selenium.getText("id=count");
        while (text.contains("loading")) {
            Thread.sleep(1000);
            text = selenium.getText("id=count");
        }
        return text;
    }
}
