/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.exceptions.entity;

import java.io.Serializable;
import java.util.Collection;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.OneToMany;
import javax.persistence.Table;
import org.hibernate.annotations.CacheConcurrencyStrategy;

/**
 *
 * @author Jan Horvath
 */
@Entity
@Table(name = "nbversion")
@org.hibernate.annotations.Cache(usage = CacheConcurrencyStrategy.NONSTRICT_READ_WRITE, include="non-lazy")
@NamedQueries({@NamedQuery(name = "Nbversion.findByVersion", query = "SELECT n FROM Nbversion n WHERE n.version = :name"),
@NamedQuery(name = "Nbversion.all", query = "SELECT n FROM Nbversion n")
})
public class Nbversion implements Serializable {
    
    private static final String NETBEANS_IDE_PREFIX = "NetBeans IDE ";
    private static final String NETBEANS_PLATFORM_PREFIX = "NetBeans Platform ";
    
    @Id
    @Column(name = "ID", nullable = false)
    private Integer id;
    @Column(name = "VERSION", nullable = false, unique= true)
    private String version;
    @OneToMany(mappedBy = "nbversionId")
    private Collection<ProductVersion> productVersionCollection;
    public static final String DEV_VERSION = "Dev";
    public static final String DEFAULT_VERSION = "7.4";
    
    public Nbversion() {
    }

    public Nbversion(Integer id) {
        this.id = id;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getVersion() {
        return version;
    }

    public void setVersion(String version) {
        this.version = version;
    }
    
    public void setLongVersion(String productVersion){
        String result = productVersion.replaceAll("\\(.*\\)", "");
        result = result.replaceAll("\\d{6,}", "").trim();
        if (result.startsWith(NETBEANS_IDE_PREFIX)) {
            result = result.substring(NETBEANS_IDE_PREFIX.length(), result.length());
        }
        if (result.startsWith(NETBEANS_PLATFORM_PREFIX)) {
            result = result.substring(NETBEANS_PLATFORM_PREFIX.length(), result.length());
        }
        setVersion(result.trim());
    }

    public Collection<ProductVersion> getProductVersionCollection() {
        return productVersionCollection;
    }

    public void setProductVersionCollection(Collection<ProductVersion> productVersionCollection) {
        this.productVersionCollection = productVersionCollection;
    }

    /**
     * return 6.0, 6.0.1, 6.1, 6.5, ....
     */
    public String getVersionNumber(){
        return getVersionNumber(version);
    }

    public static String getVersionNumber(String versionStr){
        if (versionStr.contains(DEV_VERSION) || versionStr.isEmpty()){
            return DEFAULT_VERSION;
        }
        int i = 0;
        while (Character.isDigit(versionStr.charAt(i)) || (versionStr.charAt(i)  == '.')){
            i++;
            if (i == versionStr.length()){
                break;
            }
        }
        if (i == 0){
            return null;
        }
        return versionStr.substring(0, i);
    }


    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null
                  ? id.hashCode(): 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        /*
         * TODO: Warning - this method won't work in the case the id fields are not set
         */
        if (!(object instanceof Nbversion)) {
            return false;
        }
        Nbversion other = (Nbversion) object;
        if (this.id != other.id && (this.id == null || !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "org.netbeans.modules.exceptions.entity.Nbversion[id=" + id + "]";
    }

}
