/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.modules.exceptions.utils;

import java.io.File;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;
import org.netbeans.modules.exceptions.entity.Submit;

/**
 *
 * @author jlehotsky
 */
public class UploadResourceUtils {
    
    private static final Logger LOG = Logger.getLogger(UploadResourceUtils.class.getName());
    private static final String ADDINFO_DIR_SUFFIX="_addinfo";
    
    public static String getAddInfoRootUploadDir() {
        return getRootUploadDir(ADDINFO_DIR_SUFFIX);
    }

    public static String getUploadDirPath(String rootUploadDir, String id) {
        if (id == null){
            return rootUploadDir;
        }
        File uploadDir = new File(rootUploadDir, getSubDirName(id));
        uploadDir.mkdirs();
        return uploadDir.getAbsolutePath();
    }
    
    private static final int DIRECTORY_DEPTH = 2;
    private static final int DIR_INT_COUNT = 3;
    private static String getSubDirName(String id) {
        String hash = Integer.toString(Math.abs(id.hashCode()));
        String name = File.separator;
        for (int depth = 0; depth < DIRECTORY_DEPTH; depth++){
            int length = Math.min(hash.length(), DIR_INT_COUNT);
            name = name + hash.substring(0, length) + File.separator;
            hash = hash.substring(length);
            if (hash.length() == 0){
                break;
            }
        }
        return name;
    }

    
    private static String getRootUploadDir(String suffix){
        String rootUploadDirPath = getRootUploadDirPath();
        if (rootUploadDirPath == null)
            return null;
        File uploadDir = new File(rootUploadDirPath);
        File parentFile = uploadDir.getParentFile();
        File targetUploadDir = new File(parentFile, uploadDir.getName() + suffix);
        targetUploadDir.mkdir();
        return targetUploadDir.getAbsolutePath();
    }
    
    private static String rootUploadDir = null;
    
    public static String getRootUploadDirPath(){
        if (rootUploadDir == null){
            rootUploadDir = getVariable("dir", String.class);
        }
        return rootUploadDir;
    }

        /**
     * Reads value for a variable defined in the "env-entry" section of web.xml
     *
     * @param name name of the variabl
     * @param type the expected type of the variable
     * @return value for the variable or null if there is no, or there is not the right type
     */
    public static <T> T getVariable(String name, Class<T> type) {
        try {
            Context initCtx = new InitialContext();
            Context c = (Context) initCtx.lookup("java:comp/env"); // NOI18N
            c = (Context)c.lookup("analytics"); // NOI18N
            return type.cast(c.lookup(name));
        } catch (NamingException ex) {
            LOG.log(Level.WARNING, "no env variable for " + name, ex);
//            printContext("", null);
            return null;
        } catch (ClassCastException ex) {
            LOG.log(Level.WARNING, "wrong env variable for " + name + " we need " + type.getName(), ex);
            return null;
        }
    }

}
