/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */

package org.netbeans.server.uihandler.api.bugs;

import java.util.HashMap;
import java.util.Map;
import javax.persistence.EntityManager;
import org.netbeans.modules.exceptions.entity.Exceptions;
import org.netbeans.modules.exceptions.entity.Slowness;
import org.netbeans.modules.exceptions.entity.Submit;

/**
 * Default fallback implementation for filter provider.
 * 
 * It nevers allows insertion of an exception/slowness report into any bugtracker.
 * This class is used as a fallback provider in the case there is no filter 
 * provider service found.
 * 
 * @author jlehotsky
 */
public class DefaultInsertionFilterProvider implements BugTrackerInsertionFilterProvider {

    private FilterMap filters;
    private FilterMap reopenFilters;

    public Long getDuplicatesNo() {
        // The value returned here is not important, we don't allow any insertion
        // regardless of higher logic that uses this API call
        return (Long)1l;
    }

    public void setBugReporter(BugReporterFacade bugReporter) {
        // no implementation, we don't need bug reporter facade to get any
        // information from bug tracker.
    }

    private static class FilterMap {
        private Map<Class, BugTrackerInsertionFilter[]> mapa = new HashMap<Class, BugTrackerInsertionFilter[]>();

        <Q extends Submit> void put(Class<Q> cls, BugTrackerInsertionFilter<Q>[] filters){
            mapa.put(cls, filters);
        }

        <Q extends Submit> BugTrackerInsertionFilter<Q>[] get(Class<Q> cls){
            return (BugTrackerInsertionFilter<Q>[]) mapa.get(cls);
        }

    }
    
    public <T extends Submit> BugTrackerInsertionFilter<T>[] getFilters(Class<T> clazz) {
        if (filters == null) {
            synchronized (DefaultInsertionFilterProvider.class) {
                filters = new FilterMap();
                BugTrackerInsertionFilter<Exceptions>[] excFilters = new BugTrackerInsertionFilter[]{ 
                new NeverAllowFilter()
                };
                filters.put(Exceptions.class, excFilters);
                BugTrackerInsertionFilter<Slowness>[] slownFilters = new BugTrackerInsertionFilter[]{
                new NeverAllowFilter()
                };
                filters.put(Slowness.class, slownFilters);
            }
        }
        return filters.get(clazz);
    }
    
    public <T extends Submit> BugTrackerInsertionFilter<T>[] getReopenFilters(Class<T> clazz) {
        if (reopenFilters == null) {
            synchronized (DefaultInsertionFilterProvider.class) {
                reopenFilters = new FilterMap();
                BugTrackerInsertionFilter<Exceptions>[] excFilters = new BugTrackerInsertionFilter[]{
                new NeverAllowFilter()
                };
                reopenFilters.put(Exceptions.class, excFilters);
                BugTrackerInsertionFilter<Slowness>[] slownFilters = new BugTrackerInsertionFilter[]{
                new NeverAllowFilter()
                };
                reopenFilters.put(Slowness.class, slownFilters);
            }
        }
        return reopenFilters.get(clazz);
    }


    private static class NeverAllowFilter<T extends Submit> implements BugTrackerInsertionFilter<T> {
        public BugTrackerInsertionFilter.InsertionResult isInsertable(EntityManager em, Submit exc) {
            return BugTrackerInsertionFilter.InsertionResult.CANCEL;
        }

    }

}
