USE exceptions;

SET FOREIGN_KEY_CHECKS = 0;

CREATE TABLE SUSPICIOUS_METHOD_MAPPING (
        ORIGINAL_METHOD_NAME VARCHAR(100) PRIMARY KEY,
        MAPPED_METHOD_NAME VARCHAR(100) NOT NULL
)ENGINE=INNODB;

CREATE TABLE SOURCEJAR(
	ID INT PRIMARY KEY,
	NAME VARCHAR(200) NOT NULL UNIQUE
) ENGINE=INNODB;


CREATE TABLE SOURCEJAR_MAPPING(
        SOURCEJAR VARCHAR (200) PRIMARY KEY, -- CAN CONTAIN REGEXP -> NO REFERENCE TO SOURCEJAR
        COMPONENT VARCHAR (50),
        SUBCOMPONENT VARCHAR (100),
        PRODUCT INT
) ENGINE=INNODB;

CREATE TABLE PACKAGE_MAPPING(
        PACKAGE VARCHAR (200) PRIMARY KEY,
        COMPONENT VARCHAR (50),
        SUBCOMPONENT VARCHAR (100),
        PRODUCT INT
) ENGINE=INNODB;

CREATE TABLE METHOD(
    ID INT PRIMARY KEY, 
    NAME VARCHAR(200) NOT NULL UNIQUE,
    SOURCEJAR INT,
    FOREIGN KEY (SOURCEJAR) REFERENCES SOURCEJAR(ID)
) ENGINE=INNODB;


CREATE TABLE NBUSER(
    ID INT PRIMARY KEY,
    NAME VARCHAR(30) NOT NULL UNIQUE
    -- EMAIL VARCHAR(30)   - SHOULD BE KNOWN ..NBUSERNAME@NETBEANS.ORG
) ENGINE=INNODB;


CREATE TABLE REPORT (
    ID INT PRIMARY KEY,
    REPORTTYPE VARCHAR(50),
    THROTTLED INT DEFAULT 0,
    COMPONENT VARCHAR(255),
    SUBCOMPONENT VARCHAR(255),
    PRODUCT INT,
    ISSUE_ID INT,
    STATUS INT
) ENGINE=INNODB;

CREATE TABLE REPORT_COMMENT(
    ID INT NOT NULL,
    REPORT_ID INT NOT NULL,
    NBUSER_ID INT NOT NULL,
    COMMENT VARCHAR(1000),
    COMMENT_DATE TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    PRIMARY KEY (ID),
    FOREIGN KEY (REPORT_ID) REFERENCES REPORT(ID),
    FOREIGN KEY (NBUSER_ID) REFERENCES NBUSER(ID)
) ENGINE=INNODB;

CREATE TABLE VM (
    ID INT PRIMARY KEY,
    NAME VARCHAR(255) UNIQUE,
    SHORT_NAME VARCHAR(32)
) ENGINE=INNODB;

CREATE TABLE OPERATINGSYSTEM (
    ID INT PRIMARY KEY,
    NAME VARCHAR(255),
    VERSION VARCHAR(64),
    ARCH VARCHAR(32),
    UNIQUE(NAME, VERSION, ARCH)
) ENGINE=INNODB;

CREATE TABLE EXCEPTIONS(
    ID INT PRIMARY KEY,
    SUMMARY VARCHAR(200),           -- EXCEPTION SUMMARY
    STACKTRACE INT UNIQUE,
    REPORTDATE TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    NBUSER_ID INT,
    BUILD DECIMAL(14,0),
    LOGFILE_ID INT,
    REPORT_ID INT NOT NULL,
    VM_ID INT,
    OPERATINGSYSTEM_ID INT,
    TEST_NAME VARCHAR(300),
    SOURCE VARCHAR(64),
    BUGID INT,
    BUILD_ID INT,
    TAGS VARCHAR(200),
    FOREIGN KEY (NBUSER_ID) REFERENCES NBUSER(ID),
    FOREIGN KEY (STACKTRACE) REFERENCES STACKTRACE(ID),
    FOREIGN KEY (LOGFILE_ID) REFERENCES LOGFILE(ID),
    FOREIGN KEY (REPORT_ID) REFERENCES REPORT(ID),
    FOREIGN KEY (VM_ID) REFERENCES VM(ID),
    FOREIGN KEY (OPERATINGSYSTEM_ID) REFERENCES OPERATINGSYSTEM(ID),
    FOREIGN KEY (BUILD_ID) REFERENCES BUILD_INFO(ID)
) ENGINE=INNODB;

CREATE TABLE BUILD_INFO(
    ID INT PRIMARY KEY,
    VCS INT,
    IDENTIFICATOR VARCHAR(128),
    VERSION VARCHAR(128),
    BUILD_DATE DATE
) ENGINE=INNODB;

CREATE TABLE STACKTRACE(    -- REPREZENTS THROWN EXCEPTION
    ID INT PRIMARY KEY,  
    MESSAGE VARCHAR(1024),   -- ERROR MESSAGE
    CLASS VARCHAR(1024),     -- EXCEPTION CLASS (EX. :JAVA.LANG.NULLPOINTEREXCEPTION)
    ANNOTATION INT UNIQUE,   -- NULL REPREZENTS NO ANNOTATIONS, MORE ANNOTATIONS ARE ADDED RECURSIVELY
    FOREIGN KEY (ANNOTATION) REFERENCES STACKTRACE(ID)
) ENGINE=INNODB;

CREATE TABLE DUPLICATES_LOG(
    ID INT PRIMARY KEY,
    EXCEPTIONS INT NOT NULL,
    OLD_REPORT INT NOT NULL,
    CHANGE_DATE TIMESTAMP DEFAULT CURRENT_TIMESTAMP,
    FOREIGN KEY (OLD_REPORT) REFERENCES REPORT(ID),
    FOREIGN KEY (EXCEPTIONS) REFERENCES EXCEPTIONS(ID)
) ENGINE=INNODB;

CREATE TABLE COMMENT(
    ID INT PRIMARY KEY,
    EXCEPTION_ID INT,
    NBUSER_ID INT,
    COMMENT VARCHAR(1000),
    COMMENT_DATE TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
--    FOREIGN KEY (EXCEPTION_ID) REFERENCES EXCEPTIONS(ID), EXCEPTIONS OR SLOWNESS
    FOREIGN KEY (NBUSER_ID) REFERENCES NBUSER(ID)
) ENGINE=INNODB;

CREATE TABLE JARFILE(
    ID INT PRIMARY KEY,
    NAME VARCHAR(200) NOT NULL UNIQUE
) ENGINE=INNODB;


CREATE TABLE LINE(          -- REPREZENTS ONE LINE OF STACKTRACE
    STACKTRACE_ID INT,      -- STACKTRACEID
    METHOD_ID INT,
    LINENUMBER INT, 
    LINE_ORDER INT,         -- GIVE US A SEQUENCE NUMBER IN STACK_TRACE. ONE LINE CAN BE FOR FEW TIMES IN ONE STACKTRACE
    JARFILE_ID INT,
    LINE_HASHCODE INT,
    GENERATECOMPONENT INT DEFAULT NULL,
    PRIMARY KEY (STACKTRACE_ID, LINENUMBER, METHOD_ID, LINE_ORDER),
    FOREIGN KEY (STACKTRACE_ID) REFERENCES STACKTRACE(ID),
    FOREIGN KEY (METHOD_ID) REFERENCES METHOD(ID),
    FOREIGN KEY (JARFILE_ID) REFERENCES JARFILE(ID)
) ENGINE=INNODB;


CREATE TABLE HASHCODES(
    CODE INT,
    EXCEPTIONID INT PRIMARY KEY,
    FOREIGN KEY (EXCEPTIONID) REFERENCES EXCEPTIONS(ID)
) ENGINE=INNODB;

CREATE TABLE NBVERSION (
    ID INT PRIMARY KEY,
    VERSION VARCHAR(64) NOT NULL UNIQUE
) ENGINE=INNODB;

CREATE TABLE PRODUCT_VERSION (
    ID INT PRIMARY KEY,
    NBVERSION_ID INT,
    PRODUCT_VERSION VARCHAR(255) NOT NULL UNIQUE,
    FOREIGN KEY (NBVERSION_ID) REFERENCES NBVERSION(ID)
) ENGINE=INNODB;

CREATE TABLE LOGFILE (
    ID INT PRIMARY KEY,
    NBUSER_ID INT,
    PRODUCT_VERSION_ID INT,
    IP DECIMAL(12),
    BUILDNUMBER DECIMAL(14),    -- IT IS A PIECE OF PRODUCT VERSION
    USERDIR VARCHAR(255) NOT NULL,
    UPLOAD_NUMBER INT NOT NULL,
    DATE_CREATED TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    CHANGESET BIGINT,

    FOREIGN KEY (NBUSER_ID) REFERENCES NBUSER(ID),
    FOREIGN KEY (PRODUCT_VERSION_ID) REFERENCES PRODUCT_VERSION(ID),
    UNIQUE (USERDIR, UPLOAD_NUMBER)
) ENGINE=INNODB;

CREATE INDEX GET_LOG_INFO ON LOGFILE (USERDIR, UPLOAD_NUMBER); -- USED ALSO IN PREPARE PAGE CONTEXT

CREATE TABLE SLOWNESS (
    ID INT PRIMARY KEY,
    SUMMARY VARCHAR(200),           -- EXCEPTION SUMMARY
    REPORTDATE TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    ACTION_TIME INT NOT NULL,
    SUSPICIOUS_METHOD INT NOT NULL,
    LATEST_ACTION VARCHAR(200),
    SLOWNESS_TYPE VARCHAR(30),
    NBUSER_ID INT,
    LOGFILE_ID INT,
    REPORT_ID INT NOT NULL,
    VM_ID INT,
    OPERATINGSYSTEM_ID INT,
    BUILD_ID INT,
    TEST_NAME VARCHAR (300),
    FLAGS INT,
    TAGS VARCHAR (200),
    FOREIGN KEY (NBUSER_ID) REFERENCES NBUSER(ID),
    FOREIGN KEY (LOGFILE_ID) REFERENCES LOGFILE(ID),
    FOREIGN KEY (REPORT_ID) REFERENCES REPORT(ID),
    FOREIGN KEY (VM_ID) REFERENCES VM(ID),
    FOREIGN KEY (SUSPICIOUS_METHOD) REFERENCES METHOD(ID),
    FOREIGN KEY (OPERATINGSYSTEM_ID) REFERENCES OPERATINGSYSTEM(ID),
    FOREIGN KEY (BUILD_ID) REFERENCES BUILD_INFO(ID)
    
) ENGINE=INNODB;

CREATE TABLE MAPPING(
    OLD_COMPONENT VARCHAR(50),
    OLD_SUBCOMPONENT VARCHAR(50),
    NEW_COMPONENT VARCHAR(50),
    NEW_SUBCOMPONENT VARCHAR(50),
    PRIMARY KEY(OLD_COMPONENT, OLD_SUBCOMPONENT)
) ENGINE=INNODB;

CREATE TABLE DIRECTUSER(
    NAME VARCHAR(30) PRIMARY KEY,
    MESSAGE_PREFIX VARCHAR(30)
) ENGINE=INNODB;

CREATE TABLE INNOCENTCLASS(
    CLASSNAME VARCHAR(100) PRIMARY KEY
) ENGINE=INNODB;

CREATE TABLE PATCH(
    ID INT PRIMARY KEY,
    PATCHNAME VARCHAR(30),
    MODULENAME VARCHAR(100),
    SPECVERSION VARCHAR(20),
    NBVERSION_ID INT,
    FOREIGN KEY (NBVERSION_ID) REFERENCES NBVERSION(ID)
) ENGINE=INNODB;

CREATE TABLE ISSUE_BUILD(
    ISSUE_NUMBER INT PRIMARY KEY,
    BUILD_NUMBER BIGINT
) ENGINE=INNODB;

CREATE TABLE SEQUENCE( -- sequence table for autogenerated keys
   SEQ_NAME VARCHAR(50) PRIMARY KEY,
   SEQ_COUNT DECIMAL
) ENGINE=INNODB;

INSERT INTO SEQUENCE(SEQ_NAME, SEQ_COUNT) values ('SEQ_GEN', 1);

-- Find the parent bug
DROP FUNCTION IF EXISTS getIssueByExceptionFnc;
DELIMITER |
-- CREATE FUNCTION getIssueByExceptionFnc(exception_url VARCHAR(256)) RETURNS INT
--    BEGIN
--        DECLARE id INT;
--        DECLARE dup INT;
--        DECLARE cnt INT;
--
--        BEGIN
--            DECLARE done BOOLEAN DEFAULT FALSE;
--            DECLARE cur CURSOR FOR
--            SELECT issue_id FROM issue WHERE issue_file_loc LIKE exception_url;
--            DECLARE CONTINUE HANDLER FOR NOT FOUND SET done = TRUE;
--            OPEN cur;
--            process_loop: LOOP
--                FETCH cur INTO dup;
--                -- select the first one
--                LEAVE process_loop;
--            END LOOP;
--            CLOSE cur;
--        END;
--
--        SET id = dup;
--        SET cnt = 0;
--        -- SELECT dup;
--        -- select duplicate_of bug
--        REPEAT
--            SET dup = id;
--            SET id = 0;
--            SELECT duplicate_of INTO id FROM duplicate WHERE issue_id = dup;
--            -- SELECT dup, id;
--            SET cnt = cnt + 1;
--            IF cnt > 20 THEN
--                SET id = 0;
--                SET dup = -1;
--            END IF;
--            UNTIL id = 0
--        END REPEAT;
--    RETURN dup;
--    END |
-- end of procedure
DELIMITER ;

-- following tables are skipped by the backup script

CREATE TABLE PREFERENCES (
    ID INT PRIMARY KEY,
    STATISTIC INT,
    LOGFILE INT,
    NAME INT,
    VALUE INT,
    DIRECT_VALUE INT,
    PREFIX INT,
    FOREIGN KEY (LOGFILE) REFERENCES LOGFILE(ID),
    FOREIGN KEY (STATISTIC) REFERENCES STATISTIC(ID),
    FOREIGN KEY (NAME) REFERENCES PREFNAME(ID),
    FOREIGN KEY (PREFIX) REFERENCES PREFNAME(ID),
    FOREIGN KEY (VALUE) REFERENCES PREFVALUE(ID)
) ENGINE=INNODB;

/*CREATE INDEX PREFEREBCES_WITHOUTKEY ON PREFERENCES(LOGFILE, STATISTIC, PREFIX);*/

CREATE TABLE LOGFILEPARSED (
    ID INT PRIMARY KEY,
    STATISTIC INT NOT NULL,
    LOGFILE INT NOT NULL,
    REVISION INT NOT NULL,
    FOREIGN KEY (LOGFILE) REFERENCES LOGFILE(ID),
    FOREIGN KEY (STATISTIC) REFERENCES STATISTIC(ID)
) ENGINE=INNODB;

CREATE INDEX LOGFILEPARSED_WITHOUTKEY ON LOGFILEPARSED(LOGFILE, STATISTIC);

CREATE TABLE STATISTIC(
    ID INT PRIMARY KEY,
    STATNAME VARCHAR(64) NOT NULL UNIQUE
)ENGINE=INNODB;

CREATE TABLE PREFNAME(
    ID INT PRIMARY KEY,
    NAME VARCHAR(80) NOT NULL UNIQUE
) ENGINE=INNODB;

CREATE TABLE PREFSET(
    PREFNAME INT,
    STATISTIC INT,
    ITEM_INDEX INT,
    FOREIGN KEY (PREFNAME) REFERENCES PREFNAME(ID),
    FOREIGN KEY (STATISTIC) REFERENCES STATISTIC(ID),
    PRIMARY KEY (PREFNAME, STATISTIC, ITEM_INDEX)
) ENGINE=INNODB;


CREATE TABLE PREFVALUE(
    ID INT PRIMARY KEY,
    VALUE VARCHAR(8192)
) ENGINE=INNODB;

CREATE TABLE SRGFAILURE(
    ID INT PRIMARY KEY,
    SUMMARY VARCHAR(200),           -- EXCEPTION SUMMARY
    STACKTRACE INT UNIQUE,
    REPORTDATE TIMESTAMP NOT NULL DEFAULT CURRENT_TIMESTAMP,
    NBUSER_ID INT,
    BUILD DECIMAL(14,0),
    LOGFILE_ID INT,
    REPORT_ID INT NOT NULL,
    VM_ID INT,
    OPERATINGSYSTEM_ID INT,
    TEST_NAME VARCHAR(300),
    SOURCE VARCHAR(64),
    BUGID INT,
    BUILD_ID INT,
    LABEL VARCHAR(512),
    MESSAGE VARCHAR(512),
    FOREIGN KEY (NBUSER_ID) REFERENCES NBUSER(ID),
    FOREIGN KEY (STACKTRACE) REFERENCES STACKTRACE(ID),
    FOREIGN KEY (LOGFILE_ID) REFERENCES LOGFILE(ID),
    FOREIGN KEY (REPORT_ID) REFERENCES REPORT(ID),
    FOREIGN KEY (VM_ID) REFERENCES VM(ID),
    FOREIGN KEY (OPERATINGSYSTEM_ID) REFERENCES OPERATINGSYSTEM(ID),
    FOREIGN KEY (BUILD_ID) REFERENCES BUILD_INFO(ID)
) ENGINE=INNODB;

SET FOREIGN_KEY_CHECKS = 1;




-- Next time when restoring the backup, create indexes after the backup is imported

CREATE INDEX SOURCEINDEX ON SOURCEJAR( NAME );

CREATE INDEX METHODINDEX ON METHOD( NAME );

CREATE INDEX NBUSERINDEX ON NBUSER( NAME );

CREATE INDEX JARFILEINDEX ON JARFILE( NAME );

CREATE INDEX LINE_HASHCODE ON LINE(LINE_HASHCODE);

CREATE INDEX LINE_METHOD ON LINE(METHOD_ID);

CREATE INDEX LINE_JARFILE ON LINE(JARFILE_ID);

CREATE INDEX HASHCODES_CODE ON HASHCODES(CODE);

CREATE INDEX PREFNAME_NAME ON PREFNAME(NAME);

CREATE INDEX PREFVALUE_VALUE ON PREFVALUE(VALUE);

CREATE INDEX TESTNAMEINDEX ON EXCEPTIONS(TEST_NAME);
