/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.modules.exceptions.utils;

import java.io.File;
import java.util.Map;
import java.util.logging.Level;
import java.util.logging.Logger;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import javax.persistence.EntityManager;
import org.netbeans.modules.exceptions.entity.Logfile;
import org.netbeans.server.snapshots.ProfilerData;
import org.netbeans.server.uihandler.DbPreferences;
import org.netbeans.server.uihandler.LogFileTask;
import org.netbeans.server.uihandler.statistics.CPUInfo;
import org.netbeans.server.uihandler.statistics.Memory;
import org.netbeans.server.uihandler.statistics.SnapshotStatistics;
import org.netbeans.server.uihandler.Utils;

/**
 *
 * @author Jan Horvath <jhorvath@netbeans.org>
 */
public class LogfileTools {

    public static Integer getUserCPUCount(EntityManager em, Logfile lf) {
        CPUInfo info = new CPUInfo();
        Preferences pref = DbPreferences.root(lf, info, em);
        try {
            Map<Integer, Integer> result = info.read(pref).getCounts();
            if (!result.isEmpty()) {
                return result.keySet().iterator().next();
            }
            Logfile previous = getPreviousFromThisUser(em, lf);
            if (previous != null) {
                return getUserCPUCount(em, previous);
            }
        } catch (BackingStoreException ex) {
            Logger.getLogger(Logfile.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    private static Logfile getPreviousFromThisUser(EntityManager em, Logfile logfile) {
        if (logfile.getUploadNumber() > 0) {
            return LogFileTask.getLogInfo(logfile.getUserdir(), logfile.getUploadNumber() - 1, em);
        } else {
            return null;
        }
    }
    
    public static ProfilerData getProfilerStatistics(EntityManager em, Logfile logfile){
        SnapshotStatistics stats = new SnapshotStatistics();
        Preferences pref = DbPreferences.root(logfile, stats, em);
        try {
            SnapshotStatistics.SnapshotData data = stats.read(pref);
            return ProfilerData.create(data);
        } catch (BackingStoreException ex) {
            Logger.getLogger(Logfile.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;

    }
    
    public static Integer getUserMemory(EntityManager em, Logfile logfile) {
        Memory memory = new Memory();
        Preferences pref = DbPreferences.root(logfile, memory, em);
        try {
            Map<Long, Integer> result = memory.read(pref);
            if (!result.isEmpty()){
                Long value = result.keySet().iterator().next();
                value = value / 1024 / 1024;
                return value.intValue();
            }
            Logfile previous = getPreviousFromThisUser(em, logfile);
            if (previous != null){
                return getUserMemory(em, previous);
            }
        } catch (BackingStoreException ex) {
            Logger.getLogger(Logfile.class.getName()).log(Level.SEVERE, null, ex);
        }
        return null;
    }

    public static File getMessagesFile(Logfile logfile) {
        String dirPath = Utils.getUploadDirPath(Utils.getMessagesRootUploadDir(), logfile.getUserdir());
        return new File(dirPath, logfile.getFileName());
    }
    
    public static File getEventsFile(Logfile logfile) {
        String dirPath = Utils.getUploadDirPath(Utils.getEventsRootUploadDir(), logfile.getUserdir());
        return new File(dirPath, logfile.getFileName());
    }
    
    public static File getAddInfoFile(Logfile logfile) {
        String dirPath = Utils.getUploadDirPath(Utils.getAddInfoRootUploadDir(), logfile.getUserdir());
        return new File(dirPath, logfile.getFileName());
    }

    public static File getHeapDumpFile(Logfile logfile) {
        String dirPath = Utils.getUploadDirPath(Utils.getHeapRootUploadDir(), logfile.getUserdir());
        return new File(dirPath, logfile.getFileName());
    }

    public static File getLoggerFile(Logfile logfile) {
        String dirPath = Utils.getUploadDirPath(Utils.getRootUploadDirPath(), logfile.getUserdir());
        return new File(dirPath, logfile.getFileName());
    }

    public static File getNPSFile(Logfile logfile) {
        String dirPath = Utils.getUploadDirPath(Utils.getSlownessRootUploadDir(), logfile.getUserdir());
        return new File(dirPath, logfile.getFileName());
    }
    
}
