/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * Oracle and Java are registered trademarks of Oracle and/or its affiliates.
 * Other names may be trademarks of their respective owners.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Oracle designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Oracle in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2013 Sun Microsystems, Inc.
 */
package org.netbeans.server.componentsmatch;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.net.URL;
import java.nio.charset.Charset;
import java.util.HashMap;
import java.util.logging.Level;
import java.util.logging.Logger;
import org.json.simple.JSONArray;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.netbeans.modules.exceptions.utils.Component;

/**
 *
 * @author tomas
 */
final class JarToComponentMapping {

    private static final JarToComponentMapping instance = new JarToComponentMapping();
    private HashMap<String, Component> jarToComponentMap;
    private static final String MAPPING_SERVICE_URL = "http://services.netbeans.org/dashboard/web/a_modules.php?all";
    private static final Logger LOG = Logger.getLogger(JarToComponentMapping.class.getName());

    private JarToComponentMapping() {
        jarToComponentMap = new HashMap<String, Component>();
    }

    public static JarToComponentMapping getInstance() {
        return instance;
    }

    public boolean isLoaded() {
        return !jarToComponentMap.isEmpty();
    }

    public Component getComponentForJar(String sourcejar) {
        return jarToComponentMap.get(sourcejar);
    }

    public boolean refresh() {
        boolean refreshed = false;
        HashMap<String, Component> reloadedMap = new HashMap<String, Component>();
        try {
            JSONArray a = readJsonFromUrl(MAPPING_SERVICE_URL);
            LOG.log(Level.FINEST, "Read mapping JSON from {0}", MAPPING_SERVICE_URL);
            for (Object item : a) {
                JSONObject mapping = (JSONObject) item;
                Component component = new Component((String) mapping.get("product"), (String) mapping.get("component"));
                JSONArray jars = (JSONArray) mapping.get("jars");
                assert !jars.isEmpty() : "have some jar(s) for mapping " + mapping;
                for (Object o : jars) {
                    // one BZ component can have multiple jars
                    String jar = (String) o;
                    reloadedMap.put(jar, component);
                    LOG.log(Level.FINEST, "Read component:{0}", component.toString());
                }
            }
        } catch (AssertionError ae) {
            LOG.log(Level.SEVERE, ae.getMessage(), ae);
        } catch (ClassCastException cce) {
            LOG.log(Level.SEVERE, "Unexpected structure of sent JSON", cce);
        } catch (IOException ioe) {
            LOG.log(Level.WARNING, "Error reading mapping from " + MAPPING_SERVICE_URL, ioe);
        } catch (Exception ex) {
            LOG.log(Level.SEVERE, ex.getMessage(), ex);
        }
        if (!reloadedMap.isEmpty()) {
            jarToComponentMap = reloadedMap;
            refreshed = true;
        }
        return refreshed;
    }

    private JSONArray readJsonFromUrl(String url) throws Exception {
        final InputStream is = new URL(url).openStream();
        BufferedReader rd = new BufferedReader(new InputStreamReader(is, Charset.forName("UTF-8")));
        JSONParser parser = new JSONParser();
        Object obj = parser.parse(rd);
        JSONArray array = (JSONArray) obj;
        return array;
    }
}
