/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.server.componentsmatch;

import org.netbeans.modules.exceptions.utils.Component;
import java.util.Collections;
import java.util.Comparator;
import java.util.List;
import javax.persistence.EntityManager;
import org.netbeans.modules.exceptions.entity.InnocentClass;
import org.netbeans.modules.exceptions.entity.Method;
import org.netbeans.modules.exceptions.entity.PackageMapping;
import org.netbeans.modules.exceptions.utils.PersistenceUtils;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Jan Horvath <jhorvath@netbeans.org>
 */
@ServiceProvider(service=Matcher.class)
public class PackageMatcher extends Matcher {
    private List<InnocentClass> innocents;
    private List<PackageMapping> mappings;

    private static final String PARAMETERS_CLASS_NAME = "org.openide.util.Parameters";

    public PackageMatcher() {
        reload();
    }
    
    @Override
    public void reload() {
        innocents = PersistenceUtils.getInstance().getAll(InnocentClass.class);
        mappings = PersistenceUtils.getInstance().getAll(PackageMapping.class);
        Collections.sort(mappings, new Comparator(){

            @Override
            public int compare(Object o1, Object o2) {
                if (o1 instanceof Comparable && o2 instanceof Comparable) {
                    return - ((Comparable) o1).compareTo(o2);
                }
                return -1;
            }
        });
    }

    @Override
    public Component match(EntityManager em, StackTraceElement[] stes) {
        String classToSkip = null;
        for (int i=0; i<stes.length; i++){
            if (PARAMETERS_CLASS_NAME.equals(stes[i].getClassName())){
                classToSkip = stes[i+1].getClassName();
                continue;
            }
            if ((classToSkip != null) && (stes[i].getClassName().equals(classToSkip))){
                continue;
            }
            String methodName = Method.getMethodNameFromSTE(stes[i]);
            if (isInnocent(methodName)) {
                continue;
            }
            Component comp = findComponentForElement(em, stes[i]);
            
            if (comp != null){
                return comp;
            }
        }
        return null;
    }

    private boolean isInnocent(String className){
        if (innocents != null) {
            for (InnocentClass innocent : innocents) {
                if (className.contains(innocent.getClassname())){
                    return true;
                }
            }
        }
        return false;
    }
    
    @Override
    public Component getRealComponent(EntityManager em, Component expectedComponent) {
        return expectedComponent;
    }

    private Component findComponentForElement(EntityManager em, StackTraceElement stackTraceElement) {
        String className = stackTraceElement.getClassName();
        if (className == null) {
            return null;
        }
        for (PackageMapping m : mappings) {
            String pkg = m.getPackage();
            if (className.startsWith(pkg)) {
                if (className.length() > pkg.length() && className.charAt(pkg.length()) != '.') {
                    continue;
                }
                Component comp = new Component(m.getComponent(), m.getSubcomponent());
                comp.setProduct(m.getProduct());
                return comp;
            }
        }
        return null;
    }
    
}
