/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.server.uihandler;

import java.io.ByteArrayOutputStream;
import java.io.DataOutputStream;
import java.io.IOException;
import java.net.URLEncoder;
import java.util.LinkedList;
import java.util.Queue;
import java.util.Timer;
import java.util.TimerTask;
import java.util.concurrent.atomic.AtomicBoolean;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.persistence.EntityManager;
import javax.servlet.AsyncContext;
import javax.servlet.ServletException;
import javax.servlet.ServletRequest;
import javax.servlet.ServletResponse;
import org.netbeans.server.uihandler.api.bugs.BugReporterException;
import org.netbeans.modules.exceptions.entity.Exceptions;
import org.netbeans.modules.exceptions.entity.Nbuser;
import org.netbeans.modules.exceptions.entity.Report;
import org.netbeans.modules.exceptions.entity.Submit;
import org.netbeans.server.uihandler.api.bugs.BugReporterFacade;
import org.netbeans.server.uihandler.api.bugs.BugReporterFactory;
import org.netbeans.modules.exceptions.utils.PersistenceUtils;
import org.netbeans.server.snapshots.SnapshotManager;
import org.netbeans.server.uihandler.api.Authenticator.AuthToken;
import org.netbeans.server.uihandler.api.Issue;
import org.netbeans.server.uihandler.api.SubmitInterface;
import static org.netbeans.server.uihandler.api.bugs.BugReporterFacade.LOG;
import org.netbeans.server.uihandler.api.bugs.BugTrackerInsertionFilter;
import org.netbeans.server.uihandler.api.bugs.BugTrackerInsertionFilter.InsertionResult;
import org.netbeans.server.uihandler.api.bugs.BugTrackerInsertionFilterProvider;
import org.netbeans.server.uihandler.api.bugs.BugTrackerInsertionDelayProvider;
import org.netbeans.server.uihandler.api.bugs.BugTrackerProvidersFactory;
import org.netbeans.server.utils.Persistable;
import org.netbeans.server.utils.Persistable.TransactionResult;
import org.openide.util.RequestProcessor;
import org.openide.util.Task;
import org.openide.util.TaskListener;

/**
 *
 * @author Jindrich Sedek
 */
public final class IZInsertion {

//    private static final Logger LOG = Logger.getLogger(IZInsertion.class.getName());
    public static final Logger LOG = Logger.getLogger("org.netbeans.modules.exceptions.bugs.reporter");

    private static final String SCHEDULE_ON_PRIMARY_MSG = "[sbm {0}] : scheduling request on secondary queue";    
    private static final String SCHEDULEON_SECONDARY_MSG = "[sbm {0}] : scheduling request on primary queue";
    
    private static int REPORTER_TIMEOUT = 3 * 60 * 1000; // 3min
    private static final int MESSAGE_AMOUNT = 1;
    private static final int MESSAGE_2_AMOUNT = 2;
    private static final int P2_PRIORITY_AMOUNT = 6;
    private static final int P1_PRIORITY_AMOUNT = 10;
    private static final String P1_PRIORITY_STR = "P1";// NOI18N
    private static final String P2_PRIORITY_STR = "P2";// NOI18N
    private static final String P3_PRIORITY_STR = "P3";// NOI18N
    
    private static final Queue<IssuezillaInsertionRequest> requestQueue = new LinkedList<IssuezillaInsertionRequest>();
      // Secondary queue is meant for dealing with request that might come in large numbers and amounts and their
      // processing doesn't have as high priority as user requests (in terms of waiting for feedback)
      //    usually this means automatic reporting systems
    private static final Queue<IssuezillaInsertionRequest> requestSecondaryQueue = new LinkedList<IssuezillaInsertionRequest>();
    private static final AtomicBoolean isIZProcessing = new AtomicBoolean(false);
    private static final RequestProcessor izRequestProcessor = new RequestProcessor("Bug tracker Insertion", 1);
    private static final RequestProcessor izSecondaryRequestProcessor = new RequestProcessor("Bug tracker Insertion secondary", 1);
    private static final Timer timer = new Timer("IZ Insertion");
    private static boolean queuesEnabled = true;
    private static boolean bugFiling = false;
    private static boolean testingReopen = false;

    private final BugReporterFacade bugReporter;
    private final Submit sbm;
    private final AuthToken userToken;
    private final String extraCC, extraMessage;
    
    BugTrackerInsertionFilterProvider insertionFilterProvider;
    BugTrackerInsertionDelayProvider timeoutProvider;
    private final AsyncContext asyncContext;

    IZInsertion(Submit sbm, AuthToken userToken, AsyncContext asyncContext) {
        this(sbm, userToken, asyncContext, null, null);
    }

    public IZInsertion(Submit sbm, AuthToken userToken, AsyncContext asyncContext, String extraCC, String extraMessage) {
        this.sbm = sbm;
        bugReporter = BugReporterFactory.getReporter();
        insertionFilterProvider = BugTrackerProvidersFactory.getInstance().getInsertionFilterProvider();
        timeoutProvider = BugTrackerProvidersFactory.getInstance().getInsertionTimeoutProvider(
                new BugTrackerInsertionDelayProvider() {
                    @Override
                    public int getDelay() {
                        return REPORTER_TIMEOUT;
                    }
                    @Override
                    public int getSecondaryDelay() {
                        return REPORTER_TIMEOUT;
                    }
                }
        );
        
        insertionFilterProvider.setBugReporter(bugReporter);
        this.extraCC = extraCC;
        this.extraMessage = extraMessage;
        if (isGuest(sbm.getNbuserId())) {
            this.userToken = null;
        } else {
            this.userToken = userToken;
        }
        this.asyncContext = asyncContext;
        if (!isIZProcessing.getAndSet(true)){
            timer.schedule(new IZInsertionTask(), 0, timeoutProvider.getDelay());
            timer.schedule(new IZSecondaryInsertionTask(), 0, timeoutProvider.getSecondaryDelay());
        }
    }
    
    public static boolean areQueuesEnabled() {
        return queuesEnabled;
    }
    
    public static void enableQueues(boolean enable) {
        queuesEnabled = enable;
    }
    
    public static int getRequestQueueLength() {
        return requestQueue.size();
    }

    public static int getSecondaryRequestQueueLength() {
        return requestSecondaryQueue.size();
    }

    public static boolean isBugFiling() {
        return IZInsertion.bugFiling;
    }
    
    public static boolean isTestingReopen() {
        return IZInsertion.testingReopen;
    }

    public static void setBugFiling(boolean bugFiling) {
        IZInsertion.bugFiling = bugFiling;
        LOG.info("Bug filing turned " + (bugFiling ? "on." : "off."));
    }
    
    public static void setTestingReopen(boolean testingReopen) {
        IZInsertion.testingReopen = testingReopen;
        LOG.info("Testing reopen turned " + (testingReopen ? "on." : "off."));
    }

    
    public void start() {
        InsertionResult result = InsertionResult.NONE;
        EntityManager em = LogFileTask.getRunningTask().getTaskEntityManager();
        LOG.log(Level.INFO, "[sbm {0}] Checking for insertion filtering for report {0}", new Object[] { sbm.getId().toString() } );
        for (BugTrackerInsertionFilter filter : insertionFilterProvider.getFilters(sbm.getClass())) {
            result = filter.isInsertable(em, sbm);
            LOG.log(Level.INFO, "[sbm {2}] Filter + {0} returned {1}", new Object[]{filter.getClass().getName(), result.name(), sbm.getId().toString()});
            if (result.equals(InsertionResult.CANCEL)){
                break;
            }
            if (!result.equals(InsertionResult.NONE)){
                break;
            }
        }
        
        final boolean isSrgPostprocess = sbm.getReportType().equals("SRG") && !result.equals(InsertionResult.INSERT);
        
        if (!result.equals(InsertionResult.NONE) && !result.equals(InsertionResult.CANCEL)){
            final IssuezillaInsertionRequest request =  doStart(em, result);
            if (request != null){
                if (sbm.isForSecondaryQueue()) {
                    LogFileTask.getRunningTask().addTaskListener(new TaskListener() {

                        public void taskFinished(Task arg0) {
                        
                            LOG.log(Level.INFO, "[sbm {0}] : scheduling request on secondary"
                                + " queue", new Object[] { sbm.getId().toString() });
                            requestSecondaryQueue.add(request);
                            if (isSrgPostprocess) {
                                LOG.log(Level.INFO, "[sbm {0}] : scheduling SRG postprocess request"
                                    + " on secondary queue", new Object[] { sbm.getId().toString() });
                                requestSecondaryQueue.add(new IssuezillaInsertionRequest(userToken, asyncContext,
                                    request.submit, InsertionResult.UPDATE_TRACKING_GROUP, null, null, false));
                            }
                        }
                    });
                } else {
                    LogFileTask.getRunningTask().addTaskListener(new TaskListener() {

                        public void taskFinished(Task arg0) {
                            LOG.log(Level.INFO, SCHEDULEON_SECONDARY_MSG, new Object[] { sbm.getId().toString() });
                            requestQueue.add(request);
                            if (isSrgPostprocess) {
                                LOG.log(Level.INFO, "[sbm {0}] : scheduling SRG postprocess request"
                                    + " on primary queue", new Object[] { sbm.getId().toString() });
                                requestQueue.add(new IssuezillaInsertionRequest(userToken, asyncContext,
                                    request.submit, InsertionResult.UPDATE_TRACKING_GROUP, null, null, false));
                            }

                        }
                    });
                }
            }
        } else {
            // No action on BugDB due to filters, but we need to perform udpate of tracking groups anyway
            if (isSrgPostprocess) {
                LOG.log(Level.INFO, "[sbm {0}] : shceduling standalone (no other BugDb action)"
                    + " SRG postprocess request", new Object[] { sbm.getId().toString() } );
                requestQueue.add(new IssuezillaInsertionRequest(userToken, asyncContext,
                                        sbm, InsertionResult.UPDATE_TRACKING_GROUP, null, null, false));
            }
        }
        
    }

    void startReopenInsert() {
        LOG.log(Level.INFO, "[sbm {0}] Starting \"reopen\" BugDb insert.", sbm.getId().toString());
        EntityManager em = LogFileTask.getRunningTask().getTaskEntityManager();
        for (BugTrackerInsertionFilter filter : insertionFilterProvider.getReopenFilters(sbm.getClass())) {
            if ((filter.isInsertable(em, sbm)).equals(InsertionResult.CANCEL)){
                LOG.log(Level.WARNING, "[sbm {1}] Canceling reopen based on result of {0}", new Object[] { filter.getClass().getName(), sbm.getId().toString() });
                return;
            }
        }
        IssuezillaInsertionRequest request = doStart(em, InsertionResult.INSERT);
        if (request != null){
            if (sbm.isForSecondaryQueue()) {
                LOG.log(Level.INFO, SCHEDULE_ON_PRIMARY_MSG, new Object[] { sbm.getId() });
               requestSecondaryQueue.add(request);
            } else {
                LOG.log(Level.INFO, SCHEDULEON_SECONDARY_MSG, new Object[] { sbm.getId() });
               requestQueue.add(request);
            }
        }
    }

    public void startForceInsert(EntityManager em) {
        LOG.log(Level.INFO, "Starting forced BugDb insert.");
        IssuezillaInsertionRequest request = doStart(em, InsertionResult.INSERT);
        if (request != null){
            if (sbm.isForSecondaryQueue()) {
                LOG.log(Level.INFO, SCHEDULE_ON_PRIMARY_MSG, new Object[] { sbm.getId() });
               requestSecondaryQueue.add(request);
            } else {
                LOG.log(Level.INFO, SCHEDULEON_SECONDARY_MSG, new Object[] { sbm.getId() });
               requestQueue.add(request);
            }
        }
    }
    
    private synchronized  IssuezillaInsertionRequest doStart(EntityManager em, InsertionResult result){
        if (sbm.getReportId().inIssuezillaTransfer() && InsertionResult.INSERT.equals(result)){
            LOG.log(Level.INFO, "[sbm {0}] BugDb insert cancelled - already in transfer", sbm.getId().toString());
            return null;
        }
        if (result.equals(InsertionResult.INSERT) && !sbm.getReportId().isInIssuezilla() && isBugFiling()) {
            sbm.getReportId().setIssuezillaTransfer(true);
        }
        Submit mergedSbm = em.merge(sbm);
        return new IssuezillaInsertionRequest(userToken, asyncContext, mergedSbm, result, extraMessage, extraCC, true);
    }

    private static boolean isGuest(Nbuser user){
        return Utils.isGuest(user.getName());
    }

    private Long getDuplicatesNo() {
        return insertionFilterProvider.getDuplicatesNo();
    }

    private int getDuplicatesAmount(int duplicatesCount) {
        return duplicatesCount / getDuplicatesNo().intValue();
    }

    private class IssuezillaCommentRequest extends Persistable.Query {
        
        private Submit submit;
        
        public IssuezillaCommentRequest(Submit submit) {
            this.submit = submit;
        }
        
        public TransactionResult runQuery(EntityManager em) {
                    
            bugReporter.postTextComment(submit.getReportId().getIssueId(), getManyDuplicatesComment( em, submit ));
            return TransactionResult.NONE;
        }
        
    }
    
    // "Issuezilla" in this context means any bug tracker (either bugzilla, BugDB or any other)
    private class IssuezillaInsertionRequest extends Persistable.Query implements Runnable {
        private static final int MAX_IZ_INVOCATIONS = 4;
        private int invocationCount;
        private final AuthToken userToken;
        private final AsyncContext asyncContext;
        private final Submit submit;
        private final InsertionResult insertionResult;
        private final String extraCC, extraMessage;
        private final boolean isReopen;
        
        public IssuezillaInsertionRequest(AuthToken userToken, AsyncContext asyncContext, 
                Submit sbm, InsertionResult insertionResult, 
                String extraMessage, String extraCC, boolean isReopen) {
            this.userToken = userToken;
            this.asyncContext = asyncContext;
            this.submit = sbm;
            this.insertionResult = insertionResult;
            this.invocationCount = 0;
            this.extraCC = extraCC;
            this.extraMessage = extraMessage;
            this.isReopen = isReopen;
        }

        private String getPasswd(){
            if (userToken != null){
                return userToken.getPasswd();
            }else{
                return null;
            }
        }

        public void run() {
            invocationCount++;
            try {
                if (isBugFiling()) {
                    switch (insertionResult) {
                        case INSERT:
                            LOG.log(Level.INFO, "[sbm {0}] Trying to report exception", submit.getId().toString());
                            bugReporter.reportSubmit(submit, getPasswd(), asyncContext, extraMessage, extraCC, isReopen);
                            break;
                        case COMMENT:
                            LOG.log(Level.INFO, "[sbm {0}] Trying to add comment to exception", submit.getId().toString());
                            PersistenceUtils.processPersistable(new IssuezillaCommentRequest(submit));
                            break;
                        case ATTACHMENT:
                            LOG.log(Level.INFO, "[sbm {0}] Trying to add attachement to exception", submit.getId().toString());
                            bugReporter.createAttachment(submit, getPasswd());
                            checkPriority();
                            break;
                        case ADDTOCC:
                            addReporterToCC();
                            checkPriority();
                            break;
                        case INCREASE_PRIORITY:
                            checkPriority();
                            break;
                        case UPDATE_TRACKING_GROUP:
                            String groupName = submit.getAdditionalInfo("trackingGroupName");
                            String groupValue = submit.getAdditionalInfo("trackingGroupValue");
                            
                            LOG.log(Level.INFO, "[sbm {0}] Trying to update tracking group for: group : {1}, value : {2}", 
                                new Object[] { submit.getId().toString(), groupName, groupValue } );
                            
                            bugReporter.addOrUpdateTrackingGroup(submit.getReportId().getIssueId(), 
                                groupName, groupValue);
                            break;
                        default:
                            assert false; // should never happen
                    }
                } else {
                    LOG.log(Level.INFO, "[sbm {0}] Skipped bug database insertion for exception id: {0}. Bug filing is turned off.", submit.getId().toString());
                }
            }catch (BugReporterException e){
                if (invocationCount < MAX_IZ_INVOCATIONS){
                    Object[] params = new Object[]{submit.getId().toString(), Integer.toString(invocationCount), e.getMessage()};
                    LOG.log(Level.WARNING, "[sbm {0}] Filling issue {0} failed for {1} times, beacause of {2}. Rescheduling", params);
                    if (submit.isForSecondaryQueue()) {
                        
                        LOG.log(Level.INFO, SCHEDULEON_SECONDARY_MSG, submit.getId().toString());
                        izSecondaryRequestProcessor.post(new Runnable() {

                            public void run() {
                                requestSecondaryQueue.add(IssuezillaInsertionRequest.this);
                            }
                         }, timeoutProvider.getSecondaryDelay() * 2);
                    } else {
                        izRequestProcessor.post(new Runnable() {

                            public void run() {
                                requestQueue.add(IssuezillaInsertionRequest.this);
                            }
                         }, timeoutProvider.getDelay() * 2);
                    }
                }else if (insertionResult == InsertionResult.INSERT) {
                    issueFilingFailed(e);
                
                    Object[] params = new Object[]{submit.getId().toString(), Integer.toString(invocationCount)};
                    LOG.log(Level.SEVERE, "[sbm {0}] Filling issue {0} failed for {1} times. No more attempts.", params);
                    throw e;
                }
            }
        }
        
        private synchronized void issueFilingFailed(Exception ex) {
            
            final int id = submit.getId();
            String message = "Bug processing error - see server logs";
            if (ex != null) {
                if (ex.getCause()!=null) {
                    message = ex.getCause().getClass() + " : " + ex.getCause().getMessage();
                } else {
                    message = ex.getClass() + " : " + ex.getMessage();
                }
            }
            
            if (message.isEmpty()) {
                message = "Bug processing error - see server logs";
            }
            message = escapeSpaces(message);
            processAsyncContextReply(asyncContext, message, "0", "ERROR");

            PersistenceUtils.processPersistable(new Persistable.Transaction() {

                public TransactionResult runQuery(EntityManager em) {
                    Submit thisSubmit = em.find(submit.getClass(), id);
                    if (thisSubmit == null) {
                        return TransactionResult.NONE;
                    }
                    Report report = thisSubmit.getReportId();
                    report.setIssuezillaTransfer(false);
                    report.setIssuezillaAttemptFailed(true);

                    em.merge(report);
                    return TransactionResult.COMMIT;

                }
            });
        }
        
        private void processAsyncContextReply(AsyncContext asyncContext, String reportId, String bugNo, String action) {

            if (asyncContext == null) {
                return;
            }
            
            try {
                LOG.log(Level.INFO, "[sbm {1}] Sending async servlet reply : {0}.", new Object[] { action, submit.getId().toString() }); // NOI18N 

                ServletRequest request = asyncContext.getRequest();
                ServletResponse response = asyncContext.getResponse();
                request.setAttribute("action", action);
                request.setAttribute("reportNo", reportId);
                request.setAttribute("bugNo", bugNo);
                request.getRequestDispatcher("/uploadredirect.jsp").forward(request, response); // NOI18N
                asyncContext.complete();

            } catch (ServletException | IOException | IllegalStateException ex) {
                // Illegal state : in case response has timed out - we still need to have a bug stored in db
                //     although we have entered it late (after async server response http timeout in this case)
                //     - thereofore no rethrow

                // This is a not very clean code but there's really no check like "isComplete()" on AsyncContext API
                // (Note: if there's one in the future, it needs to be synchronided)
                if (!(ex instanceof IllegalStateException) || !ex.getMessage().contains("already completed")) {
                    LOG.log(Level.SEVERE, ex.getMessage(), ex);
                }
            }
        }

        private String escapeSpaces(String message) {
            return message.replaceAll(" ", "&nbsp;");
        }
        
        public SubmitInterface.Attachment createAttachment(Submit submit) {
            byte[] data = null;
            try {
                ByteArrayOutputStream bos = new ByteArrayOutputStream();
                SnapshotManager sm = SnapshotManager.loadSnapshot(submit.getLogfileId());
                DataOutputStream dos = new DataOutputStream(bos);
                if (sm != null){
                    sm.save(dos);
                }
                dos.close();
                data = bos.toByteArray();
            } catch (IOException ex) {
                org.openide.util.Exceptions.printStackTrace(ex);
            }
            return new SubmitInterface.Attachment("snapshot-%1$s.nps", "nps snapshot", "application/nps", data);
        }

        private void checkPriority(){
            PersistenceUtils.processPersistable(this);
        }

        private void addReporterToCC(){
            if (!userToken.getUserName().equals(Utils.getReporterUsername())){
                LOG.log(Level.INFO, "[sbm {0}] Trying to add to cc for issue", submit.getId().toString());
                bugReporter.addToCCList(submit.getReportId().getIssueId(), userToken.getUserName(), userToken.getPasswd());
            }
        }

        // increase priority if needed
        public TransactionResult runQuery(EntityManager em) {
            Submit merged = Submit.getById(em, submit.getId());
            if (!merged.getReportId().isInIssuezilla()){
                // nothing to do
                return TransactionResult.NONE;
            }
            int issuezillaId = merged.getReportId().getIssueId();
            if (!PersistenceUtils.issueIsOpen(issuezillaId)){
                return TransactionResult.NONE;
            }
            int duplicatesCount = merged.getReportId().getDuplicates(em);
            int duplicatesAmount = getDuplicatesAmount(duplicatesCount);
            if (duplicatesAmount == 0){
                return TransactionResult.NONE;
            }
            if (merged instanceof Exceptions &&
                    ((Exceptions)merged).getStacktrace().getClass1().contains(AssertionError.class.getName())){
                return TransactionResult.NONE;
            }
            if (merged instanceof Exceptions && duplicatesCount % getDuplicatesNo() == 0){
                if ((duplicatesAmount % P1_PRIORITY_AMOUNT) == 0) {
                    setIssuePriority(em, merged, issuezillaId, P1_PRIORITY_STR);
                } else if (duplicatesAmount == P2_PRIORITY_AMOUNT) {
                    Issue issue = bugReporter.getIssue(issuezillaId);
                    String priority = issue.getPriority();
                    //check if priority is reasonable
                    if (!P1_PRIORITY_STR.equals(priority) && !P2_PRIORITY_STR.equals(priority) &&
                            !P3_PRIORITY_STR.equals(priority)){
                        LOG.log(Level.WARNING, "Suspicous priority {0} at issue {1}!", new Object[]{priority, issuezillaId}); 
                    }
                    if (!P1_PRIORITY_STR.equals(priority)) {
                        setIssuePriority(em, merged, issuezillaId, P2_PRIORITY_STR);
                    }
                } else if (duplicatesAmount == MESSAGE_AMOUNT || duplicatesAmount == MESSAGE_2_AMOUNT) {
                    String commentMessage = getManyDuplicatesComment(em, merged);
                    bugReporter.postTextComment(issuezillaId, commentMessage);
                }
            }
            return TransactionResult.NONE;
        }

        private void setIssuePriority(EntityManager em, Submit exc, int issuezillaId, String priorityStr){
            Issue issue = bugReporter.getIssue(issuezillaId);
            String commentMessage = getManyDuplicatesComment(em, exc);
            if ("RESOLVED".equals(issue.getIssueStatus()) && "LATER".equals(issue.getResolution())){
                bugReporter.reopenIssue(issuezillaId, commentMessage);
            }else{
                bugReporter.postTextComment(issuezillaId, commentMessage);
            }
            bugReporter.setIssuePriority(issuezillaId, priorityStr);
        }
    }
    
    /**
     * !!!ONLY FOR TESTS!!!
     */
    static void setReporterTimeout(int timeout) {
        REPORTER_TIMEOUT = timeout;
    }

    // Note: this method is called from tests only. For proper use it 
    //       shouldn't use REPORTER_TIMEOUT directly, but get real 
    //       information from timeoutProvider - but it is not 
    //       a static provider.
    static void waitIZInsertionFinished() throws InterruptedException{
        final AtomicBoolean isEmpty = new AtomicBoolean(false);
        while (!isEmpty.get()){
            izRequestProcessor.post(new Runnable() {

                public void run() {
                    isEmpty.set(requestQueue.isEmpty());        
                }
            }, REPORTER_TIMEOUT).waitFinished();
            izSecondaryRequestProcessor.post(new Runnable() {

                public void run() {
                    isEmpty.set(requestSecondaryQueue.isEmpty());        
                }
            }, REPORTER_TIMEOUT).waitFinished();
        }
        
    }

    private static class IZInsertionTask extends TimerTask {

        public void run() {
            IssuezillaInsertionRequest firstRequest = requestQueue.poll();
            if (firstRequest != null) {
                izRequestProcessor.post(firstRequest);
            }
        }
    }
    
    private static class IZSecondaryInsertionTask extends TimerTask {

        public void run() {
            IssuezillaInsertionRequest firstRequest = requestSecondaryQueue.poll();
            if (firstRequest != null) {
                izSecondaryRequestProcessor.post(firstRequest);
            }
        }
    }

    private String getManyDuplicatesComment(EntityManager em, Submit exc) {
        return bugReporter.getManyDuplicatesComment(em, exc);
    }

}
