/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.server.uihandler.statistics;

import java.util.LinkedHashMap;
import java.util.logging.LogRecord;
import java.util.prefs.BackingStoreException;
import java.util.prefs.Preferences;
import javax.servlet.jsp.PageContext;
import org.netbeans.server.uihandler.Statistics;
import org.netbeans.server.uihandler.statistics.VersioningCommandUsage.CommandUsage;
import org.openide.util.lookup.ServiceProvider;

/**
 *
 * @author Ondra Vrabec
 */
@ServiceProvider(service=Statistics.class)
public class VersioningCommandUsage extends Statistics<CommandUsage> {

    public static final String STATISTIC_NAME = "VersioningCommand"; //NOI18N

    public VersioningCommandUsage() {
        super(STATISTIC_NAME, 0);
    }

    @Override
    protected CommandUsage newData() {
        return new CommandUsage(0, 0, 0, 0, 0, 0);
    }

    @Override
    protected CommandUsage process(LogRecord rec) {
        if ("USG_VCS_CMD".equals(rec.getMessage())) {
            String vcs = rec.getParameters()[0].toString();
            long time = Long.parseLong(rec.getParameters()[1].toString());
            long modifications = Long.parseLong(rec.getParameters()[2].toString());
            Object kind = rec.getParameters()[4];
            if ("INTERNAL".equals(kind)) {
                return CommandUsage.internal(time, modifications);
            } else if ("EXTERNAL".equals(kind)) {
                return CommandUsage.external(time, modifications);
            }
        }
        return newData();
    }
    
    @Override
    protected void registerPageContext(PageContext page, String name, CommandUsage data) {
        LinkedHashMap<String,Long> mapCount = new LinkedHashMap<String, Long>();
        LinkedHashMap<String,Long> mapTime = new LinkedHashMap<String, Long>();
        LinkedHashMap<String,Long> mapModif = new LinkedHashMap<String, Long>();
        if (data != null) {
            mapCount.put("Internal", data.internalInvocations); //NOI18N
            mapCount.put("External", data.externalInvocations); //NOI18N
            mapTime.put("Internal", data.internalTime / 1000L); //NOI18N
            mapTime.put("External", data.externalTime / 1000L); //NOI18N
            mapModif.put("Internal", data.internalModifications); //NOI18N
            mapModif.put("External", data.externalModifications); //NOI18N
        }
        page.setAttribute(name, data                                                                                                                                                                                                                                                                                                                                                                                        );
        page.setAttribute(name + "Count", mapCount);
        page.setAttribute(name + "Time", mapTime);
        page.setAttribute(name + "Modifications", mapModif);
    }

    @Override
    protected CommandUsage finishSessionUpload(String userId, int sessionNumber, boolean initialParse, CommandUsage d) {
        // here we could filter all records from one user
        return d;
    }

    @Override
    protected CommandUsage join(CommandUsage one, CommandUsage two) {
        if (one == null) {
            return two;
        }
        if (two == null) {
            return one;
        }
        return new CommandUsage(one.internalInvocations + two.internalInvocations,
                one.externalInvocations + two.externalInvocations,
                one.internalTime + two.internalTime,
                one.externalTime + two.externalTime,
                one.internalModifications + two.internalModifications,
                one.externalModifications + two.externalModifications
                );
    }

    @Override
    protected void write (Preferences pref, CommandUsage d) throws BackingStoreException {
        if (d != null) {
            pref.putLong("exCount", d.externalInvocations); // NOI18N
            pref.putLong("exTime", d.externalTime); // NOI18N
            pref.putLong("exModif", d.externalModifications); // NOI18N
            pref.putLong("inCount", d.internalInvocations); // NOI18N
            pref.putLong("inTime", d.internalTime); // NOI18N
            pref.putLong("inModif", d.internalModifications); // NOI18N
        }
    }

    @Override
    protected CommandUsage read (Preferences pref) throws BackingStoreException {
        long exCount = pref.getLong("exCount", 0);
        long exTime = pref.getLong("exTime", 0);
        long exModif = pref.getLong("exModif", 0);
        long inCount = pref.getLong("inCount", 0);
        long inTime = pref.getLong("inTime", 0);
        long inModif = pref.getLong("inModif", 0);
        return new CommandUsage(inCount, exCount, inTime, exTime, inModif, exModif);
    }
    
    public static final class CommandUsage {

        private long internalInvocations;
        private long externalInvocations;
        private long internalTime;
        private long externalTime;
        private long internalModifications;
        private long externalModifications;
        
        CommandUsage(long internalInvocations,
                long externalInvocations,
                long internalTime,
                long externalTime,
                long internalModifications,
                long externalModifications) {
            this.internalInvocations = internalInvocations;
            this.internalTime = internalTime;
            this.internalModifications = internalModifications;
            this.externalInvocations = externalInvocations;
            this.externalTime = externalTime;
            this.externalModifications = externalModifications;
        }

        private static CommandUsage internal (long time, long modifications) {
            return new CommandUsage(1, 0, time, 0, modifications, 0);
        }

        private static CommandUsage external (long time, long modifications) {
            return new CommandUsage(0, 1, 0, time, 0, modifications);
        }

        @Override
        public String toString() {
            return new StringBuilder("Internal count: ").append(internalInvocations)
                    .append(", time: ").append(internalTime)
                    .append(", changes: ").append(internalModifications)
                    .append("External count: ").append(externalInvocations)
                    .append(", time: ").append(externalTime)
                    .append(", changes: ").append(externalModifications)
                    .toString();
        }
    }
}
