/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.web.action;

import org.netbeans.server.uihandler.IZInsertion;
import org.netbeans.server.uihandler.api.bugs.BugReporterFacade;
import org.netbeans.server.uihandler.api.bugs.BugReporterFactory;
import org.netbeans.server.uihandler.api.bugs.ReportThrottleFactory;
import org.netbeans.server.uihandler.api.bugs.ReportThrottleFactoryProvider;

/**
 *
 * @author jlehotsky
 */
public class AdminFacade {
    
    private static final AdminFacade INSTANCE = new AdminFacade();
    private ReportThrottleFactory throttleFactory;
    private BugReporterFacade bugReporter;
    
    private AdminFacade() {
    }
    
    public static AdminFacade getInstance() {
        return INSTANCE;
    }

    public boolean isProduction() {
        return isThrottling() && isBugFiling() && getNewBugStatus() == 11 && areTaskQueuesEnabled();
    }

    public boolean isThrottling() {
        intializeThrottlingService();
        return throttleFactory.isThrottling();
    }

    public int getNewBugStatus() {
        initializeBugReporter();        
        return bugReporter.adminGetNewBugStatus();
    }

    public void setNewBugStatus(int newBugStatus) {
        initializeBugReporter();        
        bugReporter.adminSetNewBugStatus(newBugStatus);
        // we need to reset connection because bug status is cached in pre-generated query:
        forceResetBugDbConnection();
    }
        
    public int getJDevJobs() {
        return IZInsertion.getRequestQueueLength();
    }

    public int getSRGJobs() {
        return IZInsertion.getSecondaryRequestQueueLength();
    }

    public void setThrottling(boolean throttling) {
        intializeThrottlingService();
        throttleFactory.setThrottling(throttling);
    }

    public boolean areTaskQueuesEnabled() {    
        return IZInsertion.areQueuesEnabled();
    }

    public void setTaskQueuesEnabled(boolean queuesEnabled) {
        IZInsertion.enableQueues(queuesEnabled);
    }

    public void forceResetBugDbConnection() {
        initializeBugReporter();
        bugReporter.adminEnsureConnectionAlive();    
    }

    private void intializeThrottlingService() {
        if (throttleFactory == null) {
            throttleFactory = ReportThrottleFactoryProvider.getReportThrottleFactory();
        }
    }

    private void initializeBugReporter() {
        if (bugReporter == null) {
            bugReporter = BugReporterFactory.getReporter();
        
        }
    }

    boolean isBugFiling() {
        return IZInsertion.isBugFiling();
    }
    
    void setBugFiling(boolean bugFiling) {
        IZInsertion.setBugFiling(bugFiling);
    }

}
