/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package org.netbeans.server.uihandler.statistics.vcs;

/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Copyright 2007 Sun Microsystems, Inc. All rights reserved.
 */


import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.logging.Level;
import javax.servlet.jsp.PageContext;
import static junit.framework.Assert.assertEquals;
import org.netbeans.junit.Log;
import org.netbeans.junit.NbTestCase;
import org.netbeans.server.uihandler.DatabaseTestCase;
import org.netbeans.server.uihandler.LogsManager;
import org.netbeans.server.uihandler.LogsManagerTest;

/**
 *
 * @author Jaroslav Tulach
 */
public class VersioningCommandsTest extends NbTestCase {
    private static LogsManager result;
    private CharSequence seq;

    
    public VersioningCommandsTest(String testName) {
        super(testName);
    }
    @Override
    protected Level logLevel() {
        return Level.WARNING;
    }
    
    private File logs() throws IOException {
        File f = new File(getWorkDir(), "logs");
        f.mkdirs();
        return f;
    }
    
    
    @Override
    protected void setUp() throws Exception {
        clearWorkDir();
        if (result == null) {
            DatabaseTestCase.initPersitenceUtils(getWorkDir());

            result = LogsManagerTest.createManager(logs());
            result.waitVerificationFinished();

            result.preparePageContext(LogsManagerTest.createPageContext(), "", null);

            File log = LogsManagerTest.extractResourceAs(VersioningCommandsTest.class, logs(), "vcs.log", "log444");

            result.addLog(log, "127.0.0.1");
        }
        
        seq = Log.enable("org.netbeans.server.uihandler.LogsManager", Level.WARNING);
    }
    
    @Override
    protected void tearDown() throws Exception {
        super.tearDown();
    }

    public void testUser444VersioningCommands() throws Exception {
        PageContext png = LogsManagerTest.createPageContext();
        result.preparePageContext(png, "log444", null);

        Object o = png.getAttribute("userVersioningCommandCount");
        assertNotNull("statistic created", o);
        assertEquals("Right class: " + o.getClass(), true, Map.class.isAssignableFrom(o.getClass()));
        Map inv = (Map) o;
        assertEquals("Two keys present: " + inv, 2, inv.size());
        assertEquals("2 internal commands: " + inv, 2L, inv.get("Internal"));
        assertEquals("2 external commands: " + inv, 2L, inv.get("External"));
        
        o = png.getAttribute("userVersioningCommandModifications");
        assertNotNull("statistic created", o);
        assertEquals("Right class: " + o.getClass(), true, Map.class.isAssignableFrom(o.getClass()));
        inv = (Map) o;
        assertEquals("Two keys present: " + inv, 2, inv.size());
        assertEquals("149 files modified internally: " + inv, 149L, inv.get("Internal"));
        assertEquals("282 files modified externally: " + inv, 282L, inv.get("External"));

        o = png.getAttribute("userVersioningCommandTime");
        assertNotNull("statistic created", o);
        assertEquals("Right class: " + o.getClass(), true, Map.class.isAssignableFrom(o.getClass()));
        inv = (Map) o;
        assertEquals("Two keys present: " + inv, 2, inv.size());
        assertEquals("377s spent internally: " + inv, 377262 / 1000L, inv.get("Internal"));
        assertEquals("95s spent externally: " + inv, 95279 / 1000L, inv.get("External"));
    }

}
