(function ($) {
    var Rating = function (options) {
        this.init('rating', options, Rating.defaults);
    };

    //inherit from Abstract input
    $.fn.editableutils.inherit(Rating, $.fn.editabletypes.abstractinput);

    $.extend(Rating.prototype, {
        /**
Renders input from tpl

@method render()
**/
        render: function() {
            this.$input = this.$tpl.find('button');
        },
        
        /**
Default method to show value in element. Can be overwritten by display option.
@method value2html(value, element)
**/
        value2html: function(value, element) {
            if(typeof value === 'undefined') {
                $(element).empty();
                return;
            }
       
            if (value == 0) {
                html = '<i class="icon-remove"></i>';
            } else if (value == 1) {
                html = '<i class="icon-ok"></i>';
            } else if (value == 2) {
                html = '';
            } else {
                html = '<i class="icon-edit"></i>';
            }
            
            $(element).html(html); 
        },
        
        /**
Gets value from element's html
@method html2value(html)
**/
        html2value: function(html) {
          
            return null;
        },
      
        /**
Converts value to string.
It is used in internal comparing (not for sending to server).
@method value2str(value)
**/
        value2str: function(value) {
            return value;
        },
       
        /*
Converts string to value. Used for reading value from 'data-value' attribute.
@method str2value(str)
*/
        str2value: function(str) {
            /*
this is mainly for parsing value defined in data-value attribute.
If you will always set value by javascript, no need to overwrite it
*/
            return str;
        },
       
        /**
Sets value of input.
@method value2input(value)
@param {mixed} value
**/
        value2input: function(value) {
            if (value == 0) {
                this.$input.filter('[name="wrong"]').button("toggle");
            } else if (value == 1) {
                this.$input.filter('[name="correct"]').button("toggle");
            } 
        },
       
        /**
Returns value of input.
@method input2value()
**/
        input2value: function() {
            if (this.$input.filter('[name="wrong"]').attr('class').search('active') > -1) {
                return 0;
            }
            if (this.$input.filter('[name="correct"]').attr('class').search('active') > -1) {
                return 1;
            }
            return -1;
        },
       
        /**
Activates input: sets focus on the first field.
@method activate()
**/
        activate: function() {
            this.$input.filter('[name="wrong"]').button();
            this.$input.filter('[name="correct"]').button();
        },
       
        /**
Attaches handler to submit form in case of 'showbuttons=false' mode
@method autosubmit()
**/
        autosubmit: function() {
            this.$input.keydown(function (e) {
                if (e.which === 13) {
                    $(this).closest('form').submit();
                }
            });
        }
    });

    Rating.defaults = $.extend({}, $.fn.editabletypes.abstractinput.defaults, {
        tpl: '<div class="editable-rating">    <div class="btn-group" id="rating-grp" data-toggle="buttons-radio">' +
        '<button type="button" name="correct" class="btn btn-primary">Correct</button>' +
        '<button type="button" name="wrong" class="btn btn-primary">Wrong</button>' +
        '</div>',
             
        inputclass: ''
    });

    $.fn.editabletypes.rating = Rating;

}(window.jQuery));