/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.jdbcimpl;

import org.netbeans.mdr.persistence.*;
import org.netbeans.mdr.util.*;

import java.util.*;
import java.io.*;

/**
 * JdbcSinglevaluedIndex implements the MDR SinglevaluedIndex interface
 * using JDBC.
 *
 * @author John V. Sichi
 * @version $Id$
 */
class JdbcSinglevaluedIndex
    extends JdbcIndex implements SinglevaluedIndex
{
    protected LazyPreparedStatement sqlValuesIterator;
    protected LazyPreparedStatement sqlValuesSize;
    protected LazyPreparedStatement sqlValuesContains;
    protected LazyPreparedStatement sqlUpdate;
    
    protected void defineSql()
    {
        super.defineSql();

        sqlValuesIterator = new LazyPreparedStatement(
            "select " + valColName + " from " + tableName);
        
        sqlValuesSize = new LazyPreparedStatement(
            "select count(*) from " + tableName);
        
        sqlValuesContains = new LazyPreparedStatement(
            "select count(*) from " + tableName + " where "
            + valColName + " = ?");
        
        sqlUpdate = new LazyPreparedStatement(
            "update " + tableName + " set " + valColName
            + " = ? where " + keyColName + " = ?");
    }
    
    protected boolean isKeyUnique()
    {
        return true;
    }

    // implement SinglevaluedIndex
    public boolean put(Object key,Object value) throws StorageException
    {
        return putImpl(key,value);
    }
    
    protected boolean putImpl(Object key,Object value) throws StorageException
    {
        // REVIEW:  this optimizes for update rather than insert; could
        // use UPSERT on databases that support it
        int rowCount = storage.executeUpdate(
            sqlUpdate,new Object[]{value,key});
        // TODO:  assert rowCount == 0 or 1
        if (rowCount == 0) {
            // key did not exist; insert instead
            addImpl(key,value);
            return false;
        } else {
            // key existed and has already been updated
            return true;
        }
    }
    
    // implement SinglevaluedIndex
    public void replace(Object key, Object value)
        throws StorageException, StorageBadRequestException
    {
        replaceImpl(key,value);
    }
    
    protected void replaceImpl(Object key, Object value)
        throws StorageException, StorageBadRequestException
    {
        if (!putImpl(key,value)) {
            throw new StorageBadRequestException(
                "Cannot replace item that does not exist in the index.");
        }
    }
    
    // implement SinglevaluedIndex
    public Object get(Object key)
        throws StorageException, StorageBadRequestException
    {
        Object obj = getIfExists(key);
        if (obj == null) {
            throw new StorageBadRequestException ("Item not found: " + key);
        }
        return obj;
    }

    // implement SinglevaluedIndex
    public Object getObject(Object key, SinglevaluedIndex repos)
        throws StorageException
    {
        // NOTE: If JdbcPrimaryIndex did no caching, we could optimize this
        // with an SQL join.  But we can't join against the cache.  Same goes
        // for all of the other calls that take a repos argument.
        if (keyType == Storage.EntryType.MOFID) {
            return repos.get(get(key));
        } else {
            return get(key);
        }
    }

    // implement SinglevaluedIndex
    public Object getIfExists(Object key) throws StorageException
    {
        Iterator iter = storage.getResultSetIterator(
            sqlFind,new Object[]{key},getValueType());
        if (!iter.hasNext()) {
            return null;
        }
        return iter.next();
    }
    
    // implement SinglevaluedIndex
    public Object getObjectIfExists(Object key, SinglevaluedIndex repos)
        throws StorageException
    {
    	Object val = getIfExists(key);
        if (val == null) {
            return null;
        } else {
            if (keyType == Storage.EntryType.MOFID) {
                return repos.get(val);
            } else {
                return val;
            }
        }
    }

    // implement SinglevaluedIndex
    public Collection values()
        throws StorageException
    {
        return new JdbcCollection(
            storage,
            getValueType(),
            sqlValuesIterator,sqlValuesSize,sqlValuesContains);
    }
    
    // implement SinglevaluedIndex
    public Collection queryByKeyPrefix(
        Object prefix, SinglevaluedIndex repos) throws StorageException
    {
        // TODO:  optimize with LIKE
        throw new RuntimeException("oops, not yet implemented");
    }
}

// End JdbcSinglevaluedIndex.java
