/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is Forte for Java, Community Edition. The Initial
 * Developer of the Original Software is Sun Microsystems, Inc. Portions
 * Copyright 1997-2006 Sun Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.mdrxml.looks;

import java.awt.datatransfer.Transferable;
import java.awt.datatransfer.StringSelection;
import java.io.IOException;
import java.util.Collection;
import java.util.Iterator;
import javax.jmi.reflect.*;
import javax.jmi.model.*;
import org.openide.util.NbBundle;
import org.netbeans.api.looks.*;
import org.netbeans.spi.looks.*;
import org.netbeans.api.mdr.*;
import xmlmodel.XmlmodelPackage;
import xmlmodel.RootNodeClass;
import xmlmodel.RootNode;
import org.netbeans.modules.mdrxml.util.XMLModelEventTranslator;
import org.netbeans.modules.mdrxml.util.NodeEventTranslator;
/**
 *
 * @author  Tomas Zezula
 */
public class DocumentsLook extends DefaultLook {
    
    private static final String ICON = "org/netbeans/modules/mdrxml/resources/packageproxy";
    
    /** Creates a new instance of DocumentsLook */
    public DocumentsLook() {
        super (NbBundle.getMessage (DocumentsLook.class, "TXT_DocumentsLook"));
    }
    
    public Object attachTo (Look.NodeSubstitute substitute) {
        super.attachTo (substitute);
        XMLModelEventTranslator translator = XMLModelEventTranslator.addXMLPackage(substitute);
        return new NodeEventTranslator (translator,(RefBaseObject)substitute.getRepresentedObject(),substitute);
    }
    
    public String getName (Look.NodeSubstitute substitute) {
        RefPackage pkg = (RefPackage) substitute.getRepresentedObject ();
        MDRepository repo = ((MDRObject)pkg).repository();
        String[] names = repo.getExtentNames ();
        for (int i=0; i< names.length; i++) {
            if (repo.getExtent (names[i]).equals (pkg))
                return names[i];
        }
        return ((MofPackage)pkg.refMetaObject()).getName();
    }
    
    public String getDisplayName (Look.NodeSubstitute substitute) {
        return this.getName (substitute);
    }
    
    public String iconBase (Look.NodeSubstitute substitute) {
        return ICON;
    }
    
    public Object[] getChildObjects (Look.NodeSubstitute substitute) {
        XmlmodelPackage pkg = (XmlmodelPackage) substitute.getRepresentedObject ();
        MDRepository repo = ((MDRObject)pkg).repository ();
        repo.beginTrans (false);
        try {
            RootNodeClass rnClass = pkg.getRootNode ();
            Collection rootNodes = rnClass.refAllOfClass ();
            Object[] result = new Object [rootNodes.size()];
            Iterator it = rootNodes.iterator ();
            for (int i=0; it.hasNext(); i++) {
                result[i] = it.next();
            }
            return result;
        }finally {
            repo.endTrans ();
        }
    }
    
    public Transferable clipboardCopy (Look.NodeSubstitute substitute) throws IOException {
        return new StringSelection (this.getName(substitute));
    }
    
    public boolean canCopy (Look.NodeSubstitute substitute) {
        return true;
    }
    
}
