/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdrshell;

import java.util.Collection;
import java.util.Iterator;
import java.io.PrintStream;

/** Helper singleton for StringBuffer
 *
 * @author  Petr Hrebejk
 * @version
 */

class Support {

    /** Support is a singleton.
    */
    private Support() {
    }

    /** Aligns the StringBuffer to length given by position
    */
    public static void ensurePosition( StringBuffer sb, int position ) {

        if ( sb.length() < position ) {

            while ( sb.length() < position ) {
                sb.append( " " );
            }
        }
        else if ( sb.length() > position ) {
            sb.setLength( position );
        }
    }
    
    
    /** Default print function for results of commands. 
    */
    public static void defaultPrint( PrintStream ps, Object object ) {
        defaultPrint( ps, 0, object );
    }
    
    /** Default print function for results of commands at given position on
    * line.
    */
    public static void defaultPrint( PrintStream ps, int offset, Object object ) {
        
        if ( object == null ) {
            ps.println("null");
            return;
        }

        if ( object instanceof java.util.Iterator ) {
            printIndent( ps, offset );
            ps.print( "ITERATOR : {" );
            for( Iterator it = (Iterator)object; it.hasNext(); ) {
                defaultPrint( ps, offset + 2, it.next() );
            }
            ps.println();
            printIndent( ps, offset );
            ps.println( "} // end of ITERATOR" );
        }
        else if ( object.getClass().isArray() ) {
            printIndent( ps, offset );
            ps.print( "ARRAY : [" );
            for ( int i = 0; i < java.lang.reflect.Array.getLength( object ); i++ ) {
                //printIndent( ps, offset );
                defaultPrint( ps, offset + 2, java.lang.reflect.Array.get( object, i ) );
                //ps.println( java.lang.reflect.Array.get( object, i ).toString() );
            }
            ps.println();
            printIndent( ps, offset );
            ps.println( "] // end of ARRAY" );    
        }
        else if ( object instanceof Throwable ) {
            ((Throwable)object).printStackTrace();
        }
        else if ( object instanceof Collection  ) {
            printIndent( ps, offset );
            ps.print( "COLLECTION : {" );
            for( Iterator it = ((Collection)object).iterator(); it.hasNext(); ) {
                defaultPrint( ps, offset + 2, it.next() );
            }
            ps.println();
            printIndent( ps, offset );
            ps.println( "} // end of COLLECTION" );
        }
        else {
            printIndent( ps, offset );
            ps.println( object.toString() );
        }
        
    }

    
    private static void printIndent( PrintStream ps, int indent ) {
        for( ; indent > 0; indent -- ) {
            ps.print( " " );
        }
    }
    
}
