/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.jmiimpl.mof.model;

import java.util.*;
import javax.jmi.model.*;
import javax.jmi.reflect.*;

import org.netbeans.mdr.util.*;
import org.netbeans.mdr.storagemodel.*;
import org.netbeans.mdr.persistence.*;
import org.netbeans.mdr.handlers.*;

/**
 * Implements MOF operations
 *
 * @author mmatula
 */
public abstract class NamespaceImpl extends ModelElementImpl implements Namespace {

    protected NamespaceImpl(StorableObject storable) {
        super(storable);
    }

    private static final HashMap contains = new HashMap();

    public static void clearContains() {
        contains.clear();
    }

    protected static StorableAssociation resolveContains(StorableFeatured storable) throws StorageException {
        StorableAssociation result = (StorableAssociation) contains.get(storable.getOutermostPackageId());

        if (result == null) {
            result = storable.getClassProxy().getReferenceDescriptor(MOFConstants.SH_MODEL_MODEL_ELEMENT_CONTAINER).getAssociation();
            contains.put(storable.getOutermostPackageId(), result);
        }

        return result;
    }

    // --- operations

    public ModelElement lookupElement(String name) throws NameNotFoundException {
        _lock(false);
        try {
            Collection contents;

    //        StorableFeatured storable = (StorableFeatured) _getDelegate();
    //        Collection items = storable.getMdrStorage().objectsFromAdditionalIndex(storable.getOutermostPackageId(), "ModelElement.name", name);
    //
    //        if (items != null) {
    //            try {
    //                StorableAssociation assoc = resolveContains(storable);
    //                contents = (Collection) assoc.queryObjects(assoc.getEnd1Name(), storable.getMofId());
    //            } catch (StorageException e) {
    //                throw Logger.getDefault().annotate(new DebugException(), e);
    //            }
    //            Object result;
    //            for (Iterator it = items.iterator(); it.hasNext();) {
    //                result = it.next();
    //                if (contents.contains(result)) {
    //                    return (ModelElement) BaseObjectHandler.getHandler((StorableBaseObject) result);
    //                }
    //            }
    //        } else {
                contents = getContents();

                for(Iterator it = contents.iterator(); it.hasNext();) {
                    ModelElement el = (ModelElement) it.next();
                    if (el.getName().equals(name)) {
                        return el;
                    }
                }
    //        }
        } finally {
            _unlock();
        }

        throw new NameNotFoundException(name);
    }

    public ModelElement resolveQualifiedName(List qualifiedName) throws NameNotResolvedException {
        if (qualifiedName.size() < 1) {
            throw new DebugException("No qualified name provided.");
        }
        
        ModelElement element;
        Iterator qnIterator = qualifiedName.iterator();
        String name = (String) qnIterator.next();
        ArrayList rest = new ArrayList();

        _lock(false);
        try {
            try {
                element = lookupElement(name);
            } catch (NameNotFoundException e) {
                throw new NameNotResolvedException("InvalidName", qualifiedName);
            }

            for (; qnIterator.hasNext();) {
                rest.add(qnIterator.next());
            }

            if (rest.size() >= 1) {
                if (element instanceof Namespace) {
                    element = ((Namespace) element).resolveQualifiedName(rest);
                } else {
                    throw new NameNotResolvedException("NotNameSpace", qualifiedName);
                }
            }
        } finally {
            _unlock();
        }

        return element;
    }

    public boolean nameIsValid(String proposedName) {
        _lock(false);
        try {
            if (this instanceof GeneralizableElement) {
                ((GeneralizableElement) this).lookupElementExtended(proposedName);
            } else {
                this.lookupElement(proposedName);
            }
            return false;
        } catch (NameNotFoundException e) {
            return true;
        } finally {
            _unlock();
        }
   }

    public List findElementsByType(javax.jmi.model.MofClass ofType, boolean includeSubtypes ) {
        _lock(false);
        try {
            ArrayList result = new ArrayList();
            Collection contents = getContents();
            RefObject element;

            for (Iterator it = contents.iterator(); it.hasNext();) {
                element = (RefObject) it.next();
                if (element.refIsInstanceOf(ofType, includeSubtypes)) {
                    result.add(element);
                }
            }

            return result;
        } finally {
            _unlock();
        }
    }

    // --- derived attributes

}
