/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence;

import java.io.*;

/**
* This is thrown when a StorageException needs to be thrown across a boundary
* that does not allow checked exceptions: for instance, when using a
* Collections interface to iterate through a set of MDR objects.
* The StorageException is converted to a RuntimeStorageException.
* A RuntimeStorageException returns the stack trace information of the
* original StorageException occurred, since this is far more useful
* than the stack trace of the point where the exception was converted.
*/

public class RuntimeStorageException extends RuntimeException {

    /** The original StorageException that we are converting to a 
    * RuntimeException 
    */
    private StorageException storageExcept;

    /** this constructs a RuntimeStorageException from an StorageException
    * @param err the StorageException
    */
    public RuntimeStorageException(StorageException err) {
	super(err.getMessage());
	storageExcept = err;
    }

    /** return the original StorageException's message */
    public String getMessage() {
	return storageExcept.getMessage();
    }
	
    /** return the original StorageException's localized message */
    public String getLocalizedMessage() {
	return storageExcept.getLocalizedMessage();
    }

    /** print the original StorageException's stack trace */
    public void printStackTrace() {
	storageExcept.printStackTrace();
    }

    /** print the original StorageException's stack trace */
    public void printStackTrace(PrintStream ps) {
	storageExcept.printStackTrace(ps);
    }

    /** print the original StorageException's stack trace */
    public void printStackTrace(PrintWriter pw) {
	storageExcept.printStackTrace(pw);
    }
	
}
