/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreeindex;

import org.netbeans.mdr.persistence.*;
import org.netbeans.mdr.persistence.btreeimpl.btreestorage.BtreeStorage;
/**
 * Source of pages for a Btree.
 *
 * @author	Dana Bergen
 * @version	1.0
 */
public interface BtreePageSource extends MofidGenerator {

    /**
     * Get an EntryTypeInfo for the datatype of this BtreePageSource's page IDs.
     */
    public EntryTypeInfo getPageIdInfo();

    /**
     * Retrieve a page.
     *
     * @param	pageId	byte array buffer containing pageId
     * @param	btree	btree to which the page belongs
     *
     * @return	the BtreePage
     * @exception	StorageException
     */
    public BtreePage getPage(byte[] pageId, Btree btree) 
    					throws StorageException;
    /**
     * Creates a new page and returns it.
     *
     * @param	btree	btree to which the page belongs
     *
     * @return	the BtreePage
     * @exception	StorageException
     */
    public BtreePage newPage(Btree btree) throws StorageException;
    /**
     * Creates a new oversize-key page and returns it.
     *
     * @param	btree	btree to which the page belongs
     *
     * @return	the BigKeyPage
     * @exception	StorageException
     */
    public BigKeyPage newBigKeyPage(Btree btree) throws StorageException;
    /**
     * Retrieve the root page for this btree.
     *
     * @param	btree	Btree
     * @return	the root BtreePage
     * @exception	StorageException
     */
    public BtreePage getRootPage(Btree btree) throws StorageException;
    /**
     * Notify the BtreePageSource that the caller is done using this page.
     *
     * @param page	BtreePage
     */
    public void unpinPage(BtreePage page);
    /**
     * Notify the BtreePageSource that the caller is done using this page.
     *
     * @param page	BigKeyPage
     * @exception	StorageException
     */
    public void unpinPage(BigKeyPage page) throws StorageException;
    /**
     * Notify the BtreePageSource that the caller is going to modify this page.
     *
     * @param page	BtreePage
     * @exception	StorageException
     */
    public void dirtyPage(BtreePage page) throws StorageException;
    /**
     * Fills in the provided buffer with a value that represents a null page ID.
     *
     * @param	pageId	byte array to be filled in
     */
    public void setNoPage(byte[] pageId);
    /**
     * Test whether the passed-in pageId is equal to the null page ID.
     *
     * @return	true if the pageId is the null page ID, otherwise false
     */
    public boolean isNoPage(byte[] pageId);
    /**
     * Returns the length of a page ID.
     *
     * @return	length of a page ID from this BtreePageSource
     */
    public int getPageIdLength();
    /**
     * Returns the size of a page.
     *
     * @return	size of a page from this BtreePageSource
     */
    public int getPageSize();
    
    public BtreeStorage getStorage ();
    
}
