/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreeindex;

import org.netbeans.mdr.persistence.btreeimpl.btreestorage.*;	/* for Converter */

/**
 * EntryTypeInfo implementation for integer data.
 */

public class IntInfo extends EntryTypeInfo {

    public String typeName() {
        return "Integer";
    }

    /**
     * Store an integer in a byte array
     *
     * @param i	Integer to be stored
     *
     * @return	byte array containing the integer
     */
    public byte[] toBuffer(Object i) {

	byte[] buffer = new byte[4];

	if (!(i instanceof Integer)) {
	    return null;
	}
	Converter.writeInt(buffer, 0, ((Integer) i).intValue());
	return buffer;
    }

    /**
     * Reads an integer from a byte array
     *
     * @param buffer	byte array containing an integer
     *
     * @return		new Integer containing the value read from the byte array
     */
    public Object fromBuffer(byte[] buffer) {

	return new Integer(Converter.readInt(buffer, 0));
    }

    /**
     * Returns an int read from the 4 bytes starting at the location
     * passed in.
     *
     */
    public int fromBuffer(byte[] buffer, int offset) {

	return Converter.readInt(buffer, offset);
    }

    /**
     * Compares two integers stored in byte arrays
     *
     * @param	key1Buffer	byte array containing integer search key
     * @param	key2Buffer	byte array containing integer target key
     * @param	offset		offset into key2Buffer of target key
     * @param	length		should always be 4
     *
     * @return	Returns one of:
     * <p>EQUAL if the two keys are equal
     * <p>GREATER if search key is greater than target key
     * <p>LESS if search key is less than target key
     */
    public byte compare(byte[] key1Buffer, byte[] key2Buffer, int offset,
    							int length) {
	int key1, key2;

	key1 = Converter.readInt(key1Buffer, 0);
	key2 = Converter.readInt(key2Buffer, offset);

	if (key1 == key2) {
	    return EQUAL;
	} else if (key1 > key2) {
	    return GREATER;
	} else {
	    return LESS;
	}
    }
	
    /**
     * Returns the length of an integer
     * 
     * @return	always returns 4
     */
    public int getLength() {
        return 4;
    }

    public boolean isFixedLength() {
        return true;
    }

}
