/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.btreeimpl.btreeindex;

import java.io.PrintStream;

public class TreeMetrics {

    private int depth;
    private int numOfPages;
    private long totalBytes;
    private long usedBytes;
    private long numOfEntries;
    private long numOfLeafEntries;

    public TreeMetrics(int depth, int numOfPages, long totalBytes, long usedBytes, long numOfEntries,
        long numberOfLeafEntries) {

        this.depth = depth;
        this.numOfPages = numOfPages;
        this.totalBytes = totalBytes;
        this.usedBytes = usedBytes;
        this.numOfEntries = numOfEntries;
        this.numOfLeafEntries = numberOfLeafEntries;
    }

    /*
    public TreeMetrics() {
        this.depth = 0;
        this.numOfPages = 0;
        this.totalBytes = 0;
        this.usedBytes = 0;
        this.numOfEntries = 0;
    }
     */
    
    public int getDepth() {
        return depth;
    }
    
    public int getNumberOfPages() {
        return numOfPages;
    }
    
    public double getLoadFactor() {
        return totalBytes > 0 ? (double)usedBytes / (double)totalBytes : 0;
    }

    public long getTotalBytes() {
        return totalBytes;
    }

    public long getUsedBytes() {
        return usedBytes;
    }
    
    public void setDepth(int depth) {
        this.depth = depth;
    }

    public void setNumberOfPages(int numOfPages) {
        this.numOfPages = numOfPages;
    }

    public void setTotalBytes(long totalBytes) {
        this.totalBytes = totalBytes;
    }

    public void setUsedBytes(long usedBytes) {
        this.usedBytes = usedBytes;
    }

    public void print(PrintStream stream) {
        stream.println("depth: " + depth); // NOI18N
        stream.println("#pages: " + numOfPages); // NOI18N
        stream.println("#entries: " + numOfEntries); // NOI18N
        stream.println("#leaf entries: " + numOfLeafEntries); // NOI18N
        stream.println("used bytes: " + usedBytes); // NOI18N
        stream.println("total bytes: " + totalBytes); // NOI18N
        stream.println("load factor: " + getLoadFactor()); // NOI18N
    }
    
    public static TreeMetrics computeParent(TreeMetrics[] m, long tBytes, long uBytes, long entries) {
        int d = 0;
        int pages = 0;
        long bytes = 0;
        long used = 0;
        long entrs = 0;
        long leafEntries = 0;
        
        for (int x = 0; x < m.length; x++) {
            if (m[x].getDepth() > d) {
                d = m[x].getDepth();
            }
            pages += m[x].getNumberOfPages();
            bytes += m[x].getTotalBytes();
            used += m[x].getUsedBytes();
            entrs += m[x].getNumOfEntries();
            leafEntries += m[x].getNumOfLeafEntries();
        }
        return new TreeMetrics(d + 1, pages + 1, bytes + tBytes, used + uBytes, entrs + entries, leafEntries);
    }

    public long getNumOfEntries() {
        return numOfEntries;
    }

    public void setNumOfEntries(long numOfEntries) {
        this.numOfEntries = numOfEntries;
    }

    public long getNumOfLeafEntries() {
        return numOfLeafEntries;
    }

    public void setNumOfLeafEntries(long numOfLeafEntries) {
        this.numOfLeafEntries = numOfLeafEntries;
    }
    
}   
