/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.persistence.memoryimpl;

import org.netbeans.mdr.persistence.MOFID;
import org.netbeans.mdr.persistence.Storage;
import org.netbeans.mdr.util.IOUtils;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

/**
 *
 * @author  mmatula
 * @version
 */
public class Utils extends Object {
    public static void write(OutputStream outputStream, Object object, Storage storage) throws IOException {
        if (object instanceof Map) {
            outputStream.write(IOUtils.T_MAP);
            Map temp = (Map) object;
            IOUtils.writeInt(outputStream, temp.size());
            Map.Entry key;
            for (Iterator it = temp.entrySet().iterator(); it.hasNext();) {
                key = (Map.Entry) it.next();
                write(outputStream, key.getKey(), storage);
                write(outputStream, key.getValue(), storage);
            }
        } else if (object instanceof Collection) {
            outputStream.write(IOUtils.T_COLLECTION);
            Collection col = (Collection) object;
            IOUtils.writeInt(outputStream, col.size());
            for (Iterator it = col.iterator(); it.hasNext();) {
                write(outputStream, it.next(), storage);
            }
        } else if (object instanceof MOFID) {
            outputStream.write(IOUtils.T_MOFID);
            IOUtils.writeMOFID(outputStream, (MOFID)object, storage);
        } else {
            IOUtils.write(outputStream, object);
        }
    }
    
    public static Object read(InputStream inputStream, Storage storage) throws IOException {
        int type = inputStream.read();
        switch (type) {
            case IOUtils.T_MAP: {
                int size = IOUtils.readInt(inputStream);
                Map result = new HashMap(size);
                for (int i = 0; i < size; i++) {
                    result.put(read(inputStream, storage), read(inputStream, storage));
                }
                return result;
            } 
            case IOUtils.T_COLLECTION: {
                int size = IOUtils.readInt(inputStream);
                java.util.List result = new java.util.ArrayList(size);
                for (int i = 0; i < size; i++) {
                    result.add(read(inputStream, storage));
                }
                return result;
            }
            case IOUtils.T_MOFID: {
                return IOUtils.readMOFID(inputStream, storage);
            } 
            default: {
                return IOUtils.read(inputStream, null, null, type);
            }
        }
    }
}
