/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.mdr.util;

import java.util.*;

/**
 *
 * @author Martin Matula
 * @version
 */
public class ComposedLazyList extends AbstractList {
    private final List lists = new ArrayList();
    private int elementCount = 0;

    /** Creates new ComposedLazyList */
    public ComposedLazyList() {
        super();
    }

    public ComposedLazyList(Collection col) {
        this();
        addAll(col);
    }

    public boolean addAll(Collection col) {
        List result = (col instanceof List) ? (List) col : new ArrayList(col);
        
        this.modCount++;
        
        elementCount += result.size();
        return lists.add(result);
    }
    
    public int size() {
        return elementCount;
    }
    
    public Object get(int index) {
        int listUBound = 0;
        int listLBound = 0;
        List currentList = null;
        
        for (Iterator it = lists.iterator(); it.hasNext() && listUBound <= index; listUBound += currentList.size()) {
            currentList = (List) it.next();
            listLBound = listUBound;
        }

        return currentList.get(index - listLBound);
    }
    
    public Iterator iterator() {
        return new LazyIterator();
    }
    
    private class LazyIterator implements Iterator {
        Iterator listIterator;
        Iterator currentIterator;
        
        private LazyIterator() {
            listIterator = ComposedLazyList.this.lists.iterator();
            currentIterator = null;
        }
            
        public void remove() {
            throw new UnsupportedOperationException();
        }
        
        public Object next() {
            hasNext();
            return currentIterator.next();
        }
        
        public boolean hasNext() {
            if (currentIterator == null) {
                if (listIterator.hasNext()) {
                    currentIterator = ((List) listIterator.next()).iterator();
                } else {
                    return false;
                }
            }
            
            while (!currentIterator.hasNext()) {
                if (listIterator.hasNext()) {
                    currentIterator = ((List) listIterator.next()).iterator();
                } else {
                    return false;
                }
            }
            return true;
        }
    }
}
