/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 */


package org.foo.wavsupport;

import java.awt.event.ActionEvent;
import java.io.IOException;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import javax.swing.AbstractAction;
import org.foo.wavsupport.WavDataObject;
import org.foo.wavsupport.api.WavViewDescriptor;
import org.foo.wavsupport.ui.WavPanelMultiViewDescriptor;
import org.netbeans.core.spi.multiview.CloseOperationHandler;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewFactory;
import org.openide.DialogDisplayer;
import org.openide.ErrorManager;
import org.openide.NotifyDescriptor;
import org.openide.cookies.CloseCookie;
import org.openide.cookies.OpenCookie;
import org.openide.loaders.MultiDataObject;
import org.openide.loaders.OpenSupport;
import org.openide.util.Lookup;
import org.openide.windows.CloneableTopComponent;

/**
 *
 * @author runger
 */
public class WavOpenSupport extends OpenSupport implements OpenCookie, CloseCookie
{
    public WavOpenSupport(MultiDataObject.Entry entry)
    {
        super(entry);
    }
    
    
    protected CloneableTopComponent createCloneableTopComponent()
    {
        // get all instances of views for the multiview window
        
        WavViewDescriptor main = new WavPanelMultiViewDescriptor();
        
        List<WavViewDescriptor> all = new ArrayList<WavViewDescriptor>();
        all.add(main);
        
        Lookup.Template template = new Lookup.Template(WavViewDescriptor.class);
        Lookup.Result result = Lookup.getDefault().lookup(template);
        for (Object wvd : result.allInstances())
        {
            all.add((WavViewDescriptor)wvd);
        }
        
        // initialize all the views with data
        
        WavDataObject dobj = (WavDataObject)entry.getDataObject();
        
        for (WavViewDescriptor wvd : all)
        {
            wvd.setWavDataObject(dobj);
        }
        
        // create the multiview
        
        WavViewDescriptor[] allArray = new WavViewDescriptor[all.size()];
        all.toArray(allArray);
        
        CloneableTopComponent tc = MultiViewFactory.createCloneableMultiView(
                allArray, main, new CloseHandler());
        
        tc.setDisplayName(dobj.getName());
        
        return tc;
    }
    
    private static class CloseHandler implements CloseOperationHandler, Serializable
    {
        private static final long serialVersionUID = 47L;
        public boolean resolveCloseOperation(CloseOperationState[] elements)
        {
            NotifyDescriptor nd = new NotifyDescriptor.Confirmation(
                    "Save before closing?");
            DialogDisplayer.getDefault().notify(nd);
            
            if (nd.getValue().equals(NotifyDescriptor.YES_OPTION))
            {
                for (CloseOperationState element : elements)
                {
                    element.getProceedAction().actionPerformed(
                            new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "xxx"));
                }
                return true;
            }
            else if (nd.getValue().equals(NotifyDescriptor.NO_OPTION))
            {
                for (CloseOperationState element : elements)
                {
                    element.getDiscardAction().actionPerformed(
                            new ActionEvent(this, ActionEvent.ACTION_PERFORMED, "xxx"));
                }
                return true;
            }
            else
            {
                // Cancel
                return false;
            }
        }
    }
}
