/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 */

package org.foo.wavsupport.ui;

import java.beans.PropertyVetoException;
import java.io.IOException;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.netbeans.core.spi.multiview.MultiViewFactory;
import org.openide.ErrorManager;
import org.openide.awt.UndoRedo;
import org.openide.cookies.SaveCookie;
import org.openide.loaders.DataObject;
import org.openide.nodes.Node;
import org.openide.util.HelpCtx;
import org.openide.util.Lookup;
import org.openide.util.actions.NodeAction;

/**
 *
 * @author runger
 */
public class WavComponent implements MultiViewElement {
    
    private static final CloseOperationState CLOSE_OPERATION_STATE = createCloseOperationState();
    private transient WavPanel wavPanel;
    
    public WavComponent(DataObject dobj)
    {
        super();
        wavPanel = new WavPanel(dobj);
    }

    public Action[] getActions() {
        return new Action[0];
    }

    public Lookup getLookup() {
        return wavPanel.getWavDataObject().getNodeDelegate().getLookup();
    }

    public UndoRedo getUndoRedo() {
        return new UndoRedo.Manager();
    }

    public JComponent getVisualRepresentation() {
        return wavPanel;
    }

    public JComponent getToolbarRepresentation() {
        // don't need any widgets on the toolbar
        return new JPanel();
    }

    public CloseOperationState canCloseElement() {
        if (wavPanel.getWavDataObject().isModified())
        {
            return CLOSE_OPERATION_STATE;
        }
        else
        {
            return CloseOperationState.STATE_OK;
        }
    }

    public void setMultiViewCallback(MultiViewElementCallback multiViewElementCallback) {
        // don't need this
    }
    
    // similar semantics as the equivalent methods in TopComponent
    public void componentDeactivated() {}
    public void componentActivated() {}
    public void componentHidden() {}
    public void componentShowing() {}
    public void componentClosed() {}
    public void componentOpened() {}

    public Object writeReplace() {
        return null;
    }
    
    private static CloseOperationState createCloseOperationState()
    {
        return MultiViewFactory.createUnsafeCloseState(
                "File Not Saved", new ProceedAction(), new DiscardAction());
    }

    private static class ProceedAction extends NodeAction
    {
        protected void performAction(Node[] node) {
            try
            {
                if (node != null && node.length > 0)
                {
                    SaveCookie sc = (SaveCookie)node[0].getCookie(SaveCookie.class);
                    sc.save();
                }
            }
            catch(IOException ex)
            {
                ErrorManager.getDefault().notify(ex);
            }
        }

        protected boolean enable(Node[] node) {
            return true;
        }

        public String getName() {
            return "Save";
        }

        public HelpCtx getHelpCtx() {
            return null;
        }
    }
    
    private static class DiscardAction extends NodeAction
    {
        protected void performAction(Node[] node) {
            if (node != null && node.length > 0)
            {
                DataObject dobj = (DataObject)node[0].getCookie(DataObject.class);
                try
                {
                    // throw away what's in memory.  The DataObject will
                    // be recreated from disk.
                    dobj.setValid(false);
                }
                catch (PropertyVetoException ex)
                {
                    ErrorManager.getDefault().notify(ex);
                }
            }
        }

        protected boolean enable(Node[] node) {
            return true;
        }

        public String getName() {
            return "Discard";
        }

        public HelpCtx getHelpCtx() {
            return null;
        }
    }
}
