/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.cardsui;

import java.util.Arrays;
import java.util.HashSet;
import java.util.Set;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.netbeans.examples.cards.api.Card;
import org.netbeans.examples.cards.api.Pile;
import org.openide.nodes.AbstractNode;
import org.openide.nodes.Children;
import org.openide.nodes.Node;
import org.openide.util.WeakListeners;
import org.openide.util.lookup.AbstractLookup;
import org.openide.util.lookup.InstanceContent;

/**
 *
 * @author Tim Boudreau
 */
public class PileNode extends AbstractNode implements ChangeListener {
    private final InstanceContent content;    
    public PileNode (Pile pile) {
        this (pile, new InstanceContent());
    }
    
    private PileNode(Pile pile, InstanceContent content) {
        super (new PileChildren (pile), new AbstractLookup(content));
        content.set (Arrays.asList (pile, pile.peekTopCard(), this), null);
        this.content = content;
        pile.addChangeListener(WeakListeners.change (this, pile));
    }
    
    public void stateChanged(ChangeEvent e) {
        Set <Object> newContent = new HashSet <Object> ();
        newContent.add (this);
        newContent.add (getPile());
        Card card = getPile().peekTopCard();
        if (card != null) {
            newContent.add (card);
        }
        content.set (newContent, null);
    }
    
    private Pile getPile() {
        return (Pile) getLookup().lookup (Pile.class);
    }

    private static class PileChildren extends Children.Keys<Card> implements ChangeListener {
        private Pile pile;
        public PileChildren (Pile pile) {
            this.pile = pile;
            pile.addChangeListener (WeakListeners.change (this, pile));
        }
        
        protected void addNotify() {
            setKeys (pile.getCards());
        }
        
        public Node[] createNodes(Card card) {
            return new Node[] { new CardNode (card) };
            //Try catch needed if superclass of CardNode is set to
            //BeanNode
//            try {
//                return new Node[] { new CardNode (card) };
//            } catch (Exception e) {
//                return new Node[0];
//            }
        }
        
        public void stateChanged (ChangeEvent e) {
            setKeys (pile.getCards());
        }
    }
}
