/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.manifestsupport;

import java.awt.Image;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import org.netbeans.examples.manifestsupport.api.ManifestProvider;
import org.openide.filesystems.FileChangeAdapter;
import org.openide.filesystems.FileChangeListener;
import org.openide.filesystems.FileEvent;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileUtil;
import org.openide.loaders.DataNode;
import org.openide.loaders.DataObject;
import org.openide.nodes.Children;
import org.openide.util.RequestProcessor;
import org.openide.util.Utilities;
import org.openide.util.WeakListeners;

public class ManifestDataNode extends DataNode implements Runnable, PropertyChangeListener {

    private static final String IMAGE_ICON_BASE = "org/netbeans/examples/" +
            "manifestsupport/mficon.png";
    private static final String ERROR_BADGE_PATH = "org/netbeans/examples/" +
            "manifestsupport/errorbadge.png";
    private Boolean isValid = null;
    private RequestProcessor.Task validityCheckTask = null;
    private FileChangeListener fileListener = new IconUpdateChecker();

    public ManifestDataNode(ManifestDataObject obj) {
        super(obj, Children.LEAF);
        setIconBaseWithExtension(IMAGE_ICON_BASE);

        FileObject file = obj.getPrimaryFile();
        file.addFileChangeListener(FileUtil.weakFileChangeListener(fileListener,
                file));
        obj.addPropertyChangeListener(WeakListeners.propertyChange(this, obj));
    }

    public Image getIcon(int type) {
        Image original = super.getIcon(type);
        if (isValidManifest()) {
            return original;
        } else {
            Image errorBadge = Utilities.loadImage(ERROR_BADGE_PATH);
            return Utilities.mergeImages(original, errorBadge, 5, 5);
        }
    }

    private synchronized boolean isValidManifest() {
        if (isValid == null && validityCheckTask == null) {
            postValidityCheckInBackground();
        }
        return isValid == null ? true : isValid.booleanValue();
    }

    private synchronized void postValidityCheckInBackground() {
        if (validityCheckTask == null) {
            validityCheckTask = RequestProcessor.getDefault().post(this, 500);
        }
    }

    public void run() {
        try {
            checkValidManifest();
        } finally {
            synchronized (this) {
                validityCheckTask = null;
            }
        }
    }

    private void checkValidManifest() {
        ManifestProvider provider = (ManifestProvider) getLookup().lookup(ManifestProvider.class);
        boolean iconChanged;
        synchronized (this) {
            boolean wasValid = isValidManifest();
            isValid = provider.getManifest() != null ? Boolean.TRUE : Boolean.FALSE;

            iconChanged = isValid.booleanValue() != wasValid;
        }
        //Fire the icon change outside the synchronized block
        if (iconChanged) {
            fireIconChange();
        }
    }

    private class IconUpdateChecker extends FileChangeAdapter {

        public void fileChanged(FileEvent fe) {
            postValidityCheckInBackground();
        }
    }

    public void propertyChange(PropertyChangeEvent evt) {
        if (DataObject.PROP_PRIMARY_FILE.equals(evt.getPropertyName())) {
            FileObject nue = (FileObject) evt.getNewValue();
            if (nue != null) {
                nue.addFileChangeListener(
                        FileUtil.weakFileChangeListener(fileListener, nue));
            }
        }
    }
}
