/*
 * Customer.java
 *
 * Created on September 21, 2006, 2:18 PM
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package org.netbeans.examples.masterdetail.api;

import java.beans.PropertyVetoException;
import java.util.Arrays;
import org.openide.util.NbBundle;

/**
 *
 * @author Tim Boudreau
 */
public final class Customer {
    public static final String PROP_CITY = "city";
    public static final String PROP_STATE = "state";
    public static final String PROP_FIRST_NAME = "firstName";
    public static final String PROP_LAST_NAME = "lastName";
    public static final String PROP_ADDRESS = "address";
    public static final String PROP_POSTAL_CODE = "postalCode";
    
    public static final String[] PROPS = new String[] {
        Customer.PROP_CITY,
        Customer.PROP_STATE,
        Customer.PROP_FIRST_NAME,
        Customer.PROP_LAST_NAME,
        Customer.PROP_ADDRESS,
        Customer.PROP_POSTAL_CODE,
    };
    
    
    /** Creates a new instance of Customer */
    public Customer() {
        this ("", "", "", "", "", "");
    }
    
    public Customer(String firstName, String lastName, String address, 
            String city, String state, String postalCode) {
        this.firstName = firstName;
        this.lastName = lastName;
        this.address = address;
        this.city = city;
        this.state = state;
        this.postalCode = postalCode;
        addVetoableChangeListener(new CustomerValidator());
    }
    
    /**
     *Semi useless method which gives our context action something to do.
     */
    public void clear() {
        String oldPostalCode = postalCode;
        String oldFirstName = firstName;
        String oldLastName = lastName;
        String oldAddress = address;
        String oldState = state;
        this.firstName = "";
        this.lastName = "";
        this.address = "";
        this.city = "";
        this.state = "";
        this.postalCode = "";
        propertyChangeSupport.firePropertyChange (PROP_ADDRESS, oldAddress, "");
        propertyChangeSupport.firePropertyChange (PROP_POSTAL_CODE, oldPostalCode, 
                "");
        propertyChangeSupport.firePropertyChange (PROP_LAST_NAME, oldLastName, 
                "");
        propertyChangeSupport.firePropertyChange (PROP_STATE, oldState, "");
        propertyChangeSupport.firePropertyChange (PROP_FIRST_NAME, oldFirstName, 
                "");
    }
    
    public static String getDisplayName (String prop) {
        if (PROP_FIRST_NAME.equals(prop)) {
            prop = "FIRST_NAME";
        }
        if (PROP_LAST_NAME.equals(prop)) {
            prop = "LAST_NAME";
        }
        if (PROP_POSTAL_CODE.equals(prop)) {
            prop = "POSTAL_CODE";
        }
        return NbBundle.getMessage (Customer.class, prop.toUpperCase());
    }
    
    public String toString() {
        return NbBundle.getMessage (Customer.class,
                "FMT_CustomerName", new Object[] { firstName, lastName });
    }

    /**
     * Holds value of property firstName.
     */
    private String firstName;

    /**
     * Utility field used by bound properties.
     */
    private java.beans.PropertyChangeSupport propertyChangeSupport =  new java.beans.PropertyChangeSupport(this);

    /**
     * Utility field used by constrained properties.
     */
    private java.beans.VetoableChangeSupport vetoableChangeSupport =  new java.beans.VetoableChangeSupport(this);

    /**
     * Adds a PropertyChangeListener to the listener list.
     * @param l The listener to add.
     */
    public void addPropertyChangeListener(java.beans.PropertyChangeListener l) {
        propertyChangeSupport.addPropertyChangeListener(l);
    }

    /**
     * Removes a PropertyChangeListener from the listener list.
     * @param l The listener to remove.
     */
    public void removePropertyChangeListener(java.beans.PropertyChangeListener l) {
        propertyChangeSupport.removePropertyChangeListener(l);
    }

    /**
     * Adds a VetoableChangeListener to the listener list.
     * @param l The listener to add.
     */
    public void addVetoableChangeListener(java.beans.VetoableChangeListener l) {
        vetoableChangeSupport.addVetoableChangeListener (l);
    }

    /**
     * Removes a VetoableChangeListener from the listener list.
     * @param l The listener to remove.
     */
    public void removeVetoableChangeListener(java.beans.VetoableChangeListener l) {
        vetoableChangeSupport.removeVetoableChangeListener (l);
    }

    /**
     * Getter for property name.
     * @return Value of property name.
     */
    public String getFirstName() {
        return this.firstName;
    }

    /**
     * Setter for property name.
     * @param name New value of property name.
     * 
     * @throws PropertyVetoException if some vetoable listeners reject the new value
     */
    public void setFirstName(String name) throws java.beans.PropertyVetoException {
        String oldName = this.firstName;
        vetoableChangeSupport.fireVetoableChange(PROP_FIRST_NAME, oldName, name);
        this.firstName = name;
        propertyChangeSupport.firePropertyChange (PROP_FIRST_NAME, oldName, name);
    }

    /**
     * Holds value of property city.
     */
    private String city;

    /**
     * Getter for property city.
     * @return Value of property city.
     */
    public String getCity() {
        return this.city;
    }

    /**
     * Setter for property city.
     * @param city New value of property city.
     * 
     * @throws PropertyVetoException if some vetoable listeners reject the new value
     */
    public void setCity(String city) throws java.beans.PropertyVetoException {
        String oldCity = this.city;
        vetoableChangeSupport.fireVetoableChange(PROP_CITY, oldCity, city);
        this.city = city;
        propertyChangeSupport.firePropertyChange (PROP_CITY, oldCity, city);
    }

    /**
     * Holds value of property state.
     */
    private String state;

    /**
     * Getter for property state.
     * @return Value of property state.
     */
    public String getState() {
        return this.state;
    }

    /**
     * Setter for property state.
     * @param state New value of property state.
     * 
     * @throws PropertyVetoException if some vetoable listeners reject the new value
     */
    public void setState(String state) throws java.beans.PropertyVetoException {
        String oldState = this.state;
        vetoableChangeSupport.fireVetoableChange(PROP_STATE, oldState, state);
        this.state = state;
        propertyChangeSupport.firePropertyChange (PROP_STATE, oldState, state);
    }

    /**
     * Holds value of property lastName.
     */
    private String lastName;

    /**
     * Getter for property lastname.
     * @return Value of property lastname.
     */
    public String getLastName() {
        return this.lastName;
    }
    

    /**
     * Setter for property lastname.
     * @param lastname New value of property lastname.
     * 
     * @throws PropertyVetoException if some vetoable listeners reject the new value
     */
    public void setLastName(String lastName) throws java.beans.PropertyVetoException {
        String oldLastname = this.lastName;
        vetoableChangeSupport.fireVetoableChange(PROP_LAST_NAME, oldLastname, lastName);
        this.lastName = lastName;
        propertyChangeSupport.firePropertyChange (PROP_LAST_NAME, oldLastname, lastName);
    }

    /**
     * Holds value of property postalCode.
     */
    private String postalCode;

    /**
     * Getter for property postalCode.
     * @return Value of property postalCode.
     */
    public String getPostalCode() {
        return this.postalCode;
    }

    /**
     * Setter for property postalCode.
     * @param postalCode New value of property postalCode.
     * 
     * @throws PropertyVetoException if some vetoable listeners reject the new value
     */
    public void setPostalCode(String postalCode) throws java.beans.PropertyVetoException {
        String oldPostalCode = this.postalCode;
        vetoableChangeSupport.fireVetoableChange(PROP_POSTAL_CODE, oldPostalCode, postalCode);
        this.postalCode = postalCode;
        propertyChangeSupport.firePropertyChange (PROP_POSTAL_CODE, oldPostalCode, postalCode);
    }

    /**
     * Holds value of property address.
     */
    private String address;

    /**
     * Getter for property address.
     * @return Value of property address.
     */
    public String getAddress() {
        return this.address;
    }

    /**
     * Setter for property address.
     * @param address New value of property address.
     * 
     * @throws PropertyVetoException if some vetoable listeners reject the new value
     */
    public void setAddress(String address) throws java.beans.PropertyVetoException {
        String oldAddress = this.address;
        vetoableChangeSupport.fireVetoableChange(PROP_ADDRESS, oldAddress, address);
        this.address = address;
        propertyChangeSupport.firePropertyChange (PROP_FIRST_NAME, oldAddress, address);
    }

    /**
     * Returns null if this record is valid, a description of the problem if not.
     */
    public String getProblem() {
        try {
            vetoableChangeSupport.fireVetoableChange(PROP_FIRST_NAME, firstName, firstName);
            vetoableChangeSupport.fireVetoableChange(PROP_LAST_NAME, lastName, lastName);
            vetoableChangeSupport.fireVetoableChange(PROP_ADDRESS, address, address);
            vetoableChangeSupport.fireVetoableChange(PROP_CITY, city, city);
            vetoableChangeSupport.fireVetoableChange(PROP_STATE, state, state);
            vetoableChangeSupport.fireVetoableChange(PROP_POSTAL_CODE, postalCode, postalCode);
        } catch (PropertyVetoException veto) {
            return veto.getLocalizedMessage();
        }
        return null;
    }

    public void setProperty (String name, String value) throws PropertyVetoException {
        int ix = Arrays.asList(PROPS).indexOf (name);
        switch (ix) {
            case 0 :
                setCity ( value );
                break;
            case 1 :
                setState ( value );
                break;
            case 2 :
                setFirstName ( value );
                break;
            case 3 :
                setLastName ( value );
                break;
            case 4 :
                setAddress ( value );
                break;
            case 5 :
                setPostalCode ( value );
                break;
            default : 
                throw new IllegalArgumentException ( name );
        }
    }
    
}
