/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.nbtidyintegration;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.util.ArrayList;
import java.util.List;
import java.util.regex.Matcher;
import org.openide.cookies.LineCookie;
import org.openide.text.Annotatable;
import org.openide.text.Annotation;
import org.openide.text.Line;

/**
 * Provide Annotations.
 *
 * @author trabber@ppart.de
 */
public class TidyAnnotation extends Annotation {

    private static List<Annotation> annotations = new ArrayList<Annotation>();
    private static String[] annoType = {
        "org-netbeans-examples-nbtidyintegration-tidyerrorannotation",
        "org-netbeans-examples-nbtidyintegration-tidywarningannotation"
    };
    private String reason;
    private int column;
    private int severity = 0;

    public static TidyAnnotation create(String severity, int column, String reason) {
        TidyAnnotation annotation = new TidyAnnotation(severity, column, reason);
        annotations.add(annotation);
        return annotation;
    }

    public static void clear() {
        for (Annotation annotation : annotations) {
            annotation.detach();
        }
    }

    public static void remove(TidyAnnotation annotation) {
        annotations.remove(annotation);
    }

    /**
     * Creates a new instance of TidyErrorAnnotation
     */
    private TidyAnnotation(String severity, int column, String reason) {
        this.severity = severity.contains("Err") ? 0 : 1;
        this.reason = reason;
        this.column = column;
    }

    /**
     * Define the Tidy Annotation type
     *
     * @return Constant String "TidyErrorAnnotation"
     */
    public String getAnnotationType() {
        return annoType[severity];
    }

    /** Provide the Tidy error message as a description.
     * @return Annotation Reason*/
    public String getShortDescription() {
        return reason + " (" + "Column: " + column + ")";
    }

    /** Create an annotation for a line from match string*/
    public static void createAnnotation(final LineCookie lc, final Matcher matcher)
            throws IndexOutOfBoundsException, NumberFormatException {
        String lineNumberString = matcher.group(1);
        int lineNumber = Integer.parseInt(lineNumberString) - 1;
        String columnNumberString = matcher.group(2);
        int columnNumber = Integer.parseInt(columnNumberString) - 1;
        String severity = matcher.group(3);
        String reason = matcher.group(4);
        try {
            final Line line = lc.getLineSet().getOriginal(lineNumber);
            final TidyAnnotation annotation = TidyAnnotation.create(severity, columnNumber, reason);

            annotation.attach(line);
            line.addPropertyChangeListener(new PropertyChangeListener() {

                public void propertyChange(PropertyChangeEvent ev) {
                    String type = ev.getPropertyName();
                    if (type == null || type.equals(Annotatable.PROP_TEXT)) {
                        // User edited the line, assume error should be cleared.
                        line.removePropertyChangeListener(this);
                        annotation.detach();
                        TidyAnnotation.remove(annotation);
                    }
                }
            });
        } catch (IndexOutOfBoundsException e) {
            // might happen, if state of file is not saved. ignore
        }

    }
}
