/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.nbtidyintegration;

import java.util.prefs.Preferences;

/**
 *
 * @author jentra
 */
public class TidySettings {

    public static String KEY_CONVERSION_UPPER = "conversion.upper";
    public static String KEY_CONVERSION_BARE = "conversion.bare";
    public static String KEY_CONVERSION_CLEAN = "conversion.clean";
    public static String KEY_CONVERSION_NUMERIC = "conversion.numeric";
    public static String KEY_CONVERSION_OMIT = "conversion.omit";
    public static String KEY_CONVERSION_WRAP = "conversion.wrap";
    public static String KEY_CONVERSION_WRAP_COLUMN = "conversion.wrapColumn";
    public static String KEY_CONVERSION_XML = "conversion.xml";
    public static String KEY_CONVERSION_ENCODING = "conversion.encoding";
    public static String KEY_CONVERSION_FORMAT_OUTPUT = "conversion.formatOutput";
    public static String KEY_CONVERSION_INDENT = "conversion.indent";
    public static String KEY_CONVERSION_ACCESS_CHECK_LEVEL = "conversion.accessCheckLevel";
    public static String CONVERSION_FORMAT_OUTPUT_ASIS = "as is";
    public static String CONVERSION_FORMAT_OUTPUT_XHTML = "xhtml";
    public static String CONVERSION_FORMAT_OUTPUT_HTML = "html";
    public static String DEFAULT_CONVERSION_ENCODING = "ascii";
    public static boolean DEFAULT_CONVERSION_INDENT = Boolean.TRUE;
    public static boolean DEFAULT_CONVERSION_CLEAN = Boolean.TRUE;
    public static boolean DEFAULT_CONVERSION_BARE = Boolean.FALSE;
    public static boolean DEFAULT_CONVERSION_UPPER = Boolean.FALSE;
    public static boolean DEFAULT_CONVERSION_NUMERIC = Boolean.FALSE;
    public static boolean DEFAULT_CONVERSION_OMIT = Boolean.FALSE;
    public static boolean DEFAULT_CONVERSION_XML = Boolean.FALSE;
    public static String DEFAULT_CONVERSION_FORMAT_OUTPUT = CONVERSION_FORMAT_OUTPUT_ASIS;
    public static boolean DEFAULT_CONVERSION_WRAP = Boolean.TRUE;
    public static int DEFAULT_CONVERSION_WRAP_COLUMN = 80;
    public static int DEFAULT_CONVERSION_ACCESS_CHECK_LEVEL = 0;
    public static String[] ENCODINGS = {
        "raw", //output values above 127 without conversion to entities
        "ascii", //use ISO-8859-1 for input, US-ASCII for output
        "latin0", //use ISO-8859-15 for input, US-ASCII for output
        "latin1", //use ISO-8859-1 for both input and output
        "iso2022", //use ISO-2022 for both input and output
        "utf8", //use UTF-8 for both input and output
        "mac", //use MacRoman for input, US-ASCII for output
        "win1252", //use Windows-1252 for input, US-ASCII for output
        "ibm858", //use IBM-858 (CP850+Euro) for input, US-ASCII for output
        "utf16le", //use UTF-16LE for both input and output
        "utf16be", //use UTF-16BE for both input and output
        "utf16", //use UTF-16 for both input and output
        "big5", //use Big5 for both input and output
        "shiftjis" //use Shift_JIS for both input and output
    };
    public static String[] FORMAT_OUTPUT_OPTIONS = {
        CONVERSION_FORMAT_OUTPUT_ASIS,
        CONVERSION_FORMAT_OUTPUT_HTML,
        CONVERSION_FORMAT_OUTPUT_XHTML
    };

    /** Don't create an instance */
    private TidySettings() {
    }

    public static String generateCommandLineArguments() {
        Preferences settings = Preferences.userNodeForPackage(TidySettings.class);
        StringBuilder sb = new StringBuilder();

        sb.append(" -").append(settings.get(KEY_CONVERSION_ENCODING, DEFAULT_CONVERSION_ENCODING)).append(" ");

        if (settings.getBoolean(KEY_CONVERSION_UPPER, DEFAULT_CONVERSION_UPPER)) {
            sb.append("-upper ");
        }
        if (settings.getBoolean(KEY_CONVERSION_CLEAN, DEFAULT_CONVERSION_CLEAN)) {
            sb.append("-clean ");
        }
        if (settings.getBoolean(KEY_CONVERSION_NUMERIC, DEFAULT_CONVERSION_NUMERIC)) {
            sb.append("-numeric ");
        }
        if (settings.getBoolean(KEY_CONVERSION_BARE, DEFAULT_CONVERSION_BARE)) {
            sb.append("-bare ");
        }
        if (settings.getBoolean(KEY_CONVERSION_OMIT, DEFAULT_CONVERSION_OMIT)) {
            sb.append("-omit ");
        }
        if (settings.getBoolean(KEY_CONVERSION_WRAP, DEFAULT_CONVERSION_WRAP)) {
            sb.append("-wrap ").append(settings.getInt(KEY_CONVERSION_WRAP_COLUMN, DEFAULT_CONVERSION_WRAP_COLUMN)).append(" ");
        }
        if (settings.getBoolean(KEY_CONVERSION_INDENT, DEFAULT_CONVERSION_INDENT)) {
            sb.append("-indent ");
        }
        if (settings.getBoolean(KEY_CONVERSION_XML, DEFAULT_CONVERSION_XML)) {
            sb.append("-xml ");
        }
        String format_output = settings.get(KEY_CONVERSION_FORMAT_OUTPUT, DEFAULT_CONVERSION_FORMAT_OUTPUT);
        if (!format_output.equals(CONVERSION_FORMAT_OUTPUT_ASIS)) {
            sb.append("-as").append(format_output).append(" ");
        }

        sb.append("-access ").append(settings.getInt(KEY_CONVERSION_ACCESS_CHECK_LEVEL,
                DEFAULT_CONVERSION_ACCESS_CHECK_LEVEL));

        return sb.toString();
    }

    public static void setDefaultConversionValues() {
        Preferences settings = Preferences.userNodeForPackage(TidyRunnable.class);
        settings.put(KEY_CONVERSION_ENCODING, DEFAULT_CONVERSION_ENCODING);
        settings.putBoolean(KEY_CONVERSION_INDENT, DEFAULT_CONVERSION_INDENT);
        settings.putBoolean(KEY_CONVERSION_CLEAN, DEFAULT_CONVERSION_CLEAN);
        settings.putBoolean(KEY_CONVERSION_BARE, DEFAULT_CONVERSION_BARE);
        settings.putBoolean(KEY_CONVERSION_UPPER, DEFAULT_CONVERSION_UPPER);
        settings.putBoolean(KEY_CONVERSION_NUMERIC, DEFAULT_CONVERSION_NUMERIC);
        settings.putBoolean(KEY_CONVERSION_OMIT, DEFAULT_CONVERSION_OMIT);
        settings.putBoolean(KEY_CONVERSION_XML, DEFAULT_CONVERSION_XML);
        settings.put(KEY_CONVERSION_FORMAT_OUTPUT, DEFAULT_CONVERSION_FORMAT_OUTPUT);
        settings.putBoolean(KEY_CONVERSION_WRAP, DEFAULT_CONVERSION_WRAP);
        settings.putInt(KEY_CONVERSION_WRAP_COLUMN, DEFAULT_CONVERSION_WRAP_COLUMN);
        settings.putInt(KEY_CONVERSION_ACCESS_CHECK_LEVEL, DEFAULT_CONVERSION_ACCESS_CHECK_LEVEL);
    }

    public static Object[] getEncodings() {
        return ENCODINGS;
    }

    public static Object[] getFormatOutputOptions() {
        return FORMAT_OUTPUT_OPTIONS;
    }
}
