/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.examples.xmlpseudonavigator;

import java.io.IOException;
import javax.swing.JEditorPane;
import javax.swing.SwingUtilities;
import org.openide.cookies.EditCookie;
import org.openide.cookies.EditorCookie;
import org.openide.cookies.OpenCookie;
import org.openide.loaders.DataObject;
import org.openide.windows.TopComponent;

/*
 * Objects in an XmlFileListModel
 */
final class XmlListModelItem {

    private final XmlFileListModel model;
    private final int position;
    private final String text;

    public XmlListModelItem(XmlFileListModel model, String text, int position) {
        this.model = model;
        this.text = text;
        this.position = position;
    }

    public String toString() {
        return text;
    }

    public int getPosition() {
        return position;
    }

    public void selected() throws IOException {
        if (this.model.file.isValid()) {
            DataObject ob = DataObject.find(model.getFile());
            //Make sure the file is open
            if (!openFileInEditor(ob)) {
                return;
            }
            // in NB 6 no cast needed from Lookup
            //EditorCookie ed = (EditorCookie) ob.getNodeDelegate().getLookup().lookup(
            //        EditorCookie.class);
            EditorCookie ed = (EditorCookie) ob.getCookie(
                    EditorCookie.class);
            if (ed != null) {
                ed.openDocument();
                JEditorPane pane = ed.getOpenedPanes()[0];
                pane.setSelectionStart(position);
                pane.setSelectionEnd(position);
                TopComponent tc = (TopComponent) SwingUtilities.getAncestorOfClass(TopComponent.class,
                        pane);
                if (tc != null) { //it won't be
                    tc.requestActive();
                }
            }
        }
    }

    private boolean openFileInEditor(DataObject ob) {
        EditCookie ck = (EditCookie) ob.getCookie(EditCookie.class);
        if (ck != null) {
            ck.edit();
            return true;
        }
        OpenCookie oc = (OpenCookie) ob.getCookie(OpenCookie.class);
        if (oc != null) {
            oc.open();
            return true;
        }
        return false;
    }
}