/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2008 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.web.wicket.framework;

import java.util.HashMap;
import java.util.Map;
import org.netbeans.api.java.project.JavaProjectConstants;
import org.netbeans.api.project.FileOwnerQuery;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectUtils;
import org.netbeans.api.project.SourceGroup;
import org.netbeans.api.project.libraries.Library;
import org.netbeans.api.project.libraries.LibraryManager;

import org.netbeans.modules.j2ee.dd.api.common.InitParam;
import org.netbeans.modules.j2ee.dd.api.web.DDProvider;
import org.netbeans.modules.j2ee.dd.api.web.Servlet;
import org.netbeans.modules.j2ee.dd.api.web.ServletMapping;
import org.netbeans.modules.j2ee.dd.api.web.WebApp;
import org.netbeans.modules.web.api.webmodule.WebModule;
import org.netbeans.modules.web.spi.webmodule.FrameworkConfigurationPanel;
import org.netbeans.modules.web.spi.webmodule.WebFrameworkProvider;

import org.netbeans.spi.java.project.classpath.ProjectClassPathExtender;
import org.netbeans.spi.project.support.ant.AntProjectHelper;
import org.netbeans.spi.project.support.ant.EditableProperties;

import org.openide.ErrorManager;

import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileUtil;

import org.openide.util.MapFormat;
import org.openide.util.NbBundle;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.math.BigInteger;
import java.text.DateFormat;
import java.util.Date;
import java.util.Set;

/**
 *
 * @author Petr Pisl, Geertjan Wielenga
 */
public class WicketFrameworkProvider extends WebFrameworkProvider {

    private final static String DEFAULT_APP_CLASS = "WicketApplication";     //NOI18N
    private final static String DEFAULT_MAIN_PACKAGE = "com.myapp.wicket";   //NOI18N
    private final static String DEFAULT_HOME_PAGE = "Home";                  //NOI18N
    private final static String DEFAULT_DUMMY_POJO = "WicketPojo";           //NOI18N
    private static final String ENCODING = "UTF-8"; // NOI18N
    private WicketConfigurationPanel panel;

    public WicketFrameworkProvider() {
        super(NbBundle.getMessage(WicketFrameworkProvider.class, "Wicket_Name"), //NOI18N
                NbBundle.getMessage(WicketFrameworkProvider.class, "Wicket_Description")); //NOI18N
    }

    public Set extend(WebModule webModule) {
        if (!WicketConfigUtilities.wicketLibraryExists()) {
            try {
                WicketConfigUtilities.createWicketLibrary(panel.getWicketFolder());
            } catch (IOException e) {
                ErrorManager.getDefault().notify(e);
            }
        }

        FileObject documentBase = webModule.getDocumentBase();
        Library lib = LibraryManager.getDefault().getLibrary(WicketConfigUtilities.LIB_WICKET_NAME);

        if (documentBase != null && lib != null) {
            Project project = FileOwnerQuery.getOwner(documentBase);
            ProjectClassPathExtender cpExtender = (ProjectClassPathExtender) project.getLookup().lookup(ProjectClassPathExtender.class);

            if (cpExtender != null) {
                try {
                    cpExtender.addLibrary(lib);
                } catch (IOException ioe) {
                    ErrorManager.getDefault().notify(ioe);
                }
            } else {
                ErrorManager.getDefault().log("WebProjectClassPathExtender not found in the project lookup of project: " + FileUtil.toFile(project.getProjectDirectory()));
            }

            FileObject webInf = webModule.getWebInf();
            if (webInf != null) {
                try {
                    FileSystem fs = webInf.getFileSystem();
                    fs.runAtomicAction(new CreateWicketFiles(webModule));
                } catch (IOException exc) {
                    ErrorManager.getDefault().notify(exc);
                    return null;
                }
            }
        }

        return null;
    }

    public boolean isInWebModule(WebModule webModule) {
        return WicketConfigUtilities.getActionServlet(webModule.getDeploymentDescriptor()) != null;
    }

    public File[] getConfigurationFiles(WebModule webModule) {
        return new File[0];
    }

    public FrameworkConfigurationPanel getConfigurationPanel(WebModule webModule) {
        panel = new WicketConfigurationPanel();
        boolean defaultValue = ((webModule == null) || !isInWebModule(webModule));

        if (defaultValue) {
            // get configuration panel with default value
            panel.setAppClass(DEFAULT_APP_CLASS);
            panel.setMainPackage(DEFAULT_MAIN_PACKAGE);
            panel.setHomePage(DEFAULT_HOME_PAGE);
            panel.setDummyPojo(DEFAULT_DUMMY_POJO);
        } else {
            // get configuration panel with values from the wm
            Servlet servlet = WicketConfigUtilities.getActionServlet(webModule.getDeploymentDescriptor());
            panel.setServletName(servlet.getServletName());
            panel.setURLPattern(WicketConfigUtilities.getActionServletMapping(webModule.getDeploymentDescriptor()));
        }

        return panel;
    }

    private String readResource(String name) throws IOException {
        return readTextResource(getClass().getResourceAsStream("resources/" + name), ENCODING); // NOI18N
    }

    private byte[] readBinaryResource(String name) throws IOException {
        InputStream is = getClass().getResourceAsStream("resources/" + name); // NOI18N
        ByteArrayOutputStream os = new ByteArrayOutputStream();
        try {
            FileUtil.copy(is, os);
        } finally {
            is.close();
        }
        return os.toByteArray();
    }

    private static String readTextResource(InputStream is, String encoding)
            throws IOException {
        StringBuffer sb = new StringBuffer();
        String lineSep = System.getProperty("line.separator"); //NOI18N
        BufferedReader br = new BufferedReader(new InputStreamReader(is, encoding));
        try {
            String line = br.readLine();
            while (line != null) {
                sb.append(line);
                sb.append(lineSep);
                line = br.readLine();
            }
        } finally {
            br.close();
        }
        return sb.toString();
    }

    private static void createFile(FileObject targetFolder, String name, String content) throws IOException {
        FileObject file = FileUtil.createData(targetFolder, name);
        writeFile(file, content, ENCODING);
    }

    private static void createFile(FileObject targetFolder, String name, byte[] content) throws IOException {
        FileObject file = FileUtil.createData(targetFolder, name);
        FileLock lock = file.lock();
        try {
            OutputStream os = file.getOutputStream(lock);
            try {
                os.write(content);
            } finally {
                os.close();
            }
        } finally {
            lock.releaseLock();
        }
    }

    private static void writeFile(FileObject file, String content, String encoding)
            throws IOException {
        FileLock lock = file.lock();
        try {
            BufferedWriter bw = new BufferedWriter(new OutputStreamWriter(file.getOutputStream(lock), encoding));
            try {
                bw.write(content);
            } finally {
                bw.close();
            }
        } finally {
            lock.releaseLock();
        }
    }

    private static EditableProperties loadProperties(FileObject propsFO)
            throws IOException {
        EditableProperties props = new EditableProperties(true);
        InputStream is = propsFO.getInputStream();

        try {
            props.load(is);
        } finally {
            is.close();
        }

        return props;
    }

    private static void storeProperties(FileObject propsFO, EditableProperties props)
            throws IOException {
        FileLock lock = propsFO.lock();
        try {
            OutputStream os = propsFO.getOutputStream(lock);
            try {
                props.store(os);
            } finally {
                os.close();
            }
        } finally {
            lock.releaseLock();
        }
    }

    private class CreateWicketFiles implements FileSystem.AtomicAction {

        private WebModule wm;
        private FileObject documentBase;
        private Project project;
        private FileObject targetFolder;
        private Map defaultArgs;

        public CreateWicketFiles(WebModule wm) {
            this.wm = wm;
        }

        private void createAppClass() throws IOException {
            String template = readResource("WicketApplication.template"); // NOI18N
            String appClassName = panel.getAppClass();
            String appClassFileName = appClassName + ".java"; //NOI18N

            Map args = new HashMap(defaultArgs);
            args.put("NAME", appClassName.replace('.', '/')); // NOI18N
            String content = formatTemplate(template, args);
            createFile(targetFolder, appClassFileName, content);
        }

        private void createDummyPojo() throws IOException {
            String template = readResource("WicketPojo.template"); //NOI18N
            String dummyPojoName = panel.getDummyPojo();
            String dummyPojoFileName = dummyPojoName + ".java"; // NOI18N

            Map args = new HashMap(defaultArgs);
            args.put("NAME", dummyPojoName.replace('.', '/'));
            String content = formatTemplate(template, args);
            createFile(targetFolder, dummyPojoFileName, content);
        }

        private void createHomePage() throws IOException {
            // Creating the home web page HTML file
            String template = readResource("WicketApplicationPage.html"); //NOI18N
            String homePageName = panel.getHomePage();
            String homePageHtmlFileName = homePageName + ".html"; // NOI18N

            String content = formatTemplate(template, defaultArgs);
            // If we generate example formatting, add stylesheet reference and main navigation.
            if (panel.isExampleHeader()) {
                String endLine = System.getProperty("line.separator"); //NOI18N

                String find = "</title>"; // NOI18N
                if (content.indexOf(find) > 0) {
                    StringBuffer replace = new StringBuffer();
                    replace.append(find);
                    replace.append(endLine);
                    replace.append("    <link rel='stylesheet' type='text/css' href='style.css'/>"); //NOI18N
                    content = content.replaceFirst(find, replace.toString());
                }

                find = "<body>"; // NOI18N
                if (content.indexOf(find) > 0) {
                    StringBuffer replace = new StringBuffer();
                    replace.append(find);
                    replace.append(endLine);
                    replace.append("    <span wicket:id='mainNavigation'/>"); //NOI18N
                    content = content.replaceFirst(find, replace.toString());
                }
            }
            createFile(targetFolder, homePageHtmlFileName, content);

            // Creating the home web page Java file
            template = readResource("WicketApplicationPage.template"); //NOI18N
            String homePageClassFileName = homePageName + ".java"; // NOI18N

            HashMap args = new HashMap(defaultArgs);
            args.put("NAME", homePageName); // NOI18N
            if (panel.isExampleHeader()) {
                args.put("NAME_OF_IMPORT_CLASS", ""); // NOI18N
                args.put("NAME_OF_IMPORT_CLASS_SHORT", "WicketExamplePage"); // NOI18N
            } else {
                args.put("NAME_OF_IMPORT_CLASS", "import wicket.markup.html.WebPage;"); // NOI18N
                args.put("NAME_OF_IMPORT_CLASS_SHORT", "WebPage"); // NOI18N
            }
            content = formatTemplate(template, args);
            createFile(targetFolder, homePageClassFileName, content); //NOI18N
        }

        private void createExamplePage() throws IOException {
            // Creating WicketExampleHeader.html
            String content = readResource("WicketExampleHeader.html"); //NOI18N
            createFile(targetFolder, "WicketExampleHeader.html", content); //NOI18N

            // Creating WicketExampleHeader.java
            String template = readResource("WicketExampleHeader.template"); // NOI18N
            content = formatTemplate(template, defaultArgs);
            createFile(targetFolder, "WicketExampleHeader.java", content); //NOI18N

            // Creating WicketExamplePage.java
            template = readResource("WicketExamplePage.template"); //NOI18N
            content = formatTemplate(template, defaultArgs);
            createFile(targetFolder, "WicketExamplePage.java", content); //NOI18N
        }

        private void createStyleCss() throws IOException {
            String content = readResource("style.css"); //NOI18N
            createFile(documentBase, "style.css", content); //NOI18
        }

        private void createWicketLogo() throws IOException {
            byte[] content = readBinaryResource("wicket-logo.png"); //NOI18N
            createFile(documentBase, "wicket-logo.png", content); //NOI18N
        }

        private void writeURLPattern() throws IOException {
            FileObject projectprop = project.getProjectDirectory().getFileObject(AntProjectHelper.PROJECT_PROPERTIES_PATH);
            EditableProperties ep = loadProperties(projectprop);
            ep.setProperty("client.urlPart", panel.getURLPattern()); // NOI18N
            storeProperties(projectprop, ep);
        }

        private void writeDeploymentDescriptor() throws IOException {
            FileObject dd = wm.getDeploymentDescriptor();
            WebApp ddRoot = DDProvider.getDefault().getDDRootCopy(dd);
            if (ddRoot != null) {
                try {
                    Servlet servlet = (Servlet) ddRoot.createBean("Servlet"); //NOI18
                    servlet.setServletName("WicketApplication"); //NOI18N
                    servlet.setServletClass("wicket.protocol.http.WicketServlet"); //NOI18N
                    ddRoot.addServlet(servlet);

                    InitParam param = (InitParam) servlet.createBean("InitParam"); //NOI18N
                    param.setParamName("applicationClassName"); //NOI18N
                    param.setParamValue(panel.getMainPackage() + "." + panel.getAppClass()); //NOI18N
                    servlet.addInitParam(param);
                    param = (InitParam) servlet.createBean("InitParam"); //NOI18N
                    param.setParamName("debug"); //NOI18N
                    param.setParamValue("2"); //NOI18N
                    servlet.addInitParam(param);
                    param = (InitParam) servlet.createBean("InitParam"); //NOI18N
                    param.setParamName("detail"); //NOI18N
                    param.setParamValue("2"); //NOI18N\
                    servlet.addInitParam(param);
                    servlet.setLoadOnStartup(new BigInteger("2")); //NOI18N

                    ServletMapping mapping = (ServletMapping) ddRoot.createBean("ServletMapping"); //NOI18N
                    mapping.setServletName(panel.getServletName()); //NOI18N
                    mapping.setUrlPattern(panel.getURLPattern()); //NOI18N
                    ddRoot.addServletMapping(mapping);
                    ddRoot.write(dd);
                } catch (ClassNotFoundException cnfe) {
                    ErrorManager.getDefault().notify(cnfe);
                }
            }
        }

        private String formatTemplate(String template, Map args) {
            MapFormat formatter = new MapFormat(args);
            formatter.setLeftBrace("__"); // NOI18N
            formatter.setRightBrace("__"); // NOI18N
            formatter.setExactMatch(false);
            return formatter.format(template);
        }

        public void run() throws IOException {
            documentBase = wm.getDocumentBase();
            if (documentBase == null) {
                return;
            }
            project = FileOwnerQuery.getOwner(documentBase);
            if (project == null) {
                return;
            }
            SourceGroup[] sourceGroups = ProjectUtils.getSources(project).getSourceGroups(JavaProjectConstants.SOURCES_TYPE_JAVA);
            if (sourceGroups.length == 0) {
                return;
            }
            FileObject targetRoot = sourceGroups[0].getRootFolder();
            String mainPackage = panel.getMainPackage();
            targetFolder = FileUtil.createFolder(targetRoot, mainPackage.replace('.', '/')); // NOI18N

            defaultArgs = new HashMap();
            defaultArgs.put("USER", System.getProperty("user.name")); // NOI18N
            defaultArgs.put("DATE", DateFormat.getDateInstance(DateFormat.LONG).format(new Date())); // NOI18N
            defaultArgs.put("TIME", DateFormat.getTimeInstance(DateFormat.SHORT).format(new Date())); // NOI18N
            defaultArgs.put("NAME_OF_PACKAGE", panel.getMainPackage()); // NOI18N
            defaultArgs.put("NAME_OF_HOME_PAGE", panel.getHomePage().replace('.', '/')); // NOI18N

            createAppClass();
            createHomePage();
            createDummyPojo();
            if (panel.isExampleHeader()) {
                createExamplePage();
                createStyleCss();
                createWicketLogo();
            }
            changeIndexJSP();

            writeDeploymentDescriptor();
            writeURLPattern();
        }

        /** Changes the index.jsp file to mention that Wicket does not use JSP
         * pages. The message is added after the <h1>JSP Page</h1> string, if any.
         */
        private void changeIndexJSP() throws IOException {
            FileObject indexjsp = documentBase.getFileObject("index.jsp"); //NOI18N
            if (indexjsp == null) {
                return;
            }
            String content = readTextResource(indexjsp.getInputStream(), "UTF-8"); //NOI18N
            String find = "<h1>JSP Page</h1>"; // NOI18N
            String endLine = System.getProperty("line.separator"); //NOI18N
            if (content.indexOf(find) > 0) {
                StringBuffer replace = new StringBuffer();
                replace.append(find);
                replace.append(endLine);
                replace.append("    <br/>"); //NOI18N
                replace.append(endLine);
                replace.append("    <p><font color='red'>"); //NOI18N
                replace.append(NbBundle.getMessage(WicketFrameworkProvider.class, "LBL_WICKET_WELCOME_PAGE"));
                replace.append("</font></p>"); //NOI18N
                content = content.replaceFirst(find, replace.toString());
                writeFile(indexjsp, content, "UTF-8"); //NOI18N
            }
        }
    }
}
