/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.examples.modules.minicomposer;

import java.io.IOException;
import java.util.Arrays;
import java.util.Collections;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import org.openide.ErrorManager;

import org.openide.nodes.*;
import org.openide.util.Task;
import org.openide.util.TaskListener;

/** Children of a score node.
 * One child node for each note.
 * A key is a Score.Note.
 */
public class ScoreChildren extends Children.Keys implements ChangeListener, TaskListener {
    private final ScoreCookie cookie;
    private boolean added = false;
    public ScoreChildren(ScoreCookie cookie) {
        this.cookie = cookie;
    }
    protected Node[] createNodes(Object key) {
        Score.Note n = (Score.Note)key;
        return new Node[] {new NoteNode(n)};
    }
    protected void addNotify() {
        super.addNotify();
        cookie.addChangeListener(this);
        added = true;
        stateChanged(null);
    }
    protected void removeNotify() {
        cookie.removeChangeListener(this);
        added = false;
        setKeys(Collections.EMPTY_SET);
        super.removeNotify();
    }
    public void stateChanged(ChangeEvent e) {
        if (added) {
            cookie.prepare().addTaskListener(this);
        }
    }
    public void taskFinished(Task task) {
        if (cookie.isValid()) {
            try {
                Score s = cookie.getScore();
                int c = s.getSize();
                Score.Note[] notes = new Score.Note[c];
                for (int i = 0; i < c; i++) {
                    notes[i] = s.getNote(i);
                }
                setKeys(notes);
            } catch (IOException ioe) {
                // ignore
            }
        }
    }
    public Index getIndex() {
        return new IndexImpl();
    }
    private class IndexImpl extends Index.Support {
        public Node[] getNodes() {
            return ScoreChildren.this.getNodes();
        }
        public int getNodesCount() {
            return getNodes().length;
        }
        public void reorder(int[] perm) {
            try {
                Score s = cookie.getScore();
                if (s.getSize() != perm.length) throw new IllegalArgumentException();
                Score.Note[] notes = new Score.Note[perm.length];
                for (int i = 0; i < perm.length; i++) {
                    notes[perm[i]] = s.getNote(i);
                }
                cookie.setScore(new Score(Arrays.asList(notes)));
                fireChangeEvent(new ChangeEvent(IndexImpl.this));
            } catch (IOException ioe) {
                ErrorManager.getDefault().notify(ioe);
            }
        }
    }
}
