/*
DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.

Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.


The contents of this file are subject to the terms of either the GNU
General Public License Version 2 only ("GPL") or the Common
Development and Distribution License("CDDL") (collectively, the
"License"). You may not use this file except in compliance with the
License. You can obtain a copy of the License at
http://www.netbeans.org/cddl-gplv2.html
or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
specific language governing permissions and limitations under the
License.  When distributing the software, include this License Header
Notice in each file and include the License file at
nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
particular file as subject to the "Classpath" exception as provided
by Sun in the GPL Version 2 section of the License file that
accompanied this code. If applicable, add the following below the
License Header, with the fields enclosed by brackets [] replaced by
your own identifying information:
"Portions Copyrighted [year] [name of copyright owner]"

Contributor(s):

The Original Software is NetBeans. The Initial Developer of the Original
Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
Microsystems, Inc. All Rights Reserved.

If you wish your version of this file to be governed by only the CDDL
or only the GPL Version 2, indicate your decision by adding
"[Contributor] elects to include this software in this distribution
under the [CDDL or GPL Version 2] license." If you do not indicate a
single choice of license, a recipient has the option to distribute
your version of this file under either the CDDL, the GPL Version 2 or
to extend the choice of license to its licensees as provided above.
However, if you add GPL Version 2 code and therefore, elected the GPL
Version 2 license, then the option applies only if the new code is
made subject to such option by the copyright holder.
*/
package org.netbeans.modules.snipe;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import org.openide.util.Utilities;
import org.openide.windows.IOProvider;
import org.openide.windows.OutputWriter;

/**
 * Implementation of a Snipe Editor panel.
 *
 * @author Rich Unger
 */
public class SnipeEditor {
    private SnipeDataObject m_snipe;

    public SnipeEditor() {
    }

    public SnipeDataObject getSnipe() {
        return m_snipe;
    }

    public void setSnipe(SnipeDataObject dobj) {
        m_snipe = dobj;
    }

    public void open() {
        log("Opening Snipe Editor");
    } 

    public boolean close() { 
        log("Closing Snipe Editor");
        return true;
    } 

    public JComponent getPanel() { 
        return new SnipePanel();
    } 

    private void log(String str) { 
        OutputWriter out = IOProvider.getDefault().getStdOut();
        out.println(str);
        out.flush();
    } 

    private class SnipePanel extends JPanel { 
        private Image m_image;
        private int m_imagex, m_imagey;

        public SnipePanel() { 
            super();
            m_image = Utilities.loadImage("org/netbeans/modules/snipe/resources/snipe.gif");
            m_imagex = m_image.getWidth(this);
            m_imagey = m_image.getHeight(this);
            addMouseListener(new MouseAdapter() { 
                public void mouseClicked(MouseEvent evt) { 
                    Point p = getImageLocation();
                    int x = evt.getX();
                    int y = evt.getY();

                    if( p.x <= x && x <= (p.x + m_imagex) &&
                        p.y <= y && y <= (p.y + m_imagey) ) { 
                        log("You shot the snipe!");
                    } 
                }
            }); 
        } 

        public void paint(Graphics g) { 
            Point p = getImageLocation();
            g.drawImage(m_image, p.x, p.y, Color.white, this);
        } 

        private Point getImageLocation() { 
            int panelx = getWidth();
            int panely = getHeight();
            int x = Math.max(0, (panelx - m_imagex) / 2);
            int y = Math.max(0, (panely - m_imagey) / 2);
            return new Point(x, y);
        } 
    } 
} 
