/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system;

import javax.servlet.http.HttpServletRequest;
import org.netbeans.pluginportal.system.impl.ModifyPlugin;

/**
 * This interface represents the Plugin System.
 * @author David Botterill
 */
public interface PluginSystem {
    /**
     * This is a netbeans.org login hack that refreshes the request to use for logging
     * into netbeans.org.
     * @param request - the HttpServletRequest to use for logging into netbeans.org
     */
    public void refreshRequest(HttpServletRequest request); //nb.org lockin hack
    /**
     * This method is used to add a plugin to the plugin system.
     * @param plugin - the Plugin to be added.  This plugin should be created through the
     * PluginSystem.createPlugin() method.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - The PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin addPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is used for removing a Plugin from the plugin system.  Note that all
     * comments and ratings for the given plugin will also be removed.
     * @param plugin - the Plugin to be removed.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removePlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is used to modify a Plugin.
     * @param plugin -  The Plugin to be modified.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin modifyPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * Increment the given Plugin's download number by one.
     * @param plugin - The Plugin that will have the number of downloads incremented by one.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void incrementPluginDownloads(Plugin plugin) throws PluginException;
    
    /**
     * Add a Rating to a given Plugin.
     * @param plugin - The Plugin to add the given Rating to.
     * @param newRating - The Rating to add to the given Plugin
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin or the Rating is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin addRating(Plugin plugin, Rating newRating, String authenticationToken, String userid) throws PluginException;
    
    /**
     * Removes the given Rating.
     * @param rating - The Rating to remove.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Rating is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removeRating(Rating rating, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is for adding a Comment to a Plugin.
     * @param plugin - The Plugin to add the given Comment to.
     * @param newComment - The Comment to add to the given Plugin.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null or the Comment, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin addComment(Plugin plugin, Comment newComment,String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is for removing a comment from a Plugin.
     * @param comment - The Comment to remove.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Comment is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removeComment(Comment comment,String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method will remove all OtherCategory references from a Plugin.
     * @param plugin - The Plugin from which to remove all the OtherCategorys.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removeOtherCategories(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method will add an OtherCategory to a given Plugin.
     * @param plugin - The Plugin to add the given OtherCategory.
     * @param othercategory - The OtherCategory to add to the given Plugin.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PlugException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void addOtherCategory(Plugin plugin, OtherCategory othercategory, String authenticationToken, String userid) throws PluginException;
    /**
     * This method will return a Plugin [] containing the Top rated Plugin records.
     * @param numberOfRecords - int the number of top rated Plugin records to return.
     * @return - Plugin [] of the Plugins with the highest ratings.  The number of Plugin
     * records returned are based on the numberOfRecords parameter.
     */
    public Plugin[] getTopRatedPlugins(int numberOfRecords);
    
    /**
     * This method will return a Plugin [] containing the Top downloaded Plugin records.
     * @param numberOfRecords - int the number of top downloaded Plugin records to return.
     * @return - Plugin [] of the Plugins with the most downloads.  The number of Plugin
     * records returned are based on the numberOfRecords parameter.
     */
    public Plugin[] getTopDownloadsPlugins(int numberOfRecords);
    
    /**
     * This method will return a Plugin [] containing recently updated Plugin records.
     * @param numberOfRecords - int the number of recently updated Plugin records to return.
     * @return - Plugin [] of Plugins that have recently been updated.  The number of Plugin
     * records returned are based on the numberOfRecords parameter.
     */
    public Plugin[] getRecentlyUpdatedPlugins(int numberOfRecords);
    
    /**
     * This method will return a Plugin [] containing recently added Plugin records.
     * @param numberOfRecords - int the number of recently added Plugin records to return.
     * @return - Plugin [] of Plugins that have recently been added.  The number of Plugin
     * records returned are based on the numberOfRecords parameter.
     */
    public Plugin[] getRecentlyAddedPlugins(int numberOfRecords);
    
    /**
     * This method returns an array of CategoryCount records.
     * @return - an array of CategoryCount records.
     */
    public CategoryCount [] getCategoryCounts();
    
    /**
     * This method will return an array of Plugin records encompassing ALL the plugins
     * in the Plugin System.
     * @return - an array of Plugin records containing all the Plugins in the Plugin System.
     */
    public Plugin[] getAllPlugins();
    
    
    
    /**
     * This method returns a specific Plugin for a given pluginid.
     * @param inPlugin - the Long plugin id for the Plugin to fetch.
     * @return - the Plugin corresponding to the given pluginid.
     */
    public Plugin getPlugin(Long inPlugin);
    
    /**
     * This method will return a Plugin based on a given name
     * @param pluginname - the plugin name to use to look for a Plugin.
     * @return - Plugin with a name matching the given name.
     */
    public Plugin getPluginByName(String pluginname);
    
    /**
     * This method will return an array of Strings representing categories sorted by name.
     * @return - an array of Strings representing the Categories sorted by name.
     */
    public String[] getCategoriesByName();
    
    /**
     * This method will return an array of Strings representing categories sorted by count.
     * @return - an array of Strings representing the Categories sorted by name.
     */
    public String[] getCategoriesByCount();
    
    /**
     * This method will return an array of Plugins for a given category.
     * @param category - The category to use to return an array of Plugins with this category.
     * @param showAll - if "true" this will show all quality levels of Plugins, if "false" only
     * production quality Plugins will be returned.
     * @return - an array of Plugins with the given category.
     */
    public Plugin[] getPluginsByCategory(String category,boolean showAll);
    
    
    /**
     * This method returns a Rating for a given Plugin and a given userid.
     * @param pluginid - Long pluginid to be used to check for a Rating for a given userid.
     * @param userid -  String userid to be used to find a user Rating on a given Plugin.
     * @return - Rating for a given pluginid and userid.
     */
    public Rating getRatingForUser(Long pluginid, String userid);
    
    /**
     * This method will return ALL the Comment instances for a given Plugin.
     * @param pluginid - Long pluginid for the Plugin to get the Comment instances for.
     * @return - an array of Comment instances for the given Plugin sorted ascending
     * by the date entered.
     */
    public Comment[] getComments(int pluginid);
    
    /**
     * This method returns an array of Plugin instances for a given userid
     * @param userid - String userid, the userid will be used to filter the Plugin instances to return.
     * @return -  an array of Plugin instances with the Author equal to the given userid.
     * Please note that the term "Author" and "Owner" are interchangeable.
     */
    public Plugin[] getPluginsByAuthor(String userid);
    
    /**
     * This method returns DisplayCategory instances.
     * @return - an array of DisplayCategory instances.
     * @deprecated - The OtherCategorys includes the NBM manifest "Display" category.
     */
    public DisplayCategory [] getDisplayCategories();
    
    /**
     * This method returns an array of ALL OtherCategory instances in the Plugin system.
     * @return - an array of All OtherCategory instances.
     */
    public OtherCategory [] getOtherCategories();
    
    /**
     * This method returns a list of categories that match a given prefix.
     * @param prefix - the prefix to use to match categories against.
     * @return - an array of Strings that represent category names matching the given prefix.
     */
    public String [] matchOtherCategoryPrefix(String prefix);
    
    /**
     * This method will return an array of Plugin instances based on a given search string.
     * The following fields will be searched using the search string:
     * <ul>
     *    <li>Author
     *    </li>
     *    <li>Description
     *    </li>
     *    <li>Home Page URL
     *    </li>    
     *    <li>Keywords
     *    </li>
     *    <li>Plugin Name
     *    </li>
     *    <li>Summary
     *    </li>
     *    <li>Target Platforms
     *         </li>
     *         <li>Plugin Version
     *         </li>
     *         <li>Category Name
     *         </li>
     *         <li>Versions Supported
     *         </li>
     * </ul>
     * @param searchString - the search string to use to filter the return set of Plugin instances.
     * @return - an array of Plugin instances that match the criteria in the given search string.
     */
    public Plugin [] getPlugins(String searchString);
    
    /**
     *
     * @return
     */
    public CategoryRange getCategoryRange();
    
    /**
     * This method creates a Plugin.
     * @return - a Plugin
     */
    public Plugin createPlugin();
    
    /**
     * This method creates a Comment.
     * @return - a newly created Comment.
     */
    public Comment createComment();
    
    /**
     * This method creates a Rating.
     * @return - a newly created Rating
     */
    public Rating createRating();
    
    /**
     * Create an OtherCategory.
     * @return - a newly created OtherCategory
     */
    public OtherCategory createOtherCategory();
    
    /**
     * This method allows a user to login to the plugin system.
     * @param userid - userid to login  under
     * @param password - password for given userid
     * @return - returns an authentication token to be used when making API calls that
     * require authentication.
     */
    public String login(String userid, String password);
    /**
     * This method logs off the given userid from the Plugin System.
     * @param userid - userid to be logged off.
     */
    public void logoff(String userid);
    /**
     * This method will check whether the given userid is authenticated using the
     * authentication token
     * @param userid - userid to validate for authentication
     * @param authenticationToken - the token to use to validate whether the given userid is authenticated.
     * @return - true = user is authenticated with the given token,
     * false = user is NOT authenticated with the given token.
     */
    public boolean isAuthenticated(String userid, String authenticationToken);
    
}
