/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/
package org.netbeans.pluginportal.system.impl;

import org.netbeans.pluginportal.system.*;

/**
 * This interface defines functionality for modifying a plugin.
 */
public interface ModifyPlugin {
    
    /**
     * This method is used to add a plugin to the plugin system.
     * @param plugin - the Plugin to be added.  This plugin should be created through the
     * PluginSystem.createPlugin() method.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means 
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - The PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin addPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is used for removing a Plugin from the plugin system.  Note that all
     * comments and ratings for the given plugin will also be removed.
     * @param plugin - the Plugin to be removed.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removePlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is used to modify a Plugin.
     * @param plugin -  The Plugin to be modified.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means 
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin modifyPlugin(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * Increment the given Plugin's download number by one.
     * @param plugin - The Plugin that will have the number of downloads incremented by one.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void incrementPluginDownloads(Plugin plugin) throws PluginException;
    
    /**
     * Add a Rating to a given Plugin.
     * @param plugin - The Plugin to add the given Rating to.
     * @param newRating - The Rating to add to the given Plugin
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means 
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin or the Rating is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin addRating(Plugin plugin, Rating newRating, String authenticationToken, String userid) throws PluginException;
    
    /**
     * Removes the given Rating.
     * @param rating - The Rating to remove.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Rating is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removeRating(Rating rating, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is for adding a Comment to a Plugin.
     * @param plugin - The Plugin to add the given Comment to.
     * @param newComment - The Comment to add to the given Plugin.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @return - returns a "managed" Plugin.  The term "managed" is a JPA term that means 
     * JPA knows the identity of the Plugin so it can later be modified or deleted.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null or the Comment, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public Plugin addComment(Plugin plugin, Comment newComment,String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method is for removing a comment from a Plugin.
     * @param comment - The Comment to remove.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Comment is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removeComment(Comment comment,String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method will remove all OtherCategory references from a Plugin.
     * @param plugin - The Plugin from which to remove all the OtherCategorys.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PluginException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void removeOtherCategories(Plugin plugin, String authenticationToken, String userid) throws PluginException;
    
    /**
     * This method will add an OtherCategory to a given Plugin.
     * @param plugin - The Plugin to add the given OtherCategory.
     * @param othercategory - The OtherCategory to add to the given Plugin.
     * @param authenticationToken - the authentication token created for the user when the user logged in
     * via PluginSystem.login() method.
     * @param userid - the userid to validate the authentication token against.
     * @throws org.netbeans.pluginportal.system.PluginException - If the Plugin is null, a PluginException will be thrown.  Also, the PlugException can be an error specific to the persistance layer used
     * (i.e. JPA) or a PluginAuthenticationException if the user is not authorized
     * to perform this function.
     */
    public void addOtherCategory(Plugin plugin, OtherCategory othercategory, String authenticationToken, String userid) throws PluginException;
}
