/**
 *
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 **/

package org.netbeans.pluginportal.system.impl;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.NoResultException;
import javax.persistence.NonUniqueResultException;
import javax.persistence.Query;
import org.netbeans.pluginportal.data.CommentImpl;
import org.netbeans.pluginportal.data.DisplayCategoryImpl;
import org.netbeans.pluginportal.data.OtherCategoryImpl;
import org.netbeans.pluginportal.data.PluginImpl;
import org.netbeans.pluginportal.system.CategoryCount;
import org.netbeans.pluginportal.system.CategoryRange;
import org.netbeans.pluginportal.system.Comment;
import org.netbeans.pluginportal.system.DisplayCategory;
import org.netbeans.pluginportal.system.OtherCategory;
import org.netbeans.pluginportal.system.Plugin;
import org.netbeans.pluginportal.system.PluginException;
import org.netbeans.pluginportal.system.QualityLevel;
import org.netbeans.pluginportal.system.Rating;
import org.netbeans.pluginportal.system.WeightedCategory;

/**
 *
 * @author David Botterill
 */
public class QueryManager implements QueryPlugin {
    private EntityManagerFactory emf;
    
    /** Creates a new instance of QueryManager */
    public QueryManager(EntityManagerFactory inEmf) {
        setEmf(inEmf);
    }
    
    protected EntityManager getEntityManager() {
        EntityManager returnEM = null;
        /**
         *@TODO: Not sure if we can use the same entitymanager with multiple threads
         *I've seen warnings about doing that.
         *for now, just create a new instance every time.
         */
        
        if(null != this.getEmf()) {
            returnEM = this.getEmf().createEntityManager();
        } else {
            System.err.println("Error! EntityManagerFactory == null.");
        }
        
        return returnEM;
    }
    
    public Plugin[] getAllPlugins() {
        String select = "select p from PluginImpl p";
        Query query = getEntityManager().createQuery(select);
        List<CommentImpl> resultList = query.getResultList();
        
        return (Plugin [])resultList.toArray(new Plugin[0]);
    }
    
    
    public Plugin getPlugin(Long inPlugin) {
        String select = "select p from PluginImpl p where p.pluginid=" + inPlugin;
        Query query = getEntityManager().createQuery(select);
        PluginImpl returnPlugin=null;
        try {
            returnPlugin = (PluginImpl) query.getSingleResult();
        } catch(NoResultException nre) {
            System.err.println(this.getClass().getName() + ".returnPlugin: No Result trying to get Plugin ID=" + inPlugin);
        }
        return returnPlugin;
    }
    
    public Plugin getPluginByName(String pluginname) {
        String select = "select p from PluginImpl p where upper(p.plugin_name)='" + pluginname.replaceAll("'","''").toUpperCase() + "'";
        Query query = getEntityManager().createQuery(select);
        Plugin returnPlugin=null;
        try {
            returnPlugin = (Plugin) query.getSingleResult();
        } catch(NoResultException nre) {
            /**
             * Valid if user has not rated the plugin.
             */
        } catch(NonUniqueResultException nure) {
            System.err.print("Found a non-unique plugin name entry for pluginid=" + returnPlugin.getPluginid() + " name=" + returnPlugin.getPlugin_name());
        }
        return returnPlugin;
    }
    
    
    public String[] getCategoriesByName() {
        throw new RuntimeException(new PluginException("getCategoriesByName method not implemented!"));
    }
    
    public String[] getCategoriesByCount() {
        throw new RuntimeException(new PluginException("getCategoriesByCount method not implemented!"));
    }
    
    public Plugin[] getPluginsByCategory(String category, boolean showAll) {
        if(null == category) {
            return null;
        }
        /**
         * Make sure we deal with single quotes.
         */
        category = category.replaceAll("'","''");
//        String select = "select p from PluginImpl p, DisplayCategoryImpl d where p.categoryid=d.id and d.displaycategory_name='" + category + "'";
//        if(!showAll) {
//            select += " and p.quality_level='" + QualityLevel.PRODUCTION + "'";
//        }
//        select += " order by p.plugin_name";
//        Query query = getEntityManager().createQuery(select);
//        List<PluginImpl> resultList = query.getResultList();
        
        String select = "select p from PluginImpl p, OtherCategoryImpl o where p.pluginid=o.plugin.pluginid and o.categoryname='" + category + "'";
        if(!showAll) {
            select += " and p.quality_level='" + QualityLevel.PRODUCTION + "'";
        }
        select += " order by p.plugin_name";
        Query query = getEntityManager().createQuery(select);
        List<PluginImpl> otherResultList = query.getResultList();
        
//        resultList.addAll(otherResultList);
        
        return (Plugin [])otherResultList.toArray(new Plugin[0]);
    }
   
    
    public Rating getRatingForUser(Long pluginid, String userid) {
        String select = "select r from RatingImpl r where r.plugin.pluginid=" + pluginid +
                " and r.userid='" + userid + "'";
        Query query = getEntityManager().createQuery(select);
        Rating returnRating=null;
        try {
            returnRating = (Rating) query.getSingleResult();
        } catch(NoResultException nre) {
            /**
             * Valid if user has not rated the plugin.
             */
        } catch(NonUniqueResultException nure) {
            System.err.print("Found a non-unique rating entry for pluginid=" + pluginid + " userid=" + userid);
        }
        return returnRating;
        
    }
    
    public Comment[] getComments(int pluginid) {
        String select = "select c from CommentImpl c where c.plugin.pluginid=" + pluginid;
        select += " order by c.date_entered desc";
        Query query = getEntityManager().createQuery(select);
        List<CommentImpl> resultList = query.getResultList();
        
        return (Comment [])resultList.toArray(new Comment[0]);
    }
    
    public Plugin[] getPluginsByAuthor(String userid) {
        String select = "select p from PluginImpl p where p.author_userid='" + userid + "'";
        Query query = getEntityManager().createQuery(select);
        List<PluginImpl> resultList = query.getResultList();
        
        return (Plugin [])resultList.toArray(new Plugin[0]);
    }
    
    public DisplayCategory[] getDisplayCategories() {
        DisplayCategory [] displayCategories = null;
        
        String select = "select d from DisplayCategoryImpl d order by d.displaycategory_name ";
        EntityManager em = this.getEntityManager();
        Query query = em.createQuery(select);
        List<DisplayCategoryImpl> resultList = query.getResultList();
        
        ArrayList<DisplayCategoryImpl> categories = new ArrayList(resultList);
        displayCategories = (DisplayCategory [])categories.toArray(new DisplayCategory[0]);
        
        return displayCategories;
    }
    
    public OtherCategory[] getOtherCategories() {
        String select = "select o from OtherCategoryImpl o ";
        EntityManager em = this.getEntityManager();
        Query query = em.createQuery(select);
        List<OtherCategoryImpl> resultList = query.getResultList();
        
        ArrayList<OtherCategoryImpl> categories = new ArrayList(resultList);
        return (OtherCategory [])categories.toArray(new OtherCategory[0]);
    }
    
    public String[] matchOtherCategoryPrefix(String prefix) {
        String select = "select distinct categoryname from OtherCategoryImpl o where UPPER(o.categoryname) like '" + prefix.toUpperCase() + "%'";
        EntityManager em = this.getEntityManager();
        Query query = em.createNativeQuery(select);
        List resultList = query.getResultList();
        Iterator resultIterator = resultList.iterator();
        Vector currentResult = null;
        ArrayList<String> catList = new ArrayList();
        while(resultIterator.hasNext()) {
            currentResult = (Vector)resultIterator.next();
            String categoryName = (String)currentResult.get(0); //category_name
            catList.add(categoryName);
        }
        
        return (String [])catList.toArray(new String[0]);
    }
    
    public Plugin[] getPlugins(String searchString) {
        String select = "select p from PluginImpl p left outer join p.othercategories o left outer join p.comments c";
        if(null != searchString) {
            select +=   " where (" +
                    " upper(p.author_userid) like '%" + searchString.toUpperCase() + "%' or " +
                    " upper(p.description) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(p.home_page_url) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(p.keywords) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(p.plugin_name) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(p.summary) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(p.target_platforms) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(p.version) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(o.categoryname) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(c.comment_summary) like '%" + searchString.toUpperCase() + "%' or" +
                    " upper(p.versions_supported) like '%" + searchString.toUpperCase() + "%')";
        }
        Query query = getEntityManager().createQuery(select);
        List<PluginImpl> resultList = query.getResultList();
        
        return (Plugin [])resultList.toArray(new Plugin[0]);
    }
    
    public CategoryRange getCategoryRange() {
        
        EntityManager em = this.getEntityManager();
        String select = "select max(cc.cat_count) as top, min(cc.cat_count) as bottom from othercategoryimpl o," +
                " (select distinct categoryname,count(categoryname) as cat_count from othercategoryimpl cc group by categoryname) as cc" +
                " where o.categoryname=cc.categoryname";
        Query query = em.createNativeQuery(select);
        List resultList = query.getResultList();
        Iterator resultIterator = resultList.iterator();
        Vector currentResult = null;
        ArrayList<String> list = new ArrayList();
        Integer othermax = null; //max
        Integer othermin = null; //min
        
        if(resultIterator.hasNext()) {
            currentResult = (Vector)resultIterator.next();
            othermax = (Integer)currentResult.get(0); //max
            othermin = (Integer)currentResult.get(1); //min
        }
        
        
        CategoryRange downloadRange = new CategoryRange();
        if(null != othermax && null != othermin) {
            downloadRange.setMax(othermax.intValue());
            downloadRange.setMin(othermin.intValue());
        }
        
        return downloadRange;
        
    }
    
    public EntityManagerFactory getEmf() {
        return emf;
    }
    
    public void setEmf(EntityManagerFactory emf) {
        this.emf = emf;
    }
    
}
