/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.find;

import java.util.ArrayList;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import org.netbeans.testspeceditor.beans.Value;

/**
 *
 * @author ehucka
 */
public class FindPattern {
    
    private String pattern;
    private boolean matchCase;
    private boolean wholeWord;
    private boolean regExp;
    private boolean selectedNodes;
    
    
     
    public FindPattern(String value, boolean matchCase, boolean wholeWord, boolean regExp, boolean selectedNodes) {
        this.pattern=value;
        this.matchCase=matchCase;
        this.wholeWord=wholeWord;
        this.regExp=regExp;
        this.selectedNodes=selectedNodes;
    }
    
    public String getValue() {
        return pattern;
    }
    
    public boolean isMatchCase() {
        return matchCase;
    }
    
    public boolean isWholeWord() {
        return wholeWord;
    }
    
    public boolean isRegExp() {
        return regExp;
    }
    
    public boolean isSelectedNodes() {
        return selectedNodes;
    }
    
    public FoundOccurance[] find(String value) {
        ArrayList<FoundOccurance> ret=new ArrayList(32);
        
        if (!regExp) {
            String v=value;
            String pat=pattern;
            if (!matchCase) {
                v=v.toLowerCase();
                pat=pat.toLowerCase();
            }
            int start=-1;
            int end=-1;
            start=v.indexOf(pat);
            boolean match;
            while (start > -1 && start < v.length()-1) {
                end=start+pat.length();
                match=true;
                if (wholeWord) {
                    if (start > 0 && Character.isUnicodeIdentifierPart(v.charAt(start-1))) {
                        match=false;
                    }
                    if (end < v.length()-1 && Character.isUnicodeIdentifierPart(v.charAt(end))) {
                        match=false;
                    }
                }
                if (match) {
                    FoundOccurance oc=createFoundOccurance(start, end, value);
                    ret.add(oc);
                }
                start=v.indexOf(pat, start+1);
            }
        } else {//regexp
            Pattern pat=Pattern.compile(pattern, Pattern.DOTALL|((matchCase)?0:Pattern.CASE_INSENSITIVE));
            Matcher m=pat.matcher(value);
            while (m.find()) {
                FoundOccurance oc=createFoundOccurance(m.start(), m.end(), value);
                ret.add(oc);
            }
        }
        
        return ret.toArray(new FoundOccurance[ret.size()]);
    }
    
    protected FoundOccurance createFoundOccurance(int start, int end, String text) {
        return new FoundOccurance(start, end, text);
    }
}
