/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.find;

import org.netbeans.testspeceditor.beans.TestCase;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.beans.TestSuite;
import org.netbeans.testspeceditor.beans.Value;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.events.BeanChangeSupport;

/**
 *
 * @author ehucka
 */
public class FoundBean implements Bean {
    
    public static final String TREE_PATH_DIVIDER = " | ";
    
    private Value value;
    private Bean presented;
    private String name;
    private String label;
    private FoundOccurance[] occurances;
    
    transient BeanChangeSupport support = new BeanChangeSupport(this, false);
    
     
    public FoundBean(Value bean, Bean presented, String label, FoundOccurance[] occurances) {
        this.presented = presented;
        this.value = bean;
        this.label = label;
        createName(label);
        this.occurances=occurances;
        for (int i = 0; i < occurances.length; i++) {
            occurances[i].setBean(this);
        }
    }
    
    public int getOccurancesSize() {
        return getOccurances().length;
    }
    
    public BeanChangeSupport getBeanSupport() {
        return support;
    }
    
    public Bean copy() {
        return this;
    }
    
    public Bean getPresentedBean() {
        return presented;
    }
    
    public Value getValue() {
        return value;
    }
    
    public String getName() {
        return name;
    }
    
    public FoundOccurance[] getOccurances() {
        return occurances;
    }
    
    protected void createName(String label) {
        StringBuilder sb=new StringBuilder();
        if (presented instanceof TestSpec) {
            sb.append(((TestSpec)presented).getModule());
        } else if (presented instanceof TestSuite) {
            sb.append(((TestSuite)presented).getTitle());
        } else if (presented instanceof TestCase) {
            sb.append(((TestCase)presented).getTitle());
        }
        sb.append(TREE_PATH_DIVIDER);
        sb.append(label);
        name=sb.toString();
    }
    
    public String getLabel() {
        return label;
    }
    
    public int update() {
        FoundOccurance[] occurs=FindEngine.getInstance().getPattern().find(value.getContent());
        if (occurs.length == 0) {
            return 0;
        }
        if (occurs.length < occurances.length) {
            FoundOccurance[] oldo=occurances;
            occurances=new FoundOccurance[occurs.length];
            System.arraycopy(oldo, 0, occurances, 0, occurs.length);
            for (int i = oldo.length-1;i >= occurs.length;i--) {
                support.childrenChanged(oldo[i], i, -1);
            }
        }
        for (int i = 0; i < occurances.length; i++) {
            occurances[i].update(occurs[i]);
        }
        return occurances.length;
    }
}
