/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.find;

import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.events.BeanChangeSupport;

/**
 *
 * @author ehucka
 */
public class FoundOccurance implements Bean {
    
    static final int CONTEXT = 10;
    
    private FoundBean bean;
    private int start;
    private int end;
    private boolean predots, postdots;
    
    private String prefix;
    private String suffix;
    private String pattern;
    private String htmlText=null;
    
    transient BeanChangeSupport support = new BeanChangeSupport(this, false);
    
    /**
     * Creates a new instance of FoundOccurance
     */
    public FoundOccurance(int start, int end, String value) {
        this.start=start;
        this.end=end;
        setContext(value);
    }
    
    public void setContext(String value) {
        int st=start-CONTEXT;
        if (st < 0) {
            st=0;
        }
        predots=(st > 0);
        int en=end+CONTEXT;
        if (en > value.length()) en=value.length();
        postdots=(en < value.length());
        prefix=value.substring(st, start);
        suffix=value.substring(end, en);
        pattern=value.substring(start, end);
    }
    
    public int getStart() {
        return start;
    }
    
    public int getEnd() {
        return end;
    }
    
    public BeanChangeSupport getBeanSupport() {
        return support;
    }
    
    public Bean copy() {
        return this;
    }
    
    public String getHtmlText() {
        if (htmlText == null) {
            htmlText=createHtmlText();
        }
        return htmlText;
    }
    
    protected String createHtmlText() {
        StringBuilder sb=new StringBuilder();
        sb.append("<html>");
        if (isPredots())
            sb.append("...");
        sb.append(prefix.replace("<", "&lt;"));
        sb.append("<b>");
        sb.append(pattern.replace("<", "&lt;"));
        sb.append("</b>");
        sb.append(suffix.replace("<", "&lt;"));
        if (isPostdots())
            sb.append("...");
        return sb.toString();
    }
    
    public String getPrefix() {
        return prefix;
    }
    
    public String getSuffix() {
        return suffix;
    }
    
    public String getPattern() {
        return pattern;
    }
    
    public FoundBean getBean() {
        return bean;
    }
    
    public void setBean(FoundBean bean) {
        this.bean = bean;
    }
    
    public void update(FoundOccurance occurance) {
        String old=getHtmlText();
        start=occurance.start;
        end=occurance.end;
        setContext(bean.getValue().getContent());
        support.propertyChanged("pattern", old, getHtmlText());
    }
    
    public boolean isPredots() {
        return predots;
    }
    
    public boolean isPostdots() {
        return postdots;
    }
}
