/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.actions;

import java.util.Arrays;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.LinkedList;
import javax.swing.JOptionPane;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.beans.TestCase;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.beans.TestSuite;
import org.netbeans.testspeceditor.options.Options;
import org.netbeans.testspeceditor.ui.dialogs.PackCasesDialog;
import org.netbeans.testspeceditor.ui.tree.CasesNode;
import org.netbeans.testspeceditor.ui.tree.Node;
import org.netbeans.testspeceditor.ui.tree.TestCaseNode;
import org.netbeans.testspeceditor.ui.tree.TestSpecNode;
import org.netbeans.testspeceditor.ui.tree.TestSuiteNode;

/**
 *
 * @author ehucka
 */
public class PackCasesAction implements NodeAction {
    
    
    public PackCasesAction() {
    }
    
    public void perform(Node[] nodes) {
        if (isEnabled(nodes)) {
            TestSpec spec = Application.getInstance().getDocument().getTestSpec();
            HashMap<String, TestCase> cases = new HashMap<String, TestCase>();
            //get all cases
            TestCase[] cs = spec.getTestCases();
            for (TestCase tc : cs) {
                cases.put(tc.getCase_id(), tc);
            }
            //remove all used in suites
            TestSuite[] suites = spec.getTestSuites();
            for (TestSuite suite : suites) {
                TestCase[] tcases = suite.getCases();
                for (TestCase tc : tcases) {
                    cases.remove(tc.getCase_id());
                }
            }
            if (cases.size() > 0) {
                //show all remaining
                TestCase[] remaining = cases.values().toArray(new TestCase[cases.size()]);
                Arrays.sort(remaining,new Comparator<TestCase>() {
                    public int compare(TestCase o1, TestCase o2) {
                        return o1.getTitle().compareTo(o2.getTitle());
                    }
                });
                PackCasesDialog dlg = new PackCasesDialog(remaining);
                dlg.setVisible(true);
            } else {
                JOptionPane.showMessageDialog(Application.getInstance().getUi(),
                        "There are not any unused test cases.");
            }
        }
    }
    
    public boolean isEnabled(Node[] nodes) {
        //all have the same parent
        if (nodes == null || nodes.length != 1) return false;
        if (nodes[0] instanceof CasesNode || nodes[0] instanceof TestSpecNode) return true;
        return true;
    }
    
    public String getName() {
        return "Pack Cases";
    }
    
    public String getHelpCtx() {
        return "Remove all test cases not used in any suite";
    }
    
    public char getMnemonic() {
        return 'p';
    }
    
    public javax.swing.KeyStroke getKeyStroke() {
        return null;
    }
    
    public int compareTo(Object o) {
        return getName().compareTo(((Action)o).getName());
    }
}
