/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.dialogs;

import java.awt.Color;
import java.util.ArrayList;
import javax.swing.DefaultListCellRenderer;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.ListModel;
import javax.swing.event.ListDataListener;
import org.netbeans.testspeceditor.Application;
import org.netbeans.testspeceditor.beans.TestCase;
import org.netbeans.testspeceditor.beans.TestSuite;

/**
 *
 * @author  ehucka
 */
public class CasesDialog extends javax.swing.JDialog {
    
    TestSuite suite;
    
    /** Creates new form CasesDialog */
    public CasesDialog(TestSuite suite) {
        super(Application.getInstance().getUi(), true);
        this.suite=suite;
        initComponents();
        JFrame parent=Application.getInstance().getUi();
        int w=getWidth()+10;
        int h=getHeight();
        
        if (w > parent.getWidth()) {
            w=parent.getWidth();
        }
        if (h > parent.getHeight()) {
            h=parent.getHeight();
        }
        setSize(w, h);
        setLocation(parent.getX()+(parent.getWidth()-getWidth())/2,
                parent.getY()+(parent.getHeight()-getHeight())/2);
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    // <editor-fold defaultstate="collapsed" desc=" Generated Code ">//GEN-BEGIN:initComponents
    private void initComponents() {

        jPanel1 = new javax.swing.JPanel();
        jScrollPane1 = new javax.swing.JScrollPane();
        casesList = new javax.swing.JList();
        jPanel2 = new javax.swing.JPanel();
        confirmB = new javax.swing.JButton();
        stornoB = new javax.swing.JButton();

        FormListener formListener = new FormListener();

        setDefaultCloseOperation(javax.swing.WindowConstants.DISPOSE_ON_CLOSE);
        setTitle("Select Test Cases");
        setModal(true);

        jPanel1.setBorder(javax.swing.BorderFactory.createEmptyBorder(10, 10, 1, 10));
        jPanel1.setLayout(new java.awt.BorderLayout());

        casesList.setModel(new MyModel());
        casesList.setToolTipText("Select Test Cases to add them into the Test Suite.");
        casesList.setCellRenderer(new MyRenderer());
        jScrollPane1.setViewportView(casesList);

        jPanel1.add(jScrollPane1, java.awt.BorderLayout.CENTER);

        getContentPane().add(jPanel1, java.awt.BorderLayout.CENTER);

        jPanel2.setLayout(new java.awt.FlowLayout(java.awt.FlowLayout.CENTER, 10, 10));

        confirmB.setText("Add");
        confirmB.setToolTipText("Add selected Test Cases to the Test Suite.");
        confirmB.setMargin(new java.awt.Insets(2, 3, 2, 3));
        confirmB.setPreferredSize(new java.awt.Dimension(64, 21));
        confirmB.addActionListener(formListener);
        jPanel2.add(confirmB);

        stornoB.setText("Close");
        stornoB.setMargin(new java.awt.Insets(2, 3, 2, 3));
        stornoB.setPreferredSize(new java.awt.Dimension(64, 21));
        stornoB.addActionListener(formListener);
        jPanel2.add(stornoB);

        getContentPane().add(jPanel2, java.awt.BorderLayout.SOUTH);

        pack();
    }

    // Code for dispatching events from components to event handlers.

    private class FormListener implements java.awt.event.ActionListener {
        FormListener() {}
        public void actionPerformed(java.awt.event.ActionEvent evt) {
            if (evt.getSource() == confirmB) {
                CasesDialog.this.confirmBActionPerformed(evt);
            }
            else if (evt.getSource() == stornoB) {
                CasesDialog.this.stornoBActionPerformed(evt);
            }
        }
    }// </editor-fold>//GEN-END:initComponents
    
    private void stornoBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_stornoBActionPerformed
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                setVisible(false);
                dispose();
            }
        });
    }//GEN-LAST:event_stornoBActionPerformed
    
    private void confirmBActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_confirmBActionPerformed
        Object[] objs=casesList.getSelectedValues();
        for (Object ob:objs) {
            TestCase cs=(TestCase)ob;
            if (!suite.hasCase(cs))
                suite.addCase(cs);
        }
        
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                setVisible(false);
                dispose();
            }
        });
    }//GEN-LAST:event_confirmBActionPerformed
    
    
    /**
     * @param args the command line arguments
     */
    public void display() {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                setVisible(true);
            }
        });
    }
    
    class MyRenderer extends DefaultListCellRenderer {
        Color suiteC=new Color(255,255,204);
        Color suiteselC=new Color(240,240,255);
        
        public java.awt.Component getListCellRendererComponent(javax.swing.JList list, Object value, int index, boolean isSelected, boolean cellHasFocus) {
            java.awt.Component retValue;
            TestCase tc=(TestCase)value;
            
            retValue = super.getListCellRendererComponent(list, value, index, isSelected, cellHasFocus);
            ((JLabel)retValue).setText("<html>&nbsp;"+tc.getTitle());
            if (suite.hasCase(tc)) {
                if (isSelected)
                    retValue.setBackground(suiteselC);
                else
                    retValue.setBackground(suiteC);
            }
            return retValue;
        }
    }
    
    class MyModel implements ListModel {
        
        ArrayList<ListDataListener> listeners=new ArrayList(4);
        
        public Object getElementAt(int index) {
            return Application.getInstance().getDocument().getTestSpec().getTestCase(index);
        }
        
        public void removeListDataListener(javax.swing.event.ListDataListener l) {
            listeners.remove(l);
        }
        
        public void addListDataListener(javax.swing.event.ListDataListener l) {
            listeners.add(l);
        }
        
        public int getSize() {
            return Application.getInstance().getDocument().getTestSpec().getTestCasesSize();
        }
        
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    protected javax.swing.JList casesList;
    protected javax.swing.JButton confirmB;
    protected javax.swing.JPanel jPanel1;
    protected javax.swing.JPanel jPanel2;
    protected javax.swing.JScrollPane jScrollPane1;
    protected javax.swing.JButton stornoB;
    // End of variables declaration//GEN-END:variables
    
}
