/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.edits;


import java.awt.Color;
import java.util.ArrayList;
import javax.swing.SwingUtilities;
import javax.swing.event.DocumentEvent;
import javax.swing.event.DocumentListener;
import javax.swing.text.*;
import org.netbeans.testspeceditor.find.FoundOccurance;
import org.netbeans.testspeceditor.find.HighLightingSupport;

/**
 *
 * @author  eh103527
 */
public class XMLEditorKit extends StyledEditorKit {
    
    Color tagColor=new Color(55,55,255);
    Color specialTagColor=new Color(255,0,255);
    Color attrNameColor=new Color(0,128,0);
    Color attrValueColor=new Color(204,0,107);
    Color highlightColor=new Color(255, 255, 150);
    
    SimpleAttributeSet nullAttribs = new SimpleAttributeSet();
    SimpleAttributeSet tagsAttribs = new SimpleAttributeSet();
    SimpleAttributeSet specialTagsAttribs = new SimpleAttributeSet();
    SimpleAttributeSet attrNameAttribs = new SimpleAttributeSet();
    SimpleAttributeSet attrValueAttribs = new SimpleAttributeSet();
    SimpleAttributeSet highlightAttribs = new SimpleAttributeSet();
    
    boolean debug = false;
    
     
    public XMLEditorKit() {
        super();
        StyleConstants.setBold(tagsAttribs, true);
        StyleConstants.setForeground(tagsAttribs, tagColor);
        StyleConstants.setBold(specialTagsAttribs, true);
        StyleConstants.setForeground(specialTagsAttribs, specialTagColor);
        StyleConstants.setBold(attrNameAttribs, false);
        StyleConstants.setForeground(attrNameAttribs,attrNameColor);
        StyleConstants.setBold(attrValueAttribs, false);
        StyleConstants.setForeground(attrValueAttribs,attrValueColor);
        StyleConstants.setBackground(highlightAttribs, highlightColor);
    }
    
    public String getContentType() {
        return "text/xml";
    }
    
    public Document createDefaultDocument() {
        Document doc=super.createDefaultDocument();
        DocListener list=new DocListener();
        doc.addDocumentListener(list);
        StyledDocument sd=(StyledDocument)doc;
        return doc;
    }
    
    public void recoloring(HTMLEditor editor) {
        recoloring(editor.getDocument());
    }
    
    protected void recoloring(final Document doc) {
        SwingUtilities.invokeLater(new Runnable() {
            public void run() {
                try {
                    StyledDocument sd=(StyledDocument)doc;
                    String text=doc.getText(0,doc.getLength());
                    ArrayList<XMLElement> elements=parseXML(text);
                    
                    int offs=0;
                    for (XMLElement el:elements) {
                        offs=text.indexOf(el.text,offs);
                        switch (el.type) {
                            case XMLElement.TEXT:
                                sd.setCharacterAttributes(offs, el.text.length(), nullAttribs, true);
                                offs+=el.text.length();
                                break;
                            case XMLElement.HTML_TAG:
                                sd.setCharacterAttributes(offs, el.text.length(), tagsAttribs, false);
                                offs+=el.tagName.length()+1;
                                break;
                            case XMLElement.ATTRIBUTE_NAME:
                                sd.setCharacterAttributes(offs, el.text.length(), attrNameAttribs, true);
                                offs+=el.text.length();
                                break;
                            case XMLElement.ATTRIBUTE_VALUE:
                                sd.setCharacterAttributes(offs, el.text.length(), attrValueAttribs, true);
                                offs+=el.text.length();
                                break;
                        }
                    }
                    if (HighLightingSupport.getInstance().isHighlight()) {
                        FoundOccurance[] occurs=HighLightingSupport.getInstance().getPattern().find(text);
                        for (int i = 0; i < occurs.length; i++) {
                            sd.setCharacterAttributes(occurs[i].getStart(), occurs[i].getEnd()-occurs[i].getStart(),
                                    highlightAttribs, true);
                        }
                    }
                } catch (Exception ex) {
                    ex.printStackTrace();
                }
            }
        });
    }
    
    class DocListener implements DocumentListener {
        public void insertUpdate(DocumentEvent e) {
            recoloring(e.getDocument());
        }
        
        public void removeUpdate(DocumentEvent e) {
            recoloring(e.getDocument());
        }
        
        public void changedUpdate(DocumentEvent e) {
        }
    }
    
    protected ArrayList<XMLElement> parseXML(String content) {
        int end,start;
        
        StringBuilder sb=new StringBuilder(content);
        ArrayList<String> tags=new ArrayList();
        start=sb.indexOf("<");
        if (start == -1) {
            tags.add(content);
        }
        if (start > 0) {
            tags.add(sb.substring(0,start));
            sb.delete(0,start);
            start=sb.indexOf("<");
        }
        end=0;
        char tc=0;
        while (start > -1) {
            end=sb.indexOf(">");
            if (end < 0) {
                end=sb.length();
            } else {
                end++;
            }
            tags.add(sb.substring(start,end));
            sb.delete(start,end);
            start=sb.indexOf("<");
            if (start == -1) {
                tags.add(sb.toString());
            }
            if (start > 0) {
                tags.add(sb.substring(0,start));
                sb.delete(0,start);
                start=sb.indexOf("<");
            }
        }
        if (debug) System.out.println("tags "+tags);
        ArrayList<XMLElement> elements=new ArrayList(tags.size());
        for (String line:tags) {
            if (line.length() == 0) {
                continue;
            }
            String s=new String(line);
            s=s.trim();
            if (s.indexOf('<') != 0 ) {//|| s.lastIndexOf('>') != s.length()-1) {  //value
                if (s.indexOf("&") > -1 && s.indexOf(";") > -1) {
                    parseEscapes(s, elements);
                } else {
                    elements.add(new XMLElement(line, XMLElement.TEXT));
                }
                continue;
            }
            s=s.replace('\t',' ');
            s=s.replace('<',' ');
            s=s.replace('>',' ');
            s=s.trim();
            if (s.length() == 0) {
                elements.add(new XMLElement(line, XMLElement.TEXT));
                continue;
            }
            if (s.length() > 0 && s.charAt(s.length()-1) == '/')
                s=s.substring(0,s.length()-1);
            if (s.length() > 1 && s.charAt(0) == '/') {  //one line tag or end tag
                s=s.substring(1);
            }
            int q;
            q=s.indexOf(' ');
            if (q == -1)
                q=s.length();
            String tagName=s.substring(0,q);
            
            if (FormatSupport.getInstance().isHTMLTag(tagName)) {
                elements.add(new XMLElement(line,tagName, XMLElement.HTML_TAG));
                String attribName,attribValue;
                s=s.substring(q,s.length());
                q=s.indexOf('"');
                while (q > -1) {
                    attribName=s.substring(0,q);
                    attribName=attribName.replace('=',' ');
                    attribName=attribName.trim();
                    elements.add(new XMLElement(attribName, XMLElement.ATTRIBUTE_NAME));
                    s=s.substring(q+1,s.length());
                    q=s.indexOf('"');
                    if (q > -1) {
                        attribValue=s.substring(0,q);
                        attribValue=attribValue.trim();
                        elements.add(new XMLElement(attribValue, XMLElement.ATTRIBUTE_VALUE));
                        s=s.substring(q+1,s.length());
                        q=s.indexOf('"');
                    }
                }
            } else {
                elements.add(new XMLElement(line, XMLElement.TEXT));
            }
        }
        if (debug) System.out.println("elements "+elements);
        return elements;
    }
    
    private void parseEscapes(String s, ArrayList<XMLElement> elements) {
        int ind=-1,prev=0, end=0;
        do {
            ind=s.indexOf('&', ind+1);
            if (ind > -1) {
                end=s.indexOf(';', ind);
                if (end > -1) {
                    String t=s.substring(ind, end+1);
                    if (FormatSupport.getInstance().isEscape(t)) {
                        if (ind > 0) {
                            elements.add(new XMLElement(s.substring(prev, ind), XMLElement.TEXT));
                        }
                        elements.add(new XMLElement(s.substring(ind, end+1), XMLElement.ATTRIBUTE_VALUE));
                        prev=end+1;
                    }
                    ind=end;
                }
            }
        } while (ind > -1);
        if (prev < s.length())
            elements.add(new XMLElement(s.substring(prev), XMLElement.TEXT));
    }
    
    private static int countChars(String string,char c,int from,int to) {
        int ret=0;
        for (int i=from;i < to;i++) {
            char t=string.charAt(i);
            if (t == c)
                ret++;
        }
        return ret;
    }
    
    static class XMLElement {
        
        public static final int TEXT = 0;
        public static final int HTML_TAG = 2;
        public static final int ATTRIBUTE_NAME = 4;
        public static final int ATTRIBUTE_VALUE = 5;
        
        public int type=0;
        public String text;
        public String tagName="";
        
        public XMLElement(String value,int type) {
            text=value;
            this.type=type;
        }
        
        public XMLElement(String value,String tagName,int type) {
            text=value;
            this.tagName=tagName;
            this.type=type;
        }
        
        public String toString() {
            return "\""+text+"\" "+String.valueOf(type);
        }
    }
}
