/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree;


import java.awt.Component;
import java.awt.Font;
import java.awt.FontMetrics;
import java.awt.Rectangle;
import javax.swing.JTree;
import javax.swing.tree.DefaultTreeCellRenderer;
import org.netbeans.testspeceditor.options.Options;


/**
 *
 * @author ehucka
 */
public class TreeRenderer extends DefaultTreeCellRenderer {
    
    int threeDots = -1;
    Font boldFont = null;
    
    public TreeRenderer() {
    }
    
    public Component getTreeCellRendererComponent(JTree tree, Object value,
            boolean sel, boolean expanded, boolean leaf, int row, boolean hasFocus) {
        String text = value.toString();
        if (value instanceof TestActionNode || value instanceof TestResultNode) { //hack for actions length
            DataNode node = (DataNode)value;
            Rectangle rowB, treeB;
            rowB = tree.getPathBounds(((Node)node.getParent()).getTreePath());
            treeB = tree.getVisibleRect();
            FontMetrics metrics = tree.getFontMetrics(tree.getFont());
            if (threeDots < 0 ) {
                threeDots = metrics.stringWidth("...")*2;
            }
            int w = (int)rowB.getX()+node.getIcon().getIconWidth()+getIconTextGap();
            int length = metrics.stringWidth(text);
            if (treeB.getWidth() < w+length) {
                int index = -1;
                int len = (int)w+threeDots;
                for (int i=0;i < text.length();i++) {
                    int l = metrics.charWidth(text.charAt(i));
                    if (l+len < treeB.getWidth()) {
                        len += l;
                        index = i;
                    } else {
                        break;
                    }
                }
                if (index > 0)
                    text = text.substring(0, index-1)+"...";
                else
                    text = "...";
            }
            text="<html><body>"+text;
        }
        if (value instanceof TestCaseNode || value instanceof TestSuiteNode) {
            String id = null;
            if (value instanceof TestCaseNode) {
                id = ((TestCaseNode)value).getTestCase().getCase_id();
            } else {
                id = ((TestSuiteNode)value).getTestSuite().getSuite_id();
            }
            if (id.equals(Options.getInstance().getLatestNode())) {
                if (boldFont == null) {
                    boldFont = new Font(tree.getFont().getName(), Font.BOLD, tree.getFont().getSize());
                }
                setFont(boldFont);
            } else {
                setFont(tree.getFont());
            }
        } else {
            setFont(tree.getFont());
        }
        super.getTreeCellRendererComponent(tree, text, sel, expanded, leaf, row, hasFocus);
        if (Node.class.isAssignableFrom(value.getClass())) {
            setIcon(((Node)value).getIcon());
        }
        return this;
    }
    
}
