/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.ui.tree.find;

import javax.swing.Icon;
import javax.swing.event.TreeModelEvent;
import org.netbeans.testspeceditor.beans.TestAction;
import org.netbeans.testspeceditor.beans.TestCase;
import org.netbeans.testspeceditor.beans.TestResult;
import org.netbeans.testspeceditor.beans.TestSpec;
import org.netbeans.testspeceditor.beans.TestSuite;
import org.netbeans.testspeceditor.events.Bean;
import org.netbeans.testspeceditor.find.FoundBean;
import org.netbeans.testspeceditor.find.FoundOccurance;
import org.netbeans.testspeceditor.ui.tree.DataNode;
import org.netbeans.testspeceditor.ui.tree.TestActionNode;
import org.netbeans.testspeceditor.ui.tree.TestCaseNode;
import org.netbeans.testspeceditor.ui.tree.TestResultNode;
import org.netbeans.testspeceditor.ui.tree.TestSpecNode;
import org.netbeans.testspeceditor.ui.tree.TestSuiteNode;

/**
 *
 * @author ehucka
 */
public class FoundBeanNode<T extends FoundBean> extends DataNode<T> {
    
    
    FoundTreeModel treemodel;
    
    /**
     * Creates a new instance of FoundBeanNode
     */
    public FoundBeanNode(FoundDataNode par, T ts, FoundTreeModel model) {
        super(ts, par, true);
        this.treemodel=model;
        init();
        ts.getBeanSupport().addBeanChangeListener(this);
    }
    
    private void init() {
        FoundOccurance[] occurs=getFoundBean().getOccurances();
        for (int i = 0; i < occurs.length; i++) {
            addChild(new FoundOccuranceNode(this, occurs[i], treemodel));
        }
    }
    
    public FoundBean getFoundBean() {
        return getData();
    }
    
    public FoundTreeModel getTreeModel() {
        return treemodel;
    }
    
    public String toString() {
        return getFoundBean().getName();
    }
    
    public void childrenChanged(Bean source, Bean child, int oldIndex, int newIndex) {
        if (source == getFoundBean() && child instanceof FoundOccurance) {
            if (newIndex == -1) { //old
                FoundOccuranceNode<FoundOccurance> node=(FoundOccuranceNode)(getChildAt(oldIndex));
                if (node != null && node.getOccurance() == child) {
                    int inx=remove(node);
                    node.release();
                    treeNodesRemoved(new TreeModelEvent(getTreeModel(), getTreePath(), new int[]{inx}, new Object[] {node}));
                }
            }
        }
    }
    
    public void propertyChanged(Bean source, String propertyName, Object oldValue, Object newValue) {
    }
    
    public Icon getIcon() {
        if (getFoundBean().getPresentedBean() instanceof TestSpec) {
            return TestSpecNode.icon;
        } else if (getFoundBean().getPresentedBean() instanceof TestAction) {
            return TestActionNode.icon;
        } else if (getFoundBean().getPresentedBean() instanceof TestCase) {
            return TestCaseNode.icon;
        } else if (getFoundBean().getPresentedBean() instanceof TestResult) {
            return TestResultNode.icon;
        } else if (getFoundBean().getPresentedBean() instanceof TestSuite) {
            return TestSuiteNode.icon;
        }
        return null;
    }
    
    public void release() {
        getFoundBean().getBeanSupport().removeBeanChangeListener(this);
    }
}
