/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 * 
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 * 
 * 
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 * 
 * Contributor(s):
 * 
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2007 Sun
 * Microsystems, Inc. All Rights Reserved.
 * 
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.testspeceditor.xmlparser;

import java.io.IOException;
import java.io.InputStream;
import java.nio.ByteBuffer;
import java.nio.CharBuffer;
import java.nio.charset.Charset;
import java.nio.charset.CharsetDecoder;
import java.nio.charset.CoderResult;
import java.nio.charset.CodingErrorAction;

/**
 *
 * @author ehucka
 */
public class CharSource {
    
    public static int BUFFER_LENGTH = 4096;
    
    InputStream is;
    byte[] buffer = new byte[BUFFER_LENGTH];
    CharsetDecoder decoder;
    CharBuffer charBuf;
    ByteBuffer byteBuf;
    boolean finish = false;
    boolean hasnext = true;
    
    public CharSource(InputStream is) {
        this.is = is;
        decoder = Charset.forName(XMLDocument.DEFAULT_CHARSET).newDecoder();
        decoder.onMalformedInput(CodingErrorAction.REPORT);
        decoder.onUnmappableCharacter(CodingErrorAction.REPORT);
        byteBuf = ByteBuffer.wrap(buffer);
        charBuf = CharBuffer.allocate(BUFFER_LENGTH);
        byteBuf.limit(0);
        charBuf.position(0);
        charBuf.limit(0);
    }
    
    public char next() throws IOException {
        if (charBuf.position() == charBuf.limit()) {
            throw new IOException("Buffer underflow, check its state by hasNext method.");
        }
        return charBuf.get();
    }
    
    public boolean hasNext() throws IOException {
        if (charBuf.position() >= charBuf.limit()) {
            readBuffer();
            decode();
        }
        return !finish && charBuf.position() < charBuf.limit();
    }
    
    private void readBuffer() throws IOException {
        //copy remaining to the begin of buffer
        if (byteBuf.position() < byteBuf.limit()) {
            int ind = 0;
            for (int i=byteBuf.position();i < byteBuf.limit();i++) {
                buffer[ind++]=buffer[i];
            }
            byteBuf.position(ind);
        } else {
            byteBuf.position(0);
        }
        int scanned = is.read(buffer, byteBuf.position(), buffer.length-4-byteBuf.position());
        if (scanned == -1) {
            finish = true;
            return;
        }
        byteBuf.limit(byteBuf.position()+scanned);
        byteBuf.position(0);
    }
    
    private void decode() {
        if (finish) return;
        try {
            charBuf.clear();
            CoderResult res = decoder.decode(byteBuf, charBuf, false);
            charBuf.flip();
            
            int add = 0;
            while (res.isMalformed() && add < 3) {
                is.read(buffer, byteBuf.limit(), 1);
                add++;
                res = decoder.decode(byteBuf, charBuf, false);
            }
            if (res.isMalformed()) {
                System.err.println("Malformed character.");;
            }
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }
}
