DROP TABLE IF EXISTS CATEGORY;
DROP TABLE IF EXISTS QUIZ;
DROP TABLE IF EXISTS QUIZ_QUESTION;
DROP TABLE IF EXISTS NBUSER;
DROP TABLE IF EXISTS RESPONSE;
DROP TABLE IF EXISTS RESPONSE_ITEM;
DROP TABLE IF EXISTS WINNERS;
DROP TABLE IF EXISTS USERDIR_LOGGING;

CREATE TABLE USERDIR_LOGGING(
    ID int PRIMARY KEY,
    OLD_NBID VARCHAR(255), -- old NetBeans user identifier
    WHEN_CHANGED TIMESTAMP, -- time when the id was changed
    EMAIL VARCHAR(50) -- user's e-mail
);

CREATE TABLE CATEGORY(
    ID INT PRIMARY KEY, -- quiz question category identifier
    NAME VARCHAR(255) -- name of category
);

CREATE TABLE QUIZ(
    ID INT PRIMARY KEY, -- quiz identifier
    STARTDATE DATE NOT NULL, -- quiz start date
    ENDDATE DATE NOT NULL, -- quiz end date
    CONTENT_SOURCE VARCHAR(255), -- quiz content source
    NOTIFIED INT, -- was e-mail notification send
    CLOSED INT, -- is quiz closed
    VERSION INT, -- version of quiz (persisietnce)
    PRIZE VARCHAR(100)
);
CREATE TABLE QUIZ_QUESTION(
    QUIZ_ID INT, -- quiz identifier
    RECORD INT, -- question index
    QUESTION VARCHAR(255), -- question
    QUESTION_TYPE INT, -- question type: choice, word, sentence
    RESPONSE_TEXT VARCHAR(255), -- correct answer
    CATEGORY INT, -- question category
    FOREIGN KEY (CATEGORY) REFERENCES CATEGORY(ID),
    FOREIGN KEY(QUIZ_ID) REFERENCES QUIZ(ID),
    PRIMARY KEY(QUIZ_ID, RECORD)
);
CREATE TABLE NBUSER(
    ID INT  PRIMARY KEY, -- user identifier
    NBID VARCHAR(255) NOT NULL UNIQUE, -- NB IDE user identifier
    USERNAME VARCHAR(100), -- name
    EMAIL VARCHAR(50), -- e-mail
    ADDRESS VARCHAR(200), -- address (only country is stored now)
    UNIVERSITY VARCHAR(200), -- university
    IP VARCHAR(30), -- IP adress
    SEND_NOTIFICATION INT
);

CREATE TABLE RESPONSE(
    ID INT PRIMARY KEY, -- response identifier
    NBUSER INT NOT NULL, -- user identifier
    QUIZ INT NOT NULL, -- quiz
    SUMBITTIME TIMESTAMP, -- when the response was submitted
    FOREIGN KEY (NBUSER) REFERENCES NBUSER(ID),
    FOREIGN KEY (QUIZ) REFERENCES QUIZ(ID),
    UNIQUE(NBUSER, QUIZ)
);

CREATE INDEX RESPONSE_USER ON RESPONSE(NBUSER);

CREATE TABLE RESPONSE_ITEM(
    RESPONSE INT, -- response
    ITEM_INDEX INT NOT NULL, -- response index
    ITEM_TEXT VARCHAR(255), -- response
    IS_CORRECT INT, -- is response correct
    FOREIGN KEY (RESPONSE) REFERENCES RESPONSE(ID),
    PRIMARY KEY(RESPONSE, ITEM_INDEX)
);

CREATE TABLE WINNERS(/* maybe more than just one winner ?? */
    NBUSER INT NOT NULL, -- user ID
    QUIZ INT PRIMARY KEY, -- quiz
    FOREIGN KEY (NBUSER) REFERENCES NBUSER(ID),
    FOREIGN KEY (QUIZ) REFERENCES QUIZ(ID)
);

-- fill categories
INSERT INTO CATEGORY VALUES(1, 'Back to Basics');
INSERT INTO CATEGORY VALUES(2, 'Threading and concurrency');
INSERT INTO CATEGORY VALUES(3, 'Scripting and other Languagesv');
INSERT INTO CATEGORY VALUES(4, 'AWT, Swing, etc.');
INSERT INTO CATEGORY VALUES(5, 'JavaFX');
INSERT INTO CATEGORY VALUES(6, 'Web including Web 2.0');
INSERT INTO CATEGORY VALUES(7, 'Java EE including web services and SOA');
INSERT INTO CATEGORY VALUES(8, 'Miscellaneous');
INSERT INTO CATEGORY VALUES(9, 'Gadgets, devices and Geeky stuff');
INSERT INTO CATEGORY VALUES(10, 'Java ME');
INSERT INTO CATEGORY VALUES(11, 'Java SE core APIs');
INSERT INTO CATEGORY VALUES(12, 'Testing, Profiling, etc.');
INSERT INTO CATEGORY VALUES(13, 'NetBeans Tips and Tricks');


