/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.quiz;

import java.util.Collections;
import java.util.List;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import org.netbeans.modules.quiz.entities.NbUser;

/**
 *
 * @author Jindrich Sedek
 */
public class Leaders {

    public static List<Leader> getLeaders(String category) {
        EntityManager em = PersistenceUtils.getInstance().createEntityManager();
        Query q = null;
        if ((category == null) || ("".equals(category)) || "all".equalsIgnoreCase(category)) {
            q = em.createNamedQuery("Leaders.all");
            Long count = (Long) em.createNamedQuery("QuizQuestion.count").getSingleResult();
            q.setParameter("level", count * 6 / 10);
        } else {
            Integer categoryId = Integer.parseInt(category);
            q = em.createNamedQuery("Leaders.byCategoryId");
            q.setParameter("category", categoryId);
            Long count = (Long) em.createNamedQuery("QuizQuestion.countByCategoryId")
                    .setParameter("category", categoryId).getSingleResult();
            q.setParameter("level", count * 7 / 10);
        }
        q.setMaxResults(100);
        List<Leader> data = q.getResultList();
        em.close();
        Collections.sort(data);
        return data;
    }

    public static class Leader implements Comparable<Leader> {

        private String name;
        private Integer quizCount;
        private Integer answers;
        private Integer correctAnswers;

        public Leader(NbUser user, Long answers, Long correctAnswers, Long quizCount) {
            super();
            this.name = user.getUsername();
            this.answers = answers.intValue();
            this.correctAnswers = correctAnswers.intValue();
            this.quizCount = quizCount.intValue();
        }

        public String getName() {
            return name;
        }

        public void setName(String name) {
            this.name = name;
        }

        public Integer getQuizCount() {
            return quizCount;
        }

        public void setQuizCount(Integer quizCount) {
            this.quizCount = quizCount;
        }

        public Integer getAnswers() {
            return answers;
        }

        public void setAnswers(Integer answers) {
            this.answers = answers;
        }

        public Integer getCorrectAnswers() {
            return correctAnswers;
        }

        public void setCorrectAnswers(Integer correctAnswers) {
            this.correctAnswers = correctAnswers;
        }

        public int compareTo(Leader other) {
            return other.correctAnswers.compareTo(correctAnswers);
        }
    }
}