/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz;

import java.io.IOException;
import java.text.MessageFormat;
import java.util.Date;
import java.util.List;
import java.util.Properties;
import java.util.ResourceBundle;
import java.util.concurrent.Executor;
import java.util.concurrent.Executors;
import java.util.logging.Level;
import java.util.logging.Logger;
import javax.mail.Message;
import javax.mail.MessagingException;
import javax.mail.Session;
import javax.mail.Transport;
import javax.mail.internet.MimeMessage;
import javax.persistence.EntityManager;
import javax.persistence.Query;
import javax.persistence.RollbackException;
import org.netbeans.modules.quiz.entities.NbUser;
import org.netbeans.modules.quiz.entities.Quiz;

/**
 *
 * @author ms113234
 */
public class Mailer {

    private static final ResourceBundle BUNDLE = ResourceBundle.getBundle("org.netbeans.modules.quiz.Bundle");
    private static final String ACCEPTED_BODY = BUNDLE.getString("response.accepted.email");
    private static final String ACCEPTED_SUBJECT = BUNDLE.getString("response.accepted.email.subject");
    private static final Executor mailExecutor =  Executors.newSingleThreadExecutor();
    private static volatile Properties mailProperties;
    private static final Logger LOG = Logger.getLogger(Mailer.class.getName());
    
    public static void notifyNewQuiz() {
        PersistenceUtils pu = PersistenceUtils.getInstance();
        Quiz quiz = pu.getActualQuiz();
        if (quiz == null) {
            return;
        }
        boolean notify = false;
        EntityManager em = pu.createEntityManager();
        em.getTransaction().begin();
        if (quiz.getNotified() == 0) {
            notify = true;
            quiz.setNotified(9);
            quiz = em.merge(quiz);
        }
        try {
            em.getTransaction().commit();
        } catch (RollbackException re) {
            // probbably another thread executes it so don't notify 
            notify = false;
            LOG.log(Level.SEVERE, null, re);
        }
        if (notify) {
            String text = BUNDLE.getString("new.quiz.email");
            String subject = BUNDLE.getString("new.quiz.email.subject");
            subject = MessageFormat.format(subject, quiz.getId());
            Query q = em.createNamedQuery("NbUser.findBySendNotification");
            q.setParameter("sendNotification", Integer.valueOf(1));
            List<NbUser> recipients = q.getResultList();
            LOG.info("Going to notify: "+recipients.size()+" quizers.");
            Session session = Session.getInstance(getMailProperties(), null);
            for (NbUser user : recipients) {
                try {
                    sendMail(user, subject, MessageFormat.format(text, user.getEmail()), session);
                    LOG.info("NewQuizNotification sent to: "+user.getEmail());
                } catch (Exception ex) {
                    LOG.log(Level.SEVERE, null, ex);
                }
            }
            em.getTransaction().begin();
            quiz.setNotified(1);
            em.merge(quiz);
            em.getTransaction().commit();
            em.close();
        }
    }

    public static void sendResponseConfiramtion(Quiz quiz, NbUser user) {
        mailExecutor.execute(new ResponseConfirmationTask(quiz, user));
    }
    
    private static class ResponseConfirmationTask implements Runnable {
        private Quiz quiz;
        private NbUser user;

        public ResponseConfirmationTask(Quiz quiz, NbUser user) {
            this.quiz = quiz;
            this.user = user;
        }
        
        public void run() {
                String subject = MessageFormat.format(ACCEPTED_SUBJECT, quiz.getId());
                String text = MessageFormat.format(ACCEPTED_BODY, new Object[]{
                            quiz.getId(), new Date()});
                sendMail(user, subject, text);
        }
    }

    public static void sendMail(NbUser to, String subject, String text) {
        Session session = Session.getInstance(getMailProperties(), null);
        sendMail(to, subject, text, session);
    }

    private static void sendMail(NbUser user, String subject, String text, Session session) {
        try {
            MimeMessage msg = new MimeMessage(session);
            msg.setFrom();
            msg.setRecipients(Message.RecipientType.TO, user.getEmail());
            msg.setSubject(subject);
            msg.setSentDate(new Date());
            msg.setText(text);
            Transport.send(msg);
        } catch (MessagingException mex) {
            LOG.log(Level.SEVERE, null, mex);
        }

    }

    private static Properties getMailProperties() {
        if (mailProperties == null) {
            synchronized (Mailer.class) {
                mailProperties = new Properties();
                try {
                    mailProperties.load(Mailer.class.getResourceAsStream("Mail.properties"));
                } catch (IOException ex) {
                    LOG.log(Level.SEVERE, null, ex);
                }
            }
        }
        return mailProperties;
    }
}
