/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz.entities;

import java.io.Serializable;
import java.util.Collection;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.Id;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.NoResultException;
import javax.persistence.OneToMany;
import javax.persistence.Query;
import javax.persistence.Table;
import org.netbeans.modules.quiz.PersistenceUtils;

/**
 *
 * @author Jindrich Sedek
 */
@Entity
@Table(name = "NBUSER")
@NamedQueries({@NamedQuery(name = "NbUser.findById", query = "SELECT n FROM NbUser n WHERE n.id = :id"),
    @NamedQuery(name = "NbUser.findByNbId", query = "SELECT n FROM NbUser n WHERE n.nbId = :nbId"),
    @NamedQuery(name = "NbUser.findByUsername", query = "SELECT n FROM NbUser n WHERE n.name = :name"),
    @NamedQuery(name = "NbUser.findByEmail", query = "SELECT n FROM NbUser n WHERE n.email = :email"),
    @NamedQuery(name = "NbUser.findByAddress", query = "SELECT n FROM NbUser n WHERE n.address = :address"),
    @NamedQuery(name = "NbUser.findBySendNotification",
    query = "SELECT n FROM NbUser n WHERE n.sendNotification = :sendNotification"),
    @NamedQuery(name = "NbUser.findByIp", query = "SELECT n FROM NbUser n WHERE n.ip = :ip")
})
public class NbUser implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @Column(name = "ID", nullable = false)
    private Integer id;
    @Column(name = "NBID", nullable = false, unique = true)
    private String nbId;
    @Column(name = "USERNAME")
    private String name;
    @Column(name = "EMAIL")
    private String email;
    @Column(name = "ADDRESS")
    private String address;
    @Column(name = "UNIVERSITY")
    private String university;
    @Column(name = "IP")
    private String ip;
    @Column(name = "SEND_NOTIFICATION")
    private Integer sendNotification;
    @OneToMany(mappedBy = "nbuser")
    private Collection<Response> responseCollection;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "nbuser")
    private Collection<Winners> winnersCollection;

    public NbUser() {
    }

    public NbUser(Integer id) {
        this.id = id;
    }

    public NbUser(String nbId, String name, String email, String address, String university, String ip) {
        this.id = PersistenceUtils.getNextId(NbUser.class);
        this.nbId = nbId;
        this.email = email;
        this.address = address;
        this.ip = ip;
        this.university = university;
        this.name = name;
        this.sendNotification = 1;
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public String getNbId() {
        return nbId;
    }

    public void setNbId(String nbId) {
        this.nbId = nbId;
    }

    public String getUsername() {
        return name;
    }

    public void setUsername(String username) {
        this.name = username;
    }

    public String getEmail() {
        return email;
    }

    public void setEmail(String email) {
        this.email = email;
    }

    public String getAddress() {
        return address;
    }

    public void setAddress(String address) {
        this.address = address;
    }

    public String getIp() {
        return ip;
    }

    public void setIp(String ip) {
        this.ip = ip;
    }

    public Collection<Response> getResponseCollection() {
        return responseCollection;
    }

    public void setResponseCollection(Collection<Response> responseCollection) {
        this.responseCollection = responseCollection;
    }

    public Collection<Winners> getWinnersCollection() {
        return winnersCollection;
    }

    public void setWinnersCollection(Collection<Winners> winnersCollection) {
        this.winnersCollection = winnersCollection;
    }

    public String getUniversity() {
        return university;
    }

    public void setUniversity(String university) {
        this.university = university;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null ? id.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof NbUser)) {
            return false;
        }
        NbUser other = (NbUser) object;
        if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "org.netbeans.modules.quiz.entities.NbUser[id=" + id + "]";
    }
    
    public static NbUser findById(EntityManager em, String id){
        Query q = em.createNamedQuery("NbUser.findByNbId").setParameter("nbId", id);
        NbUser user = null;
        try {
            user = (NbUser) q.getSingleResult();
        } catch (NoResultException exc) {
            return null;
        }
        return user;
    }

    public Integer getSendNotification() {
        return sendNotification;
    }

    public void setSendNotification(Integer sendNotification) {
        this.sendNotification = sendNotification;
    }
}
