/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz.entities;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import java.util.Iterator;
import javax.persistence.CascadeType;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.EntityManager;
import javax.persistence.FetchType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.NamedQueries;
import javax.persistence.NamedQuery;
import javax.persistence.NoResultException;
import javax.persistence.OneToMany;
import javax.persistence.Query;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import org.netbeans.modules.quiz.PersistenceUtils;

/**
 *
 * @author Jindrich Sedek
 */
@Entity
@Table(name = "RESPONSE")
@NamedQueries({@NamedQuery(name = "Response.findByNbUser", query = "SELECT r FROM Response r WHERE r.nbuser = :nbuser"),
    @NamedQuery(name = "Response.findByQuiz", query = "SELECT r FROM Response r WHERE r.quiz = :quiz"),
    @NamedQuery(name = "Response.findByUserSubmit", query = "SELECT r FROM Response r WHERE r.quiz = :quiz AND r.nbuser = :nbuser"),
    @NamedQuery(name = "Response.getAllResponsesGroupedByQuiz", query= "SELECT r.quiz.id, COUNT(r) FROM Response r GROUP BY r.quiz.id")
})
public class Response implements Serializable {

    private static final long serialVersionUID = 1L;
    @Id
    @Column(name = "ID", nullable = false)
    private Integer id;
    @Column(name = "SUMBITTIME")
    @Temporal(TemporalType.TIMESTAMP)
    private Date sumbittime;
    @JoinColumn(name = "NBUSER", referencedColumnName = "ID")
    @ManyToOne
    private NbUser nbuser;
    @JoinColumn(name = "QUIZ", referencedColumnName = "ID")
    @ManyToOne
    private Quiz quiz;
    @OneToMany(cascade = CascadeType.ALL, mappedBy = "response")
    private Collection<ResponseItem> responseItemCollection;

    public Response() {
    }

    public Response(NbUser user, Quiz quiz) {
        id = PersistenceUtils.getNextId(Response.class);
        this.quiz = quiz;
        this.nbuser = user;
        this.sumbittime = new Date();
    }

    public Integer getId() {
        return id;
    }

    public void setId(Integer id) {
        this.id = id;
    }

    public Date getSumbittime() {
        return sumbittime;
    }

    public void setSumbittime(Date sumbittime) {
        this.sumbittime = sumbittime;
    }

    public NbUser getNbuser() {
        return nbuser;
    }

    public void setNbuser(NbUser nbuser) {
        this.nbuser = nbuser;
    }

    public Quiz getQuiz() {
        return quiz;
    }

    public void setQuiz(Quiz quiz) {
        this.quiz = quiz;
    }

    public Collection<ResponseItem> getResponseItemCollection() {
        return responseItemCollection;
    }

    public void setResponseItemCollection(Collection<ResponseItem> responseItemCollection) {
        this.responseItemCollection = responseItemCollection;
    }

    @Override
    public int hashCode() {
        int hash = 0;
        hash += (id != null ? id.hashCode() : 0);
        return hash;
    }

    @Override
    public boolean equals(Object object) {
        // TODO: Warning - this method won't work in the case the id fields are not set
        if (!(object instanceof Response)) {
            return false;
        }
        Response other = (Response) object;
        if ((this.id == null && other.id != null) || (this.id != null && !this.id.equals(other.id))) {
            return false;
        }
        return true;
    }

    @Override
    public String toString() {
        return "org.netbeans.modules.quiz.entities.Response[id=" + id + "]";
    }

    public static boolean isAlreadySubmitted(NbUser user, Quiz quiz){
        EntityManager em = PersistenceUtils.getInstance().createEntityManager();
        boolean result = true;
        Query query = em.createNamedQuery("Response.findByUserSubmit");
        query.setParameter("quiz", quiz);
        query.setParameter("nbuser", user);
        try {
            query.getSingleResult();
        } catch (NoResultException exc) {
            result = false;
        }
        em.close();
        return result;
    }
    
    public Collection<ResponseItem> getCorrectResponseItemCollection() {
        ArrayList<ResponseItem> correctResponses = new ArrayList();
        for (Iterator<ResponseItem> it = responseItemCollection.iterator(); it.hasNext();) {
            ResponseItem responseItem = it.next();
            if(responseItem.isCorrect()) 
              correctResponses.add(responseItem);
        }

        return correctResponses;
    }

}
