/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz;

import org.netbeans.modules.quiz.stats.QuizStatsDAO;
import java.io.BufferedWriter;
import java.io.FileWriter;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Random;
import javax.persistence.EntityManager;
import org.junit.After;
import org.junit.AfterClass;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Ignore;
import org.junit.Test;
import org.netbeans.modules.quiz.Leaders.Leader;
import static org.junit.Assert.*;
import org.netbeans.modules.quiz.entities.Quiz;
import org.netbeans.modules.quiz.entities.ResponseItem;
import org.netbeans.modules.quiz.stats.QuestionItem;

/**
 *
 * @author pblaha
 */
public class PerformanceTest {

    private static PersistenceUtils pu;
    private static EntityManager em;
    private static final String NEW_LN = System.getProperty("line.separator");
    private static final String SQL_FILE_NAME = "perfData.sql";
    static boolean GEN_SQL = false;

    public PerformanceTest() {
    }

    @BeforeClass
    public static void setUpClass() throws Exception {
        PersistenceUtils.setDefaultPu(PersistenceUtils.PERF_TU);
        pu = PersistenceUtils.getInstance();
        em = pu.createEntityManager();
    }

    @AfterClass
    public static void tearDownClass() throws Exception {
    }

    @Before
    public void setUp() {
    }

    @After
    public void tearDown() {
    }

    @Test
    @Ignore
    public void genereatePerfData() throws Exception {
        // Create Performance data
        if (GEN_SQL) {
            BufferedWriter out = new BufferedWriter(new FileWriter(SQL_FILE_NAME));

            // delete the data
            out.write("DELETE FROM QUIZ_QUESTION;DELETE FROM QUIZ;DELETE FROM NBUSER;" + NEW_LN);
            out.write("DELETE FROM RESPONSE;DELETE FROM RESPONSE_ITEM;DELETE FROM WINNERS;" + NEW_LN);

            // create quizzes and questions
            for (int i = 1; i < 11; i++) {
                out.write("INSERT INTO QUIZ VALUES (" + i + ",NOW(),NOW(),NULL,0,0,0,\"Test\");" + NEW_LN);
                for (int j = 1; j < 5; j++) {
                    out.write("INSERT INTO QUIZ_QUESTION VALUES (" + i + "," + j + ",\"Q" + j + "," + i + "\",0,\"R1\"," + (new Random().nextInt(12) + 1) + ");" + NEW_LN);
                }
            }

            // users
            int k = 1;
            for (int i = 1; i < 7000; i++) {
                out.write("INSERT INTO NBUSER VALUES (" + i + ",\"NB_J2EE_" + i + "\",\"Test\",\"Test" + i + "@sun.com\",NULL,NULL,\"127.0.0.1\",1);" + NEW_LN);

                // response all quizzes

                for (int j = 1; j < 11; j++) {
                    out.write("INSERT INTO RESPONSE VALUES (" + k + "," + i + ", " + j + ",NOW());" + NEW_LN);
                    for (int l = 1; l < 5; l++) {
                        out.write("INSERT INTO RESPONSE_ITEM VALUES (" + k + ", " + l + ",\"a\"," + new Random().nextInt(2) + ");" + NEW_LN);
                    }
                    k++;
                }

            }

            // winners
            for (int i = 1; i < 11; i++) {
                out.write("INSERT INTO WINNERS VALUES (" + new Random().nextInt(7000) + "," + i + ");" + NEW_LN);
            }

            out.close();
        }
    }

    @Test
    public void testGetAllQuizzes() throws Exception {
        long start = System.currentTimeMillis();
        List<Quiz> quizzes = PersistenceUtils.getAll(Quiz.class);
        printExecTime(start, System.currentTimeMillis(), "testGetAllQuizzes()");
        assertEquals("Probably, performance data are missing in perf DB.", 10, quizzes.size());
    }

    @Test
    @Ignore
    public void testGetAllResponses() throws Exception {
        long start = System.currentTimeMillis();
        List<Quiz> quizzes = PersistenceUtils.getAll(Quiz.class);
        for (Iterator<Quiz> it = quizzes.iterator(); it.hasNext();) {
            Quiz quiz = it.next();
            List<ResponseItem> responses = PersistenceUtils.findResponsesByQuiz(quiz);
            printExecTime(start, System.currentTimeMillis(), "testGetAllResponses(), Q" + quiz.getId());
        }
        printExecTime(start, System.currentTimeMillis(), "testGetAllResponses()");
    }
    
    @Test
    public void testGetAllCorrectResponse() throws Exception {
        long start = System.currentTimeMillis();
        QuizStatsDAO stats = new QuizStatsDAO();
        List<QuestionItem> results = stats.getCorrectQuestions();
        printExecTime(start, System.currentTimeMillis(), "testGetAllCorrectResponse()");
    }

    @Test
    public void testGetTotalResponses() throws Exception {
        long start = System.currentTimeMillis();
        QuizStatsDAO stats = new QuizStatsDAO();
        Map<Integer, Long> results = stats.getTotalResponses();
        printExecTime(start, System.currentTimeMillis(), "testGetTotalResponses()");

    }
    
    @Test
    public void testLeaders() throws Exception {
        long start = System.currentTimeMillis();
        List<Leader> leaders = Leaders.getLeaders(null);
        printExecTime(start, System.currentTimeMillis(), "testGetTotalResponses()");
    }

    private void printExecTime(long start, long stop, String methodName) {
        System.out.println("Exec time " + methodName + ": " + (stop - start));
    }
}