/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2008 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */
package org.netbeans.modules.quiz.entities;

import java.util.ArrayList;
import java.util.Collection;
import java.util.Date;
import javax.persistence.EntityManager;
import org.junit.Before;
import org.junit.BeforeClass;
import org.junit.Test;
import org.netbeans.modules.quiz.PersistenceUtils;
import org.netbeans.modules.quiz.QuestionType;
import static org.junit.Assert.*;

/**
 *
 * @author ms113234
 */
public class QuizTest {

    private static PersistenceUtils pu;
    private static EntityManager em;
    
    @BeforeClass
    public static void setUpClass() throws Exception {
        PersistenceUtils.setDefaultPu(PersistenceUtils.TEST_PU);
        pu = PersistenceUtils.getInstance();
        em = pu.createEntityManager();

        // create data and new Quiz & User
        em.getTransaction().begin();
        for (int i = 0; i < 5; i++) {
            em.persist(new NbUser("usr"+i, "name", "email", "address", "university", "ip"));
        }
        
        // quiz0
        Quiz quiz = new Quiz(0, new Date(2008, 11, 15), new Date(2006, 10, 14), "Prize");
        ArrayList<QuizQuestion> questions = new ArrayList();
        questions.add(new QuizQuestion(quiz, 1, QuestionType.CHOICE, "a", "q1 ?"));
        questions.add(new QuizQuestion(quiz, 2, QuestionType.WORD, "method = a + b"));
        questions.add(new QuizQuestion(quiz, 3, QuestionType.CHOICE, "d"));
        quiz.setQuizQuestionCollection(questions);
        em.persist(quiz);
        
        // usr0 responses one invalid
        Response resp = new Response(NbUser.findById(em, "usr0"), quiz);
        ArrayList<ResponseItem> rItems = new ArrayList();
        rItems.add(new ResponseItem(resp, 1, "a", true)); // correct
        rItems.add(new ResponseItem(resp, 2, "method=a+b", true)); // correct
        rItems.add(new ResponseItem(resp, 3, "a", false)); //false
        resp.setResponseItemCollection(rItems);
        em.persist(resp);
        
        // usr1 + usr2 all responses correct
        resp = new Response(NbUser.findById(em, "usr1"), quiz);
        rItems = new ArrayList();
        rItems.add(new ResponseItem(resp, 1, "a", true)); // correct
        rItems.add(new ResponseItem(resp, 2, "method=a+b", true)); // correct
        rItems.add(new ResponseItem(resp, 3, "d", true)); //false
        resp.setResponseItemCollection(rItems);
        em.persist(resp);
        resp = new Response(NbUser.findById(em, "usr2"), quiz);
        rItems = new ArrayList();
        rItems.add(new ResponseItem(resp, 1, "a", true)); // correct
        rItems.add(new ResponseItem(resp, 2, "method=a+b", true)); // correct
        rItems.add(new ResponseItem(resp, 3, "d", true)); //false
        resp.setResponseItemCollection(rItems);
        em.persist(resp);

        // quiz1
        quiz = new Quiz(1, new Date(2008, 11, 15), new Date(2006, 10, 14), "Prize");
        questions = new ArrayList();
        questions.add(new QuizQuestion(quiz, 1, QuestionType.CHOICE, "a", "q1 ?"));
        questions.add(new QuizQuestion(quiz, 2, QuestionType.WORD, "method = a + b"));
        questions.add(new QuizQuestion(quiz, 3, QuestionType.CHOICE, "d"));
        quiz.setQuizQuestionCollection(questions);
        em.persist(quiz);
        
        // usr3 + usr4 all responses correct
        resp = new Response(NbUser.findById(em, "usr3"), quiz);
        rItems = new ArrayList();
        rItems.add(new ResponseItem(resp, 1, "a", true)); // correct
        rItems.add(new ResponseItem(resp, 2, "method=a+b", true)); // correct
        rItems.add(new ResponseItem(resp, 3, "d", true)); //false
        resp.setResponseItemCollection(rItems);
        em.persist(resp);
        resp = new Response(NbUser.findById(em, "usr4"), quiz);
        rItems = new ArrayList();
        rItems.add(new ResponseItem(resp, 1, "a", true)); // correct
        rItems.add(new ResponseItem(resp, 2, "method=a+b", true)); // correct
        rItems.add(new ResponseItem(resp, 3, "d", true)); //false
        resp.setResponseItemCollection(rItems);
        em.persist(resp);

        em.getTransaction().commit();
        em.close();
   }

    @Before
    public void setUp() {
        System.out.println("=================");
    }

    /**
     * Test of getWinnerCandidateCollection method, of class Quiz.
     */
    @Test
    public void testGetWinnerCandidateCollection() {
        em = pu.createEntityManager();
        Quiz quiz = Quiz.findById(em, 0);
        Collection<NbUser> candidates = quiz.getWinnerCandidatesList();
        // the collection should contain 2 candidates usr1 and usr2
        // usr0 have one worng item at quiz0
        // usr3 + usr4 correctly respond the quiz1 but do not respond quiz0
        assertEquals(candidates.size(), 2);
        assertTrue(candidates.contains(NbUser.findById(em, "usr1")));
        assertTrue(candidates.contains(NbUser.findById(em, "usr2")));
    }

}