/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted;

import java.awt.event.ActionListener;
import java.beans.*;
import org.netbeans.modules.regsup.logic.LogicAutomaton;

/**
 *
 * @author  Petr Slegr
 * @version 25.04.02
 */
public class AutomatonEditorStatusBar extends javax.swing.JPanel {
    
    private LogicAutomaton automaton;
    
    /** Creates new form AutomatonEditorStatusBar */
    public AutomatonEditorStatusBar (LogicAutomaton automaton) {
        initComponents ();
        // correspondens that LogicAutomaton.shouldDetermin is defaultly true
        dfaToggleButton.setSelected (true);
        nodeCountLabel.setText ("Nodes " + String.valueOf (automaton.getNodeCount ()) + " ");
        edgeCountLabel.setText ("Edges " + String.valueOf (automaton.getEdgeCount ()) + " ");
        //
        this.automaton = automaton;
        automaton.addPropertyChangeListener (new AutomatonListenPoint ());
    }
    
    void addDFAButtonListener (ActionListener listener) {
        // on this button listens also this class' inner class
        dfaToggleButton.addActionListener (listener);
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class AutomatonListenPoint implements PropertyChangeListener {
        
        /**
         * This method gets called when a bound property is changed.
         * @param evt A PropertyChangeEvent object describing the event source
         *  	and the property that has changed.
         */
        public void propertyChange (PropertyChangeEvent evt) {
            if (evt.getPropertyName ().equals (LogicAutomaton.PROP_DETERMIN)) {
                if (automaton.isDetermin ()) {
                    dfaStatusLabel.setText ("DFA");
                } else {
                    dfaStatusLabel.setText ("NFA");
                }
            }
            else if (evt.getPropertyName ().equals (LogicAutomaton.PROP_NODE_COUNT)) {
                nodeCountLabel.setText ("Nodes " + String.valueOf (automaton.getNodeCount ()));
            }
            else if (evt.getPropertyName ().equals (LogicAutomaton.PROP_EDGE_COUNT)) {
                edgeCountLabel.setText ("Edges " + String.valueOf (automaton.getEdgeCount ()) + " ");
            }
        }
        
    }
    
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        statusBarPanel = new javax.swing.JPanel();
        nodeCountLabel = new javax.swing.JLabel();
        edgeCountLabel = new javax.swing.JLabel();
        previousStateButton = new javax.swing.JButton();
        nextStateButton = new javax.swing.JButton();
        dfaToggleButton = new javax.swing.JToggleButton();
        dfaStatusLabel = new javax.swing.JLabel();
        empyLabel = new javax.swing.JLabel();

        setLayout(new java.awt.GridBagLayout());

        statusBarPanel.setLayout(new java.awt.GridBagLayout());

        nodeCountLabel.setText("Nodes ");
        nodeCountLabel.setToolTipText("Number of nodes in this automaton (not including nested)");
        nodeCountLabel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(nodeCountLabel, gridBagConstraints);

        edgeCountLabel.setText("Edges ");
        edgeCountLabel.setToolTipText("Number of edges in this automaton (not including nested)");
        edgeCountLabel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(edgeCountLabel, gridBagConstraints);

        previousStateButton.setText("P");
        previousStateButton.setToolTipText("");
        previousStateButton.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        previousStateButton.setMaximumSize(new java.awt.Dimension(26, 26));
        previousStateButton.setMinimumSize(new java.awt.Dimension(26, 26));
        previousStateButton.setPreferredSize(new java.awt.Dimension(26, 26));
        previousStateButton.setEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 2;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(previousStateButton, gridBagConstraints);

        nextStateButton.setText("N");
        nextStateButton.setToolTipText("");
        nextStateButton.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        nextStateButton.setPreferredSize(new java.awt.Dimension(26, 26));
        nextStateButton.setEnabled(false);
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 3;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.VERTICAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        statusBarPanel.add(nextStateButton, gridBagConstraints);

        dfaToggleButton.setSelected(true);
        dfaToggleButton.setText("DFA");
        dfaToggleButton.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                dfaToggleButtonActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 4;
        gridBagConstraints.gridy = 0;
        statusBarPanel.add(dfaToggleButton, gridBagConstraints);

        dfaStatusLabel.setHorizontalAlignment(javax.swing.SwingConstants.CENTER);
        dfaStatusLabel.setText("DFA");
        dfaStatusLabel.setToolTipText("Label displaying whether edited automaton is actually DFA orNFA");
        dfaStatusLabel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        dfaStatusLabel.setMaximumSize(new java.awt.Dimension(56, 26));
        dfaStatusLabel.setMinimumSize(new java.awt.Dimension(56, 26));
        dfaStatusLabel.setPreferredSize(new java.awt.Dimension(56, 26));
        dfaStatusLabel.setHorizontalTextPosition(javax.swing.SwingConstants.CENTER);
        statusBarPanel.add(dfaStatusLabel, new java.awt.GridBagConstraints());

        empyLabel.setBorder(new javax.swing.border.BevelBorder(javax.swing.border.BevelBorder.LOWERED));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 6;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.BOTH;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.WEST;
        gridBagConstraints.weightx = 1.0;
        statusBarPanel.add(empyLabel, gridBagConstraints);

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 0;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        gridBagConstraints.anchor = java.awt.GridBagConstraints.SOUTHWEST;
        gridBagConstraints.weightx = 1.0;
        add(statusBarPanel, gridBagConstraints);

    }//GEN-END:initComponents
    
    private void dfaToggleButtonActionPerformed (java.awt.event.ActionEvent evt) {//GEN-FIRST:event_dfaToggleButtonActionPerformed
        if (dfaToggleButton.isSelected ()) {
            dfaToggleButton.setText ("DFA");
        } else {
            dfaToggleButton.setText ("NFA");
        }
    }//GEN-LAST:event_dfaToggleButtonActionPerformed
    
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton nextStateButton;
    private javax.swing.JButton previousStateButton;
    private javax.swing.JToggleButton dfaToggleButton;
    private javax.swing.JLabel nodeCountLabel;
    private javax.swing.JLabel empyLabel;
    private javax.swing.JLabel dfaStatusLabel;
    private javax.swing.JPanel statusBarPanel;
    private javax.swing.JLabel edgeCountLabel;
    // End of variables declaration//GEN-END:variables
    
}
