/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted;

import org.netbeans.modules.regsup.auted.actions.*;
import org.netbeans.modules.regsup.logic.*;
import java.util.*;
import java.awt.Point;
import java.awt.Graphics;
import javax.swing.JFrame;
import org.openide.util.SharedClassObject;
import org.openide.util.actions.CallableSystemAction;
import org.netbeans.modules.regsup.debug.RexDebugger;

/** Main manager of regsup.
 *
 * @author  Petr Slegr
 * @version 21.8.01/2
 */
public class MainManager {
    
    /** Says that regsup has been started from IDE
     */
    public static final int RUN_IDE = 0;
    
    /** Says that regsup has been started from OS
     */
    public static final int RUN_ALONE = 1;
    
    private static MainManager inst;
    
    /*
    private EditorWindow currentWindow;
     */
    
    private int runMode = -1;
    
    private AutomatonEditor currentEditor;
    
    private RexDebugger currentDebugger;
    
    /**
     * Maps container to editor
     */
    private Hashtable editors;
    
    /**
     */
    private Hashtable debuggers = new Hashtable ();
    
    /**
     * Opened AutomatonEditors
     * !!!!!! na zruseni
     */
    private ArrayList opens;
    
    /**
     */
    private ArrayList elementActions;

    /**
     */
    private ArrayList addActions;
    
    /**
     */
    private ArrayList debugActions = new ArrayList ();
    
    /**
     * Other actions
     */
    private ArrayList actions;
    
    /** Actions with special position in popup or not to be displayed in popup, e.g.
     * DeleteAction, UndoAction.
     */
    private ArrayList specialActions = new ArrayList ();
    
    private String package_prefix = "org.netbeans.modules.regsup.auted.actions.";
    
    private String[] actionNames = {
        "AddNodeAction", // Add|Add node
        //        "AddExpressionAction", //Add|Add expression
        "AddFromTemplateAction",
        "AddConnectedNodeAction",
        "AddConnectedNodeBelowAction",
        "AddContainerAction",
        "DeleteAction",
        "EditAction",
        //"FinishDebuggerAction",
        "CreateEdgeAction",
        "CreateSelfEdgeAction",
        "OpenContainerAction",
        "RevertEdgeAction",
        "ToggleFinalNodeAction",
        "ToggleStartingNodeAction",
        "ToggleBreakpointAction",
        "StepIntoAction",
        "StepOverAction",
        "UndoAction",
        "RedoAction"
    };
    
    UndoAction undoAction;
    
    RedoAction redoAction;
    
    DeleteAction deleteAction;
    
    private ArrayList templates;
    
    private String templatesPackagePrefix = "org.netbeans.modules.regsup.auted.templates.";
    
    private String[] templateNames = {
        "StringTemplate"
    };
    
    /**
     */
    // !!!!???? prejmenovat na activated?
    private VisualAutomatonElement[] selectedElements;
    
    /**
     */
    private AutomatonEditorPanel currentPanel;
    
    private AutomatonEditorPanelListener automatonEditorPanelListener = 
    new AutomatonEditorPanelListenPoint ();
    
    /** This method can called only one time
     */
    public void setRunMode (int mode) {
        if (runMode == -1) {
            // not seted yet
            if (mode == RUN_ALONE || mode == RUN_IDE) {
                runMode = mode;
            } else {
                throw new IllegalArgumentException ("Unknown mode: " + mode);
            }
        } else {
            throw new IllegalStateException ("Run mode already set");
        }
    }
    
    /**
     */
    public int getRunMode () {
        return runMode;
    }
    
    /** Creates new Main */
    private MainManager () {
        editors = new Hashtable ();
        //debuggers = new ArrayList ();
        opens = new ArrayList ();
        elementActions = new ArrayList ();
        addActions = new ArrayList ();
        actions = new ArrayList ();
        templates = new ArrayList ();
        prepareActions ();
        GeometryFactoryRegistry.setFactory (VisualAutomatonNode.class, new NodeGeometryFactory ());
        GeometryFactoryRegistry.setFactory (VisualAutomatonContainer.class, new ContainerGeometryFactory ());
    }
    
    private void prepareActions () {
        for (int i=0; i< actionNames.length; i++) {
            //Object o = SharedClassObject.findObject (AddNodeAction.class, true);
            Class clazz = null;
            try {
                clazz = Class.forName (package_prefix + actionNames[i]);
            }
            catch (ClassNotFoundException cnfe) {
                cnfe.printStackTrace ();
            }
            Object obj = SharedClassObject.findObject (clazz, true);
            // !! testovat obj na null ??
            if (obj instanceof ElementAction) {
                elementActions.add (obj);
            }
            if (obj instanceof AddAction) {
                addActions.add (obj);
            } else {
                actions.add (obj);
            }
        }
        //
        undoAction = (UndoAction) SharedClassObject.findObject (UndoAction.class, true);
        redoAction = (RedoAction) SharedClassObject.findObject (RedoAction.class, true);
        deleteAction = (DeleteAction) SharedClassObject.findObject (DeleteAction.class, true);
        specialActions.add (deleteAction);
        specialActions.add (SharedClassObject.findObject (EditAction.class, true));
        debugActions.add (SharedClassObject.findObject (FinishDebuggerAction.class, true));
    }
    
    public CallableSystemAction getAction (Class clazz) {
        return (CallableSystemAction) SharedClassObject.findObject (clazz, true);
    }
    
    public Set getElementActions () {
        return new HashSet (elementActions);
    }
    
    public Set getAddActions () {
        return new HashSet (addActions);
    }
    
    public Set getDebugActions () {
        return new HashSet (debugActions);
    }
    
    // !!!!!
    public Set specialActions () {
        return new HashSet (specialActions);
    }
    
    private void prepareTemplates () {
        for (int i = 0; i < templateNames.length; i++) {
            try {
                Class clazz = Class.forName (templatesPackagePrefix + templateNames[i]);
                templates.add (clazz);
            }
            catch (ClassNotFoundException e) {
                e.printStackTrace ();
            }
        }
    }
    
    public List getTemplates () {
        return new ArrayList (templates);
    }
    
    /** Returns instance
     */
    public static MainManager getDefault () {
        if (null == inst)
            inst = new MainManager ();
        return inst;
    }
    
    /** Checks if editor for specified container is not created yet and then create
     * and register it.
     * Otherwise return proper editor.
     */
    public synchronized AutomatonEditor createEditor (VisualAutomatonContainer container) {
        Object res = editors.get (container);
        if (res == null) {
            AutomatonEditor editor = new AutomatonEditor (container);
            editors.put (container, editor);
            //System.out.println ("editor created");
            return editor;
        } else {
            //System.out.println ("editor reused");
            return (AutomatonEditor) res;
        }
    }
    
    /** Always creates a new debugger for specified editor.
     */
    public synchronized RexDebugger createDebugger (AutomatonEditor editor) {
        RexDebugger debugger = new RexDebugger (editor);
        //debuggers.add (debugger);
        debuggers.put (debugger, editor);
        return debugger;
    }
    
    /**
     * @return true if there is some debugger for editor, i.e. the editor is debugged 
     */
    public boolean existsDebuggerFor (AutomatonEditor editor) {
        return debuggers.containsValue (editor);
    }
    
    public synchronized void setCurrentDebugger (RexDebugger debugger) {
        if (! debuggers.containsKey (debugger)) {
            throw new IllegalArgumentException ("debugger is not registered: " + debugger);
        }
        currentDebugger = debugger;
    }
    
    public synchronized RexDebugger getCurrentDebugger () {
        return currentDebugger;
    }
    
    /**
     */
    public synchronized void disposeEditor (AutomatonEditor editor) {
        if (! editors.containsValue (editor)) {
            throw new IllegalArgumentException ("editor is not properly created: " + editor);
        }
        editors.remove (editor.getContent ()); // unregisters editor
    }
    
    /**
     * @deprecated
     */
    public void registerEditor (AutomatonEditor editor) {
        boolean res = opens.add (editor);
        if (!res) {
            throw new IllegalStateException ("editor already registered: " + editor);
        }
    }
    
    /**
     */
    public Set getEditors () {
        //return Collections.enumeration (editors.values ());
        // !!?? neklonuje se dvakrat??
        return new HashSet (editors.values ());
    }
    
    /**
     */
    public void setCurrentEditor (AutomatonEditor editor) {
        if (! editors.containsValue (editor)) {
            throw new IllegalArgumentException ("editor is not properly registered: " + editor);
        }
        currentEditor = editor;
        //System.out.println("current editor set to: " + editor);
    }
    
    /** "Selected" is synonym for "activated".
     */
    public void setSelectedElements (VisualAutomatonElement[] selected) {
        selectedElements = selected;
    }
    
    /**
     */
    public VisualAutomatonElement[] getSelectedElements () {
        return selectedElements;
    }

    
    /// !!! hack aby APanel nemusel drzel VisualAutomatonContainer
    ///???? pouziva se?
    AutomatonEditor findEditorByContainer (VisualAutomatonContainer container) {
        for (int i = 0; i < opens.size (); i++ ) {
            AutomatonEditor editor = (AutomatonEditor) opens.get (i);
            if (editor.getContent () == container) {
                return editor;
            }
        }
        throw new IllegalArgumentException ("editor with this container is not registered: " + container);
    }
    
    /** May return null if no editor is open.
     * @return AutomatonEditor or null
     */
    public AutomatonEditor getCurrentEditor () {
        return currentEditor;
    }
    

    /** May return null if no panel has been opened.
     */
    public AutomatonEditorPanel getCurrentPanel () {
        //System.out.println("CURRENT PANEL: " + currentPanel);
        return currentPanel;
    }
    
    public void registerAutomatonEditorPanelOwner (AutomatonEditorPanelOwner owner) {
        owner.addAutomatonEditorPanelListener (automatonEditorPanelListener);
    }
    
    /**
     */
    void shutDown () {
        System.exit (0);
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class AutomatonEditorPanelListenPoint implements AutomatonEditorPanelListener {
        
        public void activated (AutomatonEditorPanel panel) {
            currentPanel = panel;
        }
        
        public void deactivated (AutomatonEditorPanel panel) {
            currentPanel = null;
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private static void debugSetup (VisualAutomatonContainer container) {
        //
        VisualAutomatonNode node1 = container.createNode (new Point (40, 40));
        node1.setStarting (true);
        VisualAutomatonNode node2 = container.createNode (new Point (120, 40));
        //container.createEdge (node1, node2, new ComplexRexEdgeElement ('a','c'));
        container.createEdge (node1, node2, new ComplexRexEdgeElement ('a'));
        VisualAutomatonNode node3 = container.createNode (new Point (200, 40));
        //container.createEdge (node2, node3, new ComplexRexEdgeElement ('h','k'));
        container.createEdge (node2, node3, new ComplexRexEdgeElement ('h'));
        VisualAutomatonNode node4 = container.createNode (new Point (200, 100));
        node4.setFinalNode (true);
        //container.createEdge (node2, node4, new ComplexRexEdgeElement ('c','c'));
        container.createEdge (node2, node4, new ComplexRexEdgeElement ('c'));
        VisualAutomatonNode node5 = container.createNode (new Point (280, 40));
        //container.createEdge (node2, node4, new ComplexRexEdgeElement ('c','c'));
        container.createEdge (node3, node5, new ComplexRexEdgeElement ('o'));
        VisualAutomatonNode node6 = container.createNode (new Point (360, 40));
        container.createEdge (node5, node6, new ComplexRexEdgeElement ('j'));
        node6.setFinalNode (true);
        //
        VisualAutomatonContainer cont = container.createContainer (new Point (200, 200));
        VisualAutomatonNode node11 = cont.createNode (new Point (40, 40));
        node11.setStarting (true);
        VisualAutomatonNode node12 = cont.createNode (new Point (120, 80));
        node12.setFinalNode (true);
        //container.createEdge (node1, node2, new ComplexRexEdgeElement ('a','c'));
        cont.createEdge (node11, node12, new ComplexRexEdgeElement ('h'));
        //
        cont.setFinalNode (true);
        container.createEdge (node1, cont, new ComplexRexEdgeElement ('a', 'd'));
    }
    
    private static void debugSetup2 (VisualAutomatonContainer container) {
        VisualAutomatonNode node1 = container.createNode (new Point (40, 40));
        node1.setStarting (true);
        //
        VisualAutomatonContainer cont = container.createContainer (new Point (200, 200));
        VisualAutomatonNode node11 = cont.createNode (new Point (40, 40));
        node11.setStarting (true);
        VisualAutomatonNode node12 = cont.createNode (new Point (120, 80));
        VisualAutomatonNode node13 = cont.createNode (new Point (200, 80));
        node13.setFinalNode (true);
        cont.createEdge (node11, node12, new ComplexRexEdgeElement ('h'));
        cont.createEdge (node12, node13, new ComplexRexEdgeElement ('o'));
        //
        container.createEdge (node1, cont, new ComplexRexEdgeElement ('a'));
        //
        VisualAutomatonNode node3 = container.createNode (new Point (360, 40));
        container.createEdge (cont, node3, new ComplexRexEdgeElement ('j'));
        node3.setFinalNode (true);
    }
    
    /** Sets a -> ( h -> o) -> j
     * @param container Root container
     */
    private static void debugSetup3 (VisualAutomatonContainer container) {
        VisualAutomatonNode node1 = container.createNode (new Point (40, 40));
        node1.setStarting (true);
        //
        VisualAutomatonContainer cont = container.createContainer (new Point (200, 200));
        VisualAutomatonNode node11 = cont.createNode (new Point (40, 40));
        node11.setStarting (true);
        VisualAutomatonNode node12 = cont.createNode (new Point (120, 80));
        VisualAutomatonNode node13 = cont.createNode (new Point (200, 80));
        node13.setFinalNode (true);
        cont.createEdge (node11, node12, new ComplexRexEdgeElement ('h'));
        cont.createEdge (node12, node13, new ComplexRexEdgeElement ('o'));
        //
        container.createEdge (node1, cont, new ComplexRexEdgeElement ('a'));
        //
        VisualAutomatonNode node3 = container.createNode (new Point (360, 40));
        container.createEdge (cont, node3, new ComplexRexEdgeElement ('j'));
        node3.setFinalNode (true);
    }
    
    /** Sets START -> a -> ( h -> o) -> k -> FINAL1
     *             -> a -> (h -> o) -> j -> FINAL2
     * @param container Root container
     */
    // ! public z debug duvodu
    public static void debugSetup31 (VisualAutomatonContainer container) {
        VisualAutomatonNode node1 = container.createNode (new Point (40, 40));
        node1.setStarting (true);
        //
        {
            VisualAutomatonContainer cont = container.createContainer (new Point (200, 200));
            VisualAutomatonNode node11 = cont.createNode (new Point (40, 40));
            node11.setStarting (true);
            VisualAutomatonNode node12 = cont.createNode (new Point (120, 80));
            VisualAutomatonNode node13 = cont.createNode (new Point (200, 80));
            node13.setFinalNode (true);
            cont.createEdge (node11, node12, new ComplexRexEdgeElement ('h'));
            cont.createEdge (node12, node13, new ComplexRexEdgeElement ('o'));
            //
            container.createEdge (node1, cont, new ComplexRexEdgeElement ('a'));
            VisualAutomatonNode node3 = container.createNode (new Point (360, 40));
            container.createEdge (cont, node3, new ComplexRexEdgeElement ('j'));
            node3.setFinalNode (true);
            
        }
        //
        {
            VisualAutomatonContainer cont2 = container.createContainer (new Point (200, 300));
            VisualAutomatonNode node11 = cont2.createNode (new Point (40, 40));
            node11.setStarting (true);
            VisualAutomatonNode node12 = cont2.createNode (new Point (120, 80));
            VisualAutomatonNode node13 = cont2.createNode (new Point (200, 80));
            node13.setFinalNode (true);
            cont2.createEdge (node11, node12, new ComplexRexEdgeElement ('h'));
            cont2.createEdge (node12, node13, new ComplexRexEdgeElement ('o'));
            //
            container.createEdge (node1, cont2, new ComplexRexEdgeElement ('a'));
            VisualAutomatonNode node3 = container.createNode (new Point (360, 100));
            container.createEdge (cont2, node3, new ComplexRexEdgeElement ('k'));
            //container.createSelfEdge (node3, new ComplexRexEdgeElement ('a'));
            node3.setFinalNode (true);
        }
        //
    }
    
    /**
     * @param args the command line arguments
     */
    public static void main (String args[]) {
        // !!! udelat non-static metodu start, ktera toto vsechno udela
    }
    
}
