/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.auted;

import java.beans.*;
import java.awt.*;
import java.awt.geom.*;

/**
 * @author Petr Slegr
 * @version 18.05.02/2
 */
// !!! vyresit poslouchani
public class NodeGeometry extends ConnectableGeometry implements PropertyChangeListener {
    
    // for final node
    protected Shape innerCircle;
    
    // for starting node
    protected Shape outerCircle1;
    
    protected Shape outerCircle2;    
    
    protected String name;
    
    protected double textX;
    
    protected double textY;
    
    protected VisualAutomatonNode node;
    
    protected static PointPlus[] relativeSnapPoints;
    
    protected static final int SNAP_POINTS_COUNT = 16;
    
    protected PointPlus[] snapPoints;
    
    static {
        prepareRelativeSnapPoints (VisualAutomatonNode.size);
    }
    
    /**
     */
    NodeGeometry (VisualAutomatonNode node) {
        this.node = node;
        this.name = String.valueOf (node.getID ());
        //Graphics g = MainManager.getDefault ().getSystemGraphics ();
        //int height = g.getFontMetrics ().getHeight ();
        //int width = SwingUtilities.computeStringWidth (g.getFontMetrics (), name);
        int height = 10;
        int width = 10;
        int hLimit = 12; // !! const // (30 - 10) / 2 // max height
        int max = (int) (Math.sqrt (15*15  - hLimit*hLimit)); // !! const
        int maxWidth = 2 * max;
        int dif = 30 - maxWidth;
        //
        //Rectangle2D bound = g.getFontMetrics ().getStringBounds (name, g);
        double x = node.position.getX ();
        double y = node.position.getY ();
            /*
            Shape circle = new Ellipse2D.Double (x, y, 30, 30);
            if (! circle.contains (bound)) {
                shape = new RoundRectangle2D.Double (x, y, width + 2 * dif, 30, 15, 15); // !! const
            } else {
                shape = circle;
            }
             */
        if (width > maxWidth) {
            shape = new RoundRectangle2D.Double (x, y, width + 2 * dif, 30, 15, 15); // !! const
        } else {
            shape = new Ellipse2D.Double (x, y, node.size, node.size);
        }
        textX = x + dif;
        textY = y + (30 - height)/2 + height; // y + (30-height)/2 + height/2
        // final
        if (node.isFinalNode ()) {
            innerCircle = new Ellipse2D.Double (node.getPosition ().getX () + 5,
            node.getPosition ().getY () + 5,
            20, 20); // !! const
        }
        // starting
        if (node.isStarting ()) {
            outerCircle1 = new Ellipse2D.Double (node.getPosition ().getX () - 1, 
            node.getPosition ().getY () - 1,
            32, 32); // !! const
            outerCircle2 = new Ellipse2D.Double (node.getPosition ().getX () - 2, 
            node.getPosition ().getY () - 2,
            34, 34); // !! const
        }
        //
        snapPoints = new PointPlus [SNAP_POINTS_COUNT];
        for (int i = 0; i < SNAP_POINTS_COUNT; i++) {
            snapPoints[i] = relativeSnapPoints[i].add (((Ellipse2D) shape).getCenterX (),
            ((Ellipse2D) shape).getCenterY ());
            //System.out.println("absolute: " + snapPoints[i]);
        }
    }
    
    private static void prepareRelativeSnapPoints (int nodeSize) {
        relativeSnapPoints = new PointPlus [SNAP_POINTS_COUNT];
        double elemAlfa = Math.PI / SNAP_POINTS_COUNT * 2.0;
        double alfa = elemAlfa;
        double r = nodeSize / 2.0;
        for (int i = 0; i < SNAP_POINTS_COUNT; i++) {
            double x = r * Math.cos (alfa);
            double y = r * Math.sin (alfa);
            relativeSnapPoints[i] = new PointPlus (x, y);
            //System.out.println("relative: " + relativeSnapPoints[i]);
            alfa += elemAlfa;
        }
    }
    
    /**
     */
    public void paint (Graphics2D g) {
        g.draw (shape);
        if (innerCircle != null) {
            g.draw (innerCircle);
        }
        if (outerCircle1 != null && outerCircle2 != null) {
            g.draw (outerCircle1);
            g.draw (outerCircle2);
        }
        g.drawString (name, (int) textX, (int) textY);
    }
    
    /**
     */
    public boolean isOn (Point position) {
        return shape.contains (position);
    }
    
    public boolean isOnForSelection (Point position) {
        return isOn (position);
    }
    
    /**
     */
    public Point[] getSnapPoints () {
        return (Point []) snapPoints.clone ();
    }
    
    /**
     * This method gets called when a bound property is changed.
     * @param evt A PropertyChangeEvent object describing the event source
     * 	and the property that has changed.
     */
    public void propertyChange (PropertyChangeEvent evt) {
        if (evt.getPropertyName ().equals (VisualAutomatonNode.PROP_SIZE)) {
            int newSize = ((Integer) evt.getNewValue ()).intValue ();
            prepareRelativeSnapPoints (newSize);
        }
    }
    
}

