/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.auted.actions;

import org.openide.util.HelpCtx;
import org.openide.util.NbBundle;
import org.openide.util.actions.CallableSystemAction;
import org.netbeans.modules.regsup.auted.*;
import java.util.*;

/** General action to be performed on set of elements selected by user.
 *
 * @author  Petr Slegr
 */
public abstract class ElementAction extends CallableSystemAction {

    /** By default does nothing. To be overriden in subclasses.
     */
    public void performAction (VisualAutomatonElement[] selected) {
    }

    /**
     */
    public final void performAction () {
        //VisualAutomatonElement[] selected = MainManager.getDefault ().getSelectedElements ();
        VisualAutomatonElement[] selected = MainManager.getDefault ().
        getCurrentPanel ().getCurrentView ().getEditor ().getSelectedElements ();
        performAction (selected);
        if (selected.length > 0) {
            // assumes that all elements are from the same container
            VisualAutomatonContainer cont = selected[0].getContainer ();
            if (cont != null) {
                AutomatonEditor editor = MainManager.getDefault ().createEditor (cont);
                // !!!?
                // assumes that this call also will cause repaint of panel
                editor.resetSelectedElements ();
                return;
            }
        }
        // default repaint
        MainManager.getDefault ().getCurrentPanel ().repaint ();
    }
    
    protected String iconResource () {
        return "ActionIcon.gif";
    }

    public HelpCtx getHelpCtx () {
        return HelpCtx.DEFAULT_HELP;
        // If you will provide context help then use:
        // return new HelpCtx (AddNodeActionAction.class);
    }

    /** Test whether the action is currently enabled.
     * @return <code>true</code> if so
     */
    public abstract boolean isEnabled (VisualAutomatonElement[] selected);
    
    /** Perform extra initialization of this action's singleton.
     * PLEASE do not use constructors for this purpose!
    protected void initialize () {
	super.initialize ();
     * putProperty (Action.SHORT_DESCRIPTION, NbBundle.getMessage (AddNodeActionAction.class, "HINT_Action"));
    }
    */

}
