/*
                    Sun Public License Notice
 
The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/
 
The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.
 
Contributor(s): Petr Slegr.
 */

package org.netbeans.modules.regsup.debug;

import org.netbeans.modules.regsup.logic.*;
import org.netbeans.modules.regsup.auted.*;
import java.util.*;

/**
 *
 * @author  Petr Slegr
 * @version 28.12.01
 */
public class RexDebugger implements RexDebuggerEvent {
    
    //private ArrayList breakpoints;
    
    private LogicAutomaton automaton;
    
    private VisualAutomatonContainer rootContainer;
    
    private RexElement expr;
    
    /** Input string
     */
    private String input;
    
    /** Last read char
     */
    private char currentChar;
    
    /** Index to input string
     */
    private int stepCounter;
    
    /** Index to states. Means current state.
     */
    private int stateCounter;
    
    /**
     */
    private ArrayList states;
    
    /** Current state from debugger point of view - not from user one.
     */
    private RexDebuggerState currentState;
    
    /** Set of containers that have been steped-in and not steped-out yet
     */
    private Set currentOpenContainers = new HashSet ();
    
    /** Root automaton editor
     */
    private AutomatonEditor editor;
    
    /** Set of breakpoints currently applied
     */
    private Set appliedBreakpoints;
    
    /** Breakpoint which is applied is requesting suspend of debugging
     */
    // ???
    private boolean suspendRequest;
    
    private RexDebuggerStage stage = RexDebuggerStage.DEFAULT;
    
    /**
     */
    private boolean started;
    
    /**
     */
    private int interrupted;
    
    /**
     */
    private boolean readed;
    
    /** Cache for AutomatonEditors which have been opened by this debugger and there have been
     * seted elements as active within them.
     */
    private Set openEditors;
    
    // cache
    private RexDebuggerState nextStateCache;
    
    /** atribute of cache
     */
    private char nextStateCacheChar;
    
    /** atribute of cache
     */
    private RexDebuggerState nextStateCachePrevious;
    
    public static final int DEBUGGER_NOT_INTERRUPTED = 0;
    
    public static final int DEBUGGER_SUSPENDED = 1;
    
    public static final int DEBUGGER_HANGED = 2;
    
    private static Hashtable breakpoints = new Hashtable ();
    
    /** Utility field holding list of RexDebuggerListeners. */
    private transient ArrayList rexDebuggerListenerList;
    
    /** Use MainManager.createDebugger () instead to call this constructor.
     * @param editor root editor for some container
     * !!! predpoklada, ze editor bude obsahovat Rex expression.
     * !!!!!!!!!!! melo by se checkovat jestli je automat ok
     */
    public RexDebugger (AutomatonEditor editor) {
        this.editor = editor;
        rootContainer = editor.getContent ();
        automaton = rootContainer.getLogicAutomaton ();
        // !!!! expr = editor.getExpression ();
        states = new ArrayList ();
        openEditors = new HashSet ();
    }
    
    public static void addBreakpoint (VisualAutomatonContainer container, RexBreakpoint breakpoint) {
        Object o = breakpoints.get (container);
        Set set;
        if (o == null) {
            set = new HashSet ();
            breakpoints.put (container, set);
        } else {
            set = (Set) o;
        }
        set.add (breakpoint);
        //System.out.println("breakpoint: " + breakpoint + "for: " + container.hashCode () + " added");
    }
    
    public static void removeBreakpoint (VisualAutomatonContainer container, RexBreakpoint breakpoint) {
        if (! containsBreakpoint (container, breakpoint)) {
            throw new IllegalStateException ("breakpoint not present: " + breakpoint);
        }
        Object o = breakpoints.get (container);
        ((Set) o).remove (breakpoint);
    }
    
    public static boolean containsBreakpoint (VisualAutomatonContainer container, RexBreakpoint breakpoint) {
        Object o = breakpoints.get (container);
        if (o == null) {
            return false;
        } else {
            return ((Set) o).contains (breakpoint);
        }
    }
    
    public static Set getBreakpoints (VisualAutomatonContainer container) {
        Object o = breakpoints.get (container);
        if (o == null) {
            return Collections.EMPTY_SET;
        } else {
            return new HashSet ((Set) o);
        }
    }
    
    /**
     * @return number of last readed character plus one.
     */
    public int getStepNumber () {
        return stepCounter;
    }
    
    public boolean isStarted () {
        return started;
    }
    
    public boolean isHanged () {
        return stage.equals (RexDebuggerStage.HANGED);
    }
    
    public boolean isAccepted () {
        return stage.equals (RexDebuggerStage.ACCEPTED);
    }
    
    public RexDebuggerStage getStage () {
        return stage;
    }
    
    // in all following methods firstly specific event is fired and
    // general event is fired secondly
    // ! was bug: contrary order (general, specific)
    private void setStageStarted () {
        this.stage = RexDebuggerStage.STARTED;
        fireStarted (this);
        fireStageChanged (this);
    }
    
    private void setStageSuspended () {
        this.stage = RexDebuggerStage.SUSPENDED;
        fireSuspended (this);
        fireStageChanged (this);
    }
    
    private void setStageHanged () {
        this.stage = RexDebuggerStage.HANGED;
        fireStageChanged (this);
        fireHanged (this);
    }
    
    private void setStageAccepeted () {
        this.stage = RexDebuggerStage.ACCEPTED;
        fireStageChanged (this);
        fireInputAccepted (this);
    }
    
    private void setStageMovingForward () {
        stage = RexDebuggerStage.MOVING_FORWARD;
        // !!!??? vlastni eventa
        fireStageChanged (this);
    }
    
    private void setStageMovingBackward () {
        stage = RexDebuggerStage.MOVING_BACKWARD;
        // !!!??? vlastni eventa
        fireStageChanged (this);
    }
    
    /** Returns input string of this debugger.
     */
    public String getInput () {
        return input;
    }
    
    /** Sets new input string for this debugger. If debugger is started this method changes only
     * part of string from character behind the last readed character (but the preceding
     * part of string must be the same - it is checked).
     * @param anInput new input string (complete). Null not allowed.
     */
    public void setInput (String anInput) {
        if (anInput == null) {
            throw new IllegalArgumentException ("Null not allowed");
        }
        //System.out.println ("old input: " + input);
        if (input == null) {
            // input not set yet
            input = anInput;
        } else {
            int index = getStepNumber ();
            if (index == 0) {
                //System.out.println ("seted input: " + input);
                input = anInput;
            } else {
                //System.out.println ("index: " + index);
                String oldPart = input.substring (0, index);
                //System.out.println ("oldPart: " + oldPart);
                String check = anInput.substring (0, index);
                //System.out.println ("check: " + check);
                if (! oldPart.equals (check)) {
                    throw new IllegalArgumentException ("prefix of new input string is not the same as currently readed string");
                }
                input = oldPart + anInput.substring (index);
                //System.out.println ("seted input: " + input);
            }
        }
        //System.out.println ("new input: " + input);
    }
    
    /** Utility seter
     */
    private void setCurrentState (RexDebuggerState state) {
        currentState = state;
        //System.out.println ("CURRENT STATE changed to: " + currentState);
    }
    
    /**
     */
    public void start () {
        // upravit pro LA.getStartingNodes ()
        RexDebuggerState state = new RexDebuggerState ();
        Set startingNodes = automaton.getStartingNodes ();
        for (Iterator it = startingNodes.iterator (); it.hasNext (); ) {
            VisualAutomatonNode startingNode = rootContainer.find ((LogicAutomatonNode) it.next ());
            state.addNode (startingNode);
        }
        state.setUser (true); // ! not necessary but better due to moveBack () impl
        states.add (state);
        setCurrentState (state);
        stateCounter++;
        //System.out.println ("stateCounter: " + stateCounter);
        //System.out.println ("states.size: " + states.size ());
        // !!!! odstranit pro prechodu na stage
        started = true;
        setStageStarted ();
        //fireStarted (this);
        //notifyEditor (false);
        //System.out.println("TEST:currentStates: " + currentState.getPureNodes ());
    }
    
    /** Reads input a increments step counter
     */
    private char readInput () {
        return input.charAt (stepCounter++);
        // !!!!???? co treba firovat eventu
    }
    
    /**
     * @side sets currentChar
     */
    private char unReadInput () {
        currentChar = input.charAt (--stepCounter); // note: predecrement
        fireCharacterUnReaded (this);
        return currentChar;
    }
    
    /**
     * @side sets next-state cache
     */
    private RexDebuggerState prepareNextStateFromCurrent () {
        /*
        if (currentChar == nextStateCacheChar && currentState.equals (nextStateCachePrevious) &&
        nextStateCache != null) {
            return nextStateCache;
        }
         */
        RexDebuggerState state = prepareNextState (input.charAt (stepCounter), currentState);
        /*
        nextStateCache = state;
        nextStateCachePrevious = currentState;
        nextStateCacheChar = currentChar;
        System.out.println ("STATE: " + state);
         */
        return state;
    }
    
    /**
     * @side yes // !!! asi uz ne
     */
    private RexDebuggerState prepareNextState (char inputChar, RexDebuggerState currState) {
        RexDebuggerState nextState = new RexDebuggerState ();
        boolean noRead = false;
        //
        if (currState.containsContainers ()) {
            noRead = true;
            // "to step-in" containers
            Set cons = currState.getContainers ();
            Iterator it = cons.iterator ();
            while (it.hasNext ()) {
                VisualAutomatonContainer cont = (VisualAutomatonContainer) it.next ();
                // upravit pro LA.getStartingNodes ()
                /*
                nextState.addNode (cont.find (cont.getLogicAutomaton ().getStartingNode ()));
                 */
            }
            // container that will be marked as stepED in
            //currentOpenContainers.addAll (currState.getContainers ());
        }
        if (currState.containsFinalNodes ()) {
            noRead = true;
            // "to step-out containers
            Set cons = currState.getFinalNodes ();
            Iterator it = cons.iterator ();
            while (it.hasNext ()) {
                VisualAutomatonNode node =(VisualAutomatonNode) it.next ();
                // !!! funkcionalita presunuta do RexDebuggerState
                if (node.getSuperContainer () != null) {
                    VisualAutomatonContainer container = node.getContainer ();
                    // !!!!!!!! davat zavrene containery zvlast !!!!
                    nextState.addPureNode (container);
                    nextState.addClosedContainer (container);
                }
            }
        }
        //
        
        //
        if (noRead) {
            nextState.addPureNodes (currState.getPureNodes ());
            //System.out.println ("nextState (without read) prepared: " + nextState);
            return nextState;
        }
        // read
        //final char inputChar = readInput ();
        Set nextableNodes = currState.getPureNodes ();
        Iterator it = nextableNodes.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonNode node = (VisualAutomatonNode) it.next ();
            setNextStateForNode (nextState, node, inputChar);
        }
        nextState.setNumber (stepCounter);
        //System.out.println ("nextState (with read) prepared: " + nextState);
        return nextState;
    }
    
    /**
     */
    private RexDebuggerState prepareNextState2 (char inputChar, RexDebuggerState currState) {
        RexDebuggerState nextState = new RexDebuggerState ();
        boolean noRead = false;
        //
        if (currState.containsContainers ()) {
            noRead = true;
            // "to step-in" containers
            Set cons = currState.getContainers ();
            Iterator it = cons.iterator ();
            while (it.hasNext ()) {
                VisualAutomatonContainer cont = (VisualAutomatonContainer) it.next ();
                // upravit pro LA.getStartingNodes ()
                /*
                nextState.addNode (cont.find (cont.getLogicAutomaton ().getStartingNode ()));
                 */
            }
        }
        if (currState.containsFinalNodes ()) {
            noRead = true;
            // "to step-out containers
            Set cons = currState.getFinalNodes ();
            Iterator it = cons.iterator ();
            while (it.hasNext ()) {
                VisualAutomatonNode node =(VisualAutomatonNode) it.next ();
                // !!! funkcionalita presunuta do RexDebuggerState
                if (node.getSuperContainer () != null) {
                    nextState.addPureNode (node.getContainer ());
                }
            }
        }
        if (noRead) {
            nextState.addPureNodes (currState.getPureNodes ());
            //System.out.println ("nextState prepared: " + nextState);
            return nextState;
        }
        // read
        //final char inputChar = readInput ();
        Set nextableNodes = currState.getPureNodes ();
        Iterator it = nextableNodes.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonNode node = (VisualAutomatonNode) it.next ();
            setNextStateForNode (nextState, node, inputChar);
        }
        nextState.setNumber (stepCounter);
        //System.out.println ("nextState (with read) prepared: " + nextState);
        return nextState;
    }
    
    /**
     */
    private void setNextStateForNode (RexDebuggerState state, VisualAutomatonNode node, char inputChar) {
        Set logicNodes = new HashSet ();
        Set logicEdges = new HashSet ();
        LogicAutomatonNode logicNode = node.getLogicNode ();
        VisualAutomatonContainer currentContainer = node.getContainer ();
        logicNode.getNext (inputChar, logicEdges, logicNodes);
        Iterator it;
        it = logicEdges.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonEdge logicEdge = (LogicAutomatonEdge) it.next ();
            state.addEdge (currentContainer.find (logicEdge));
        }
        it = logicNodes.iterator ();
        while (it.hasNext ()) {
            LogicAutomatonNode logNode = (LogicAutomatonNode) it.next ();
            state.addNode (currentContainer.find (logNode));
        }
    }
    
    /*
    private void __stepWithRead () {
        char inputChar = readInput ();
        __step (inputChar);
        while (! currentState.containsPureNodesOnly ()) {
            __step (inputChar);
        }
    }
     */
    
    private void resume () {
        interrupted = DEBUGGER_NOT_INTERRUPTED;
    }
    
    /** Performs one "Core Step" in debugging
     */
    private void __step (/* char inputChar */) {
        //RexDebuggerState nextState = prepareNextState (readInput (), currentState);
        if (currentState.containsPureNodesOnly ()) {
            currentChar = readInput ();
            readed = true;
        }
        if (currentState.containsContainers ()) {
            Set containers = currentState.getContainers ();
            // selects first from container
            // !? what about different selection policy?
            VisualAutomatonContainer cont = (VisualAutomatonContainer) containers.iterator ().next ();
            currentOpenContainers.addAll (currentState.getContainers ());
            fireContainerStepedIn (cont, this);
        }
        if (currentState.containsClosedContainers ()) {
            Set containers = currentState.getClosedContainers ();
            // selects first from container
            // !? what about different selection policy?
            VisualAutomatonContainer cont = (VisualAutomatonContainer) containers.iterator ().next ();
            currentOpenContainers.removeAll (currentState.getClosedContainers ());
            fireContainerStepedOut (cont, this);
        }
        /*
        System.out.println ("currentOpenContainers: " + currentOpenContainers);
        currentOpenContainers.addAll (currentState.getContainers ());
        currentOpenContainers.removeAll (currentState.getClosedContainers ());
         */
        RexDebuggerState nextState = prepareNextState (currentChar, currentState);
        //System.out.println ("currentOpenContainers: " + currentOpenContainers);
        //
        // !!!??? reached vs. entered?
        if (nextState.containsContainers ()) {
            Set containers = nextState.getContainers ();
            // selects first from container
            // !? what about different selection policy?
            VisualAutomatonContainer cont = (VisualAutomatonContainer) containers.iterator ().next ();
            fireContainerReached (cont, this);
        }
        states.add (nextState);
        stateCounter++;
        setCurrentState (nextState);
        evalBreakpoints ();
        //notifyEditor (readed);
        if (readed) {
            // !!!??? nefirovat primo v metode readInput
            fireCharacterReaded (this);
            readed = false;
        }
        if (appliedBreakpoints.size () > 0) {
            fireBreakpointApplied (this);
            //System.out.println ("breakpoint applied");
        }
        if (suspendRequest) {
            // !!!! odstranit po migraci na stages
            interrupted = DEBUGGER_SUSPENDED;
            setStageSuspended ();
            suspendRequest = false;
        }
        if (isInputRead () && currentState.containsTopFinalNode ()) {
            interrupted = DEBUGGER_SUSPENDED;
            setStageAccepeted ();
        } else {
            //if (nextState.getNodes ().size () == 0) {
            if (! isMoveForwardPossible ()) {
                // !!!! odstranit po migraci na stages
                interrupted = DEBUGGER_HANGED;
                setStageHanged ();
            }
        }
    }
    
    
    /**
     */
    private void evalBreakpoints () {
        Set breaks = getBreakpoints (editor.getContent ());
        //System.out.println("CURRENT CONTAINER: " + editor.getContent ().hashCode ());
        //System.out.println("BREAKPOINTS FOR CURRENT: " + breaks);
        Iterator it = breaks.iterator ();
        appliedBreakpoints = new HashSet ();
        while (it.hasNext ()) {
            RexBreakpoint breakPoint = (RexBreakpoint) it.next ();
            //System.out.println ("Breakpoint: " + breakPoint);
            if (breakPoint.applies (currentState)) {
                appliedBreakpoints.add (breakPoint);
                if (breakPoint.isSuspend ()) {
                    suspendRequest = true;
                    //System.out.println("YYYYYYYYYYYYYYY");
                }
            }
        }
    }
    
    /** Notifies editor about currently active elements
     * @param reset
     */
    /*
    private void notifyEditor (boolean reset) {
        if (reset) {
            Iterator it = openEditors.iterator ();
            while (it.hasNext ()) {
                AutomatonEditor editor = (AutomatonEditor) it.next ();
                editor.resetActiveElements ();
            }
            openEditors = new HashSet (); // reset of cache
        }
        //
        Set nodes = currentState.getNodes ();
        Iterator it = nodes.iterator ();
        while (it.hasNext ()) {
            // !!! aaaa spolecny kod
            VisualAutomatonNode node = (VisualAutomatonNode) it.next ();
            VisualAutomatonContainer container = node.getContainer ();
            AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
            openEditors.add (editor);
            editor.addActiveElement (node);
        }
        it = currentState.getEdges ().iterator ();
        while (it.hasNext ()) {
            // !!! aaaa
            VisualAutomatonEdge edge = (VisualAutomatonEdge) it.next ();
            VisualAutomatonContainer container = edge.getContainer ();
            AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
            openEditors.add (editor);
            editor.addActiveElement (edge);
        }
        //
        editor.resetActiveInContainers (); // was missing
        it = currentOpenContainers.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonContainer cont = (VisualAutomatonContainer) it.next ();
            AutomatonEditor editor = MainManager.getDefault ().createEditor (cont.getContainer ());
            editor.addActiveInContainer (cont);
        }
        //
        // ! policy: makes visible first node
        VisualAutomatonNode node = (VisualAutomatonNode) nodes.iterator ().next ();
        // docasne
        //MainManager.getDefault ().createEditor (node.getContainer ()).makeElementVisible (node);
    }
     */
    
    /**
     */
    // !!!!! predelat: nemela by se volat __step, ale vnorit se jen do specifikovaneho kontejneru ???
    public void stepInto (VisualAutomatonContainer container) {
        if (! currentState.containsContainer (container)) {
            throw new IllegalStateException ("wrong debugger state");
        }
        /*
        EditorWindow window = MainManager.getDefault ().getCurrentWindow ();
        AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
        window.changeViewTo (editor);
         */
        setStageMovingForward ();
        suspendRequest = true; // !!!!!???? hack, aby se nemusel davat breakpoint
        __step ();
        currentState.setUser (true);
        setStageSuspended (); // this can be done also inside __step ()
    }
    
    /**
     */
    public void stepOver (VisualAutomatonContainer container) {
        if (! started) {
            throw new IllegalStateException ("debugger is not started: " + this);
        }
        if (! currentState.containsContainer (container)) {
            throw new IllegalStateException ("wrong debugger state: " + currentState);
        }
        setStageMovingForward ();
        RexBreakpoint br = new OverContainerBreakpoint (container);
        editor.addBreakpoint (br);
        resume ();
        while (interrupted == DEBUGGER_NOT_INTERRUPTED) {
            __step ();
        }
        editor.removeBreakpoint (br);
        currentState.setUser (true);
        setStageSuspended (); // this can be done also inside __step ()
    }
    
    /** Finds first RexDebuggerState with pure nodes only. Used in step () method to find nodes
     * for seting breakpoint.
     */
    // !!?? param inputChar neni mozna potreba
    private Set prepareBreakpointNodes (char inputChar) {
        RexDebuggerState state = prepareNextState2 (inputChar, currentState);
        while (! state.containsPureNodesOnly ()) {
            state = prepareNextState (inputChar, state);
        }
        return state.getNodes (); // same as getPureNodes ()... :-)
    }
    
    /**
     */
    public void stepAndRead () {
        if (! started) {
            throw new IllegalStateException ("debugger is not started: " + this);
        }
        if (null == input) {
            throw new IllegalStateException ("input not seted: " + this);
        }
        setStageMovingForward ();
        resume ();
        /// !!!!??? co vlastni sada systemovych breakpointu ???
        // nove reseni
        Set breakNodeBreakpoints = new HashSet ();
        char inputChar = input.charAt (stepCounter);
        // !!!! prepare sets duplicitly as prepareNextState...
        Set breakNodes = prepareBreakpointNodes (inputChar);
        Iterator it = breakNodes.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonNode n = (VisualAutomatonNode) it.next ();
            //System.out.println ("break node: " + n);
            RexBreakpoint br = NodeBreakpoint.createHiddenFor (n);
            editor.addBreakpoint (br);
            breakNodeBreakpoints.add (br);
        }
        //__stepWithRead ();
        while (interrupted == DEBUGGER_NOT_INTERRUPTED) {
            __step ();
        }
        it = breakNodeBreakpoints.iterator ();
        while (it.hasNext ()) {
            RexBreakpoint br = (RexBreakpoint) it.next ();
            editor.removeBreakpoint (br);
        }
        currentState.setUser (true);
        //
        /* nefunguje moc
        Set breakNodeBreakpoints = new HashSet ();
        char inputChar = input.charAt (stepCounter);
        // !!!! prepare sets duplicitly as prepareNextState...
        Set breakNodes = prepareBreakpointNodes (inputChar);
        System.out.println("breakpoint set: " + breakNodes);
        Iterator it = breakNodes.iterator ();
        while (it.hasNext ()) {
            VisualAutomatonNode n = (VisualAutomatonNode) it.next ();
            System.out.println ("break node: " + n);
            RexBreakpoint br = NodeBreakpoint.createHiddenFor (n);
            editor.addBreakpoint (br);
            breakNodeBreakpoints.add (br);
        }
        //
        while (interrupted == DEBUGGER_NOT_INTERRUPTED) {
            __step (inputChar);
        }
        // removes breakpoints
        it = breakNodeBreakpoints.iterator ();
        while (it.hasNext ()) {
            RexBreakpoint br = (RexBreakpoint) it.next ();
            editor.removeBreakpoint (br);
        }
         */
    }
    
    public boolean isInputRead () {
        return stepCounter == input.length ();
    }
    
    // !!! k nicemu
    public boolean containsActive (VisualAutomatonNode node) {
        // !!!!! mozna rozdelit na pure a ne-pure
        return currentState.getNodes ().contains (node);
    }
    
    /**
     */
    /*
    public void step () {
        // !!!!????
        suspendRequest = true;
        __step ();
        currentState.setUser (true); // !
    }
     */
    
    /**
     */
    public void step () {
        step (true, true);
    }
    
    /**
     */
    public void step (boolean in, boolean out) {
        // !!!!????
        //suspendRequest = true;
        setStageMovingForward ();
        __step ();
        boolean b1 = currentState.containsContainers () && in;
        boolean b2 = currentState.containsClosedContainers () && out;
        while (b1 || b2) {
            __step ();
            b1 = currentState.containsContainers () && in;
            b2 = currentState.containsClosedContainers () && out;
        }
        currentState.setUser (true);
        // !!!! pretlouka stage nastavenou v __step !!!!
        //setStageSuspended (); // this can be done also inside __step ()
    }
    
    /**
     */
    // !!!!! prejmenovat na can
    public boolean isMoveBackwardPossible () {
        if (stateCounter < 2) {
            return false;
        }
        if (RexDebuggerStage.SUSPENDED.compareTo (stage) <= 0) {
            return true;
        } else {
            return false;
        }
    }
    
    public boolean isMoveForwardPossible () {
        // input completely read
        if (isInputRead ()) {
            return false;
        }
        //char ch = currentChar;
        //currentChar = input.charAt (stepCounter + 1);
        RexDebuggerState next = prepareNextStateFromCurrent ();
        //currentChar = ch;
        System.out.println ("nextNodesAvailable: " + (next.getNodes ().size () != 0));
        return next.getNodes ().size () != 0;
    }
    
    /** Undoes (rollbacks) last user debug action
     */
    public void moveBack () {
        if (! isMoveBackwardPossible ()) {
            throw new IllegalStateException ("Cannot move back: " + this);
        }
        setStageMovingBackward ();
        /*
        if (stateCounter < 2) {
            return;
        }
         */
        // currentState
        // -> SHOULD be user
        // -> COULD be read
        if (currentState.isRead ()) {
            unReadInput ();
        }
        stateCounter--;
        RexDebuggerState state = (RexDebuggerState) states.get (stateCounter - 1);
        while (stateCounter >= 2 && ! state.isUser ()) {
            if (state.isRead ()) {
                unReadInput ();
            }
            stateCounter--;
            state = (RexDebuggerState) states.get (stateCounter - 1);
        }
        setCurrentState (state);
        //notifyEditor (true);
        setStageSuspended ();
    }
    
    /**
     */
    public void back () {
        if (! isMoveBackwardPossible ()) {
            throw new IllegalStateException ("Cannot move back: " + this);
        }
        setStageMovingBackward ();
        if (currentState.isRead ()) {
            unReadInput ();
        }
        stateCounter--;
        RexDebuggerState state = (RexDebuggerState) states.get (stateCounter - 1);
        /*
        while (stateCounter >= 2 && ! state.isUser ()) {
            if (state.isRead ()) {
                unReadInput ();
            }
            stateCounter--;
            state = (RexDebuggerState) states.get (stateCounter - 1);
        }
         */
        setCurrentState (state);
        setStageSuspended ();
    }
    
    /** Means "continue". Runs debugger until:
     * a) a breakpoint is applied
     * b) the input is accepted
     * c) the debugger is hanged
     */
    public void run () {
        while (interrupted == DEBUGGER_NOT_INTERRUPTED) {
            __step ();
        }
    }
    
    /**
     */
    public AutomatonEditor getEditor () {
        return editor;
    }
    
    /**
     */
    public RexDebuggerState getCurrentState () {
        return currentState;
    }
    
    /** Adds breakpoint to this debugger.
     *
     */
    /*
    public void addBreakpoint (RexBreakpoint brk) {
        boolean res = breakpoints.add (brk);
        if (! res) {
            throw new IllegalStateException ("breakpoint already added: " + brk);
        }
    }
     */
    
    /** Registers RexDebuggerListener to receive events.
     * @param listener The listener to register.
     */
    public synchronized void addRexDebuggerListener (RexDebuggerListener listener) {
        if (rexDebuggerListenerList == null ) {
            rexDebuggerListenerList = new ArrayList ();
        }
        rexDebuggerListenerList.add (listener);
    }
    
    /** Removes RexDebuggerListener from the list of listeners.
     * @param listener The listener to remove.
     */
    public synchronized void removeRexDebuggerListener (RexDebuggerListener listener) {
        if (rexDebuggerListenerList != null ) {
            rexDebuggerListenerList.remove (listener);
        }
    }
    
    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireInputAccepted (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).inputAccepted (event);
        }
    }
    
    /** Notifies all registered listeners about the event.
     *
     * @param event The event to be fired
     */
    private void fireHanged (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).hanged (event);
        }
    }
    
    private void fireBreakpointApplied (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).breakpointApplied (event);
        }
    }
    
    private void fireSuspended (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).suspended (event);
        }
    }
    
    private void fireStarted (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).started (event);
        }
    }
    
    private void fireCharacterReaded (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).characterReaded (event);
        }
    }
    
    private void fireCharacterUnReaded (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).characterUnReaded (event);
        }
    }
    
    private void fireContainerReached (VisualAutomatonContainer container, RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).containerReached (container, event);
        }
    }
    
    private void fireContainerStepedIn (VisualAutomatonContainer container, RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).containerStepedIn (container, event);
        }
    }
    
    private void fireContainerStepedOut (VisualAutomatonContainer container, RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).containerStepedOut (container, event);
        }
    }
    
    private void fireStageChanged (RexDebuggerEvent event) {
        ArrayList list;
        synchronized (this) {
            if (rexDebuggerListenerList == null) return;
            list = (ArrayList)rexDebuggerListenerList.clone ();
        }
        for (int i = 0; i < list.size (); i++) {
            ((RexDebuggerListener)list.get (i)).stageChanged (event);
        }
    }
    
    /** Returns this (impl of RexDebuggerEvent).
     */
    public RexDebugger getDebugger () {
        return this;
    }
    
    ////////////////////////////////////////////////////////////////////////////
    /**
     */
    // !!!!! prejmenovat na StateType
    public static class RexDebuggerStage implements Comparable {
        
        private int intern;
        
        private RexDebuggerStage (int intern) {
            this.intern = intern;
        }
        
        /** Stages are ordered as follows (poset):
         * DEFAULT < STARTED
         * STARTED < MOVING_BACKWARD
         * STARTED < MOVING_FORWARD
         * MOVING_FORWARD < SUSPENDED
         * MOVING_BACKWARD < SUSPENDED
         * SUSPENDED < HANGED
         * SUSPENDED < ACCEPTED
         */
        public int compareTo (Object obj) {
            return intern - ((RexDebuggerStage) obj).intern;
        }
        
        /** Means not started
         */
        public static final RexDebuggerStage DEFAULT = new RexDebuggerStage (0);
        
        /**
         */
        public static final RexDebuggerStage STARTED = new RexDebuggerStage (10);
        
        /**
         */
        public static final RexDebuggerStage MOVING_FORWARD = new RexDebuggerStage (20);
        
        /**
         */
        public static final RexDebuggerStage MOVING_BACKWARD = new RexDebuggerStage (21);
        
        /**
         */
        public static final RexDebuggerStage SUSPENDED = new RexDebuggerStage (30);
        
        /**
         */
        public static final RexDebuggerStage HANGED = new RexDebuggerStage (40);
        
        /**
         */
        public static final RexDebuggerStage ACCEPTED = new RexDebuggerStage (41);
        
    }
}
