/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.ide;

import org.openide.cookies.*;
import org.openide.filesystems.FileLock;
import org.openide.filesystems.FileObject;
import org.openide.loaders.DataObject;
import org.openide.text.DataEditorSupport;
import org.openide.windows.CloneableOpenSupport;
import org.openide.loaders.OpenSupport;
import org.netbeans.modules.regsup.auted.*;
import java.awt.Point;
import org.openide.windows.CloneableTopComponent;
import java.io.FileNotFoundException;
import org.xml.sax.*;
import java.util.Hashtable;
import java.io.*;

/** Support for editing a data object.
 *
 * @author Petr Slegr
 * @version 30.04.02
 */
public class RegsupOpenSupport extends OpenSupport implements OpenCookie, CloseCookie {
    
    private RegsupDataObject dataObject;
    
    /** Top-level container with dataobject data
     */
    private VisualAutomatonContainer container;
    
    /** Maps RegsupDataObject to RegsupWindow (TopComponent)
     */
    private static Hashtable windows = new Hashtable ();
    
    /** Create a new editor support.
     * @param obj the data object whose primary file will be edited as text
     */
    public RegsupOpenSupport (RegsupDataObject obj) {
        super (obj.getPrimaryEntry (), new MyEnv (obj));
        dataObject = obj;
        // Set a MIME type as needed, e.g.:
        //setMIMEType ("text/plain");
    }
    
    public void open () {
        /*
        VisualAutomatonContainer container = new VisualAutomatonContainer (null, new Point ());
         */
        RegsupWindow win = (RegsupWindow) windows.get (dataObject);
        if (win != null) {
            win.open ();
            win.requestFocus ();
            return;
        }
        FileObject fo = dataObject.getPrimaryFile ();
        AutomatonHandler handler = new AutomatonHandlerImpl ();
        try {
            AutomatonParser.parse (new InputSource (fo.getInputStream ()), handler);
        }
        catch (FileNotFoundException fnfe) {
            throw new IllegalStateException (fnfe.getClass ().toString () + fnfe.getMessage ());
        }
        catch (SAXException sex) {
            sex.printStackTrace ();
            throw new IllegalStateException (sex.getClass ().toString () + sex.getMessage ());
        }
        catch (javax.xml.parsers.ParserConfigurationException pex) {
            throw new IllegalStateException (pex.getClass ().toString () + pex.getMessage ());
        }
        catch (IOException iox) {
            throw new IllegalStateException (iox.getClass ().toString () + iox.getMessage ());
        }
        container = handler.getResult ();
        container.addContainerListener (new ModificationListenPoint ());
        AutomatonEditor editor = MainManager.getDefault ().createEditor (container);
        //MainManager.debugSetup31 (container);
        win = new RegsupWindow (dataObject, editor);
        Object res = windows.put (dataObject, win);
        win.open ();
    }
    
    /**
     */
    protected CloneableTopComponent createCloneableTopComponent () {
        return null;
    }
    
    /** Called when the document is modified.
     * Here, adding a save cookie to the object and marking it modified.
     * @return true if the modification is acceptable
     */
    /*
    protected boolean notifyModified () {
        if (! super.notifyModified ()) {
            return false;
        }
        MyDataObject obj = (MyDataObject) getDataObject ();
        if (obj.getCookie (SaveCookie.class) == null) {
            obj.setModified (true);
            // You must implement this method on the object:
            obj.addSaveCookie (new Save ());
        }
        return true;
    }
     */
    
    /** Called when the document becomes unmodified.
     * Here, removing the save cookie from the object and marking it unmodified.
     */
    /*
    protected void notifyUnmodified () {
        MyDataObject obj = (MyDataObject) getDataObject ();
        SaveCookie save = (SaveCookie) obj.getCookie (SaveCookie.class);
        if (save != null) {
            // You must implement this method on the object:
            obj.removeSaveCookie (save);
            obj.setModified (false);
        }
        super.notifyUnmodified ();
    }
     */
    
    /** A save cookie to use for the editor support.
     * When saved, saves the document to disk and marks the object unmodified.
     */
    /*
    private class Save implements SaveCookie {
        public void save () throws IOException {
            saveDocument ();
            getDataObject ().setModified (false);
        }
    }
     */
    
    ////////////////////////////////////////////////////////////////////////////
    private class ModificationListenPoint implements VisualAutomatonContainerListener {
        
        public void containerAdded (VisualAutomatonContainerEvent event) {
        }
        
        public void containerRemoved (VisualAutomatonContainerEvent event) {
        }
        
        /** General method called whenever some from other methods is called
         */
        public void contentChanged (VisualAutomatonContainerEvent event) {
            if (! dataObject.isModified ()) {
                dataObject.setModified (true);
                dataObject.addSaveCookie (new Save ());
            }
        }
        
        public void elementAdded (VisualAutomatonContainerEvent event) {
        }
        
        public void elementRemoved (VisualAutomatonContainerEvent event) {
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    private class Save implements SaveCookie {
        
        public void save () throws java.io.IOException {
            FileLock lock = null;
            try {
                FileObject fo = dataObject.getPrimaryFile ();
                lock = fo.lock ();
                OutputStream out = fo.getOutputStream (lock);
                new AutomatonWriter (container, out).write ();
                out.flush ();
                out.close ();
                SaveCookie save = (SaveCookie) dataObject.getCookie (SaveCookie.class);
                if (save != null) {
                    dataObject.removeSaveCookie (save);
                }
                dataObject.setModified (false);
            }
            finally {
                if (lock != null) {
                    lock.releaseLock ();
                }
            }
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    
    /** A description of the binding between the editor support and the object.
     * Note this may be serialized as part of the window system and so
     * should be static, and use the transient modifier where needed.
     */
    private static class MyEnv extends DataEditorSupport.Env {
        
        //private static final long serialVersionUID = ...L;
        
        /** Create a new environment based on the data object.
         * @param obj the data object to edit
         */
        public MyEnv (RegsupDataObject obj) {
            super (obj);
        }
        
        /** Get the file to edit.
         * @return the primary file normally
         */
        protected FileObject getFile () {
            return getDataObject ().getPrimaryFile ();
        }
        
        /** Lock the file to edit.
         * Should be taken from the file entry if possible, helpful during
         * e.g. deletion of the file.
         * @return a lock on the primary file normally
         * @throws IOException if the lock could not be taken
         */
        protected FileLock takeLock () throws IOException {
            return ((RegsupDataObject) getDataObject ()).getPrimaryEntry ().takeLock ();
        }
        
        /** Find the editor support this environment represents.
         * Note that we have to look it up, as keeping a direct
         * reference would not permit this environment to be serialized.
         * @return the editor support
         */
        public CloneableOpenSupport findCloneableOpenSupport () {
            return (RegsupOpenSupport) getDataObject ().getCookie (RegsupOpenSupport.class);
        }
        
    }
    
}
