/*
                    Sun Public License Notice

The contents of this file are subject to the Sun Public License Version
1.0 (the "License"). You may not use this file except in compliance with
the License. A copy of the License is available at http://www.sun.com/

The Original Code is the Regsup module.
The Initial Developer of the Original Code is Petr Slegr.
Portions created by Petr Slegr are Copyright (C) 2002.
All Rights Reserved.

Contributor(s): Petr Slegr.
*/

package org.netbeans.modules.regsup.logic.aut2expr;

import org.netbeans.modules.regsup.logic.*;
import java.util.ArrayList;

/**
 *
 * @author  Petr Slegr
 * @version 02.12.01
 */
public class DirectResolveProblem extends ResolveProblem {
    
    protected String problemName;
    
    protected boolean local;
    
    protected LogicAutomatonNode cause;
    
    protected boolean error;
    
    public DirectResolveProblem (String problemName, boolean error) {
        this (problemName, null, error);
    }
    
    public DirectResolveProblem (String problemName, LogicAutomatonNode cause,
    boolean error) {
        this.problemName = problemName;
        this.local = true;
        this.cause = cause;
        this.error = error;
    }
    
    /** Is local or global
     */
    public boolean isLocal () {
        return local;
    }
    
    /** If this problem is local return "cause", i.e. the problem node.
     * @return null if this problem is not local
     */
    public LogicAutomatonNode getCause () {
        return cause;
    }
    
    /** Is error or warning
     */
    public boolean isError () {
        return error;
    }

    public boolean isProxy () {
        return false;
    }    

    public void setName (String name) {
        problemName = name;
    }
    
    public String getName () {
        return problemName;
    }
    
    public String toString () {
        return problemName;
    }
    
    // following problems could be either errors or warnings
    
    ////////////////////////////////////////////////////////////////////////////
    public static class BlankNode extends DirectResolveProblem {
        
        public BlankNode (LogicAutomatonNode cause, boolean error) {
            super ("Node without output edge", cause, error);
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    public static class DullNode extends DirectResolveProblem {
        
        public DullNode (LogicAutomatonNode cause, boolean error) {
            super ("Node is not final but has no output edge", cause, error);
        }
        
    }
    
    ////////////////////////////////////////////////////////////////////////////
    public static class IsolatedNode extends DirectResolveProblem {
        
        public IsolatedNode (LogicAutomatonNode cause, boolean error) {
            super ("Isolated node", cause, error);
        }
    }
    
    ////////////////////////////////////////////////////////////////////////////
    public static class NonDeterminNode extends DirectResolveProblem {
        
        public NonDeterminNode (LogicAutomatonNode cause, boolean error) {
            super ("Non-deterministic node", cause, error);
        }
    }    

    ////////////////////////////////////////////////////////////////////////////
    public static class UnreachableNode extends DirectResolveProblem {
        
        public UnreachableNode (LogicAutomatonNode cause, boolean error) {
            super ("Unreachable node", cause, error);
        }
    }    

    /**
     * @param args the command line arguments
     */
    public static void main (String args[]) {
    }
    
}
