/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline;

import javax.swing.event.*;
import java.awt.event.*;
import javax.swing.tree.*;
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;

import org.openide.WizardDescriptor;
import org.openide.util.*;

import org.netbeans.modules.vcscore.*;
import org.netbeans.modules.vcscore.cmdline.RelativeMountPanel;

/**
 *
 * @author  mkleint
 * @version
 */
public class Cust4MountPanel extends RelativeMountPanel implements CvsCustomPanel, TreeSelectionListener {

    private NewCvsCustomizer customizer;
    private CvsFileSystem fs;
    private ArrayList listeners;
    private boolean isLastPanel = false;
    
    private volatile boolean isValid;

    static final long serialVersionUID = 5574652503187921114L;
    
    public Cust4MountPanel() {
        super();
        //initComponents ();
        isValid = false;
        //trRelMount.addTreeSelectionListener(this);
    }

    public String getName() {
        return org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.mountDialogLabel");
    }
    
    public void setLast(boolean last) {
        isLastPanel = last;
    }
/** last panel check.
 * @return true if last else false.
 */    
    
    public boolean isLast() {
        D("is last");
        return isLastPanel;
    }
    
    public int getNext() {
        return 2;
    }

    
    private void fireChangeState() {
        ArrayList list;
        synchronized (this) {
            if (this.listeners == null)
                return;
            list = (ArrayList) this.listeners.clone();
        }
        ChangeEvent event = new ChangeEvent (this);
        for (int i=0; i<list.size(); i++)
            ((ChangeListener)list.get (i)).stateChanged (event);
    }


    private synchronized void addChangeListener (ChangeListener l) {
        if (listeners == null) {
            listeners = new ArrayList ();
        }
        listeners.add (l);
    }

    private synchronized void removeChangeListener (ChangeListener l) {
        if (listeners != null) {
            listeners.remove (l);
        }
    }

    public void waitForCheckout() {
        isValid= false;
        final NewCvsCustomizer fCustom = customizer;
        if (customizer.isWaiting()) {
            new Thread ("CVS-WaitingForCheckout") {
                public void run() {
                    try {
                        while (true) {
                            if (fCustom.isWaiting()) {
                                setInfoLabel(org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.waitingForCheckout"));
                            } else {
                                setInfoLabel(org.openide.util.NbBundle.getBundle(NewCvsCustomizer.class).getString("NewCvsCustomizer.checkoutFinished"));
                                initTree(fCustom.rootDir, fCustom.relMount);
                                isValid = true;
                                setEnabledTree(true);
                                fireChangeState();
                                return;
                            }
                            Thread.sleep(3000);
                        }
                    } catch (InterruptedException e) {
                        initTree(fCustom.rootDir, fCustom.relMount);
                        isValid = true;
                        setEnabledTree(true);
                        fireChangeState();
                        return;
                    }
                    
                }
            }.start();
        }
        else {
            isValid = true;
            initTree(customizer.rootDir, customizer.relMount);
            setEnabledTree(true);
        }
    }
    
   private void fillTextFields() {
     BufferedReader buff = null;    
     String dirName = customizer.rootDir;
     dirName = dirName + (customizer.relMount.equals("") ? "" : (File.separator + customizer.relMount));
     if (customizer.relMount.equals("")) {
         //no relative mount point.. just working dir..
         File current = new File(dirName);
         if (!current.exists()) return;
         File[] files = current.listFiles(new FileFilter()  {
              public boolean accept(File f) {
                 if (f.isDirectory() && f.getName().equalsIgnoreCase("CVS")) return true;
                 return false;
              }
          });
          if (files.length == 0) { //found cvs dir in directory
             files = current.listFiles(); 
             File sub = files[0]; // no matter which we pick.. alll should be the same..
             dirName = sub.getAbsolutePath();
          }
     }
     dirName = dirName + File.separator + "CVS" + File.separator + "Root";
     File dirFile = new File(dirName);
     if (dirFile.exists()) {
        try { 
          buff = new BufferedReader(new InputStreamReader(new FileInputStream(dirFile.getAbsolutePath())));
          String line = buff.readLine();
          if (line.startsWith(":")) {
            line = line.substring(1);
            StringTokenizer token = new StringTokenizer(line, ":",false);
            if (token.hasMoreTokens()) {
              customizer.serverType = token.nextToken();
              if (customizer.serverType.equals("local"))  {
                 customizer.repository =  line.substring(customizer.serverType.length() + 1);
              } else {   //some kind of server..
                if (token.hasMoreTokens()) {
                   String userServer =  token.nextToken();   
                   customizer.userName = userServer.substring(0,userServer.indexOf('@'));
                   customizer.serverName = userServer.substring(userServer.indexOf('@') + 1);
                }
                if (token.hasMoreTokens())  customizer.repository = token.nextToken();
              }  
            }      
          }    
        } catch (IOException exc) { //doesn't matter - nothing will be filled in
            customizer.userName = "";
            customizer.serverName = "";
            customizer.serverType = "local";
            customizer.repository = "";
        }              
        finally {
          if (buff != null) {
             try { 
               buff.close(); 
             } catch (IOException eIO) {}  
          }    
        }    
     } 
     else {
         customizer.userName = "";
         customizer.serverName = "";
         customizer.serverType = "local";
         customizer.repository = "";
     }
   }
    
    private static void D(String debug) {
//        System.out.println("Cust4MountPanel(): "+debug);
    }


    public static class Panel implements WizardDescriptor.Panel {

        private Cust4MountPanel panel;
        private String name;


        public Panel (String name) {
            this.name = name;
        }

        /** Get the component displayed in this panel.
         * @return the component
         */
        public synchronized java.awt.Component getComponent() {
            if (this.panel == null) {
                this.panel = new Cust4MountPanel ();
                if (this.name != null)
                    this.panel.setName (this.name);
            }
            return this.panel;
        }

        /** Help for this panel.
         * When the panel is active, this is used as the help for the wizard dialog.
         * @return the help or <code>null</code> if no help is supplied
         */
        public HelpCtx getHelp() {
            return HelpCtx.DEFAULT_HELP;
        }

        /** Provides the wizard panel with the current data--either
         * the default data or already-modified settings, if the user used the previous and/or next buttons.
         * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
         * @param settings the object representing wizard panel state, as originally supplied to {@link WizardDescriptor#WizardDescriptor(WizardDescriptor.Iterator,Object)}
         */
        public void readSettings(Object settings) {
            if (settings instanceof NewCvsCustomizer) {
                Cust4MountPanel p = (Cust4MountPanel) this.getComponent();
                p.customizer = (NewCvsCustomizer)settings;
                p.fs = p.customizer.getFileSystem();
                p.isValid = false;
                p.setTreeModel(new DefaultTreeModel(new DefaultMutableTreeNode()));
                p.setEnabledTree(false);
                p.waitForCheckout();
            }
        }
        /** Provides the wizard panel with the opportunity to update the
         * settings with its current customized state.
         * Rather than updating its settings with every change in the GUI, it should collect them,
         * and then only save them when requested to by this method.
         * Also, the original settings passed to {@link #readSettings} should not be modified (mutated);
         * rather, the (copy) passed in here should be mutated according to the collected changes.
         * This method can be called multiple times on one instance of <code>WizardDescriptor.Panel</code>.
         * @param settings the object representing a settings of the wizard
         */
        public void storeSettings(Object settings) {
            if (settings instanceof NewCvsCustomizer) {
                Cust4MountPanel p = (Cust4MountPanel) this.getComponent();
                p.customizer = (NewCvsCustomizer)settings;
                p.fs = p.customizer.getFileSystem();
                p.customizer.relMount = p.getRelMount();
                if (!p.isLastPanel)
                    p.fillTextFields();
            }
        }
        /** Test whether the panel is finished and it is safe to proceed to the next one.
         * If the panel is valid, the "Next" (or "Finish") button will be enabled.
         * @return <code>true</code> if the user has entered satisfactory information
         */
        public boolean isValid() {
            Cust4MountPanel p = (Cust4MountPanel) this.getComponent();
            if (p.isValid) {
                if (p.customizer != null)
                    p.customizer.setWait(false);
            }
            return p.isValid;
        }
        /** Add a listener to changes of the panel's validity.
         * @param l the listener to add
         * @see #isValid
         */
        public synchronized void addChangeListener(ChangeListener l) {
            Cust4MountPanel p = (Cust4MountPanel) this.getComponent();
            p.addChangeListener (l);
        }
        /** Remove a listener to changes of the panel's validity.
         * @param l the listener to remove
         */
        public synchronized void removeChangeListener(ChangeListener l) {
            Cust4MountPanel p = (Cust4MountPanel) this.getComponent();
            p.removeChangeListener (l);
        }

    }



}
