/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright 1997-2007 Sun Microsystems, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * The Original Software is NetBeans. The Initial Developer of the Original
 * Software is Sun Microsystems, Inc. Portions Copyright 1997-2006 Sun
 * Microsystems, Inc. All Rights Reserved.
 *
 * If you wish your version of this file to be governed by only the CDDL
 * or only the GPL Version 2, indicate your decision by adding
 * "[Contributor] elects to include this software in this distribution
 * under the [CDDL or GPL Version 2] license." If you do not indicate a
 * single choice of license, a recipient has the option to distribute
 * your version of this file under either the CDDL, the GPL Version 2 or
 * to extend the choice of license to its licensees as provided above.
 * However, if you add GPL Version 2 code and therefore, elected the GPL
 * Version 2 license, then the option applies only if the new code is
 * made subject to such option by the copyright holder.
 */

package org.netbeans.modules.vcs.cmdline;

import java.awt.*;
import java.io.*;
import java.util.*;
import java.beans.*;
import java.text.*;
import javax.swing.*;

import org.openide.*;
import org.openide.filesystems.FileObject;
import org.openide.filesystems.FileSystem;
import org.openide.filesystems.FileSystem.Status;
import org.openide.filesystems.FileUtil;
import org.openide.filesystems.FileStateInvalidException;
import org.openide.filesystems.AbstractFileSystem;
import org.openide.filesystems.DefaultAttributes;
import org.openide.nodes.Children;
import org.openide.util.actions.*;
import org.openide.util.NbBundle;

import org.netbeans.modules.vcscore.*;
import org.netbeans.modules.vcscore.commands.VcsCommand;
import org.netbeans.modules.vcscore.commands.VcsCommandNode;
import org.netbeans.modules.vcscore.commands.CommandExecutorSupport;
import org.netbeans.modules.vcscore.cmdline.UserCommand;
import org.netbeans.modules.vcscore.settings.GeneralVcsSettings;
import org.netbeans.modules.vcscore.commands.CommandDataOutputListener;
import org.netbeans.modules.vcscore.commands.CommandOutputListener;
import org.netbeans.modules.vcscore.commands.CommandsPool;
import org.netbeans.modules.vcscore.commands.VcsCommandExecutor;
import org.netbeans.modules.vcscore.util.*;
import org.netbeans.modules.vcscore.actions.*;
import org.netbeans.modules.vcs.cmdline.passwd.CVSPasswd;

/** CVS filesystem.
 * 
 * @author Michal Fadljevic, Martin Entlicher
 */
//-------------------------------------------
public class CvsFileSystem extends VcsFileSystem implements java.beans.PropertyChangeListener {
    
    // The table of file names ehich are defaulty ignored by cvs
    public static final String[] DEFAULT_IGNORE_FILES={".#*","#*",",*","_$*","*~","*$","*.a",
                                                       "*.bak","*.BAK","*.elc","*.exe","*.ln",
                                                       "*.o","*.obj","*.olb","*.old","*.orig",
                                                       "*.rej","*.so","*.Z",".del-*",".make.state",
                                                       ".nse_depinfo","core","CVS","CVS.adm","cvslog.*",
                                                       "RCS","RCSLOG","SCCS","tags","TAGS"};
    
    
    class CvsIgnoreList implements VcsFileSystem.IgnoreListSupport {
        
        ArrayList result = null;
        
        public ArrayList createInitialIgnoreList () {
            if (result == null) {
                result = new ArrayList();
                // Start with defaultly ingored files
                for (int i=0; i<DEFAULT_IGNORE_FILES.length; i++)
                    result.add (DEFAULT_IGNORE_FILES[i]);
                // First the CVSROOT/cvsignore
                VcsCommand coIgnoreCommand =  CvsFileSystem.this.getCommand ("CHECKOUT_IGNORE_LIST"); // NOI18N
                if (coIgnoreCommand != null) {
                    Table files = new Table();
                    files.put ("", findFileObject(""));
                    VcsCommandExecutor[] executors = VcsAction.doCommand (files,coIgnoreCommand,null,CvsFileSystem.this,null,null,
                    new CommandDataOutputListener () {
                        public void outputData (String[] data) {
                            for (int i=0; i<data.length; i++) {
                                StringTokenizer tokens = new StringTokenizer(data[i]);
                                while (tokens.hasMoreElements()) {
                                    String element = tokens.nextToken().trim();
                                    if ("!".equals(element)) result.clear();
                                    else result.add (element);
                                }
                            }
                        }
                    },null,false);
                    CommandsPool pool = CvsFileSystem.this.getCommandsPool();
                    for (int i=0; i< executors.length; i++) {
                        try {
                            pool.waitToFinish (executors[i]);
                        } catch (InterruptedException iexc) {}
                    }
                    
                }
                // Next add $HOME/.cvsignore
                GeneralVcsSettings settings = (GeneralVcsSettings)GeneralVcsSettings.findObject (GeneralVcsSettings.class,true);
                if (settings != null) {
                    File home = settings.getHome();
                    if (home != null) {
                        File userIgnoreFile = new File (home,CVS_IGNORE_FILE_NAME);
                        if (userIgnoreFile != null) {
                            BufferedReader in = null;
                            try {
                                in = new BufferedReader (new FileReader (userIgnoreFile));
                                String line = null;
                                while ((line=in.readLine())!=null) {
                                    StringTokenizer tk = new StringTokenizer (line);
                                    while (tk.hasMoreTokens()) {
                                        String element = tk.nextToken().trim();
                                        if (element.length() == 0)
                                            continue;
                                        else if ("!".equals(element)) {
                                            result.clear();
                                            continue;
                                        }
                                        else
                                            result.add (element);
                                    }
                                }
                            }catch (java.io.IOException ioException) { /* Skeep file, if can not be read*/}
                            finally {
                                if (in != null)
                                    try {
                                        in.close();
                                    }catch (java.io.IOException nestedIOException){}
                            }
                        }
                    }
                }
                // Next $CVSIGNORE
                String line = System.getProperty ("env-cvsignore"); // No I18N
                if (line != null) {
                    StringTokenizer tk = new StringTokenizer (line);
                    while (tk.hasMoreTokens()) {
                        String element = tk.nextToken().trim();
                        if (element.length() == 0)
                            continue;
                        else if ("!".equals(element)) {
                            result.clear();
                            continue;
                        }
                        else
                            result.add (element);
                    }
                }
            }
            return result;
        }
        
        public ArrayList createIgnoreList (String fileName, ArrayList parentIgnoreList) {
            org.openide.filesystems.FileObject file = findResource(fileName);
            if (file == null) return parentIgnoreList;
            ArrayList result = (ArrayList) parentIgnoreList.clone();
            FileObject cvsignoreFile = file.getFileObject(CVS_IGNORE_FILE_NAME);
            if (cvsignoreFile != null) {
                BufferedReader in = null;
                try {
                    in = new BufferedReader ( new InputStreamReader(cvsignoreFile.getInputStream()));
                    String line = null;
                    while ((line=in.readLine()) != null) {
                        StringTokenizer tk = new StringTokenizer (line);
                        while (tk.hasMoreTokens()) {
                            String element = tk.nextToken().trim();
                            if (element.length() ==0)
                                continue;
                            else if ("!".equals(element)) {
                                result.clear();
                                continue;
                            }
                            else
                                result.add (element);
                        }
                    }
                }catch (java.io.IOException e) {/*skeep file, if can not be read*/}
                finally {
                    if (in != null)
                        try {
                            in.close();
                        }catch (java.io.IOException nestedIOException) {}
                }
            }
            return result;
        }
    }
    
    
    private Debug D = new Debug ("CvsFileSystem", true); // NOI18N
    
    public final static String CVS_IGNORE_FILE_NAME=".cvsignore"; //NOI18N
    
    public final static String CVS_SERVER_LOCAL = "local"; // NOI18N
    public final static String CVS_SERVER_SERVER = "server"; // NOI18N
    public final static String CVS_SERVER_PSERVER = "pserver"; // NOI18N
    public final static String CVS_SERVER_EXT = "ext"; // NOI18N
    // suppose that CVS server only exists for UNIX/Linux.
    // The same presumption is made in CvsAction.doCommand () method
    final static String PS_SERVER = "/"; // NOI18N
    final static String PS_LOCAL = java.io.File.separator;

    private static final String COMMIT_PROMPT_REMOVE_LINES_WITH = "CVS:";

    private static String last_cvsServerType = CVS_SERVER_LOCAL;
    private static String last_cvsRoot = System.getProperty("user.home"); // NOI18N
    private static String last_cvsServer = ""; // NOI18N
    private static String last_cvsUser = System.getProperty ("user.name");
    private static String last_cvsPassword = ""; // NOI18N
    private static String last_cvsModule = ""; // NOI18N
    private static String last_cvsModuleName = ""; // NOI18N
    private static String last_cvsExePath = null; // NOI18N
    private static String last_cygwinPath = null; // NOI18N
    //private static boolean last_useUnixShell = false;

    private String cvsServerType = new String(last_cvsServerType);
    private String cvsRoot = new String(last_cvsRoot);
    private String cvsServer = new String(last_cvsServer);
    private String cvsUser = new String(last_cvsUser);
    private String cvsPassword = new String(last_cvsPassword);
    private String cvsModule = new String(last_cvsModule);
    private String cvsModuleName = new String(last_cvsModuleName);
    private String cvsExePath = (last_cvsExePath == null) ? null : new String(last_cvsExePath);
    private String cygwinPath = (last_cygwinPath == null) ? null : new String(last_cygwinPath);
    private transient boolean doInitialCheckout = false;
    //private boolean useUnixShell = last_useUnixShell;

    private static final String CACHE_FILE_FOLDER = "CVS";
    private static final String CACHE_FILE_NAME = "netbeans.cmd.cache";
    
    private boolean executePreCommit = true;

    public String getCvsRoot () { return cvsRoot;}
    public String getCvsUserName () { return cvsUser; }
    public String getCvsServer () { return cvsServer; }
    public String getCvsServerType () { return cvsServerType; }
    public String getCvsModule () { return getRelativeMountPoint();}
    public String getCvsPassword () { return cvsPassword; }
    public String getCvsModuleName () { return cvsModuleName; }
    public String getCvsExePath () { return cvsExePath; }
    public String getCygwinPath () { return cygwinPath; }
    //public boolean isUseUnixShell () { return useUnixShell; }

    /*
    public void setExecutePreCommit(boolean executePreCommit) {
        this.executePreCommit = executePreCommit;
        UserCommand cmd = getCommand("COMMIT");
        cmd.setExecutePreCommands(executePreCommit);
    }
    
    public boolean isExecutePreCommit() {
        return executePreCommit;
    }
                                                        
    /**
     * The name of the directory with CVS info.
     */
    private static final String CVS_DIRNAME = "CVS";
    /**
     * The names of files in CVS directory.
     */
    private static final String[] CVS_DIRCONTENT = {"Entries", "Repository", "Root"};
    /**
     * Whether the information dialog about need of checkout is invoked.
     */
    private transient boolean needToCheckoutInvoked = false;
    
    private static org.openide.windows.InputOutput outputCVS = null; //MK
    
        


    String getCvcRootForServerType () {
        return cvsServerType.equals (CVS_SERVER_LOCAL) ? cvsRoot : cvsUser+"@"+cvsServer+":"+cvsRoot; // NOI18N
    }

    public void setCvsRoot (String root) {
        //String uroot = root;
        cvsRoot = root;
        last_cvsRoot = new String(root);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("CVS_REPOSITORY"); // NOI18N
        var.setValue (root);
        var = (VcsConfigVariable) variablesByName.get ("CVSROOT"); // NOI18N
        String result = getCvcRootForServerType ();
        if (org.openide.util.Utilities.isUnix() || isUseUnixShell()) {
            // On UNIX I have to replace all occurrences of '\' by '\\' to be correctly interpreted by the shell
            result = org.openide.util.Utilities.replaceString(result, "\\", "\\\\");
        }
        var.setValue (result);
        D.deb (result);
    }

    public void setCvsServer (String server) {
        cvsServer = server;
        last_cvsServer = new String(server);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("CVS_SERVER"); // NOI18N
        var.setValue (server);
        setCvsRoot (cvsRoot);
    }

    public void setCvsServerType (String serverType) {
        D.deb ("serverType ("+serverType+")"); // NOI18N
        cvsServerType = serverType;
        last_cvsServerType = new String(serverType);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("SERVERTYPE"); // NOI18N
        var.setValue (serverType);
        //var = (VcsConfigVariable) variablesByName.get ("PS"); // NOI18N
        //ResourceBundle bu = NbBundle.getBundle ("org.netbeans.modules.vcs.cmdline.CommandLines"); // NOI18N
        /*
        if(serverType==CVS_SERVER_LOCAL) {
          //var.setValue (PS_LOCAL);
          getCommand ("LIST").setDataRegex (bu.getString ("REGEX_LIST"));
    } else {
          //var.setValue (PS_SERVER);
          getCommand ("LIST").setDataRegex (bu.getString ("REGEX_LIST_SRV"));
    }
        */
        setCvsModule (cvsModule);
        //setCvsRoot (cvsRoot);
        boolean isPserver = CVS_SERVER_PSERVER.equals(serverType);
        VcsCommand cmd = getCommand("LOGIN");
        if (cmd != null) cmd.setProperty(VcsCommand.PROPERTY_HIDDEN, !isPserver ? Boolean.TRUE : Boolean.FALSE);
        cmd = getCommand("LOGOUT");
        if (cmd != null) cmd.setProperty(VcsCommand.PROPERTY_HIDDEN, !isPserver ? Boolean.TRUE : Boolean.FALSE);
    }

    public void setCvsUserName (String user) {
        cvsUser = user;
        last_cvsUser = new String(user);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get("CVS_USERNAME"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("CVS_USERNAME", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("CVS_USERNAME", var);
        }
        var.setValue(user);
        setCvsRoot (cvsRoot);
    }

    void setCvsPassword (String password) {
        cvsPassword = password;
    }

    public void setCvsModule (String module) {
        cvsModule = module;
     //   setRelativeMountPoint(module); // not possible due to customizer setup
        
        last_cvsModule = new String(module);
/*        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("MODULE_S"); // NOI18N
        //if (module.length() > 0) module += "/"; // NOI18N
        var.setValue (module);
        D.deb("MODULE_S = "+var.getValue()); // NOI18N
        String osName = System.getProperty("os.name");
   */     
        module = module.replace('/', File.separatorChar);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("MODULE"); // NOI18N
        var.setValue (module);
        D.deb("MODULE = "+var.getValue()); // NOI18N
      
    }

    public void setCvsModuleName (String moduleName) {
        cvsModuleName = moduleName;
        last_cvsModuleName = new String(moduleName);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("MODULE_NAME"); // NOI18N
        var.setValue (moduleName);
        D.deb("MODULE_NAME = "+moduleName);
    }

    public void setCvsExePath (String exePath) {
        if (isUseUnixShell()) cvsExePath = exePath.replace('\\', '/');
        else cvsExePath = exePath;
        D.deb("cvsExePath = "+cvsExePath);
        last_cvsExePath = new String(cvsExePath);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("CVS_EXE"); // NOI18N
        var.setValue (cvsExePath);
    }

    public void setUseUnixShell (boolean unixShell) {
        //useUnixShell = unixShell;
        //last_useUnixShell = unixShell;
        super.setUseUnixShell(unixShell);
        setCvsExePath(((VcsConfigVariable) variablesByName.get ("CVS_EXE")).getValue());
        initCommands ();
    }

    public void setCygwinPath (String path) {
        cygwinPath = path;
        last_cygwinPath = new String(path);
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("SHELL"); // NOI18N
        var.setValue (path);
        int index = path.lastIndexOf(File.separatorChar);
        D.deb("path = "+path); // NOI18N
        if (index < 0) index = 0;
        String bin = path.substring(0, index);
        D.deb("index = "+index+", bin = "+bin); // NOI18N
        bin = bin.replace(File.separatorChar, '/');
        var = (VcsConfigVariable) variablesByName.get ("CYGWINBIN"); // NOI18N
        var.setValue (bin);
    }

    //-------------------------------------------
    static final long serialVersionUID =3105697696081480308L;
    public CvsFileSystem() {
        super ();
        this.setIgnoreListSupport(new CvsIgnoreList());
        D.deb ("CvsFileSystem()"); // NOI18N
        setConfiguration ();
        setCvsServerType (last_cvsServerType);
        this.addPropertyChangeListener(this);
        needToCheckoutInvoked = false;
        addKnownStatuses();
        addBadgeIcons();
        setOutputTab();
        setCreateVersioningSystem(true);
        D.deb("constructor done.");
    }

    private void addKnownStatuses() {
        possibleFileStatusesMap.put("Up-to-date", cvsg("CTL_Status_UpToDate"));
        possibleFileStatusesMap.put("Locally Modified", cvsg("CTL_Status_LocallyModified"));
        possibleFileStatusesMap.put("Locally Added", cvsg("CTL_Status_LocallyAdded"));
        possibleFileStatusesMap.put("Locally Removed", cvsg("CTL_Status_LocallyRemoved"));
        possibleFileStatusesMap.put("Needs Checkout", cvsg("CTL_Status_NeedsCheckout"));
        possibleFileStatusesMap.put("Needs Patch", cvsg("CTL_Status_NeedsPatch"));
        possibleFileStatusesMap.put("Needs Merge", cvsg("CTL_Status_NeedsMerge"));
        possibleFileStatusesMap.put("File had conflicts on merge", cvsg("CTL_Status_FileHadConflictOnMerge"));
        possibleFileStatusesMap.put("Unknown", cvsg("CTL_Status_Unknown"));
    }
    
    private void addBadgeIcons() {
        java.awt.Image icon;
        statusIconMap.put("Up-to-date", new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/vcs/cmdline/resources/badgeUpToDate.gif")).getImage());
        statusIconMap.put("Locally Modified", icon = new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/vcs/cmdline/resources/badgeLocModified.gif")).getImage());
        statusIconMap.put("Locally Added", icon);
        statusIconMap.put("Locally Removed", new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/vcs/cmdline/resources/badgeLocRemoved.gif")).getImage());
        statusIconMap.put("Needs Checkout", icon = new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/vcs/cmdline/resources/badgeNeedsCheckout.gif")).getImage());
        statusIconMap.put("Needs Patch", icon);
        statusIconMap.put("Needs Merge", icon);
        statusIconMap.put("File had conflicts on merge", new javax.swing.ImageIcon(getClass().getResource("/org/netbeans/modules/vcs/cmdline/resources/badgeMergeConflict.gif")).getImage());
    }

    private void setNotModifiableStatusesFromVars() {
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get("NOT_MODIFIABLE_FILE_STATUSES");
        if (var != null) {
            //ArrayList notModifiableStatuses = new ArrayList();
            String[] statuses = VcsUtilities.getQuotedStrings(var.getValue());
            setNotModifiableStatuses(Arrays.asList(statuses));
        }
    }
    
    private void setConfiguration () {
        //System.out.println("CvsFileSystem   readConfiguration ()"); // NOI18N
        ResourceBundle bu = NbBundle.getBundle ("org.netbeans.modules.vcs.cmdline.CommandLines"); // NOI18N
        Vector vars = new Vector ();
        String osName;
        String os = System.getProperty("os.name");
        if(os.indexOf("Win") >= 0) { // NOI18N
            osName = "_WIN"; // NOI18N
        } else osName = "_UNIX"; // NOI18N
        D.deb("OS = "+os+" => osName = "+osName); // NOI18N

        vars.add (new VcsConfigVariable ("FS_DISPLAY_NAME", "", bu.getString ("VAR_FS_DISPLAY_NAME"), false, false, false, "")); // NOI18N
        vars.add (new VcsConfigVariable ("MODULE", "", "", false, false, false, "")); // NOI18N
        vars.add (new VcsConfigVariable ("MODULE_S", "", "", false, false, false, "")); // NOI18N
        //vars.add (new VcsConfigVariable ("MODULE_NAME", "", "", false, false, false, bu.getString ("VAR_MODULE_SELECT"+osName))); // NOI18N
        //vars.add (new VcsConfigVariable ("CP", "", bu.getString ("VAR_CP"+osName), false, false, false));
        vars.add (new VcsConfigVariable ("PS", "", ""+java.io.File.separator, false, false, false, "")); // NOI18N
        //vars.add (new VcsConfigVariable ("CPS", "", "${classpath.separator}", false, false, false)); // NOI18N
        vars.add (new VcsConfigVariable ("CVSROOT", "", "", false, false, false, "")); // NOI18N
        vars.add (new VcsConfigVariable ("CVS_REPOSITORY", "", "", false, false, false, "")); // NOI18N
        vars.add (new VcsConfigVariable ("NUR", "", "\"", false, false, false, "")); // NOI18N

        D.deb("OK 1"); // NOI18N
        vars.add (new VcsConfigVariable ("MODPATH", "", bu.getString ("VAR_MODPATH"+osName), false, false, false, ""));
        D.deb("OK 2"); // NOI18N
        vars.add (new VcsConfigVariable ("RUN", "", bu.getString ("VAR_RUN"+osName), false, false, false, ""));
        D.deb("OK 3"); // NOI18N
        vars.add (new VcsConfigVariable ("RUNCDM", "", bu.getString ("VAR_RUNCDM"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("RUNCDW", "", bu.getString ("VAR_RUNCDW"+osName), false, false, false, ""));
        D.deb("OK 4"); // NOI18N
        vars.add (new VcsConfigVariable ("WORKDIR", "", bu.getString ("VAR_WORKDIR"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("MODDIR", "", bu.getString ("VAR_WORKDIR"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("SERVERTYPE", "", "local", false, false, false, "")); // NOI18N
        vars.add (new VcsConfigVariable ("SHOWLOCALFILES", "", "true", false, false, false, "")); // NOI18N
        vars.add (new VcsConfigVariable ("WRAPPER", "", "org.netbeans.modules.vcs.cmdline.list.CvsListCommand.class", false, false, false, "")); // NOI18N
        vars.add (new VcsConfigVariable ("NOT_MODIFIABLE_FILE_STATUSES", "", bu.getString ("VAR_NOT_MODIFIABLE_FILE_STATUSES"), false, false, false, ""));
        vars.add (new VcsConfigVariable ("QUOTE", "", bu.getString ("VAR_QUOTE"+osName), false, false, false, ""));
        
        /*
        vars.add (new VcsConfigVariable ("CHECKOUT_CMD", "", bu.getString ("VAR_CHECKOUT_CMD"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("DIFF_CMD", "", bu.getString ("VAR_DIFF_CMD"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("DIFF_USR_CMD", "", bu.getString ("VAR_DIFF_USR_CMD"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("REV_DIFF_USR_CMD", "", bu.getString ("VAR_REV_DIFF_USR_CMD"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("LOG_INFO_CMD", "", bu.getString ("VAR_LOG_INFO_CMD"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("ADD_TAG_CMD", "", bu.getString ("VAR_ADD_TAG_CMD"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("CHECKOUT_MODULES_S_CMD", "", bu.getString ("VAR_CHECKOUT_MODULES_S_CMD"+osName), false, false, false, ""));
        vars.add (new VcsConfigVariable ("CHECKOUT_MODULES_C_CMD", "", bu.getString ("VAR_CHECKOUT_MODULES_C_CMD"+osName), false, false, false, ""));
         */
        if (last_cvsExePath != null) {
            vars.add (new VcsConfigVariable ("CVS_EXE", "", last_cvsExePath, false, false, false, ""));
            cvsExePath = new String(last_cvsExePath);
        } else {
            vars.add (new VcsConfigVariable ("CVS_EXE", "", bu.getString ("VAR_CVS_EXE"), false, false, false, ""));
            cvsExePath = bu.getString ("VAR_CVS_EXE");
        }
        if (last_cygwinPath != null) {
            vars.add (new VcsConfigVariable ("SHELL", "", last_cygwinPath, false, false, false, ""));
            cygwinPath = new String(last_cygwinPath);
        } else {
            vars.add (new VcsConfigVariable ("SHELL", "", bu.getString ("VAR_SHELL"), false, false, false, ""));
            cygwinPath = bu.getString ("VAR_SHELL");
        }
        vars.add (new VcsConfigVariable ("CYGWINBIN", "", bu.getString ("VAR_CYGWINBIN"), false, false, false, ""));
        D.deb("OK 5"); // NOI18N
        super.setVariables (vars);
        if (last_cygwinPath != null) {
            setCygwinPath(cygwinPath);
        }
        //D.deb("FileSeperator = "+java.io.File.separator); // NOI18N
        //D.deb("PS = "+ ((VcsConfigVariable) variablesByName.get("PS")).getValue()); // NOI18N

        D.deb("Calling initCommands()"); // NOI18N
        initCommands();
    }

    private void initCommands () {
        // do not automatically lock files in CVS, since it is in a strong contradiction
        // with CVS philosophy (concurrent development).
        setLockFilesOn(false);
        String osName;
        String os = System.getProperty("os.name");
        if(os.indexOf("Win") >= 0) { // NOI18N
            if (isUseUnixShell()) osName = "_CYGWIN"; // NOI18N
            else osName = "_WIN"; // NOI18N
        } else osName = "_UNIX"; // NOI18N
        D.deb("OS = "+os+" => osName = "+osName); // NOI18N

        ResourceBundle bu = NbBundle.getBundle ("org.netbeans.modules.vcs.cmdline.CommandLines"); // NOI18N
        //VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("CP"); // NOI18N
        //var.setValue (bu.getString ("VAR_CP"+osName));
        VcsConfigVariable var = (VcsConfigVariable) variablesByName.get ("CVS_SERVER"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("CVS_SERVER", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("CVS_SERVER", var);
        }

        var = (VcsConfigVariable) variablesByName.get ("FS_DISPLAY_NAME"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("FS_DISPLAY_NAME", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("FS_DISPLAY_NAME", var);
        }
        var.setValue (bu.getString ("VAR_FS_DISPLAY_NAME")); // NOI18N

        /*
        var = (VcsConfigVariable) variablesByName.get ("FS_SYSTEM_NAME_ANNOTATION"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("FS_SYSTEM_NAME_ANNOTATION", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("FS_SYSTEM_NAME_ANNOTATION", var);
        }
        var.setValue (bu.getString ("VAR_FS_SYSTEM_NAME_ANNOTATION"));
        
        var = (VcsConfigVariable) variablesByName.get ("FS_DISPLAY_NAME_ANNOTATION"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("FS_DISPLAY_NAME_ANNOTATION", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("FS_DISPLAY_NAME_ANNOTATION", var);
        }
        var.setValue (bu.getString ("VAR_FS_DISPLAY_NAME_ANNOTATION"));
         */

        var = (VcsConfigVariable) variablesByName.get ("NOT_MODIFIABLE_FILE_STATUSES"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("NOT_MODIFIABLE_FILE_STATUSES", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("NOT_MODIFIABLE_FILE_STATUSES", var);
        }
        var.setValue (bu.getString ("VAR_NOT_MODIFIABLE_FILE_STATUSES")); // NOI18N

        var = (VcsConfigVariable) variablesByName.get (VcsFileSystem.VAR_STATUS_SCHEDULED_ADD);
        if (var == null) {
            var = new VcsConfigVariable(VcsFileSystem.VAR_STATUS_SCHEDULED_ADD, "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put(VcsFileSystem.VAR_STATUS_SCHEDULED_ADD, var);
        }
        var.setValue (bu.getString ("VAR_STATUS_SCHEDULED_ADD")); // NOI18N
        var = (VcsConfigVariable) variablesByName.get (VcsFileSystem.VAR_STATUS_SCHEDULED_REMOVE);
        if (var == null) {
            var = new VcsConfigVariable(VcsFileSystem.VAR_STATUS_SCHEDULED_REMOVE, "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put(VcsFileSystem.VAR_STATUS_SCHEDULED_REMOVE, var);
        }
        var.setValue (bu.getString ("VAR_STATUS_SCHEDULED_REMOVE")); // NOI18N

        var = (VcsConfigVariable) variablesByName.get ("QUOTE"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("QUOTE", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("QUOTE", var);
        }
        var.setValue (bu.getString ("VAR_QUOTE"+osName)); // NOI18N

        var = (VcsConfigVariable) variablesByName.get ("MODPATH"); // NOI18N
        D.deb("var = "+var);
        var.setValue (bu.getString ("VAR_MODPATH"+osName));
        var = (VcsConfigVariable) variablesByName.get ("RUN"); // NOI18N
        var.setValue (bu.getString ("VAR_RUN"+osName));
        var = (VcsConfigVariable) variablesByName.get ("RUNCDM"); // NOI18N
        var.setValue (bu.getString ("VAR_RUNCDM"+osName));
        var = (VcsConfigVariable) variablesByName.get ("RUNCDW"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("RUNCDW", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("RUNCDW", var);
        }
        var.setValue (bu.getString ("VAR_RUNCDW"+osName));
        var = (VcsConfigVariable) variablesByName.get ("WORKDIR"); // NOI18N
        var.setValue (bu.getString ("VAR_WORKDIR"+osName));

        var = (VcsConfigVariable) variablesByName.get ("MODDIR"); // NOI18N
        if (var == null) {
            var = new VcsConfigVariable("MODDIR", "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("MODDIR", var);
        }
        var.setValue (bu.getString ("VAR_MODDIR"+osName));

        if (isUseUnixShell() || "_UNIX".equals(osName)) {
            var = (VcsConfigVariable) variablesByName.get ("ADJUST_CHARS"); // NOI18N
            if (var == null) {
                var = new VcsConfigVariable("ADJUST_CHARS", "", "", false, false, false, null);
                getVariables().add(var);
                variablesByName.put("ADJUST_CHARS", var);
            }
            var.setValue (bu.getString ("VAR_ADJUST_CHARS_UNIX"));

            var = (VcsConfigVariable) variablesByName.get ("ADJUST_VARS"); // NOI18N
            if (var == null) {
                var = new VcsConfigVariable("ADJUST_VARS", "", "", false, false, false, null);
                getVariables().add(var);
                variablesByName.put("ADJUST_VARS", var);
            }
            var.setValue (bu.getString ("VAR_ADJUST_VARS_UNIX"));
        } else {
            var = (VcsConfigVariable) variablesByName.get ("ADJUST_CHARS"); // NOI18N
            if (var != null) {
                getVariables().remove(var);
                variablesByName.remove("ADJUST_CHARS");
            }
            var = (VcsConfigVariable) variablesByName.get ("ADJUST_VARS"); // NOI18N
            if (var != null) {
                getVariables().remove(var);
                variablesByName.remove("ADJUST_VARS");
            }
        }
        
        var = (VcsConfigVariable) variablesByName.get ("WRAPPER"); // NOI18N
        var.setValue ("org.netbeans.modules.vcs.cmdline.list.CvsListCommand.class"); // NOI18N

        var = (VcsConfigVariable) variablesByName.get ("PS"); // NOI18N
        if (isUseUnixShell())
            var.setValue ("/"); // NOI18N
        else
            var.setValue (java.io.File.separator);
        var = (VcsConfigVariable) variablesByName.get ("CD"); // NOI18N
        if (isUseUnixShell())
            var.setValue ("cd"); // NOI18N
        else {
            if (org.openide.util.Utilities.isWindows()) {
                var.setValue ("cd /D"); // NOI18N
            } else {
                var.setValue ("cd"); // NOI18N
            }
        }
        var = (VcsConfigVariable) variablesByName.get(CommandExecutorSupport.GLOBAL_INPUT_DESCRIPTOR);
        if (var == null) {
            var = new VcsConfigVariable(CommandExecutorSupport.GLOBAL_INPUT_DESCRIPTOR,
                                        "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put(CommandExecutorSupport.GLOBAL_INPUT_DESCRIPTOR, var);
        }
        var.setValue(VcsUtilities.getBundleString(bu.getString ("VAR_GLOBAL_INPUT_DESCRIPTOR"))); // NOI18N
        var = (VcsConfigVariable) variablesByName.get(CommandExecutorSupport.GLOBAL_INPUT_EXPRESSION);
        if (var == null) {
            var = new VcsConfigVariable(CommandExecutorSupport.GLOBAL_INPUT_EXPRESSION,
                                        "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put(CommandExecutorSupport.GLOBAL_INPUT_EXPRESSION, var);
        }
        var.setValue(bu.getString ("VAR_GLOBAL_INPUT_EXPRESSION")); // NOI18N

        var = (VcsConfigVariable) variablesByName.get("CVS_DEF_EXE");
        if (var == null) {
            var = new VcsConfigVariable("CVS_DEF_EXE",
                                        "", "", false, false, false, null);
            getVariables().add(var);
            variablesByName.put("CVS_DEF_EXE", var);
        }
        var.setValue(bu.getString ("VAR_CVS_DEF_EXE"+osName)); // NOI18N

        getVarValueAdjustment().setAdjust(getVariablesAsHashtable());

        UserCommand cmd;
        //Vector commands = new Vector ();
        //String moduleName = " $[? MODULE_NAME] [${MODULE_NAME}] [.]"; // NOI18N
        Children.Array children = new Children.Array();
        VcsCommandNode cvsCommands = new VcsCommandNode(children, null); // the root (container) of commands, does not represent any actual command.

        cmd = new UserCommand ();
        cmd.setDisplayName(cvsg("LAB_CVSFileSystemName"));
        cmd.setName("CVS_MAIN_MENU");
        Children.Array mainChildren = new Children.Array();
        VcsCommandNode mainCommands = new VcsCommandNode(mainChildren, cmd);
        children.add(new VcsCommandNode[] { mainCommands });
        
        D.deb("UserCommand.");
        cmd= new UserCommand ();
        D.deb("cmd = "+cmd);
        cmd.setName ("LIST"); // NOI18N
        D.deb("cmd = "+cmd);
        try {
            D.deb("refresh label = "+cvsg("CMD_Refresh"));
        } catch(Exception e) {
            D.deb("Exception: "+e);
            e.printStackTrace();
        }
        cmd.setDisplayName (cvsg("CMD_Refresh")); // NOI18N
        D.deb("cmd = "+cmd);
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LIST"+osName));
        D.deb("cmd = "+cmd);
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString ("REGEX_LIST"));
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "^(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_FILE_NAME, new Integer(0));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_STATUS, new Integer(1));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_REVISION, new Integer(2));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_TIME, new Integer(3));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_DATE, new Integer(4));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_STICKY, new Integer(5));
        cmd.setProperty(VcsCommand.PROPERTY_DISABLED_ON_STATUS, bu.getString("DISABLED_ON_STATUS_LIST"));
        //cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_WITH_PARENT | VcsCommand.EXEC_SERIAL_OF_COMMAND));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        D.deb("cmd 1 = "+cmd);

        cmd= new UserCommand ();
        cmd.setName ("LIST_SUB"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_RefreshRecursively")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LIST_SUB"+osName));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString ("REGEX_LIST_SUB"));
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "^(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_FILE_NAME, new Integer(0));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_STATUS, new Integer(1));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_REVISION, new Integer(2));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_TIME, new Integer(3));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_DATE, new Integer(4));
        cmd.setProperty(UserCommand.PROPERTY_LIST_INDEX_STICKY, new Integer(5));
        cmd.setProperty(VcsCommand.PROPERTY_DISABLED_ON_STATUS, bu.getString("DISABLED_ON_STATUS_LIST"));
        //cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION_WITH, "\"LIST "+VcsCommand.EXEC_SERIAL_WITH_PARENT+"\"");
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        D.deb("cmd 2 = "+cmd);

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        cmd= new UserCommand ();
        cmd.setName ("INIT"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Init_AN"));
        cmd.setDisplayName (cvsg("CMD_Init")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_INIT"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        /*
        cmd= new UserCommand ();
        cmd.setName ("CHECKOUT_MODULE"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_CheckOut_AN"));
        cmd.setDisplayName (cvsg("CMD_CheckOutModule")/*+moduleName*); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CHECKOUT_MODULE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_MATCHED, "checkout");
        //cmd.addPreCommand(preModule);
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("EXPORT_MODULE_DATE"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Export_AN"));
        cmd.setDisplayName (cvsg("CMD_ExportModuleByDate")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_EXPORT_MODULE_DATE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("EXPORT_MODULE_REVISION"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Export_AN"));
        cmd.setDisplayName (cvsg("CMD_ExportModuleByRevision")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_EXPORT_MODULE_REVISION"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });
         */

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        cmd= new UserCommand ();
        cmd.setName ("COMMIT"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Commit_AN"));
        cmd.setDisplayName (cvsg("CMD_Commit")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_COMMIT"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_UNMATCHED, "  -l ");
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CHANGING_NUM_REVISIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_DISABLED_ON_STATUS, bu.getString("DISABLED_ON_STATUS_COMMIT"));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });
        //commands.add (cmd);

        cmd= new UserCommand ();
        cmd.setName ("COMMIT_CMD"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_COMMIT_CMD"+osName));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_COMMIT")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("COMMIT_TEMPLATE_GETTER"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_COMMIT_TEMPLATE_GETTER"+osName));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString ("REGEX_COMMIT_TEMPLATE_GETTER")); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_IGNORE_FAIL, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("UPDATE"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Update_AN"));
        cmd.setDisplayName (cvsg("CMD_Update")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_UPDATE"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_UPDATE")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_UNMATCHED, "  -l ");
        //cmd.setRefreshParent(false);
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_ON_PACKAGE | VcsCommand.EXEC_SERIAL_OF_COMMAND));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("ADD"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Add_AN"));
        cmd.setDisplayName (cvsg("CMD_Add")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_ADD"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_ADD")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION_WITH, "\"ADD_DIR "+VcsCommand.EXEC_SERIAL_WITH_PARENT+"\"");
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_ON_PACKAGE | VcsCommand.EXEC_SERIAL_OF_COMMAND));
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES_IN_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_DISABLED_ON_STATUS, bu.getString("DISABLED_ON_STATUS_ADD"));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        //cmd.setProperty(VcsCommand.PROPERTY_LOAD_ATTRS_TO_VARS, new String[] { "isBinary", "FO_BINARY", "FO_BINARY_VALUE" });
        cmd.setProperty(VcsCommand.PROPERTY_DISTINGUISH_BINARY_FILES, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("ADD_DIR"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Add_AN"));
        cmd.setDisplayName (cvsg("CMD_Add")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_ADD_DIR"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_PARENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_WITH_PARENT | VcsCommand.EXEC_SERIAL_OF_COMMAND));
        cmd.setProperty(VcsCommand.PROPERTY_NEEDS_HIERARCHICAL_ORDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES_IN_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_DISABLED_ON_STATUS, bu.getString("DISABLED_ON_STATUS_ADD_DIR"));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_ADD_DIR")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_ADD_DIR")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REMOVE"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Remove_AN"));
        cmd.setDisplayName (cvsg("CMD_Remove")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REMOVE"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_REMOVE")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_UNMATCHED, "  -l ");
        cmd.setProperty(VcsCommand.PROPERTY_CONFIRMATION_MSG, VcsUtilities.getBundleString(bu.getString("CONFIRMATION_REMOVE_MSG")));
        cmd.setProperty(VcsCommand.PROPERTY_PROCESS_ALL_FILES, Boolean.TRUE); // I want to remove all files including *.class
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("RELEASE"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_Release")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_RELEASE"));
        //cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, bu.getString ("INPUT_DESCRIPTOR_REMOVE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_PARENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_MATCHED, " ");
        cmd.setProperty(VcsCommand.PROPERTY_CONFIRMATION_MSG, VcsUtilities.getBundleString(bu.getString("CONFIRMATION_RELEASE_MSG")));
        //cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("PRE_RELEASE"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_PRE_RELEASE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "([^?].*)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        //cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        /*
        cmd= new UserCommand ();
        cmd.setName ("DELETE_FILE"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REMOVE"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_REMOVE")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_PARENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CONFIRMATION_MSG, VcsUtilities.getBundleString(bu.getString("CONFIRMATION_DELETE_FILE_MSG")));
        //cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("DELETE_DIR"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_RELEASE"));
        //cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, bu.getString ("INPUT_DESCRIPTOR_REMOVE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_PARENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CONFIRMATION_MSG, VcsUtilities.getBundleString(bu.getString("CONFIRMATION_DELETE_DIR_MSG")));
        //cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });
         */

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        cmd= new UserCommand ();
        cmd.setName ("STATUS"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Status_AN"));
        cmd.setDisplayName (cvsg("CMD_Status")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_STATUS"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_STATUS")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("LOG"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Log_AN"));
        cmd.setDisplayName (cvsg("CMD_Log")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LOG"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_LOG")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("HISTORY"); // NOI18N
        //cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Log_AN"));
        cmd.setDisplayName (cvsg("CMD_History")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_HISTORY"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_HISTORY")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        cmd= new UserCommand ();
        cmd.setName ("IMPORT"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Import_AN"));
        cmd.setDisplayName (cvsg("CMD_Import")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_IMPORT"+osName));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_IMPORT")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_PARENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_MATCHED, "checkout");
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_IMPORT")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_IMPORT")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("CHECKOUT"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_CheckOut_AN"));
        cmd.setDisplayName (cvsg("CMD_CheckOut")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CHECKOUT"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_CHECKOUT")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_PARENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_UNMATCHED, "  -l ");
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        //cmd.setRefreshParent(false);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("EXPORT"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Export_AN"));
        cmd.setDisplayName (cvsg("CMD_Export")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_EXPORT"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_EXPORT")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        //cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        Children.Array editingChildren = new Children.Array();
        cmd= new UserCommand ();
        cmd.setName("SUBMENU_EDITING");
        cmd.setDisplayName(cvsg("SubmenuEditing"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(editingChildren, cmd) });
        
        cmd= new UserCommand ();
        cmd.setName ("EDIT"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Edit_AN"));
        cmd.setDisplayName (cvsg("CMD_Edit")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_EDIT"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_EDIT")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_EDIT")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        editingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("UNEDIT"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Unedit_AN"));
        cmd.setDisplayName (cvsg("CMD_Unedit")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_UNEDIT"));
        cmd.setProperty(UserCommand.PROPERTY_INPUT, "y\n"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_CONFIRMATION_MSG, VcsUtilities.getBundleString(bu.getString("VAR_UNEDIT_MSG")));
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_UNEDIT")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_UNEDIT")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        editingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("EDITORS"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Editors_AN"));
        cmd.setDisplayName (cvsg("CMD_Editors")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_EDITORS"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        editingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        Children.Array watchingChildren = new Children.Array();
        cmd= new UserCommand ();
        cmd.setName("SUBMENU_WATCHES");
        cmd.setDisplayName(cvsg("SubmenuWatches"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(watchingChildren, cmd) });
        
        cmd= new UserCommand ();
        cmd.setName ("WATCH_ON"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_WatchOn_AN"));
        cmd.setDisplayName (cvsg("CMD_WatchOn")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_WATCH_ON"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_WATCH_ON")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_WATCH_ON")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        watchingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("WATCH_OFF"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_WatchOff_AN"));
        cmd.setDisplayName (cvsg("CMD_WatchOff")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_WATCH_OFF"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_WATCH_OFF")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_WATCH_OFF")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        watchingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("WATCH_ADD"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_WatchSet")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_WatchSet_AN"));
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_WATCH_ADD"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_WATCH_ADD")));
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.FALSE);
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_ON_PACKAGE));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        watchingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("WATCH_ADD_DIR"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_WatchSet")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_WatchSet_AN"));
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_WATCH_ADD_DIR"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_WATCH_ADD_DIR")));
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        watchingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("WATCHERS"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Watchers_AN"));
        cmd.setDisplayName (cvsg("CMD_Watchers")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_WATCHERS"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        watchingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });
        
        Children.Array lockingChildren = new Children.Array();
        cmd= new UserCommand ();
        cmd.setName("SUBMENU_LOCKING");
        cmd.setDisplayName(cvsg("SubmenuLocking"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(lockingChildren, cmd) });
        
        cmd= new UserCommand ();
        // Set the name as LOCK1, because we don't want to do automatic locks in CVS
        cmd.setName ("LOCK1"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Lock_AN"));
        cmd.setDisplayName (cvsg("CMD_Lock")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LOCK"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_LOCK")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_LOCK")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_LOCK")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        lockingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        // Set the name as UNLOCK1, because we don't want to do automatic unlocks in CVS
        cmd.setName ("UNLOCK1"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Unlock_AN"));
        cmd.setDisplayName (cvsg("CMD_Unlock")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_UNLOCK"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_UNLOCK")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_UNLOCK")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_UNLOCK")));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        lockingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        /*
        cmd = new UserCommand();
        cmd.setName("SELECTOR_REVISIONS");
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_SELECTOR_REVISIONS"/*+osName*));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
//        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString("REGEX_PRE_CHECKOUT_MODULE")); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd = new UserCommand();
        cmd.setName("SELECTOR_BRANCHES");
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_SELECTOR_BRANCHES"/*+osName*));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
//        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString("REGEX_PRE_CHECKOUT_MODULE")); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });
         */

        Children.Array taggingChildren = new Children.Array();
        cmd= new UserCommand ();
        cmd.setName("SUBMENU_TAGGING");
        cmd.setDisplayName(cvsg("SubmenuTagging"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(taggingChildren, cmd) });
        
        cmd= new UserCommand ();
        cmd.setName ("MERGE"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_MergeWithBranch_AN"));
        cmd.setDisplayName (cvsg("CMD_MergeWithBranch")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_MERGE"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_MERGE")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_UNMATCHED, "  -l ");
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        taggingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REM_STICKY"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_RemoveStickyTag_AN"));
        cmd.setDisplayName (cvsg("CMD_RemoveStickyTag")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REM_STICKY"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_RECURSIVELY_PATTERN_UNMATCHED, "  -l ");
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_RUN_ON_MULTIPLE_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_ON_PACKAGE | VcsCommand.EXEC_SERIAL_OF_COMMAND));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_REM_STICKY")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_REM_STICKY")));
        taggingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });


        cmd= new UserCommand ();
        cmd.setName ("TAGS"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_AddTag")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_ADD_TAG"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_ADD_TAG")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_CHANGING_NUM_REVISIONS, Boolean.TRUE); // when I add a branch tag
        cmd.setProperty(VcsCommand.PROPERTY_CHANGING_REVISION, Boolean.TRUE); // when add a tag to a revision
        //cmd.setChangedRevisionVariableName("REVISION_NAME");
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_ADD_TAG")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_ADD_TAG")));
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_ON_PACKAGE | VcsCommand.EXEC_SERIAL_OF_COMMAND));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        taggingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REMOVE_TAG"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_RemoveTag")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REMOVE_TAG"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_REMOVE_TAG")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_CHANGING_NUM_REVISIONS, Boolean.TRUE); // when I add a branch tag
        cmd.setProperty(VcsCommand.PROPERTY_CHANGING_REVISION, Boolean.TRUE); // when add a tag to a revision
        //cmd.setChangedRevisionVariableName("REVISION_NAME");
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_SUCCESS_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_SUCCESS_REMOVE_TAG")));
        cmd.setProperty(VcsCommand.PROPERTY_NOTIFICATION_FAIL_MSG, VcsUtilities.getBundleString(bu.getString("NOTIFICATION_FAIL_REMOVE_TAG")));
        cmd.setProperty(VcsCommand.PROPERTY_CONCURRENT_EXECUTION, new Integer(VcsCommand.EXEC_SERIAL_ON_PACKAGE | VcsCommand.EXEC_SERIAL_OF_COMMAND));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        taggingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("BRANCHES"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_ViewBranches")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_BRANCHES"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.FALSE);
        taggingChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        cmd= new UserCommand ();
        cmd.setName ("DIFF"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Diff_AN"));
        cmd.setDisplayName (cvsg("CMD_Diff")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_DIFF"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_DIFF")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString ("REGEX_DIFF")); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("DIFF_TEXT"); // NOI18N
        //cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Diff_AN"));
        cmd.setDisplayName (cvsg("CMD_Diff_Text")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_DIFF_USR_CMD"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_DIFF")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_IGNORE_FAIL, Boolean.TRUE);
        //cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        //cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("PATCH"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_Patch")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_PATCH"));
        cmd.setProperty(VcsCommand.PROPERTY_INPUT_DESCRIPTOR, VcsUtilities.getBundleString(bu.getString ("INPUT_DESCRIPTOR_PATCH")));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "$ ^"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_IGNORE_FAIL, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        // SEPARATOR
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, null) });

        boolean isPserver = CVS_SERVER_PSERVER.equals(getCvsServerType());
        cmd= new UserCommand ();
        cmd.setName ("LOGIN"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_Login")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LOGIN"));
        cmd.setProperty(VcsCommand.PROPERTY_HIDDEN, !isPserver ? Boolean.TRUE : Boolean.FALSE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("LOGOUT"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_Logout")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LOGOUT"));
        cmd.setProperty(VcsCommand.PROPERTY_HIDDEN, !isPserver ? Boolean.TRUE : Boolean.FALSE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });


        cmd= new UserCommand ();
        cmd.setName ("REVISION_VIEW"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_RevisionView")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_VIEW"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.FALSE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        /*
        Children.Array revisionChildren = new Children.Array();
        cmd= new UserCommand ();
        cmd.setName ("REVISION_EXPLORER_MENU"); // NOI18N
        cmd.setDisplayName (cvsg("CMD_RevisionExplorerMenu")); // NOI18N
        VcsCommandNode revisionCommands = new VcsCommandNode(revisionChildren, cmd);
        children.add(new VcsCommandNode[] { revisionCommands });
         */

        cmd= new UserCommand ();
        cmd.setName ("REVISION_GETTER"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_GETTER"));
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REVISION_UPDATE"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Update_AN"));
        cmd.setDisplayName (cvsg("CMD_Update")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_UPDATE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NUM_REVISIONS, new Integer(1));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REVISION_MERGE"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_MergeRevisions_AN"));
        cmd.setDisplayName (cvsg("CMD_MergeRevisions")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_MERGE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NUM_REVISIONS, new Integer(1));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REVISION_MERGE2"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_MergeRevisions2_AN"));
        cmd.setDisplayName (cvsg("CMD_MergeRevisions2")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_MERGE"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_CHECK_FOR_MODIFICATIONS, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_NUM_REVISIONS, new Integer(2));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REVISION_DIFF"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Diff_AN"));
        cmd.setDisplayName (cvsg("CMD_Diff")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_DIFF"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString ("REGEX_DIFF")); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_NUM_REVISIONS, new Integer(1));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REVISION_DIFF2"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Diff_AN"));
        cmd.setDisplayName (cvsg("CMD_Diff")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_DIFF"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString ("REGEX_DIFF")); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_NUM_REVISIONS, new Integer(2));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("REVISION_OPEN"); // NOI18N
        //cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Update_AN"));
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_OPEN"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_NUM_REVISIONS, new Integer(1));
        cmd.setProperty(VcsCommand.PROPERTY_SUPPORTS_ADVANCED_MODE, Boolean.TRUE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        /* MK - adding for wizard customizer support */
        cmd = new UserCommand();
        cmd.setName("CUSTOMIZER");
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CUSTOMIZER"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        /* MK - adding for wizard customizer support */
        cmd = new UserCommand();
        cmd.setName("GET_MODULE_NAME");
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_GET_MODULE_NAME"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });
        
        cmd= new UserCommand ();
        /* MK for the initial checkout- does checkout more modules at once */
        cmd.setName ("CHECKOUT_MODULES"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        //cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_CheckOut_AN"));
        //cmd.setDisplayName (cvsg("CMD_CheckOutModule")/*+moduleName*/); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CHECKOUT_MODULES"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_DISPLAY_PLAIN_OUTPUT, Boolean.TRUE);
        //cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });
        
        cmd = new UserCommand ();
        cmd.setName ("CHECKOUT_IGNORE_LIST");  // NOI18N
        cmd.setDisplayName (null);
        cmd.setProperty (VcsCommand.PROPERTY_EXEC,bu.getString("EXEC_CHECKOUT_IGNORE_LIST"));
        cmd.setProperty (VcsCommand.PROPERTY_IGNORE_FAIL, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd)});

        cmd= new UserCommand ();
        cmd.setName ("LOGIN_CHECK"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LOGIN_CHECK"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("CHECKOUT_CMD"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CHECKOUT_CMD"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("DIFF_USR_CMD"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_DIFF_USR_CMD"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString("REGEX_DIFF")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_IGNORE_FAIL, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("DIFF_CMD"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_DIFF_CMD"));
        cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString("REGEX_DIFF_CMD")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_IGNORE_FAIL, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("LOG_INFO_CMD"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_LOG_INFO_CMD"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("SELECTOR_REVISIONS"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, VcsUtilities.getBundleString(bu.getString ("EXEC_SELECTOR_REVISIONS")));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("SELECTOR_BRANCHES"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, VcsUtilities.getBundleString(bu.getString ("EXEC_SELECTOR_BRANCHES")));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("SELECTOR_MODULES"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_SELECTOR_MODULES"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("CHECKOUT_MODULES_S_CMD"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CHECKOUT_MODULES_S_CMD"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("CHECKOUT_MODULES_C_CMD"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CHECKOUT_MODULES_C_CMD"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("WATCH_ACTION_STATUS"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_WATCH_ACTION_STATUS"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("WATCHERS_LIST"); // NOI18N
        cmd.setDisplayName (null); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_WATCHERS_LIST"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("VCS_ADD"); // NOI18N
        cmd.setDisplayName (null);
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_VCS_ADD"));
        //cmd.setProperty(VcsCommand.PROPERTY_LOAD_ATTRS_TO_VARS, new String[] { "isBinary", "FO_BINARY", "FO_BINARY_VALUE" });
        cmd.setProperty(VcsCommand.PROPERTY_DISTINGUISH_BINARY_FILES, Boolean.TRUE);
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName ("VCS_REMOVE"); // NOI18N
        cmd.setDisplayName (null);
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_VCS_REMOVE"));
        cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName (VcsCommand.NAME_SCHEDULE_ADD + "_CMD"); // NOI18N
        cmd.setDisplayName (null);
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_SCHEDULE_ADD_CMD"));
        //cmd.setProperty(VcsCommand.PROPERTY_LOAD_ATTRS_TO_VARS, new String[] { "isBinary", "FO_BINARY", "FO_BINARY_VALUE" });
        cmd.setProperty(VcsCommand.PROPERTY_DISTINGUISH_BINARY_FILES, Boolean.TRUE);
        //cmd.setProperty(VcsCommand.PROPERTY_REFRESH_CURRENT_FOLDER, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName (VcsCommand.NAME_SCHEDULE_ADD); // NOI18N
        cmd.setDisplayName (null);
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_SCHEDULE_ADD"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName (VcsCommand.NAME_SCHEDULE_REMOVE); // NOI18N
        cmd.setDisplayName (null);
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_SCHEDULE_REMOVE"));
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        cmd= new UserCommand ();
        cmd.setName (VcsCommand.NAME_REVISION_LIST); // NOI18N
        cmd.setDisplayName (null);
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_REVISION_LIST"));
        cmd.setProperty(VcsCommand.PROPERTY_IGNORE_FAIL, Boolean.TRUE);
        mainChildren.add (new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });

        D.deb("cmd last = "+cmd);

        /*
        cmd= new UserCommand ();
        cmd.setName ("CUSTOMIZER_LAUNCH"); // NOI18N
        //cmd.setProperty(VcsCommand.PROPERTY_ADVANCED_NAME, cvsg("CMD_Diff_AN"));
        cmd.setDisplayName (cvsg("CMD_CustomizerLaunch")); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_EXEC, bu.getString ("EXEC_CUSTOMIZER_COMMAND"));
        //cmd.setProperty(UserCommand.PROPERTY_DATA_REGEX, bu.getString ("REGEX_DIFF")); // NOI18N
        //cmd.setProperty(UserCommand.PROPERTY_ERROR_REGEX, "(.*$)"); // NOI18N
        cmd.setProperty(VcsCommand.PROPERTY_ON_FILE, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_DIR, Boolean.FALSE);
        cmd.setProperty(VcsCommand.PROPERTY_ON_ROOT, Boolean.TRUE);
        children.add(new VcsCommandNode[] { new VcsCommandNode(Children.LEAF, cmd) });
         */

        setCommands (cvsCommands);
        
        if (getUserParams() == null) setUserParamsLabels(new String[] { cvsg("CTL_GlobalUserParamsLabel") });
        setUserLocalParamsLabels(new String[] { cvsg("CTL_CommandUserParamsLabel"), cvsg("CTL_CommandUserParamsLabelImport"), cvsg("CTL_CommandUserParamsLabelCheckout") });
        setNotModifiableStatusesFromVars();
        //cmd = getCommand("COMMIT");
        //cmd.setExecutePreCommands(isExecutePreCommit());
        if (attr instanceof VcsAttributes) {
            VcsActionSupporter actionSupport = ((VcsAttributes)attr).getCurrentSupporter();
            actionSupport.addSupportForAction(UpdateCommandAction.class, "UPDATE");
            actionSupport.addSupportForAction(HistoryCommandAction.class, "LOG");
            actionSupport.addSupportForAction(CommitCommandAction.class, "COMMIT");
            actionSupport.addSupportForAction(AddCommandAction.class, "ADD");
            actionSupport.addSupportForAction(RemoveCommandAction.class, "REMOVE");
        }
        
        D.deb("initCommands done.");
    }

    /**
     * Get the full file path where cache information should be stored.
     */
    public String getCacheFileName(String path) {
        File cacheDir = getFile(path + File.separator + CACHE_FILE_FOLDER);
        if (!cacheDir.exists() || !cacheDir.isDirectory()) return null;
        else return new File(cacheDir, CACHE_FILE_NAME).getAbsolutePath();
        //return getFile(path + File.separator + CACHE_FILE_FOLDER + File.separator + CACHE_FILE_NAME).getAbsolutePath();
    }
    
    /**
     * Allows some cleanup of the document which the user is asked for.
     * In CVS all lines beginning with CVS: have to be removed.
     * doc The Document
     * promptNum the order of the document
     * docIdentif some identification that can be set in settting the listener (UserCommand).
     */
    public void filePromptDocumentCleanup(javax.swing.JTextArea ta, int promptNum, Object docIdentif) {
        javax.swing.text.Document doc = ta.getDocument();
        if (docIdentif instanceof UserCommand) {
            UserCommand cmd = (UserCommand) docIdentif;
            if (cmd.getName().indexOf("COMMIT") >= 0) {
                //int numLines = ta.getRows();
                for(int line = 0; line < ta.getLineCount(); line++) {
                    try {
                        int begin = ta.getLineStartOffset(line);
                        int end = ta.getLineEndOffset(line);
                        String lineStr = doc.getText(begin, end - begin);
                        if (lineStr.regionMatches(0, COMMIT_PROMPT_REMOVE_LINES_WITH, 0, COMMIT_PROMPT_REMOVE_LINES_WITH.length())) {
                            doc.remove(begin, end - begin);
                            line--;
                        }
                    } catch (javax.swing.text.BadLocationException exc) {
                        org.openide.TopManager.getDefault().notifyException(exc);
                    }
                }
            }
        }
    }
    
    public Hashtable getVariablesByName() {
        return variablesByName;
    }

    public FilenameFilter getLocalFileFilter() {
        return new FilenameFilter() {
                   public boolean accept(File dir, String name) {
                       return !name.equalsIgnoreCase("CVS"); // NOI18N
                   }
               };
    }

    public void propertyChange (PropertyChangeEvent evt) {
        if (evt.getPropertyName() != FileSystem.PROP_VALID) return;
        if (isValid()) {
            D.deb("Filesystem added to the repository, setting refresh time to "+refreshTimeToSet); // NOI18N
            setRefreshTime(refreshTimeToSet);
            D.deb("calling WARN");
            warnDirectoriesDoNotExists();
            D.deb("calling WARN finished.");
        } else {
            D.deb("Filesystem is not valid any more, setting refresh time to 0"); // NOI18N
            setRefreshTime(0);
        }
    }

    /**
    * Test if the directory was checked out by CVS or not.
    * @param dir the directory name to test
    * @return <code>true</code> if the directory was created by CVS, <code>false</code> if not.
    */
    private boolean isCVSDirectory(File dir) {
        D.deb("TESTING CVS dir "+dir);
        File subdir = new File(dir, CVS_DIRNAME);
        if (!subdir.isDirectory()) return false;
        for(int i = 0; i < CVS_DIRCONTENT.length; i++) {
            File cvsFile = new File(subdir, CVS_DIRCONTENT[i]);
            if (!cvsFile.isFile()) return false;
        }
        return true;
    }
    
    private boolean isCVSRoot(File dir) {
        if (isCVSDirectory(dir)) return true;
        File[] subfiles = dir.listFiles();
        if (subfiles == null) return false;
        for(int i = 0; i < subfiles.length; i++) {
            if (subfiles[i].isDirectory() && isCVSDirectory(subfiles[i])) return true;
        }
        return false;
    }
    
    public boolean isCheckoutNeeded() {
      return !isCVSRoot(getRootDirectory());
    }   

    public boolean isCheckoutNeeded(File directory) {
      return !isCVSRoot(directory);
    }
    
    /*
     * Add the root directory to the cache. This prevents LIST command to be run and fail
     * when the local directory structure was not checked out by cvs.
     *
    private void addRootToCache() {
        VcsDir dir = cache.getDir("");
        if (dir == null || (!dir.isLoaded() && !dir.isLocal())) {
            dir = new VcsDir();
            dir.setPath("");
            dir.setName("");
            dir.setLoaded(true);
            cache.readDirFinished(dir, new Vector(0), false);
        }
    }
     */

    /* Scans children for given name
     */
    public String[] children (String name) {
        if (name.length() == 0) { // We're on the root
            if (!isCVSRoot(getRootDirectory())) {
                D.deb("I'm NOT in directory checked-out by CVS !!");
                if (!needToCheckoutInvoked) {
                    javax.swing.SwingUtilities.invokeLater(new Runnable() {
                                                               public void run() {
                                                                   TopManager.getDefault ().notify (new NotifyDescriptor.Message (
                                                                                                        cvsg("DLG_CVS_NeedToCheckOutFirst"), NotifyDescriptor.Message.INFORMATION_MESSAGE));
                                                                   //needToCheckoutInvoked = false; -- will appear only once to prevent disturbing users
                                                               }
                                                           });
                    needToCheckoutInvoked = true;
                }
                //addRootToCache();
                /*
                if (cache.isDir(name)) {
                    String[] vcsFiles = null;
                    String[] files = null;
                    vcsFiles = cache.getLocalFilesAndSubdirs(name);
                    D.deb("vcsFiles="+MiscStuff.arrayToString(vcsFiles)); // NOI18N

                    String p = ""; // NOI18N
                    try{
                        p = getRootDirectory().getCanonicalPath();
                    }
                    catch (IOException e){
                        //E.err(e,"getCanonicalPath() failed"); // NOI18N
                        return new String[0]; // I failed, return nothing.
                    }
                    files = cache.dirsFirst(p+File.separator+name,vcsFiles);
                    D.deb("files="+MiscStuff.arrayToString(files)); // NOI18N
                    return files;
                } else {
                    return new String[0];
                }
                */
            }
        }
        return super.children(name);
    }

    private void readObject(ObjectInputStream in) throws
        ClassNotFoundException, IOException, NotActiveException{
        in.defaultReadObject();
        last_cvsExePath = cvsExePath;
        last_cvsModule = cvsModule;
        last_cvsModuleName = cvsModuleName;
        last_cvsPassword = cvsPassword;
        last_cvsRoot = cvsRoot;
        last_cvsServer = cvsServer;
        last_cvsServerType = cvsServerType;
        last_cvsUser = cvsUser;
        last_cygwinPath = cygwinPath;
        initCommands();
        addKnownStatuses();
        addBadgeIcons();
        setOutputTab();  //MK
        this.setIgnoreListSupport (new CvsIgnoreList());
        setCreateVersioningSystem(true);
    }


    //-------------------------------------------
    private void writeObject(ObjectOutputStream out) throws IOException {
        //D.deb("writeObject() - saving bean"); // NOI18N
        // cache is transient
        out.defaultWriteObject();
    }
    
    /**
     * Notification, that the filesystem is being added to the repository
     */
    public void addNotify() {
        if (this.doInitialCheckout) {
            Table files = new Table();
            files.put("", findResource(""));
            VcsAction.doCommand(files, getCommand("CHECKOUT"), null, this);
        }
        super.addNotify();
    }    

    
    /**
     * Set whether to do initial checkout after mount or not.
     */
    public void setInitialCheckout(boolean doInitialCheckout) {
        this.doInitialCheckout = doInitialCheckout;
    }
    
    public boolean checkLogin(String connectStr, String password) 
            throws IOException, java.net.UnknownHostException {
        CVSPasswd pasFile = new CVSPasswd(getCygwinPath());
        pasFile.loadPassFile();
        //entry = new PasswdEntry();
        pasFile.remove(connectStr);
        pasFile.add(connectStr, password); //CVSPasswd.scramble(password));
        pasFile.savePassFile();
        final StringBuffer message = new StringBuffer();
        //try {
        boolean loggedIn = CVSPasswd.checkLogin(this, message);
        /*
        } catch (java.net.UnknownHostException exc) {
            setStatus(org.openide.util.NbBundle.getBundle(LoginDialog.class).getString("LoginDialog.unknownHost"));
            return;
        } catch (java.io.IOException exc) {
            setStatus(org.openide.util.NbBundle.getBundle(LoginDialog.class).getString("LoginDialog.connectionIOError"));
            return;
        } finally {
            passwordField.setEnabled(true);
            loginButton.setEnabled(true);
        }
         */
        if (!loggedIn) {
            throw new java.io.IOException() {
                public String getLocalizedMessage() {
                    return message.toString();
                }
            };
        }
        return loggedIn;
    }
    
        //-------------------------------------------
    public void debugClear(){
        if( getDebug() ){
            try{
                outputCVS.getOut().reset();
            }catch (IOException e){}
        }
    }

    public void setOutputTab() {
        if (outputCVS == null) {
            outputCVS = TopManager.getDefault().getIO("CVS");
        }    
    }    
    
    //-------------------------------------------
    public void debug(String msg){
        if( getDebug() ){
            outputCVS.getOut().println(msg);
        }
    }

    /**
     * Print an error output. Force the message to print to the Output Window.
     * The debug property is not considered.
     * @param msg the message to print out.
     */
    public void debugErr(String msg){
        outputCVS.getOut().println(msg);
        outputCVS.select();
    }

    private String cvsg(String s) {
        //D.deb("getting: "+s);
        return NbBundle.getBundle(CvsFileSystem.class).getString(s);
        //       ("org.netbeans.modules.vcs.cmdline.BundleCVS").getString (s);
    }


}

